
#ifndef ndspy_h
#define ndspy_h

#include <stdio.h>
#include <stdlib.h>
#include <uparam.h>


#define PkDspyMaxMsgSize 32760

#define PkDspyServerMsgOpen  (65 + 1)
#define PkDspyServerMsgImage (65 + 2)
#define PkDspyServerMsgData  (65 + 3)
#define PkDspyServerMsgQuery (65 + 4)
#define PkDspyServerMsgClose (65 + 5)

/*
PtDspyUnsigned64
PtDspySigned64
*/
typedef double PtDspyFloat64;
typedef float PtDspyFloat32;

#if defined(__mips)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__mips64)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__sparc)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__alpha)
#define PkDspyByteOrderNative	PkDspyByteOrderLoHi
#else
#define PkDspyByteOrderNative	PkDspyByteOrderLoHi
#endif

#if defined(__mips64) || defined(__alpha)
typedef unsigned int PtDspyUnsigned32;
typedef int PtDspySigned32;
#else
typedef unsigned long PtDspyUnsigned32;
typedef long PtDspySigned32;
#endif

typedef unsigned short PtDspyUnsigned16;
typedef short PtDspySigned16;

typedef unsigned char PtDspyUnsigned8;
typedef char PtDspySigned8;

typedef PtDspyUnsigned32 PtDspyMsgLen;
typedef PtDspyUnsigned32 PtDspyServerMessage;

#define PkDspyClassFloat 1
#define PkDspyClassInt 2

#define PkDspyNone          0

#define PkDspyFloat32		1
#define PkDspyUnsigned32	2
#define PkDspySigned32		3
#define PkDspyUnsigned16	4
#define PkDspySigned16		5
#define PkDspyUnsigned8		6
#define PkDspySigned8		7

#define PkDspyString		8
#define PkDspyMatrix		9
#define PkDspyArrayBegin	10
#define PkDspyArrayEnd      11

#define PkDspyMaskType		8191

#define PkDspyMaskOrder (PkDspyByteOrderHiLo | PkDspyByteOrderLoHi)
#define PkDspyShiftOrder	13
#define PkDspyByteOrderHiLo	8192
#define PkDspyByteOrderLoHi	16384

/* #define PkDspyOpenPortCount 8 */

/* Added by Aghiles Kheffache for row data displays >>>> */

typedef struct PtDspyFragment_s
{
	/* WARNING, if DlFragment changes in DlSampler, change this structure too!
	 */

	/* Fragment color & opacity */
	float *color;
	float *filler;

	/* depth of this fragment */
	float depth;

	/* u/v */
	float u, v;
	
	/* fragment opacity */
	float opacity[3];

	/* Next fragment on the list (in the subsample) */
	struct PtDspyFragment_s *next;

} PtDspyFragment;

typedef struct
{
	char ID[4];

	/* Samples per pixel in x & y */
	unsigned sppx, sppy;

	/* Data width & height in pixels */
	int width, height;

	/* Array of fragment lists. NumLists = width*height*sppx*sppy */
	PtDspyFragment **fragments;

} PtDspyRawData;

/* <<<< Added by Aghiles Kheffache for row data displays */


typedef struct
{
   char *name;
   unsigned type;
} PtDspyDevFormat;


typedef struct
{
	PtDspyUnsigned32 width;
	PtDspyUnsigned32 height;
	PtDspyFloat32 aspectRatio;
} PtDspySizeInfo;

typedef struct
{
  PtDspyUnsigned8 overwrite;
  PtDspyUnsigned8 unused; /* 'interactive' in prman */
} PtDspyOverwriteInfo;

typedef struct
{
  PtDspyUnsigned8 redraw;
} PtDspyRedrawInfo;

/* added by AK */
typedef struct
{
	PtDspyUnsigned8 cooked;
} PtDspyCookedInfo;

typedef enum
{
	PkSizeQuery,
	PkOverwriteQuery,
	PkNextDataQuery,
	PkRedrawQuery,
	PkCookedQuery  /* this one is not a standard prman query, AK */
} PtDspyQueryType;

typedef enum
{
	PkDspyErrorNone = 0,
	PkDspyErrorNoMemory,
	PkDspyErrorUnsupported,
	PkDspyErrorBadParams,
	PkDspyErrorNoResource,
	PkDspyErrorUndefined,
	PkDspyErrorStop
} PtDspyError;

#define PkDspyFlagsWantsScanLineOrder 1
#define PkDspyFlagsWantsEmptyBuckets 2
#define PkDspyFlagsWantsNullEmptyBuckets 4

typedef struct
{
	int flags;
} PtFlagStuff;

typedef void * PtDspyImageHandle;
typedef void * PtDspyChannel;
typedef void * PtDspyOutput;

void
DspyMemReverseCopy(unsigned char *t, const unsigned char *s, int len);
void
DspyMemReverse(unsigned char *t, int len);

void DspyError(const char *module, const char *fmt, ...);

#if defined(WIN32) || defined(_WIN32)
#define DLLEXPORT extern "C" __declspec(dllexport)
#else
#define DLLEXPORT extern "C"
#endif

DLLEXPORT PtDspyError
DspyImageOpen(PtDspyImageHandle * image,
		const char *drivername,
		const char *filename,
		int width,
		int height,
		int paramCount,
		const UserParameter *parameters,
		int iFormatCount,
		PtDspyDevFormat *format,
		PtFlagStuff *flagstuff);

DLLEXPORT PtDspyError
DspyImageData(PtDspyImageHandle image,
		int xmin,
		int xmax,
		int ymin,
		int ymax,
		int entrysize,
		const unsigned char *data);
 
DLLEXPORT PtDspyError
DspyImageClose(PtDspyImageHandle);

DLLEXPORT  PtDspyError
DspyImageReopen(PtDspyImageHandle image,
		const char *drivername,
		const char *filename,
		int width,
		int height,
		int paramCount,
		const UserParameter *parameters,
		int iFormatCount,
		PtDspyDevFormat *format,
		PtFlagStuff *flagstuff);

DLLEXPORT PtDspyError
DspyImageDelayClose(PtDspyImageHandle);

DLLEXPORT PtDspyError
DspyImageQuery(PtDspyImageHandle,
	   PtDspyQueryType,
	   size_t,
	   void *);


typedef PtDspyError
	(*PtDspyOpenFuncPtr)(PtDspyImageHandle * image,
		const char *drivername,
		const char *filename,
		int width,
		int height,
		int paramCount,
		const UserParameter *parameters,
		int iFormatCount,
		PtDspyDevFormat *format,
		PtFlagStuff *flagstuff);

typedef PtDspyError
	(*PtDspyWriteFuncPtr)(PtDspyImageHandle image,
		int xmin,
		int xmax_plus_one,
		int ymin,
		int ymax_plus_one,
		int entrysize,
		const unsigned char *data);

#undef DLLEXPORT

typedef PtDspyError
	(*PtDspyCloseFuncPtr)(PtDspyImageHandle);
typedef PtDspyError
	(*PtDspyFlushFuncPtr)(PtDspyImageHandle);

typedef PtDspyError
	(*PtDspyReopenFuncPtr)(PtDspyImageHandle image,
		const char *drivername,
		const char *filename,
		int width,
		int height,
		int paramCount,
		const UserParameter *parameters,
		int iFormatCount,
		PtDspyDevFormat *format,
		PtFlagStuff *flagstuff);

typedef PtDspyError
	(*PtDspyDelayCloseFuncPtr)(PtDspyImageHandle);

typedef PtDspyError
	(*PtDspyQueryFuncPtr)(PtDspyImageHandle,
	   PtDspyQueryType,
	   int ,
	   void *);


#endif /* ifdef ndspy_h */





