/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/*    These coded instructions, statements, and computer programs  contain    */
/*    unpublished  proprietary and  confidential information of  Centre de    */
/*    traitement  en  imagerie virtuelle inc. (CTIV), and are protected by    */
/*    Federal copyright law. They may not be disclosed to third parties or    */
/*    or duplicated in any form, in whole or in  part, without  the  prior    */
/*    written consent of CTIV.                                                */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     rx.h
// = AUTHOR(S)
//     Charles De Lean
// = VERSION
//     $Revision$
// = DATE RELEASED
//     $Date: 2005-11-16 14:43:59 -0500 (Wed, 16 Nov 2005) $
// = RCSID
//     $Id: rx.h 8239 2005-11-16 19:43:59Z olivier $
// ===========================================================================

#ifndef __dlRX_H
#define __dlRX_H

#include <ri.h>
#include <dlInterface.h>

#ifdef  __cplusplus
extern "C" {
#endif

/*
	Public (DSO) interface for noise.
*/

DL_INTERFACE extern RtInt RxNoise(
	RtInt i_inDimension,
	RtFloat *i_in,
	RtInt i_outDimension,
	RtFloat *o_out);

DL_INTERFACE extern RtInt RxPNoise(
	RtInt i_inDimension,
	RtFloat *i_in,
	RtFloat *i_period,
	RtInt i_outDimension,
	RtFloat *o_out);

DL_INTERFACE extern RtInt RxCellNoise(
	RtInt i_inDimension,
	RtFloat *i_in,
	RtInt i_outDimension,
	RtFloat *o_out);

/*
	Public (DSO) interface for textures.
*/

DL_INTERFACE extern RtInt RxEnvironmentV(
	RtString i_fileName,
	RtInt i_firstChannel,
	RtInt i_nChannels,
	RtPoint i_dir0,
	RtPoint i_dir1,
	RtPoint i_dir2,
	RtPoint i_dir3,
	RtFloat *o_result,
	RtInt i_n,
	RtToken i_tokens[],
	RtPointer i_parms[]);

DL_INTERFACE extern RtInt RxEnvironment(
	RtString i_fileName,
	RtInt i_firstChannel,
	RtInt i_nChannels,
	RtPoint i_dir0,
	RtPoint i_dir1,
	RtPoint i_dir2,
	RtPoint i_dir3,
	RtFloat *o_result,
	...);

DL_INTERFACE extern RtInt RxShadowV(
	RtString i_fileName,
	RtInt i_firstChannel,
	RtPoint i_P0,
	RtPoint i_P1,
	RtPoint i_P2,
	RtPoint i_P3,
	RtFloat *o_result,
	RtInt i_n,
	RtToken i_tokens[],
	RtPointer i_parms[]);

DL_INTERFACE extern RtInt RxShadow(
	RtString i_fileName,
	RtInt i_firstChannel,           
	RtPoint i_P0,
	RtPoint i_P1,
	RtPoint i_P2,
	RtPoint i_P3,
	RtFloat *o_result,
	...);

DL_INTERFACE extern RtInt RxTextureV(
	RtString i_fileName,
	RtInt i_firstChannel,
	RtInt i_nChannels,
	RtFloat i_s0,
	RtFloat i_t0,
	RtFloat i_s1,
	RtFloat i_t1,
	RtFloat i_s2,
	RtFloat i_t2,
	RtFloat i_s3,
	RtFloat i_t3,
	RtFloat *o_result,
	RtInt i_n,
	RtToken i_tokens[],
	RtPointer i_parms[]);

DL_INTERFACE extern RtInt RxTexture(
	RtString i_fileName,
	RtInt i_firstChannel,
	RtInt i_nChannels,
	RtFloat i_s0,
	RtFloat i_t0,
	RtFloat i_s1,
	RtFloat i_t1,
	RtFloat i_s2,
	RtFloat i_t2,
	RtFloat i_s3,
	RtFloat i_t3,
	RtFloat *o_result,
	...);

/*
	Public (DSO) interface for information retrieval.

	RxAttribute, RxOption, and RxRendererInfo return the
	current values of the named Attribute or Option.

	The caller passes in a buffer and it is filled-in with the
	resulting requested info.  Return value is 0 on success, or
	-1 if name can't be found, or N if the supplied buffer was
	too short and needs to be at least N bytes long.  Caller
	supplies the addresses of several other return values which
	are filled in:  resulttype is the RxInfoType_t of the values
	put into the result buffer, and resultcount is the number of
	those (multibyte) items.
*/

typedef enum
{
	RxInfoFloat,
	RxInfoInteger,
	RxInfoStringV,
	RxInfoColor,
	RxInfoNormal,
	RxInfoVector,
	RxInfoPoint,
	RxInfoHPoint,
	RxInfoMPoint,
	RxInfoMatrix
} RxInfoType_t;

DL_INTERFACE extern RtInt RxAttribute (
	RtString i_name,
	RtPointer o_result, 
	RtInt i_resultLen,
	RxInfoType_t *o_resulttype,
	RtInt *o_resultcount);

DL_INTERFACE extern RtInt RxOption (
	RtString i_name,
	RtPointer o_result,
	RtInt i_resultLen,
	RxInfoType_t *o_resulttype,
	RtInt *o_resultcount);

DL_INTERFACE extern RtInt RxRendererInfo (
	RtString i_name,
	RtPointer o_result,
	RtInt i_resultLen, 
	RxInfoType_t *o_resulttype,
	RtInt *o_resultcount);

/*
	Public (DSO) interface for point transformation.
*/

DL_INTERFACE extern RtInt RxTransformPoints(
	RtToken i_fromspace,
	RtToken i_tospace,
	RtInt i_n,
	RtPoint io_p[],
	RtFloat i_time);

/*
	Public (DSO) interface for file caching.
*/
DL_INTERFACE extern RtString RxCacheFile(
	RtString i_filename);

#ifdef  __cplusplus
}
#endif

#endif /* ndef __dlRX_H */
