/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/
 
// ===========================================================================
// = LIBRARY
//     shaderCompiler
// = FILENAME
//     shadeop.h
// = VERSION
//     $ $
// = DATE RELEASED
//     $Date: 2007-01-26 15:15:24 -0500 (Fri, 26 Jan 2007) $
// = RCSID
//     $Id: shadeop.h 9718 2007-01-26 20:15:24Z olivier $
// ===========================================================================

#ifndef __shadeop_H
#define __shadeop_H

#include <string.h>
#include <stdlib.h>

#ifdef _WIN32
#define DLEXPORT __declspec(dllexport)
#include <malloc.h>
#else
#define DLEXPORT
#endif

/****************************************************************************
 * How to use this file.
 *
 * This file must be included when we define a shadeop in a DSO.
 *
 * The macro SHADEOP_TABLE creates a table describing the functions 
 * defined in the DSO.  
 *
 * The macro SHADEOP_INIT creates the prototype of the initialization
 * function.
 *
 * The macro SHADEOP creates the prototype of the method function
 *
 * The macro SHADEOP_CLEANUP creates the prototype of the cleanup function.
 *
 *
 * See also end of file for an exemple.
 ****************************************************************************/


/****************************************************************************
  Struct SHADEOP_STRUCT
****************************************************************************/

typedef struct _SHADEOP_SPEC {
	char *declaration;          /* if zero length string, end of table signaled. */
	char *initfunctionname;     /* optional */
	char *cleanupfunctionname;  /* optional */
} SHADEOP_SPEC;


/****************************************************************************
   Macro SHADEOP_TABLE
****************************************************************************/

#define SHADEOP_TABLE(SHADEOP)   SHADEOP_SPEC DLEXPORT SHADEOP ## _shadeops[]


/****************************************************************************
   Macro SHADEOP_INIT
****************************************************************************/

#define SHADEOP_INIT(FUNC)       void DLEXPORT *FUNC(void)


/****************************************************************************
   Macro SHADEOP
****************************************************************************/

#define SHADEOP(FUNC)            int DLEXPORT FUNC(void *initdata, int argc, void **argv)


/****************************************************************************
   Macro SHADEOP_CLEANUP
****************************************************************************/

#define SHADEOP_CLEANUP(FUNC)    void DLEXPORT FUNC(void *initdata)


/****************************************************************************
   Macro ASSIGN_STRING

   For affectations to strings
*****************************************************************************/

#define ASSIGN_STRING(VAR, S)    { if ((VAR)) free((VAR));  (VAR) = strdup(S); }


#endif

