/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//		 3delight
// = AUTHOR(S)
//		 Franck Diard, Patrick Fournier, Aghiles Kheffache, Charles De Lean
// = VERSION
//		 $Revision$
// = DATE RELEASED
//		 $Date: 2006-06-21 15:48:40 -0400 (Wed, 21 Jun 2006) $
// = RCSID
//		 $Id: dlSL.h 9133 2006-06-21 19:48:40Z olivier $
// ===========================================================================

#ifndef __dlSL_H
#define __dlSL_H

#include "dlStopAndBool.h"
#include "dlBasicTypes.h"
#include "dlInterface.h"

class DlAttributeSet;
class DlLightInput;
class DlShadingContext;
class DlRtModule;
class DlShaderSupport;
class DlFrame3D;
class DlSpaceRegistry;
class DlSearchPath;
class DlRayInfos;
class DlRIOptions;

////////////////////////////////////////////////////////////////////////////
// Spline types
////////////////////////////////////////////////////////////////////////////

enum DlSplineBasis
{
	e_CatmullRomBasis = 0,
	e_BezierBasis = 1,
	e_BSplineBasis = 2,
	e_HermiteBasis = 3,
	e_LinearBasis = 4
};

////////////////////////////////////////////////////////////////////////////
// Message passing shader request
////////////////////////////////////////////////////////////////////////////

enum DlMessageFromShader
{
	e_AtmosphereParam,
	e_DisplacementParam,
	e_LightsourceParam,
	e_SurfaceParam,
	e_IncidentParam,
	e_OppositeParam
};

extern "C"
{
	////////////////////////////////////////////////////////////////////////////
	// Shading and lighting functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlSpecular(
		DlRGBColor *o_color,
		const DlVector3 *i_normal,
		const DlVector3 *i_viewer,
		float i_roughness,
		unsigned i_numLights,
		DlLightInput  *i_lights,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_mpIdx,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlSpecularstd(
		DlRGBColor *o_color,
		const DlVector3 *i_normal,
		const DlVector3 *i_viewer,
		float i_roughness,
		unsigned i_numLights, 
		DlLightInput  *i_lights, 
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_mpIdx,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlSpecularbrdf(
		DlRGBColor *o_color, 
		const DlVector3 *i_light,
		const DlVector3 *i_normal, 
		const DlVector3 *i_viewer, 
		float i_roughness, 
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlDiffuse(
		DlRGBColor          *o_color, 
		const DlVector3     *i_normal, 
		unsigned         i_numLights, 
		DlLightInput  *i_lights, 
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_mpIdx,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlAmbient(
		DlRGBColor *o_color, 
		unsigned i_numLights, 
		DlLightInput *i_lights,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_mpIdx,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlPhong(
		DlRGBColor *o_color, 
		const DlVector3 *i_N,
		const DlVector3 *i_V,
		float i_size,
		unsigned i_numLights, 
		DlLightInput  *i_lights, 
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_mpIdx,
		const DlShadingContext &i_context );


	////////////////////////////////////////////////////////////////////////////
	// Functions for illuminance and illuminate and gather statements
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlIlluminate(
		DlStopAndBool         *o_test,
		const DlVector3        i_appexPosition, 
		const DlVector3       *i_axis, 
		float                 *i_angle, 
		const DlVector3       *i_pos, 
		DlVector3             *o_L,
		const DlStopAndBool   *i_flag,
		unsigned               i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlIlluminateEnd(
		DlStopAndBool *i_illum_flag,
		DlVector3 *i_Ps,
		DlVector3 *i_L,
		const DlRGBColor *i_Cl, 
		const DlStopAndBool *i_flag,
		unsigned i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlSolar(
		DlStopAndBool *o_res,
		DlVector3 *i_Ps,
		const DlVector3 i_Axis, 
		float i_angle,
		DlVector3 *i_L, 
		const DlStopAndBool *i_flag,
		unsigned i_numMp,
		const DlShadingContext & );

	DL_INTERFACE void DlSolarEnd(
		DlStopAndBool *i_illum_flag,
		DlVector3 *i_Ps,
		DlVector3 *i_L,
		const DlRGBColor *i_Cl, 
		const DlStopAndBool *i_flag,
		unsigned i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlIlluminance(
		DlStopAndBool *o_test,
		const DlVector3 *i_pos, 
		const DlVector3 *i_axis,
		float *i_angle,
		const char *i_category,
		const DlStopAndBool *i_flag,
		unsigned i_numMp,
		const DlLightInput *i_lights,
		unsigned int i_lightCnt,
		unsigned int i_polyCount);

	DL_INTERFACE void DlIlluminanceCounter(
		unsigned int &io_lightCnt,
		DlStopAndBool &o_flag,
		DlVector3 **o_L,
		DlRGBColor **o_Cl,
		DlRGBColor **o_Ol,
		unsigned i_nbIn,
		const void *i_inParams[],
		unsigned i_nbOut,
		const void *i_outParams[],
		const void *i_typeParams[],
		bool i_increment,
		const DlVector3 *i_evalPts,
		const char *i_category,
		unsigned int i_numLights,
		DlLightInput *i_lights,
		unsigned i_numPts,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlGatherBegin(
		const char          *i_category,
		const DlVector3     *i_pos,
		const DlVector3     *i_duPos,
		const DlVector3     *i_dvPos,
		const DlVector3     *i_axis,
		float               *i_angle,
		int	                *io_samples,
		const void         **o_handle,
		unsigned             i_nbIn,
		const void          *i_inputs[],
		unsigned             i_nbOut,
		const void          *i_names[],
		const void          *i_res[],
		const void          *i_types[],
		const DlStopAndBool *i_flag,
		unsigned             i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlGatherCheck(
		const void          *i_handle,
		int                  i_count,
		DlStopAndBool       *o_test,
		const void          *i_args[],
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		const DlShadingContext & );

	DL_INTERFACE void DlGatherEnd(const void *i_handle);

	////////////////////////////////////////////////////////////////////////////
	// Mathematical functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlCos(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlACos(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlSin(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlASin(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlTan(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlATan(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlATan2(
		float               *o_res,
		const float         *i_value,
		const float         *i_value2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlPow(
		float               *o_res,
		const float         *i_value,
		const float         *i_pow,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlExp(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlSqrt(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlInverseSqrt(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlLog(
		float               *o_res,
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlLog2(
		float               *o_res,
		const float         *i_value,
		const float         *i_value2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlDiv1D1D(
		float               *o_res,
		const float         *i_value,
		const float         *i_value2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlDotProd(
		float               *o_res,
		const DlVector3     *i_value,
		const DlVector3     *i_value2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlCrossProd(
		DlVector3     *o_res,
		const DlVector3     *i_value,
		const DlVector3     *i_value2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlSmoothStep(
		float               *o_res,
		float                i_min, 
		float                i_max, 
		const float         *i_value,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlSpline1D(
		float      *o_res, 
		const void *i_args[],
		int i_nbArgs);

	DL_INTERFACE void DlSpline3D(
		DlVector3  *o_res, 
		const void *i_args[],
		int         i_nbArgs);

	DL_INTERFACE void DlSpline1DWithBasis(
		float         *o_res, 
		DlSplineBasis  i_basis,
		const void    *i_args[],
		int            i_nbArgs);

	DL_INTERFACE void DlSpline3DWithBasis(
		DlVector3     *o_res, 
		DlSplineBasis  i_basis,
		const void    *i_args[],
		int            i_nbArgs);

	DL_INTERFACE void DlSplineArray1D(
		float *o_res,
		const char *i_basis,
		float i_value,
		float *i_controlPts,
		unsigned i_nbPts );

	DL_INTERFACE void DlSplineArray3D(
		DlVector3 *o_res,
		const char *i_basis,
		float i_value,
		DlVector3 *i_controlPts,
		unsigned i_nbPts );

	DL_INTERFACE void DlRandom1D(
		float               *o_res,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlRandom3D(
		DlVector3           *o_res,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlFNoise1D(
		float               *o_res, 
		float               *i_f, 
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFNoise2D(
		float               *o_res, 
		 const float         *i_f1, 
		 const float         *i_f2, 
		 const DlStopAndBool *i_test, 
		 unsigned         i_numMp);


	DL_INTERFACE void DlFNoise3D(
		float               *o_res,
		 const DlVector3     *i_p,
		 const DlStopAndBool *i_test, 
		 unsigned         i_numMp);

	DL_INTERFACE void DlFNoise4D(
		float               *o_res,
		 const DlVector3     *i_p, 
		 float               *i_f,
		 const DlStopAndBool *i_test, 
		 unsigned         i_numMp);

	DL_INTERFACE void DlPNoise1D(
		DlVector3           *o_res, 
		 float               *i_f,
		 const DlStopAndBool *i_test, 
		 unsigned         i_numMp);

	DL_INTERFACE void DlPNoise2D(
		DlVector3           *o_res, 
		float               *i_f1, 
		float               *i_f2,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPNoise3D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPNoise4D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		float               *i_f,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFCellnoise1D(
		float               *o_res, 
		float               *i_f, 
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFCellnoise2D(
		float               *o_res, 
		const float         *i_f1, 
		const float         *i_f2, 
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFCellnoise3D(
		float               *o_res,
		const DlVector3     *i_p,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFCellnoise4D(
		float               *o_res,
		const DlVector3     *i_p, 
		float               *i_f,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPCellnoise1D(
		DlVector3           *o_res, 
		float               *i_f,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPCellnoise2D(
		DlVector3           *o_res, 
		float               *i_f1, 
		float               *i_f2,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPCellnoise3D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPCellnoise4D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		float               *i_f,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);


	DL_INTERFACE void DlFPnoise1D(
		float               *o_res, 
		float               *i_f, 
		float                i_period,
		const DlStopAndBool *i_test, 
		unsigned        i_numMp);

	DL_INTERFACE void DlFPnoise2D(
		float               *o_res, 
		const float         *i_f1, 
		const float         *i_f2, 
		float                i_uperiod,
		float                i_vperiod,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFPnoise3D(
		float               *o_res,
		const DlVector3     *i_p,
		const DlVector3      i_period,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlFPnoise4D(
		float               *o_res,
		const DlVector3     *i_p, 
		float               *i_f,
		const DlVector3      i_pperiod,
		float                i_tperiod,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPPnoise1D(
		DlVector3           *o_res, 
		float               *i_f,
		float                i_period,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPPnoise2D(
		DlVector3           *o_res, 
		float               *i_f1, 
		float               *i_f2,
		float                i_uperiod,
		float                i_vepriod,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPPnoise3D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		const DlVector3      i_period,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlPPnoise4D(
		DlVector3           *res, 
		const DlVector3     *i_p,
		float               *i_f,
		const DlVector3      i_pperiod,
		float                i_tperiod,
		const DlStopAndBool *i_test, 
		unsigned         i_numMp);

	DL_INTERFACE void DlDuDv1D(
		float *o_du,
		float *o_dv,
		const float *i_value,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlDuDv3D(
		DlVector3 *o_du,
		DlVector3 *o_dv,
		const DlVector3 *i_value,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlDeriv1D(
		float               *o_res,
		const float         *i_num,
		const float         *i_den,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlDeriv3D(
		DlVector3           *o_res,
		const DlVector3     *i_num,
		const float         *i_den,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context);
	
	DL_INTERFACE void DlDifferential1D(
		float* o_dValU,
		float* o_dValV,
		float* io_val,
		unsigned i_numPts,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlDifferential3D(
		DlVector3* o_dValU,
		DlVector3* o_dValV,
		DlVector3* io_val,
		unsigned i_numPts,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlMin1D(
		float      *o_res,
		const void *i_args[],
		unsigned    i_nbArgs);

	DL_INTERFACE void DlMin3D(
		DlVector3  *o_res,
		const void *i_args[],
		unsigned    i_nbArgs);

	DL_INTERFACE void DlMax1D(
		float      *o_res,
		const void *i_args[],
		unsigned    i_nbArgs);

	DL_INTERFACE void DlMax3D(DlVector3  *o_res,
	  const void *i_args[],
	  unsigned    i_nbArgs);

	DL_INTERFACE void DlClamp3D(
		DlVector3           *o_res,
		const DlVector3     *i_p,
		const DlVector3      i_min,
		const DlVector3      i_max,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlFilterstep(
		float               *o_res,
		float               *i_edge,
		float               *i_s1,
		float               *i_s2,
		const void          *i_args[],
		unsigned             i_nbArgs,
		const DlStopAndBool *i_flag,
		unsigned             i_numMp);

	DL_INTERFACE void DlBoxFilterstep(
		float               *o_res,
		float               *i_edge,
		float               *i_s1,
		float               *i_s2,
		float                i_width,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlTriangleFilterstep(
		float               *o_res,
		float               *i_edge,
		float               *i_s1,
		float               *i_s2,
		float                i_width,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlCatmullRomFilterstep(
		float               *o_res,
		float               *i_edge,
		float               *i_s1,
		float               *i_s2,
		float                i_width,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlGaussianFilterstep(
		float               *o_res,
		float               *i_edge,
		float               *i_s1,
		float               *i_s2,
		float                i_width,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	////////////////////////////////////////////////////////////////////////////
	// Geometric functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlLength(
		float               *o_res,
		const DlVector3     *i_v,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp );

	DL_INTERFACE void DlDistance(
		float               *o_res,
		const DlVector3     *i_p1, 
		const DlVector3     *i_p2,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlNormalize(
		DlVector3           *o_res, 
		const DlVector3     *i_v, 
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);

	DL_INTERFACE void DlFaceforward(
		DlVector3           *o_res, 
		const DlVector3     *i_N,
		const DlVector3     *i_I,
		const DlVector3     *i_Nref,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlTransformSpace(
		DlVector3           *o_res,
		const DlVector3     *i_point,
		const DlFrame3D     *i_space,
		const DlMatrix44    *i_matrix,
		unsigned int         i_which,
		bool                 i_fromCurrent,
		const DlStopAndBool *i_flag,
		unsigned i_numMp,
		const DlShadingContext &i_context );


	DL_INTERFACE void DlReflect(
		DlVector3           *o_res, 
		const DlVector3     *i_I,
		const DlVector3     *i_N,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlRefract(
		DlVector3 *o_res,
		const DlVector3 *i_I,
		const DlVector3 *i_N,
		float i_eta,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlFresnel(
		const DlVector3     *i_I,
		const DlVector3     *i_N,
		float                i_eta,
		float               *o_Kr,
		float               *o_Kt,
		DlVector3           *o_R,
		DlVector3           *o_T,
		const DlStopAndBool *i_flag,
		unsigned i_numMp);

	DL_INTERFACE void DlArea(
		float               *o_res,
		const DlVector3     *i_P,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlCalculateNormal(
		DlVector3 *o_res,
		const DlVector3 *i_P,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlDepth(
		float *o_z, 
		const DlVector3 *i_p,
		const DlShadingContext &i_context,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlTrace(
		DlRGBColor *o_res,
		float *o_dist,
		
		const DlVector3 *i_rayStart,
		const DlVector3 *i_dRayStartU,
		const DlVector3 *i_dRayStartV,
		
		const DlVector3 *i_rayDir,
		const DlVector3 *i_dRayDirU,
		const DlVector3 *i_dRayDirV,
		
		const void *i_args[],
		unsigned i_nbArgs,			
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlPtlined(
		float               *o_res,
		const DlVector3      i_p0,
		const DlVector3      i_p1,
		const DlVector3     *i_q,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlRotatePoint(
		DlVector3           *o_res,
		const DlVector3     *i_q,
		float                i_angle,
		const DlVector3      i_p0,
		const DlVector3      i_p1,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );


	////////////////////////////////////////////////////////////////////////////
	// Matrix functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlGetToSpaceMatrix(
		const DlFrame3D *i_space,
		DlMatrix44* o_matrix,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlInverse(
		DlMatrix44       *o_result,
		const DlMatrix44 *i_matrix);

	DL_INTERFACE void DlDeterminant(
		float            *o_result,
		const DlMatrix44 *i_matrix);

	DL_INTERFACE void DlTranslate(
		DlMatrix44       *o_result,
		const DlMatrix44 *i_matrix,
		const DlVector3  *i_pt);

	DL_INTERFACE void DlRotate(
		DlMatrix44       *o_result,
		const DlMatrix44 *i_matrix,
		const float      i_angle,
		const DlVector3  *i_axis);

	DL_INTERFACE void DlScale(
		DlMatrix44       *o_result,
		const DlMatrix44 *i_matrix,
		const DlVector3  *i_point);

	////////////////////////////////////////////////////////////////////////////
	// Color functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlHSVToRGB(
		DlRGBColor          *o_rgb, 
		const DlRGBColor    *i_c, 
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);
  
	DL_INTERFACE void DlRGBToHSV(
		DlRGBColor          *o_result, 
		const DlRGBColor    *i_c, 
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);
  
	DL_INTERFACE void DlHSLToRGB(
		DlRGBColor          *o_rgb, 
		const DlRGBColor    *i_c, 
		const DlStopAndBool *i_flag,
		unsigned         i_numMp);
  
	DL_INTERFACE void DlRGBToHSL(
		DlRGBColor          *o_result, 
		const DlRGBColor    *i_c,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp );

	DL_INTERFACE void DlHSYToRGB(
		DlRGBColor          *o_res,
		const DlRGBColor    *i_c,
		const DlStopAndBool *i_flag,
		unsigned         i_numMp );

	DL_INTERFACE void DlRGBToHSY(
		DlRGBColor *o_result, 
		const DlRGBColor *i_c,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );
	
	DL_INTERFACE void DlYIQToRGB(
		DlRGBColor *o_rgb, 
		const DlRGBColor *i_c, 
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlRGBToYIQ(
		DlRGBColor *o_result,
		const DlRGBColor *i_c,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );
  
	DL_INTERFACE void DlXYZToRGB(
		DlRGBColor *o_rgb,
		const DlRGBColor *i_c,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	DL_INTERFACE void DlRGBToXYZ(
		DlRGBColor *o_result,
		const DlRGBColor *i_c,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );
  
	DL_INTERFACE void DlTransformColor(
		DlRGBColor *o_res,
		const DlRGBColor *i_c,
		const char *i_space,
		bool i_fromRGB,
		const DlStopAndBool *i_flag,
		unsigned i_numMp );

	/* -------------------------------------------------- R A Y    T R A C I N G
	*/
	DL_INTERFACE void DlTransmission(
		DlRGBColor *o_res,

		const DlVector3 *i_from,
		const DlVector3 *i_dFromU,
		const DlVector3 *i_dFromV,

		const DlVector3 *i_to,
		const DlVector3 *i_dtoU,
		const DlVector3 *i_dtoV,

		bool i_finiteLength,
		const void *i_args[],
		unsigned i_nbArgs,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlIsShadowRay(
		float *o_res,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlIsIndirectRay(
		float *o_res,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlRayLevel(
		float *o_res,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlRayHitTest(
		float *o_res,
		const DlVector3 *i_from,
		const DlVector3 *i_to,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlOcclusion_IndirectDiffuse(
		/* output for indirectdiffuse */
		DlRGBColor *o_indirectRes,
		/* output for occlusion */
		float *o_occRes,
		/* input */
		const DlVector3 *i_pos,
		const DlVector3 *i_dir,
		float *i_samples,
		const void *i_args[],
		unsigned i_nbArgs,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlSubSurface(
		DlRGBColor *o_res,
		const DlVector3 *i_P,
		const DlVector3 *i_N,
		const void *i_args[],
		unsigned i_nbArgs,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		const DlShadingContext &i_context );

	DL_INTERFACE void DlIrradiance(
		DlRGBColor *o_res,
		const char *i_envnam,
		const DlVector3 *i_dir,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		const DlShadingContext &i_context );
	
	DL_INTERFACE void DlPhotonmap(
		DlRGBColor *o_res,
		const char *i_photonmap_name,
		const DlVector3 *i_P, const DlVector3 *i_dPdu, const DlVector3 *i_dPdv,
		const DlVector3 *i_N,
		const void *i_args[],
		unsigned i_nbArgs,
		const DlStopAndBool *i_flag,
		unsigned i_numPts,
		const DlShadingContext &i_context );

	/* ------------------------------------------------------------- S T R I N G
	*/
	DL_INTERFACE void DlPrintf(
		const void  *i_args[],
		unsigned     i_nbArgs);

	DL_INTERFACE void DlConcat(
		const char **o_result,
		const void  *i_args[],
		unsigned     i_nbArgs);

	DL_INTERFACE void DlFormat(
		const char **o_result,
		const void  *i_args[],
		unsigned     i_nbArgs);

	DL_INTERFACE void DlMatch(
		float *o_res,
		const char *i_pattern,
		const char *i_subject );

	/* --------------------------------------------------------------- O T H E R
	*/
	DL_INTERFACE void DlPack(
		float *o_res,
		const float *i_byte3,
		const float *i_byte2,
		const float *i_byte1,
		const float *i_byte0,
		const DlStopAndBool *i_test,
		unsigned i_numMp );

	DL_INTERFACE void DlUnpack(
		const float *i_packbyte,
		float *o_byte3,
		float *o_byte2,
		float *o_byte1,
		float *o_byte0,
		const DlStopAndBool *i_test,
		unsigned i_numMp );

	/* -------------------------------------------T E X T U R E    M A P P I N G
	*/
	DL_INTERFACE void DlMakeFilterstepPoints(
		const float *i_s,
		float *o_s1,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context );

	/*
		shadow(...) shadeop
	*/
	DL_INTERFACE void DlShadow(
		float                 *o_res,
		int                    i_dim,
		const char            *i_Name,
		int                    i_channel, 
		const DlVector3       *i_P,
		const DlVector3       *i_dPU,
		const DlVector3       *i_dPV,
		const void            *i_args[],
		unsigned               i_nbArgs,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlStopAndBool   *i_flag,
		const DlShadingContext &i_context );

	/*
		texture(...) shadeop
	*/
	DL_INTERFACE void DlTexture(
		float                 *o_res,
		unsigned               i_dim,
		const char            *i_Name, 
		unsigned               i_channel,
		const float           *i_s,
		const float           *i_t,
		const float           *i_dsu,
		const float           *i_dtu,
		const float           *i_dsv,
		const float           *i_dtv, 
		const void            *i_args[],
		unsigned               i_nbArgs,
		unsigned i_numPts,
		const DlStopAndBool   *i_flag,
		const DlShadingContext &i_context );

	/*
		environment(...) shadeop
	*/
	DL_INTERFACE void DlEnvironment(
		float *o_res,
		unsigned i_dim,
		const char *i_Name,
		unsigned i_channel,
		const DlVector3 *i_direction,
		const DlVector3 *i_dDirU,
		const DlVector3 *i_dDirV,
		const void *i_args[],
		unsigned i_nbArgs,
		unsigned i_numPts,
		unsigned i_polyCnt,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_sp );

	/*
		bake(...) shadeop

		PARAMETERS
		i_texture_name
		          : the texture to bake to.
		i_num_channels
		          : number of channels represented by i_value
		i_s, i_t  : texture coordinates for each value
		i_value   : the data to bake
		i_channel_name
		          : optional parameter indicating a channel name for the 
				    data.
		i_num_mps : number of data points to bake
		i_polyCnt : data index when not SIMD
	*/
	DL_INTERFACE void DlBake(
		const DlStopAndBool *i_flag,
		const char *i_texture_name,
		unsigned i_num_channels,
		float *i_s, float *i_t,
		float *i_value,
		const char *i_channelname,
		unsigned i_num_mps, 
		unsigned i_polyCnt,
		const DlShadingContext &i_sp );

	/*
		bake3d() shadeop

		o_res
			[uniform] return value of the shadeop
		i_filename
			[uniform] The file to bake to.
		i_displaychannels
			[uniform] The display channels to bake (comma separated).
		i_P, i_N
			[varying] The coordinates and normals for the baking.
		i_args, i_nbArgs
			Named parameters (see .cpp for details).
	*/
	DL_INTERFACE void DlBake3d(
		float *o_res,
		const char *i_filename,
		const char *i_displaychannels,
		const DlVector3 *i_P,
		const DlVector3 *i_N,
		const void *i_args[],
		unsigned i_nbArgs,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context );

	/*
		texture3d() shadeop

		o_res
			[uniform] return value of the shadeop
		i_filename
			[uniform] The file to read from.
		i_P, i_N
			[varying] The coordinates and normals for the lookup.
		i_args, i_nbArgs
			Named parameters (see .cpp for details).
	*/
	DL_INTERFACE void DlTexture3d(
		float *o_res,
		const char *i_filename,
		const DlVector3 *i_P,
		const DlVector3 *i_N,
		const void *i_args[],
		unsigned i_nbArgs,
		unsigned i_numPts,
		const DlStopAndBool *i_flag,
		const DlShadingContext &i_context );

	////////////////////////////////////////////////////////////////////////////
	// Message passing and informations functions
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE void DlShaderName(
		const char **o_shaderName,
		const DlShadingContext &i_sp);

	DL_INTERFACE void DlTextureInfo(
		float                *o_res,
		const char           *i_filename,
		void                 *o_data,
		const char           *i_name,
		unsigned int          i_type,
		int                   i_nbElems,
		const DlShadingContext &i_context);

	DL_INTERFACE void DlAttribute(
		float                  *o_res,
		void                   *o_data,
		const char             *i_name,
		unsigned int            i_type,
		int                     i_nbElems,
		const DlShadingContext  *i_context,
		DlAttributeSet *i_attributes);

	DL_INTERFACE void DlOption(
		float                  *o_res,
		void                   *o_data,
		const char             *i_name,
		unsigned int            i_type,
		int                     i_nbElems,
		const DlShadingContext *i_context,
		const DlRIOptions      *i_options);

	DL_INTERFACE void DlRendererInfo(
		float               *o_res,
		void                *o_data,
		const char          *i_name,
		unsigned int         i_type,
		int                  i_nbElems);

	DL_INTERFACE void DlRayInfo(
		float *o_res,
		void *o_data,
		const char *i_name,
		unsigned int i_type,
		int i_nbElems,
		const DlShadingContext &i_context);
	
	DL_INTERFACE void DlMessageParam(
		float                  *o_res,
		DlMessageFromShader     i_shader,
		const char             *i_name,
		void                   *o_info,
		int                     i_isVarying,
		unsigned int            i_type,
		int                     i_nbElems,
		const DlStopAndBool    *i_flag,
		unsigned            i_numMp,
		const DlShadingContext &i_context,
		unsigned            i_noLight );

	////////////////////////////////////////////////////////////////////////////
	// Utility functions for shaders
	////////////////////////////////////////////////////////////////////////////

	DL_INTERFACE const DlFrame3D* DlFindSpaceFromName(
		const char *i_name,
		const DlShadingContext &i_context);

	DL_INTERFACE const DlFrame3D* DlSpaceFromIdx(
		unsigned i_spaceIdx,
		const DlShadingContext &i_context);

	DL_INTERFACE const char* FindDSO(
		const DlSearchPath *i_searchPath, 
		const char         *i_fileName);

	DL_INTERFACE void ReportBadArrayIndexError(
		const char *i_dsoFile, 
		int         i_noLineDso,
		const char *i_shaderFile,
		int         i_noLineShader,
		const char *i_arrayVarName,
		float       i_index,
		int         i_size);

	DL_INTERFACE void* DlCalloc(unsigned i_nbElems, unsigned i_size);

	DL_INTERFACE void DlFree(void *i_block);

	DL_INTERFACE const char* DlStrdup(const char* i_str);

	DL_INTERFACE void DlFreeStr(const char *i_str);

	DL_INTERFACE void DlAllocArray(
		void **io_array,
		int *i_size,
		unsigned int i_type,
		bool i_isArrayVar,
		bool i_isSizeVar,
		unsigned int i_numMp,
		const DlStopAndBool *i_flag,
		unsigned int *o_nbElem);

	DL_INTERFACE void DlFreeArray(
		void **io_array,
		bool i_freeStrings,
		unsigned int i_numMp);

	DL_INTERFACE int DlSizeArray(void *i_array);

	DL_INTERFACE bool DlShapeIsLeftHanded(const DlShadingContext &i_context);
}

#endif // __dlSL_H
