/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/


// ===========================================================================
// = LIBRARY
//     3delight
// = AUTHOR(S)
//     Patrick Fournier
// = VERSION
//     $Revision$
// = DATE RELEASED
//     $Date: 2006-08-22 23:40:34 -0400 (Tue, 22 Aug 2006) $
// = RCSID
//     $Id: dlShaderInterface.h 9366 2006-08-23 03:40:34Z olivier $
// ===========================================================================


#ifndef __dlShaderInterface_H
#define __dlShaderInterface_H

class DlShaderInterface
{
public:

    typedef enum
    {
        e_typeBad = -1,			// Shader type given by dso does not match expected type

        e_typeSurface = 0,
        e_typeDisplacement = 1,
        e_typeAtmosphere = 2,
		e_typeVolume = 2,		// When compiling the shader, volume type is unknown. Suppose atmosphere.
        e_typeLight = 3,
        e_typeImager = 4,
		e_typeInterior = 5,
        e_typeUnknown,			// Unknown shader type
        e_typeTransformation
    } DlType;

	enum { k_maxShader = 6 }; // number of supported shaders
			
	typedef struct {
		unsigned int m_numVisible; // number of visible variables
		unsigned int m_numHidden; // number of hidden variables
		unsigned int m_numSpaces; // number of spaces in space list
	} DlDataSize;

	struct DlVisibleParamSpec
	{
		/* Parameter name */
		const char* m_name;
		
		/* Space where the initial value is interpreted (for shaderinfo). */
		const char* m_space;
		
		/* Parameter type; DlTokenInfo contents (except for m_arraySize which is
		   -1 for variable length arrays) */
		unsigned m_class;
		unsigned m_type;
		int m_arraySize;
		
		/* Is this an output parameter? */
		bool m_isOutput;
	};

};

#endif // __dlShaderInterface_H
