/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     shaderCompiler
// = FILENAME
//     slcShaderCCodeOp.h
// = AUTHOR(S)
//     Marco Jacques, Charles De Lean
// = VERSION
//     $Revision$
// = DATE RELEASED
//     $Date: 2007-01-26 15:15:24 -0500 (Fri, 26 Jan 2007) $
// = RCSID
//     $Id: slcShaderCCodeOp.h 9718 2007-01-26 20:15:24Z olivier $
// ===========================================================================

////////////////////////////////////////////////////////////////////////////
// This file is used only by the generated C file of the shader, not by the
// compiler itself
////////////////////////////////////////////////////////////////////////////

#ifndef __slcShaderCCodeOp_H
#define __slcShaderCCodeOp_H

////////////////////////////////////////////////////////////////////////////
// Dll initialization function needed for cygwin
////////////////////////////////////////////////////////////////////////////

#if defined(DL_COMPILING_SHADER) && defined(CYGWIN32)

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#include <stdio.h>

extern "C"
{
	BOOL APIENTRY
	DllMainCRTStartup (
		HANDLE hInst /* Library instance handle. */ ,
		DWORD reason /* Reason this function is being called. */ ,
		void *reserved /* Not used. */ );

	// Defined in minwg32 library...

	BOOL _CRT_INIT (
		HANDLE hInst /* Library instance handle. */ ,
		DWORD reason /* Reason this function is being called. */ ,
		void *reserved /* Not used. */ );
}

// Var needed by the cygwin libraries...

struct _reent *_impure_ptr = 0;

// Enter point of the dll needed by Visual-C++...

BOOL APIENTRY DllMainCRTStartup (
	 HANDLE hInst /* Library instance handle. */ ,
	 DWORD reason /* Reason this function is being called. */ ,
	 void *reserved /* Not used. */ )
{

	_CRT_INIT(hInst, reason, reserved);

	// I'm always returning true here.  CRT_INIT seems to return
	// 0, which something has gone wrong.  But, after testing
	// everything seems to be fine...

	return 1;
}

#endif

#include "dlSL.h"
#include "dlShaderInterface.h"
#include "dlDSO.h"

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <float.h>

#ifdef _WIN32
#define DL_SHADER_FUNC __declspec(dllexport)
#else
#define DL_SHADER_FUNC
#endif

/////////////////////////////////////////////////////////////////////////////
// For functions defined in DSO
/////////////////////////////////////////////////////////////////////////////

struct SlcDSOInfo {
	typedef void *(*InitFunction)(void);
	typedef int   (*MethodFunction)(void *initdata, int argc, void **argv);
	typedef void  (*CleanupFunction)(void *initdata);

	InitFunction     m_initFunc;
	MethodFunction   m_methodFunc;
	CleanupFunction  m_cleanupFunc;
	void            *m_initdata;
};

////////////////////////////////////////////////////////////////////////////
// MACROS FOR ASSIGNMENTS
////////////////////////////////////////////////////////////////////////////

#define ASSIGN_FLOAT_FLOAT(F1, F2)  (F1) = (F2)
#define ASSIGN_INT_FLOAT(I1, F2)  (I1) = (int)(F2)
#define ASSIGN_FLOAT_INT(F1, I2)  (F1) = (float)(I2)
#define ASSIGN_INT_INT(I1, I2)  (I1) = (I2)

#define ASSIGN_POINT_POINT(P1, P2) \
	((P1)[0] = (P2)[0]), ((P1)[1] = (P2)[1]), ((P1)[2] = (P2)[2])

#define ASSIGN_MATRIX_MATRIX(M1, M2)    memcpy((M1), (M2), sizeof(DlMatrix44))

#define ASSIGN_POINT_FLOAT(P1, F2) \
	((P1)[0] = F2), ((P1)[1] = F2), ((P1)[2] = F2)

#define ASSIGN_MATRIX_FLOAT(M1, F2) \
   ((M1)[0]  = (F2),  (M1)[1]  = 0,    (M1)[2]  = 0,     (M1)[3]  = 0, \
    (M1)[4]  = 0,     (M1)[5]  = (F2), (M1)[6]  = 0,     (M1)[7]  = 0, \
    (M1)[8]  = 0,     (M1)[9]  = 0,    (M1)[10] = (F2),  (M1)[11] = 0, \
    (M1)[12] = 0,     (M1)[13] = 0,    (M1)[14] = 0,     (M1)[15] = (F2))

#define ASSIGN_BOOL_BOOL(B1, B2)  ((B1) = (B2))

#define ASSIGN_BOOL_BOOL_SIMD(B1, B2) \
	memcpy((B1), (B2), numMp * sizeof(DlStopAndBool))

#define ASSIGN_BOOL_BOOL_SIMD_IF(B1, B2, B) \
{                                        \
	unsigned i;                          \
	for (i = 0; i < numMp; i++)          \
		if ((B)[i])                      \
			(B1)[i] = (B2)[i];           \
}

#define INIT_POINT(P1, F2, F3, F4) \
	((P1)[0] = (F2), (P1)[1] = (F3), (P1)[2] = (F4))

#define INIT_MATRIX(M, E0, E1, E2, E3, E4, \
	E5, E6, E7, E8, E9, E10, E11, E12, E13, E14, E15) \
   ((M)[0] = (E0), (M)[1] = (E1), (M)[2] = (E2), (M)[3] = (E3), \
    (M)[4] = (E4), (M)[5] = (E5), (M)[6] = (E6), (M)[7] = (E7), \
    (M)[8] = (E8), (M)[9] = (E9), (M)[10] = (E10), (M)[11] = (E11), \
    (M)[12] = (E12), (M)[13] = (E13), (M)[14] = (E14), (M)[15] = (E15))

#define ASSIGN_FLOAT_FLOAT_SIMD(F1, F2) \
	memcpy((F1), (F2), numMp * sizeof(float))

#define ASSIGN_POINT_POINT_SIMD(P1, P2) \
	memcpy((P1), (P2), numMp * sizeof(DlVector3))

#define ASSIGN_MATRIX_MATRIX_SIMD(M1, M2) \
	memcpy((M1), (M2), numMp * sizeof(DlMatrix44))

#define ASSIGN_FLOAT_FLOAT_SIMD_IF(F1, F2, B) \
{                                        \
	unsigned i;                          \
	for (i = 0; i < numMp; i++)          \
		if ((B)[i])                      \
			(F1)[i] = (F2)[i];           \
}

#define ASSIGN_POINT_POINT_SIMD_IF(P1, P2, B) \
{                                        \
	unsigned i;                          \
	for (i = 0; i < numMp; i++)          \
		if ((B)[i])                      \
			ASSIGN_POINT_POINT(P1[i], P2[i]); \
}

#define ASSIGN_MATRIX_MATRIX_SIMD_IF(M1, M2, B) \
{                                        \
	unsigned i;                          \
	for (i = 0; i < numMp; i++)          \
		if ((B)[i])                      \
			ASSIGN_MATRIX_MATRIX(M1[i], M2[i]); \
}

#define FREE_STRING(S) DlFreeStr((S))

#define ASSIGN_STRING_STRING(S1, S2) \
if( (S1) != (S2) ) \
{ \
	if( (S1) ) \
	{ \
		FREE_STRING( (S1) ); \
	} \
	(S1) = DlStrdup( (S2) ); \
}

#define ALLOC_ARRAY(ARRAY, SIZE, TYPE, ARRAY_VAR, SIZE_VAR, B, NB_ELEM) \
	DlAllocArray((ARRAY), (SIZE), TYPE, (ARRAY_VAR), (SIZE_VAR), numMp, B, NB_ELEM)

#define SIZE_ARRAY(ARRAY) DlSizeArray(ARRAY)

////////////////////////////////////////////////////////////////////////////
// POINTS, VECTOR AND NORMALS TRANSFORMATION
//
// Spaces are known at compile-time.
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_TRANSFORM_TO_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0,\
		0, false, 0, 1, i_context)

#define CALL_PRIM_VTRANSFORM_TO_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		1, false, 0, 1, i_context)

#define CALL_PRIM_NTRANSFORM_TO_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		2, false, 0, 1, i_context)

#define CALL_PRIM_TRANSFORM_FROM_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		0, true, 0, 1, i_context)

#define CALL_PRIM_VTRANSFORM_FROM_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		1, true, 0, 1, i_context)

#define CALL_PRIM_NTRANSFORM_FROM_CURRENT(P1, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		2, true, 0, 1, i_context)

#define CALL_PRIM_TRANSFORM_TO_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		0, false, (B), numMp, i_context)

#define CALL_PRIM_VTRANSFORM_TO_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		1, false, (B), numMp, i_context)

#define CALL_PRIM_NTRANSFORM_TO_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		2, false, (B), numMp, i_context)

#define CALL_PRIM_TRANSFORM_FROM_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		0, true, (B), numMp, i_context)

#define CALL_PRIM_VTRANSFORM_FROM_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		1, true, (B), numMp, i_context)

#define CALL_PRIM_NTRANSFORM_FROM_CURRENT_SIMD(P1, B, P2, SPACE)  \
	DlTransformSpace((P1), (P2), DlSpaceFromIdx(SPACE, i_context), 0, \
		2, true, (B), numMp, i_context)

////////////////////////////////////////////////////////////////////////////
// MACROS FOR ARITHMETIC OPERATORS
////////////////////////////////////////////////////////////////////////////

#define ADD_FLOAT_FLOAT(F1, F2, F3)  (F1) = (F2) + (F3)
#define ADD_INT_INT(I1, I2, I3)  (I1) = (I2) + (I3)

#define ADD_FLOAT_POINT(P1, F2, P3)  ((P1)[0] = (F2) + (P3)[0]), \
                                     ((P1)[1] = (F2) + (P3)[1]), \
                                     ((P1)[2] = (F2) + (P3)[2])

#define ADD_POINT_FLOAT(P1, P2, F3)  ((P1)[0] = (P2)[0] + (F3)), \
                                     ((P1)[1] = (P2)[1] + (F3)), \
                                     ((P1)[2] = (P2)[2] + (F3))

#define ADD_POINT_POINT(P1, P2, P3)  ((P1)[0] = (P2)[0] + (P3)[0]), \
                                     ((P1)[1] = (P2)[1] + (P3)[1]), \
                                     ((P1)[2] = (P2)[2] + (P3)[2])

#define SUB_FLOAT_FLOAT(F1, F2, F3)  (F1) = (F2) - (F3)
#define SUB_INT_INT(F1, F2, F3)  (F1) = (F2) - (F3)

#define SUB_FLOAT_POINT(F1, F2, P3)  ((F1)[0] = (F2) - (P3)[0]), \
                                     ((F1)[1] = (F2) - (P3)[1]), \
                                     ((F1)[2] = (F2) - (P3)[2])

#define SUB_POINT_FLOAT(P1, P2, F3)  ((P1)[0] = (P2)[0] - (F3)), \
                                     ((P1)[1] = (P2)[1] - (F3)), \
                                     ((P1)[2] = (P2)[2] - (F3))

#define SUB_POINT_POINT(P1, P2, P3)  ((P1)[0] = (P2)[0] - (P3)[0]), \
                                     ((P1)[1] = (P2)[1] - (P3)[1]), \
                                     ((P1)[2] = (P2)[2] - (P3)[2])

#define MULT_FLOAT_FLOAT(F1, F2, F3)  (F1) = (F2) * (F3)

#define MULT_FLOAT_POINT(P1, F2, P3)  ((P1)[0] = (F2) * (P3)[0]), \
                                      ((P1)[1] = (F2) * (P3)[1]), \
                                      ((P1)[2] = (F2) * (P3)[2])

#define MULT_FLOAT_MATRIX(M1, F2, M3) \
{ \
    (M1)[0] = (F2) * (M3)[0]; \
    (M1)[1] = (F2) * (M3)[1]; \
    (M1)[2] = (F2) * (M3)[2]; \
    (M1)[3] = (F2) * (M3)[3]; \
    (M1)[4] = (F2) * (M3)[4]; \
    (M1)[5] = (F2) * (M3)[5]; \
    (M1)[6] = (F2) * (M3)[6]; \
    (M1)[7] = (F2) * (M3)[7]; \
    (M1)[8] = (F2) * (M3)[8]; \
    (M1)[9] = (F2) * (M3)[9]; \
    (M1)[10] = (F2) * (M3)[10]; \
    (M1)[11] = (F2) * (M3)[11]; \
    (M1)[12] = (F2) * (M3)[12]; \
    (M1)[13] = (F2) * (M3)[13]; \
    (M1)[14] = (F2) * (M3)[14]; \
    (M1)[15] = (F2) * (M3)[15]; \
}

#define MULT_POINT_FLOAT(P1, P2, F3)  ((P1)[0] = (P2)[0] * (F3)), \
                                      ((P1)[1] = (P2)[1] * (F3)), \
                                      ((P1)[2] = (P2)[2] * (F3))

#define MULT_POINT_POINT(P1, P2, P3)  ((P1)[0] = (P2)[0] * (P3)[0]), \
                                      ((P1)[1] = (P2)[1] * (P3)[1]), \
                                      ((P1)[2] = (P2)[2] * (P3)[2])

#define MULT_MATRIX_FLOAT(M1, M3, F2)  \
{ \
    (M1)[0] = (F2) * (M3)[0]; \
    (M1)[1] = (F2) * (M3)[1]; \
    (M1)[2] = (F2) * (M3)[2]; \
    (M1)[3] = (F2) * (M3)[3]; \
    (M1)[4] = (F2) * (M3)[4]; \
    (M1)[5] = (F2) * (M3)[5]; \
    (M1)[6] = (F2) * (M3)[6]; \
    (M1)[7] = (F2) * (M3)[7]; \
    (M1)[8] = (F2) * (M3)[8]; \
    (M1)[9] = (F2) * (M3)[9]; \
    (M1)[10] = (F2) * (M3)[10]; \
    (M1)[11] = (F2) * (M3)[11]; \
    (M1)[12] = (F2) * (M3)[12]; \
    (M1)[13] = (F2) * (M3)[13]; \
    (M1)[14] = (F2) * (M3)[14]; \
    (M1)[15] = (F2) * (M3)[15]; \
}

/*
	NOTE: we need a temporary matrix in case M2=M1 or M3==M2
*/
#define MULT_MATRIX_MATRIX(M1, M2, M3) \
{ \
    DlMatrix44 t; \
\
    t[0] = (M2)[0] * (M3)[0] + (M2)[1] * (M3)[4] + (M2)[2] * (M3)[8]  + \
			(M2)[3] * (M3)[12];  \
    t[1] = (M2)[0] * (M3)[1] + (M2)[1] * (M3)[5] + (M2)[2] * (M3)[9]  + \
			(M2)[3] * (M3)[13];  \
    t[2] = (M2)[0] * (M3)[2] + (M2)[1] * (M3)[6] + (M2)[2] * (M3)[10] + \
			(M2)[3] * (M3)[14];  \
    t[3] = (M2)[0] * (M3)[3] + (M2)[1] * (M3)[7] + (M2)[2] * (M3)[11] + \
			(M2)[3] * (M3)[15];  \
\
    t[4] = (M2)[4] * (M3)[0] + (M2)[5] * (M3)[4] + (M2)[6] * (M3)[8]  + \
			(M2)[7] * (M3)[12];  \
    t[5] = (M2)[4] * (M3)[1] + (M2)[5] * (M3)[5] + (M2)[6] * (M3)[9]  + \
			(M2)[7] * (M3)[13];  \
    t[6] = (M2)[4] * (M3)[2] + (M2)[5] * (M3)[6] + (M2)[6] * (M3)[10] + \
			(M2)[7] * (M3)[14];  \
    t[7] = (M2)[4] * (M3)[3] + (M2)[5] * (M3)[7] + (M2)[6] * (M3)[11] + \
			(M2)[7] * (M3)[15];  \
\
    t[8]  = (M2)[8] * (M3)[0] + (M2)[9] * (M3)[4] + (M2)[10] * (M3)[8]  + \
			(M2)[11] * (M3)[12];  \
    t[9]  = (M2)[8] * (M3)[1] + (M2)[9] * (M3)[5] + (M2)[10] * (M3)[9]  + \
			(M2)[11] * (M3)[13];  \
    t[10] = (M2)[8] * (M3)[2] + (M2)[9] * (M3)[6] + (M2)[10] * (M3)[10] + \
			(M2)[11] * (M3)[14];  \
    t[11] = (M2)[8] * (M3)[3] + (M2)[9] * (M3)[7] + (M2)[10] * (M3)[11] + \
			(M2)[11] * (M3)[15];  \
\
    t[12] = (M2)[12] * (M3)[0] + (M2)[13] * (M3)[4] + (M2)[14] * (M3)[8]  + \
			(M2)[15] * (M3)[12];  \
    t[13] = (M2)[12] * (M3)[1] + (M2)[13] * (M3)[5] + (M2)[14] * (M3)[9]  + \
			(M2)[15] * (M3)[13];  \
    t[14] = (M2)[12] * (M3)[2] + (M2)[13] * (M3)[6] + (M2)[14] * (M3)[10] + \
			(M2)[15] * (M3)[14];  \
    t[15] = (M2)[12] * (M3)[3] + (M2)[13] * (M3)[7] + (M2)[14] * (M3)[11] + \
			(M2)[15] * (M3)[15];  \
\
    ASSIGN_MATRIX_MATRIX((M1), t); \
}

#define CONCAT_SPACE_MATRIX(M1, M2, SPACE) \
{ \
    DlMatrix44 tempM; \
	DlGetToSpaceMatrix(DlSpaceFromIdx(SPACE, i_context), &tempM, i_context); \
\
    MULT_MATRIX_MATRIX((M1), (M2), tempM); \
} \

/* ------------------------------------------ D I V I D E    P R I M I T I V E S

	NOTE:
	- We always check if the devider is 0. If it is, we set the result to
	FLT_MAX, this should be the correct behaviour for most applications. To
	be perfectly right, we should also consider the 0/0 case, which is undefied,
	but we let this check to the user inside the shader...
	
	Update (Olivier, jan 2005): As of now, division by 0 is considered a noop.
	While this may seem less logical, it is much more stable numerically (FLT_MAX
	will easily turn into infinity with simple arithmetic and then infinity
	easily turns into NaN).

	- Divide operations performred on points reuse the macros defined for
	colors (since they both have three componenets).
*/

#define DIV_FLOAT_FLOAT(F1, F2, F3) \
{ \
	if( (F3) == 0.0f ) \
	{ \
		(F1) = (F2);	\
	} \
	else \
	{ \
		(F1) = (F2) / (F3); \
	} \
}

#define DIV_FLOAT_FLOAT_SIMD(F1, B, F2, F3) DlDiv1D1D((F1), (F2), (F3), (B), numMp)

#define DIV_FLOAT_POINT(P1, F2, P3)  \
{ \
	DIV_FLOAT_FLOAT((P1)[0], F2, (P3)[0]); \
	DIV_FLOAT_FLOAT((P1)[1], F2, (P3)[1]); \
	DIV_FLOAT_FLOAT((P1)[2], F2, (P3)[2]); \
}

#define DIV_FLOAT_MATRIX(M1, F2, M3)  \
{ \
    DlMatrix44 temp; DlInverse(&temp, &(M3)); \
  \
    MULT_FLOAT_MATRIX((M1), (F2), temp);  \
}

#define DIV_POINT_FLOAT(P1, P2, F3) \
{ \
	if( (F3) == 0.0f ) \
	{ \
		ASSIGN_POINT_POINT(P1, P2); \
	} \
	else \
	{ \
		float inv = 1.0f / (F3); \
		(P1)[0] = (P2)[0] * inv; \
		(P1)[1] = (P2)[1] * inv; \
		(P1)[2] = (P2)[2] * inv; \
	} \
}

#define DIV_POINT_POINT(P1, P2, P3) \
{ \
	DIV_FLOAT_FLOAT((P1)[0], (P2)[0], (P3)[0]); \
	DIV_FLOAT_FLOAT((P1)[1], (P2)[1], (P3)[1]); \
	DIV_FLOAT_FLOAT((P1)[2], (P2)[2], (P3)[2]); \
}

#define DIV_MATRIX_FLOAT(M1, M2, F3)  \
{ \
	float temp; \
	DIV_FLOAT_FLOAT(temp, 1.0f, (F3)); \
	MULT_MATRIX_FLOAT((M1), (M2), temp);  \
}

#define DIV_MATRIX_MATRIX(M1, M2, M3) \
{  \
	DlMatrix44 temp; \
	DlInverse( &temp, &(M3) );  \
	MULT_MATRIX_MATRIX((M1), (M2), temp); \
}

#define DOT_PROD_POINT_POINT(F1, P2, P3) \
   (F1) = ((P2)[0] * (P3)[0]) + ((P2)[1] * (P3)[1]) + ((P2)[2] * (P3)[2])

#define DOT_PROD_POINT_POINT_SIMD(F1, B, P2, P3) DlDotProd((F1), (P2), (P3), (B), numMp)

#define CROSS_PROD_POINT_POINT(P1, P2, P3) \
{ \
	float temp1, temp2, temp3; \
	temp1 = (P2)[1] * (P3)[2] - (P2)[2] * (P3)[1]; \
	temp2 = (P2)[2] * (P3)[0] - (P2)[0] * (P3)[2]; \
	temp3 = (P2)[0] * (P3)[1] - (P2)[1] * (P3)[0]; \
	(P1)[0] = temp1; (P1)[1] = temp2; (P1)[2] = temp3; \
}

#define CROSS_PROD_POINT_POINT_SIMD(P1, B, P2, P3) DlCrossProd((P1), (P2), (P3), (B), numMp)

#define INVERT_FLOAT(F1, F2) (F1) = -(F2)

#define INVERT_POINT(P1, P2)     ((P1)[0] = -(P2)[0]), \
                                 ((P1)[1] = -(P2)[1]), \
                                 ((P1)[2] = -(P2)[2])

////////////////////////////////////////////////////////////////////////////
// MACROS FOR RELATIONS RETURNING BOOLEAN RESULTS (Version 2)
////////////////////////////////////////////////////////////////////////////

#define BOOL_LESS_THAN_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) < (F2)))

#define BOOL_LESS_THAN_INT_INT(B1, I1, I2) \
	SET_BOOL((B1), ((I1) < (I2)))

#define BOOL_LESS_EQUAL_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) <= (F2)))

#define BOOL_LESS_EQUAL_INT_INT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) <= (F2)))

#define BOOL_GREATER_THAN_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) > (F2)))

#define BOOL_GREATER_EQUAL_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) >= (F2)))

#define BOOL_EQUAL_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) == (F2)))

#define BOOL_NOT_EQUAL_FLOAT_FLOAT(B1, F1, F2) \
	SET_BOOL((B1), ((F1) != (F2)))

#define BOOL_EQUAL_STRING_STRING(B1, S1, S2) \
	SET_BOOL((B1), (!strcmp((S1), (S2))))

#define BOOL_NOT_EQUAL_STRING_STRING(B1, S1, S2) \
	SET_BOOL((B1), (strcmp((S1), (S2))))

#define BOOL_EQUAL_POINT_POINT(B1, P1, P2) \
	SET_BOOL((B1), \
		((P1)[0] == (P2)[0] && (P1)[1] == (P2)[1] && (P1)[2] == (P2)[2]))

#define BOOL_NOT_EQUAL_POINT_POINT(B1, P1, P2) \
	SET_BOOL((B1), \
		((P1)[0] != (P2)[0] || (P1)[1] != (P2)[1] || (P1)[2] != (P2)[2]))

#define BOOL_NOT_EQUAL_FLOAT_POINT(B1, F1, P2) \
	SET_BOOL((B1), ((F1) != (P2)[0] || (F1) != (P2)[1] || (F1) != (P2)[2]))

#define BOOL_NOT_EQUAL_POINT_FLOAT(B1, P1, F2) \
	SET_BOOL((B1), ((P1)[0] != (F2) || (P1)[1] != (F2) || (P1)[2] != (F2)))

#define BOOL_EQUAL_FLOAT_POINT(B1, F1, P2) \
	SET_BOOL((B1), ((F1) == (P2)[0] && (F1) == (P2)[1] && (F1) == (P2)[2]))

#define BOOL_EQUAL_POINT_FLOAT(B1, P1, F2) \
	SET_BOOL((B1), ((P1)[0] == (F2) && (P1)[1] == (F2) && (P1)[2] == (F2)))

#define BOOL_EQUAL_MATRIX_MATRIX(B, M1, M2) \
	SET_BOOL((B), !memcmp((M1), (M2), sizeof(DlMatrix44)))

#define BOOL_EQUAL_FLOAT_MATRIX(B, F, M) \
{ \
    DlMatrix44 t;   ASSIGN_MATRIX_FLOAT(t, (F));  \
\
    SET_BOOL((B), !memcmp((M), t, sizeof(DlMatrix44))); \
}

#define BOOL_EQUAL_MATRIX_FLOAT(B, M, F)  BOOL_EQUAL_FLOAT_MATRIX(B, F, M)

#define BOOL_NOT_EQUAL_MATRIX_MATRIX(B, M1, M2) \
	SET_BOOL((B), memcmp((M1), (M2), sizeof(DlMatrix44)))

#define BOOL_NOT_EQUAL_FLOAT_MATRIX(B, F, M) \
{ \
    DlMatrix44 t;   ASSIGN_MATRIX_FLOAT(t, (F));  \
\
    SET_BOOL((B), memcmp((M), t, sizeof(DlMatrix44))); \
}

#define BOOL_NOT_EQUAL_MATRIX_FLOAT(B, M, F) \
	BOOL_NOT_EQUAL_FLOAT_MATRIX(B, F, M)

/* ------------------------------------------------ M A T H    F U N C T I O N S
*/

#define DL_PI 3.1415926536f

#define CALL_PRIM_RADIANS(F1, F2)  (F1) = (F2) * (DL_PI / 180)

#define CALL_PRIM_DEGREES(F1, F2)  (F1) = (F2) * (180 / DL_PI)

#define CALL_PRIM_SIN(F1, F2) *(F1) = sin(*(F2))

#define CALL_PRIM_SIN_SIMD(F1, B, F2) DlSin((F1), (F2), (B), numMp)

#define CALL_PRIM_ASIN(F1, F2) \
	*(F1) = ((*(F2) < -1.0) ? DL_PI : (*(F2) > 1.0) ? 0 : asin(*(F2)))

#define CALL_PRIM_ASIN_SIMD(F1, B, F2) DlASin((F1), (F2), (B), numMp)

#define CALL_PRIM_COS(F1, F2) *(F1) = cos(*(F2))

#define CALL_PRIM_COS_SIMD(F1, B, F2) DlCos((F1), (F2), (B), numMp)

#define CALL_PRIM_ACOS(F1, F2) \
	*(F1) = ((*(F2) < -1.0) ? DL_PI : (*(F2) > 1.0) ? 0 : acos(*(F2)))

#define CALL_PRIM_ACOS_SIMD(F1, B, F2) DlACos((F1), (F2), (B), numMp)

#define CALL_PRIM_TAN(F1, F2) *(F1) = tan(*(F2))

#define CALL_PRIM_TAN_SIMD(F1, B, F2) DlTan((F1), (F2), (B), numMp)

#define CALL_PRIM_ATAN(F1, F2) *(F1) = atan(*(F2))

#define CALL_PRIM_ATAN_SIMD(F1, B, F2) DlATan((F1), (F2), (B), numMp)

#define CALL_PRIM_ATAN2(F1, F2, F3) *(F1) = atan2(*(F2), *(F3))

#define CALL_PRIM_ATAN2_SIMD(F1, B, F2, F3) DlATan2((F1), (F2), (F3), (B), numMp)

#define CALL_PRIM_POW(F1, F2, F3) \
{ \
	if( *(F2)==0.f ) \
	{ \
		*(F1)=0.f; \
	} \
	else if( *(F2)<0.f ) \
	{ \
		/* if F2 is negative, then F3 must be an integer */ \
		if( ceil(*(F3))!= *(F3) ) \
		{ \
			*(F1) = .0f; \
		} \
		else \
		{ \
			*(F1) = pow(*(F2), *(F3)); \
		} \
	} \
	else \
	{ \
		*(F1) = pow(*(F2), *(F3)); \
	} \
}

#define CALL_PRIM_POW_SIMD(F1, B, F2, F3) DlPow((F1), (F2), (F3), (B), numMp)

#define CALL_PRIM_EXP(F1, F2) *(F1) = exp(*(F2))

#define CALL_PRIM_EXP_SIMD(F1, B, F2) DlExp((F1), (F2), (B), numMp)

#define CALL_PRIM_SQRT(F1, F2) \
{ \
	if (*(F2) <= 0.0) \
		*(F1) = 0.0; \
	else \
		*(F1) = sqrt(*(F2)); \
}

#define CALL_PRIM_SQRT_SIMD(F1, B, F2) DlSqrt((F1), (F2), (B), numMp)

#define CALL_PRIM_INVERSESQRT(F1, F2) \
{ \
	if (*(F2) <= 0.0) \
		*(F1) = FLT_MAX; \
	else \
		*(F1) = 1.0 / sqrt(*(F2)); \
}

#define CALL_PRIM_INVERSESQRT_SIMD(F1, B, F2) DlInverseSqrt((F1), (F2), (B), numMp)

#define CALL_PRIM_LOG(F1, F2) \
{ if (*(F2) <= 0.0) *(F1) = -FLT_MAX; else *(F1) = log(*(F2)); }

#define CALL_PRIM_LOG_SIMD(F1, B, F2) DlLog((F1), (F2), (B), numMp)

#define CALL_PRIM_LOG2(F1, F2, F3) \
{ \
	if (*(F2) <= 0.0 || (*(F3) <= 0.0)) \
		*(F1) = 0; \
	else \
		*(F1) = (log(*(F2))) / (log(*(F3))); \
}

#define CALL_PRIM_LOG2_SIMD(F1, B, F2, F3) DlLog2((F1), (F2), (F3), (B), numMp)

#define CALL_PRIM_MOD(F1, F2, F3)  \
{ int n = (int)((F2)/(F3)); \
  (F1) = (F2) - n * (F3); \
  if ((F1) < 0) (F1) += fabs(F3); \
}

#define CALL_PRIM_ABS(F1, F2) (F1) = fabs(F2)

#define CALL_PRIM_SIGN(F1, F2) \
	(F1) = ((F2) > 0) ? 1.0 : (((F2) < 0) ? -1.0 : 0.0)

#define CALL_PRIM_MAX2_FLOAT(F1, F2, F3) \
	((F1) = ((F2) > (F3)) ? (F2) : (F3))

#define CALL_PRIM_MIN2_FLOAT(F1, F2, F3) \
	((F1) = ((F2) < (F3)) ? (F2) : (F3))

#define CALL_PRIM_CLAMP_1D(F1, F2, F3, F4)  \
      (F1) = ((F2) < (F3)) ? (F3) : (((F2) > (F4)) ? (F4) : (F2))

#define CALL_PRIM_CLAMP_3D(P1, P2, P3, P4) \
	DlClamp3D((P1), (P2), (P3), (P4), 0, 1);

#define CALL_PRIM_CLAMP_3D_SIMD(P1, B, P2, P3, P4)  \
	DlClamp3D((P1), (P2), (P3), (P4), (B), numMp);

#define CALL_PRIM_MIN_1D(F1, ARGS, NB)  DlMin1D(&(F1), (ARGS), (NB))

#define CALL_PRIM_MIN_3D(P1, ARGS, NB)  DlMin3D(&(P1), (ARGS), (NB))

#define CALL_PRIM_MAX_1D(F1, ARGS, NB)  DlMax1D(&(F1), (ARGS), (NB))

#define CALL_PRIM_MAX_3D(P1, ARGS, NB)  DlMax3D(&(P1), (ARGS), (NB))

#define CALL_PRIM_FLOOR(F1, F2)     (F1) = floor(F2)

#define CALL_PRIM_CEIL(F1, F2)      (F1) = ceil(F2)

#define CALL_PRIM_ROUND(F1, F2)     (F1) = floor((F2) + 0.5)

#define CALL_PRIM_STEP(F1, F2, F3)  (F1) = ((F3) < (F2)) ? 0 : 1

#define CALL_PRIM_SMOOTHSTEP(F1, F2, F3, F4)  \
    DlSmoothStep((F1), (F2), (F3), (F4), 0, 1)

#define CALL_PRIM_SMOOTHSTEP_SIMD(F1, B, F2, F3, F4) \
    DlSmoothStep((F1), (F2), (F3), (F4), (B),  numMp)

#define CALL_PRIM_SPLINE_1D(F1, ARGS, NB)  \
    DlSpline1DWithBasis(&(F1), e_CatmullRomBasis, (ARGS), (NB))

#define CALL_PRIM_SPLINE_3D(P1, ARGS, NB)  \
    DlSpline3DWithBasis(&(P1), e_CatmullRomBasis, (ARGS), (NB))

#define CALL_PRIM_SPLINE_BASIS_1D(F1, ARGS, NB)  DlSpline1D(&(F1), (ARGS), (NB))

#define CALL_PRIM_SPLINE_BASIS_3D(F1, ARGS, NB)  DlSpline3D(&(F1), (ARGS), (NB))

#define CALL_PRIM_SPLINE_ARRAY_1D(F1, VAL, CTRLPTS, NBPTS)  \
	DlSplineArray1D(&(F1), 0x0, VAL, CTRLPTS, NBPTS)

#define CALL_PRIM_SPLINE_ARRAY_3D(P1, VAL, CTRLPTS, NBPTS)  \
	DlSplineArray3D(&(P1), 0x0, VAL, CTRLPTS, NBPTS)

#define CALL_PRIM_SPLINE_BASIS_ARRAY_1D(F1, BASIS, VAL, CTRLPTS, NBPTS) \
	DlSplineArray1D(&(F1), BASIS, VAL, CTRLPTS, NBPTS)

#define CALL_PRIM_SPLINE_BASIS_ARRAY_3D(P1, BASIS, VAL, CTRLPTS, NBPTS) \
	DlSplineArray3D(&(P1), BASIS, VAL, CTRLPTS, NBPTS)

////////////////////////////////////////////////////////////////////////////
// Du
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_DU_FLOAT_SIMD(F1, B, F2)    \
	DlDuDv1D((F1), 0x0, (F2), numMp, (B), i_context)

#define CALL_PRIM_DU_POINT_SIMD(P1, B, P2)    \
	DlDuDv3D((P1), 0x0, (P2), numMp, (B), i_context)

////////////////////////////////////////////////////////////////////////////
// Dv
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_DV_FLOAT_SIMD(F1, B, F2)    \
	DlDuDv1D(0x0, (F1), (F2), numMp, (B), i_context)

#define CALL_PRIM_DV_POINT_SIMD(P1, B, P2)    \
	DlDuDv3D(0x0, (P1), (P2), numMp, (B), i_context)

////////////////////////////////////////////////////////////////////////////
// Deriv
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_DERIV_FLOAT_SIMD(F1, B, F2, F3) \
	DlDeriv1D((F1), (F2), (F3), numMp, (B), i_context)

#define CALL_PRIM_DERIV_POINT_SIMD(P1, B, P2, F3) \
	DlDeriv3D((P1), (P2), (F3), numMp, (B), i_context)

/*
	(internal) Differential, for derivable parameters.
*/
#define CALL_INTERNAL_DIFFERENTIAL3D_SIMD(B, P, Pdu, Pdv) \
	DlDifferential3D((Pdu), (Pdv), (P), numMp, i_context)

#define CALL_INTERNAL_DIFFERENTIAL1D_SIMD(B, F, Fdu, Fdv) \
	DlDifferential1D((Fdu), (Fdv), (F), numMp, i_context)

////////////////////////////////////////////////////////////////////////////
// Random
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_RANDOM_FLOAT(F1) DlRandom1D((F1), 0, 1, i_context)

#define CALL_PRIM_RANDOM_FLOAT_SIMD(F1, B) \
	DlRandom1D((F1), (B),  numMp, i_context)

#define CALL_PRIM_RANDOM_POINT(C1) DlRandom3D((C1), 0, 1, i_context)

#define CALL_PRIM_RANDOM_POINT_SIMD(C1, B) \
	DlRandom3D((C1), (B),  numMp, i_context)

////////////////////////////////////////////////////////////////////////////
// Noise
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_NOISE_FLOAT_1D(F1, F2) \
    DlFNoise1D((F1), (F2), 0, 1)

#define CALL_PRIM_NOISE_FLOAT_2D(F1, F2, F3) \
    DlFNoise2D((F1), (F2), (F3), 0, 1)

#define CALL_PRIM_NOISE_FLOAT_3D(F1, P2)  \
     DlFNoise3D((F1), (P2), 0, 1)

#define CALL_PRIM_NOISE_FLOAT_4D(F1, P2, F3)  \
     DlFNoise4D((F1), (P2), (F3), 0, 1)

#define CALL_PRIM_NOISE_POINT_1D(P1, F2) \
    DlPNoise1D((P1), (F2), 0, 1)

#define CALL_PRIM_NOISE_POINT_2D(P1, F2, F3) \
    DlPNoise2D((P1), (F2), (F3), 0, 1)

#define CALL_PRIM_NOISE_POINT_3D(P1, P2)  \
     DlPNoise3D((P1), (P2), 0, 1)

#define CALL_PRIM_NOISE_POINT_4D(P1, P2, F3)  \
     DlPNoise4D((P1), (P2), (F3), 0, 1)

#define CALL_PRIM_NOISE_FLOAT_1D_SIMD(F1, B, F2) \
    DlFNoise1D((F1), (F2), (B),  numMp)

#define CALL_PRIM_NOISE_FLOAT_2D_SIMD(F1, B, F2, F3) \
    DlFNoise2D((F1), (F2), (F3), (B),  numMp)

#define CALL_PRIM_NOISE_FLOAT_3D_SIMD(F1, B, P2)  \
    DlFNoise3D((F1), (P2), (B),  numMp)

#define CALL_PRIM_NOISE_FLOAT_4D_SIMD(F1, B, P2, F3)  \
    DlFNoise4D((F1), (P2), (F3), (B),  numMp)

#define CALL_PRIM_NOISE_POINT_1D_SIMD(P1, B, F2) \
    DlPNoise1D((P1), (F2), (B),  numMp)

#define CALL_PRIM_NOISE_POINT_2D_SIMD(P1, B, F2, F3) \
    DlPNoise2D((P1), (F2), (F3), (B),  numMp)

#define CALL_PRIM_NOISE_POINT_3D_SIMD(P1, B, P2)  \
     DlPNoise3D((P1), (P2), (B),  numMp)

#define CALL_PRIM_NOISE_POINT_4D_SIMD(P1, B, P2, F3)  \
     DlPNoise4D((P1), (P2), (F3), (B),  numMp)

////////////////////////////////////////////////////////////////////////////
// Cellnoise
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_CELLNOISE_FLOAT_1D(F1, F2) \
    DlFCellnoise1D((F1), (F2), 0, 1)

#define CALL_PRIM_CELLNOISE_FLOAT_2D(F1, F2, F3) \
    DlFCellnoise2D((F1), (F2), (F3), 0, 1)

#define CALL_PRIM_CELLNOISE_FLOAT_3D(F1, P2) \
     DlFCellnoise3D((F1), (P2), 0, 1)

#define CALL_PRIM_CELLNOISE_FLOAT_4D(F1, P2, F3)  \
     DlFCellnoise4D((F1), (P2), (F3), 0, 1);

#define CALL_PRIM_CELLNOISE_POINT_1D(P1, F2) \
    DlPCellnoise1D((P1), (F2), 0, 1)

#define CALL_PRIM_CELLNOISE_POINT_2D(P1, F2, F3) \
    DlPCellnoise2D((P1), (F2), (F3), 0, 1)

#define CALL_PRIM_CELLNOISE_POINT_3D(P1, P2)  \
     DlPCellnoise3D((P1), (P2), 0, 1)

#define CALL_PRIM_CELLNOISE_POINT_4D(P1, P2, F3)  \
     DlPCellnoise4D((P1), (P2), (F3), 0, 1)

#define CALL_PRIM_CELLNOISE_FLOAT_1D_SIMD(F1, B, F2) \
    DlFCellnoise1D((F1), (F2), (B), numMp)

#define CALL_PRIM_CELLNOISE_FLOAT_2D_SIMD(F1, B, F2, F3) \
    DlFCellnoise2D((F1), (F2), (F3), (B), numMp)

#define CALL_PRIM_CELLNOISE_FLOAT_3D_SIMD(F1, B, P2)  \
     DlFCellnoise3D((F1), (P2), (B), numMp)

#define CALL_PRIM_CELLNOISE_FLOAT_4D_SIMD(F1, B, P2, F3)  \
     DlFCellnoise4D((F1), (P2), (F3), (B), numMp)

#define CALL_PRIM_CELLNOISE_POINT_1D_SIMD(P1, B, F2) \
    DlPCellnoise1D((P1), (F2), (B), numMp)

#define CALL_PRIM_CELLNOISE_POINT_2D_SIMD(P1, B, F2, F3) \
    DlPCellnoise2D((P1), (F2), (F3), (B), numMp)

#define CALL_PRIM_CELLNOISE_POINT_3D_SIMD(P1, B, P2)  \
     DlPCellnoise3D((P1), (P2), (B), numMp)

#define CALL_PRIM_CELLNOISE_POINT_4D_SIMD(P1, B, P2, F3)  \
     DlPCellnoise4D((P1), (P2), (F3), (B), numMp)

////////////////////////////////////////////////////////////////////////////
// P noise
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_PNOISE_FLOAT_1D(F1, F2, P) \
	DlFPnoise1D((F1), (F2), (P), 0, 1)

#define CALL_PRIM_PNOISE_FLOAT_2D(F1, F2, F3, UP, VP) \
	DlFPnoise2D((F1), (F2), (F3), (UP), (VP), 0, 1)

#define CALL_PRIM_PNOISE_FLOAT_3D(F1, P2, P)  \
	DlFPnoise3D((F1), (P2), (P), 0, 1)

#define CALL_PRIM_PNOISE_FLOAT_4D(F1, P2, F3, PP, TP)  \
	DlFPnoise4D((F1), (P2), (F3), (PP), (TP), 0, 1)

#define CALL_PRIM_PNOISE_POINT_1D(P1, F2, P) \
	DlPPnoise1D((P1), (F2), (P), 0, 1)

#define CALL_PRIM_PNOISE_POINT_2D(P1, F2, F3, UP, VP) \
	DlPPnoise2D((P1), (F2), (F3), (UP), (VP), 0, 1)

#define CALL_PRIM_PNOISE_POINT_3D(P1, P2, P)  \
	DlPPnoise3D((P1), (P2), (P), 0, 1)

#define CALL_PRIM_PNOISE_POINT_4D(P1, P2, F3, PP, TP)  \
	DlPPnoise4D((P1), (P2), (F3), (PP), (TP), 0, 1);  \

#define CALL_PRIM_PNOISE_FLOAT_1D_SIMD(F1, B, F2, P) \
	DlFPnoise1D((F1), (F2), (P), (B), numMp)

#define CALL_PRIM_PNOISE_FLOAT_2D_SIMD(F1, B, F2, F3, UP, VP) \
	DlFPnoise2D((F1), (F2), (F3), (UP), (VP), (B), numMp)

#define CALL_PRIM_PNOISE_FLOAT_3D_SIMD(F1, B, P2, P)  \
	DlFPnoise3D((F1), (P2), (P), (B), numMp)

#define CALL_PRIM_PNOISE_FLOAT_4D_SIMD(F1, B, P2, F3, PP, TP)  \
	DlFPnoise4D((F1), (P2), (F3), (PP), (TP), (B), numMp)

#define CALL_PRIM_PNOISE_POINT_1D_SIMD(P1, B, F2, P) \
	DlPPnoise1D((P1), (F2), (P), (B), numMp)

#define CALL_PRIM_PNOISE_POINT_2D_SIMD(P1, B, F2, F3, UP, VP) \
	DlPPnoise2D((P1), (F2), (F3), (UP), (VP), (B), numMp)

#define CALL_PRIM_PNOISE_POINT_3D_SIMD(P1, B, P2, P)  \
	DlPPnoise3D((P1), (P2), (P), (B), numMp)

#define CALL_PRIM_PNOISE_POINT_4D_SIMD(P1, B, P2, F3, PP, TP)  \
	DlPPnoise4D((P1), (P2), (F3), (PP), (TP), (B), numMp)

////////////////////////////////////////////////////////////////////////////
// MACROS FOR GEOMETRIC FUNCTIONS
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_XCOMP(F1, P2)     (F1) = (P2)[0]

#define CALL_PRIM_YCOMP(F1, P2)     (F1) = (P2)[1]

#define CALL_PRIM_ZCOMP(F1, P2)     (F1) = (P2)[2]

#define CALL_PRIM_SETXCOMP(P1, F2)  (P1)[0] = (F2)

#define CALL_PRIM_SETYCOMP(P1, F2)  (P1)[1] = (F2)

#define CALL_PRIM_SETZCOMP(P1, F2)  (P1)[2] = (F2)

/* length shadeop */
#define CALL_PRIM_LENGTH(F1, P2) \
	DlLength((F1), (P2), 0, 1)

#define CALL_PRIM_LENGTH_SIMD(F1, B, P2) \
	DlLength((F1), (P2), (B),  numMp)

#define CALL_PRIM_DISTANCE(F1, P2, P3) \
	DlDistance((F1), (P2), (P3), 0, 1)

#define CALL_PRIM_DISTANCE_SIMD(F1, B, P2, P3)   \
	DlDistance((F1), (P2), (P3), (B),  numMp)

#define CALL_PRIM_AREA_SIMD(F1, B, P2) \
	DlArea((F1), (P2), numMp, (B), i_context)

/* normalize shadeop */
#define CALL_PRIM_NORMALIZE(P1, P2) DlNormalize((P1), (P2), 0, 1)

#define CALL_PRIM_NORMALIZE_SIMD(P1, B, P2) \
	DlNormalize((P1), (P2), (B), numMp)

/* ptlined shadeop */
#define CALL_PRIM_PTLINED(F, P0, P1, Q) \
	DlPtlined((F), (P0), (P1), (Q), 0, 1)

#define CALL_PRIM_PTLINED_SIMD(F, B, P0, P1, Q) \
	DlPtlined((F), (P0), (P1), Q, (B), numMp)

/* rotate shadeop */
#define CALL_PRIM_ROTATE_POINT(P, Q, ANGLE, P0, P1) \
	DlRotatePoint(&(P), &(Q), (ANGLE), (P0), (P1), 0, 1)

/* filterstep */
#define CALL_PRIM_FILTERSTEP(RES, E, S1, S2, ARGS, NB) \
{ \
   DlFilterstep((RES),(E),(S1), (S2), ARGS, NB, 0, 1); \
}

#define CALL_PRIM_FILTERSTEP_SIMD(RES, B, E, S1, S2, ARGS, NB)  \
   DlFilterstep((RES),(E),(S1), (S2), ARGS, NB, (B), numMp)

#define CALL_INTERNAL_FILTERSTEP_POINTS_SIMD(S, B, S1)  \
   DlMakeFilterstepPoints((S1), (S), numMp, (B), i_context)

////////////////////////////////////////////////////////////////////////////
// Faceforward
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_FACEFORWARD(P1, P2, P3)      \
	DlFaceforward((P1), (P2), (P3), 0, 0, 1, polyCnt, i_context) \

#define CALL_PRIM_FACEFORWARD_SIMD(P1, B, P2, P3) \
	DlFaceforward((P1), (P2), (P3), 0, (B), numMp, 0, i_context)

#define CALL_PRIM_FACEFORWARD2(P1, P2, P3, P4)      \
	DlFaceforward((P1), (P2), (P3), (P4), 0, 1, polyCnt, i_context)

#define CALL_PRIM_FACEFORWARD2_SIMD(P1, B, P2, P3, P4) \
	DlFaceforward((P1), (P2), (P3), (P4), (B), numMp, 0, i_context)

////////////////////////////////////////////////////////////////////////////
// Reflect
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_REFLECT(P1, P2, P3) DlReflect((P1), (P2), (P3), 0, 1)

#define CALL_PRIM_REFLECT_SIMD(P1, B, P2, P3) \
	DlReflect((P1), (P2), (P3), (B),  numMp)

////////////////////////////////////////////////////////////////////////////
// Refract
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_REFRACT(P1, P2, P3, F4) \
	DlRefract((P1), (P2), (P3), (F4), 0, 1)

#define CALL_PRIM_REFRACT_SIMD(P1, B, P2, P3, F4) \
	DlRefract((P1), (P2), (P3), (F4), (B),  numMp)

////////////////////////////////////////////////////////////////////////////
// Fresnel
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_FRESNEL5(P1, P2, F3, F4, F5) \
	DlFresnel((P1), (P2), (F3), (F4), (F5), 0, 0, 0, 1)

#define CALL_PRIM_FRESNEL5_SIMD(B, P1, P2, F3, F4, F5) \
	DlFresnel((P1), (P2), (F3), (F4), (F5), 0, 0, (B),  numMp)

#define CALL_PRIM_FRESNEL7(P1, P2, F3, F4, F5, P6, P7) \
	DlFresnel((P1), (P2), (F3), (F4), (F5), (P6), (P7), 0, 1)

#define CALL_PRIM_FRESNEL7_SIMD(B, P1, P2, F3, F4, F5, P6, P7) \
	DlFresnel((P1), (P2), (F3), (F4), (F5), (P6), (P7), (B),  numMp)

////////////////////////////////////////////////////////////////////////////
// transform, vtransform and ntransform
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
// Version 4
//
// Spaces are not known at compile-time.
////////////////////////////////////////////////////////

#define FIND_SPACE(S) DlFindSpaceFromName(S, i_context)

#define CALL_PRIM_TRANSFORM(P1, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 0, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 0, true, 0, 1, i_context); \
}

#define CALL_PRIM_VTRANSFORM(P1, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 1, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 1, true, 0, 1, i_context); \
}

#define CALL_PRIM_NTRANSFORM(P1, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 2, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 2, true, 0, 1, i_context); \
}

#define CALL_PRIM_TRANSFORM_SIMD(P1, B, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 0, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 0, true, (B), numMp, i_context); \
}

#define CALL_PRIM_VTRANSFORM_SIMD(P1, B, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 1, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 1, true, (B), numMp, i_context); \
}

#define CALL_PRIM_NTRANSFORM_SIMD(P1, B, S2, S3, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 2, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), toSpace, 0, 2, true, (B), numMp, i_context); \
}

#define CALL_PRIM_TRANSFORM2(P1, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 0, true, 0, 1, i_context); \
}

#define CALL_PRIM_VTRANSFORM2(P1, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 1, true, 0, 1, i_context); \
}

#define CALL_PRIM_NTRANSFORM2(P1, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 2, true, 0, 1, i_context); \
}

#define CALL_PRIM_TRANSFORM2_SIMD(P1, B, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 0, true,  (B), numMp, i_context); \
}

#define CALL_PRIM_VTRANSFORM2_SIMD(P1, B, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 1, true, (B), numMp, i_context); \
}

#define CALL_PRIM_NTRANSFORM2_SIMD(P1, B, S3, P4) \
{ \
	const DlFrame3D *toSpace = FIND_SPACE(S3); \
	DlTransformSpace((P1), (P4), toSpace, 0, 2, true, (B), numMp, i_context); \
}

////////////////////////////////////////////////////////////////////////////
// Transform functions with matrices
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_TRANSFORM_MATRIX(P1, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 0, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 0, true, 0, 1, i_context); \
}

#define CALL_PRIM_VTRANSFORM_MATRIX(P1, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 1, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 1, true, 0, 1, i_context); \
}

#define CALL_PRIM_NTRANSFORM_MATRIX(P1, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 2, false, 0, 1, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 2, true, 0, 1, i_context); \
}

#define CALL_PRIM_TRANSFORM_MATRIX_SIMD(P1, B, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 0, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 0, true, (B), numMp, i_context); \
}

#define CALL_PRIM_VTRANSFORM_MATRIX_SIMD(P1, B, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 1, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 1, true, (B), numMp, i_context); \
}

#define CALL_PRIM_NTRANSFORM_MATRIX_SIMD(P1, B, S2, M, P4) \
{ \
	const DlFrame3D *fromSpace = FIND_SPACE(S2); \
	DlTransformSpace((P1), (P4), fromSpace, 0, 2, false, (B), numMp, i_context); \
	DlTransformSpace((P1), (P1), 0, &(M), 2, true, (B), numMp, i_context); \
}

#define CALL_PRIM_TRANSFORM2_MATRIX(P1, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 0, true, 0, 1, i_context); \
}

#define CALL_PRIM_VTRANSFORM2_MATRIX(P1, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 1, true, 0, 1, i_context); \
}

#define CALL_PRIM_NTRANSFORM2_MATRIX(P1, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 2, true, 0, 1, i_context); \
}

#define CALL_PRIM_TRANSFORM2_MATRIX_SIMD(P1, B, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 0, true, (B), numMp, i_context); \
}

#define CALL_PRIM_VTRANSFORM2_MATRIX_SIMD(P1, B, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 1, true, (B), numMp, i_context); \
}

#define CALL_PRIM_NTRANSFORM2_MATRIX_SIMD(P1, B, M, P2) \
{ \
	DlTransformSpace((P1), (P2), 0, &(M), 2, true, (B), numMp, i_context); \
}

////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_DEPTH(F1, P2)   DlDepth((F1), (P2), i_context, 0, 1)

#define CALL_PRIM_DEPTH_SIMD(F1, B, P2)   \
	DlDepth((F1), (P2), i_context, (B),  numMp)

#define CALL_PRIM_CALCULATENORMAL_SIMD( P1, B, P2 )  \
	DlCalculateNormal((P1), (P2), numMp, (B), i_context )

////////////////////////////////////////////////////////////////////////////
// MACROS FOR COMPONENT ACCESS FUNCTIONS (matrices, colors/points)
////////////////////////////////////////////////////////////////////////////
/*
	NOTE:
	There is no need to do the following:
	"
	int index = int(F2);
	if( index>=0 && index<=2 )
		...
	"

	Since we can do a more optimal:

	"
	unsigned index = unsigned(F2);
	if( index <= 2 )
		...
	"

	This is valid since negative floats will 'wrap' around and become large
	unsigned values.
*/

#define CALL_PRIM_COMP(F1, C2, F3) \
{ \
	unsigned index = unsigned(F3); \
	if( index <= 2 ) \
	{ \
		(F1) = *(((float*)(&(C2))) + index); \
	} \
}

#define CALL_PRIM_SETCOMP(C1, F2, F3) \
{ \
    unsigned index = unsigned(F2); \
	if( index <= 2 ) \
	{ \
		*(((float*)(&(C1))) + index) = (F3); \
	} \
}

#define CALL_PRIM_COMP_MATRIX(F1, M, R, C)  \
{ \
    unsigned row = unsigned(R); \
	unsigned col = unsigned(C); \
    if( row <= 3 && col <= 3) \
    { \
        (F1) = (M)[ row*4 + col ]; \
    } \
}

#define CALL_PRIM_SETCOMP_MATRIX(M, R, C, F) \
{ \
    unsigned row = unsigned( R ); \
	unsigned col = unsigned( C ); \
    if( row <= 3 && col <= 3)  \
    { \
        (M)[row*4+col] = (F); \
    } \
}

#define CALL_PRIM_MIX_FLOAT_FLOAT_FLOAT(RES, float0, float1, value) \
{ \
	RES = float0*(1.0f-value) + float1*value; \
}

#define CALL_PRIM_MIX_POINT_POINT_FLOAT(RES, color0, color1, value) \
{ \
	(RES)[0] = (color0)[0]*(1.0f-value) + (color1)[0]*value; \
	(RES)[1] = (color0)[1]*(1.0f-value) + (color1)[1]*value; \
	(RES)[2] = (color0)[2]*(1.0f-value) + (color1)[2]*value; \
}

#define CALL_PRIM_MIX_POINT_POINT_POINT(RES, color0, color1, value) \
{ \
	(RES)[0] = (color0)[0]*(1.0f-(value)[0]) + (color1)[0]*(value)[0]; \
	(RES)[1] = (color0)[1]*(1.0f-(value)[1]) + (color1)[1]*(value)[1]; \
	(RES)[2] = (color0)[2]*(1.0f-(value)[2]) + (color1)[2]*(value)[2]; \
}

#define CALL_PRIM_MIX_1D(C1, F2, F3, F4)  \
    DlMix1D((C1), (F2), (F3), (F4), 0, 1); \

#define CALL_PRIM_MIX_1D_SIMD(C1, B, C2, C3, F4)  \
	DlMix1D((C1), (C2), (C3), (F4), (B),  numMp)

#define CALL_PRIM_MIX_3D(C1, C2, C3, F4) \
	DlMix3D((C1), (C2), (C3), (F4), 0, 1)

#define CALL_PRIM_MIX_3D_SIMD(C1, B, C2, C3, F4)  \
	DlMix3D((C1), (C2), (C3), (F4), (B),  numMp)

#define CALL_PRIM_CTRANSFORM_FROM_RGB(C1, C2, S) \
	DlTransformColor((C1), (C2), (S), true, 0, 1)

#define CALL_PRIM_CTRANSFORM_FROM_RGB_SIMD(C1, B, C2, S) \
	DlTransformColor((C1), (C2), (S), true, (B),  numMp)

#define CALL_PRIM_CTRANSFORM_TO_RGB(C1, C2, S) \
	DlTransformColor((C1), (C2), (S), false, 0, 1)

#define CALL_PRIM_CTRANSFORM_TO_RGB_SIMD(C1, B, C2, S) \
	DlTransformColor((C1), (C2), (S), false, (B),  numMp)

#define CALL_PRIM_CTRANSFORM(C1, S1, S2, C2) \
{ \
     CALL_PRIM_CTRANSFORM_TO_RGB(C1, C2, S1); \
     CALL_PRIM_CTRANSFORM_FROM_RGB(C1, C1, S2);  \
}

#define CALL_PRIM_CTRANSFORM_SIMD(C1, B, S1, S2, C2) \
{ \
     CALL_PRIM_CTRANSFORM_TO_RGB_SIMD(C1, B, C2, S1); \
     CALL_PRIM_CTRANSFORM_FROM_RGB_SIMD(C1, B, C1, S2);  \
}

#define CALL_PRIM_CTRANSFORM2(C1, S1, C2) \
{ \
     CALL_PRIM_CTRANSFORM_FROM_RGB(C1, C2, S1);  \
}

#define CALL_PRIM_CTRANSFORM2_SIMD(C1, B, S1, C2) \
{ \
     CALL_PRIM_CTRANSFORM_FROM_RGB_SIMD(C1, B, C2, S1);  \
}

////////////////////////////////////////////////////////////////////////////
// MACROS FOR SHADING AND LIGHTING FUNCTIONS
////////////////////////////////////////////////////////////////////////////

/* ambient()
*/
#define CALL_PRIM_AMBIENT(C1) \
	DlAmbient( \
		(C1), \
		i_numLights, i_lights, \
		0x0, 1, polyCnt, i_context )

#define CALL_PRIM_AMBIENT_SIMD(C1, B) \
	DlAmbient( \
		(C1), \
		i_numLights, i_lights, \
		(B), numMp, 0, \
		i_context )

/* ambient()
*/
#define CALL_PRIM_DIFFUSE(C1, P2) \
	DlDiffuse( \
		(C1), (P2), \
		i_numLights, i_lights, \
		0x0, 1, polyCnt, i_context )

#define CALL_PRIM_DIFFUSE_SIMD(C1, B, P2) \
	DlDiffuse( \
		(C1), (P2), \
		i_numLights, i_lights, \
		(B), numMp, 0x0, i_context )

/* specular()
*/
#define CALL_PRIM_SPECULAR(C1, P2, P3, F4) \
	DlSpecular((C1), (P2),\
		(P3), (F4), \
		i_numLights, i_lights,    \
		0x0, 1, polyCnt, i_context)

#define CALL_PRIM_SPECULAR_SIMD(C1, B, P2, P3, F4) \
	DlSpecular((C1), (P2),\
		(P3), (F4), \
		i_numLights, \
		i_lights,    \
		(B), numMp, 0, i_context )

/* specularstd()
*/
#define CALL_PRIM_SPECULARSTD(C1, P2, P3, F4) \
	DlSpecularstd((C1), (P2),\
		(P3), (F4), \
		i_numLights, i_lights,    \
		0x0, 1, polyCnt, i_context)

#define CALL_PRIM_SPECULARSTD_SIMD(C1, B, P2, P3, F4) \
	DlSpecularstd((C1), (P2),\
		(P3), (F4), \
		i_numLights, i_lights,    \
		(B), numMp, 0, i_context)

/* specularbrdf()
*/
#define CALL_PRIM_SPECULARBRDF(C, L, N, V, R) \
	DlSpecularbrdf((C), (L), \
		(N), (V), \
		(R),       \
		0, 1)

#define CALL_PRIM_SPECULARBRDF_SIMD(C, B, L, N, V, R) \
	DlSpecularbrdf((C), (L), \
		(N), (V), \
		(R),      \
		(B), numMp )

/* phong()
*/
#define CALL_PRIM_PHONG(C1, P2, P3, F4) \
	DlPhong((C1), (P2),\
		(P3), (F4), \
		i_numLights, i_lights,    \
		0x0, 1, polyCnt, i_context)

#define CALL_PRIM_PHONG_SIMD(C1, B, P2, P3, F4) \
	DlPhong((C1), (P2), \
		(P3), (F4), \
		i_numLights, \
		i_lights, \
		(B), numMp, 0, i_context )

#define CALL_PRIM_IRRADIANCE( C, ENVMAP, N ) \
	DlIrradiance((C), ENVMAP, (N), \
		0x0, 1, \
		i_context )

#define CALL_PRIM_IRRADIANCE_SIMD(C, B, ENVMAP, N ) \
	DlIrradiance( C, ENVMAP, N, \
		(B), numMp, \
		i_context )

/* ------------------------------------------------------ R A Y    T R A C I N G
*/
#define CALL_PRIM_SUBSURFACE( C, P, N, args, nbArgs ) \
	DlSubSurface((C), (P), (N), \
		args, nbArgs, \
		0x0, 1, \
		i_context )

#define CALL_PRIM_SUBSURFACE_SIMD(C, B, P, N, args, nbArgs ) \
	DlSubSurface( C, P, (N), \
		args, nbArgs, \
		(B), numMp, \
		i_context )

#define CALL_PRIM_TRACE(C1, S, D, o_dist, args, nbArgs, DSU, DSV, DDU, DDV ) \
	DlTrace((C1), (o_dist), (S), (DSU), (DSV), (D), (DDU), (DDV), \
		args, nbArgs, \
		0, 1, polyCnt, \
		i_context )

#define CALL_PRIM_TRACE_SIMD(C1, B, S, D, o_dist, args, nbArgs, DSU, DSV, DDU, DDV ) \
	DlTrace((C1), (o_dist), (S), (DSU), (DSV), (D), (DDU), (DDV), \
		args, nbArgs, \
		(B), numMp, 0,  \
		i_context )

#define CALL_PRIM_OCCLUSION(Ret, P, R, Samples, ARGS, NB) \
{ \
	DlOcclusion_IndirectDiffuse( \
		0x0, (Ret), \
		(P), (R), (Samples), ARGS, NB, \
		0x0, \
		1, polyCnt, \
		i_context ); \
}

#define CALL_PRIM_OCCLUSION_SIMD(Ret, B, P, R, Samples, ARGS, NB ) \
{ \
	DlOcclusion_IndirectDiffuse( \
		0x0, (Ret), \
		(P), (R), (Samples), ARGS, NB, \
		(B), \
		numMp, 0, \
		i_context ); \
}

#define CALL_PRIM_INDIRECTDIFFUSE( Ret, P, R, Samples, ARGS, NB ); \
{ \
	DlOcclusion_IndirectDiffuse( \
		(Ret), 0x0, \
		(P), (R), (Samples), ARGS, NB, \
		0x0, \
		1, polyCnt, \
		i_context ); \
}

#define CALL_PRIM_INDIRECTDIFFUSE_SIMD( Ret, B, P, R, Samples, ARGS, NB ) \
{ \
	DlOcclusion_IndirectDiffuse( \
		(Ret), 0x0, \
		(P), (R), (Samples), ARGS, NB, \
		(B), \
		numMp, 0, \
		i_context); \
}

#define CALL_PRIM_PHOTONMAP(C1, map, P, N, args, nbArgs, DPU, DPV ) \
	DlPhotonmap((C1), (map), (P), (DPU), (DPV), (N), \
		args, nbArgs, \
		0, 1, \
		i_context )

#define CALL_PRIM_PHOTONMAP_SIMD(C1, B, map, P, N, args, nbArgs, DPU, DPV ) \
	DlPhotonmap((C1), (map), (P), (DPU), (DPV), (N), \
		args, nbArgs, \
		(B), numMp, \
		i_context )

////////////////////////////////////////////////////////////////////////////
// MACROS FOR FUNCTIONS ON STRINGS
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_PRINTF(ARGS, NB)    DlPrintf(ARGS, NB)

#define CALL_PRIM_CONCAT(S, ARGS, NB)  DlConcat(&(S), ARGS, NB)

#define CALL_PRIM_FORMAT(S, ARGS, NB)  DlFormat(&(S), ARGS, NB)

#define CALL_PRIM_MATCH(F, S1, S2)     DlMatch(&(F), (S1), (S2))

////////////////////////////////////////////////////////////////////////////
// MACROS FOR LIGHT STATEMENTS
////////////////////////////////////////////////////////////////////////////

// For illuminate, Axis and angle are optional uniform.

#define ILLUMINATE(RES, P, AXIS, ANGLE, PS, L)  \
	DlIlluminate( \
		(RES), (P), (AXIS), (ANGLE), (PS), \
		(L), 0, 1, i_context )

#define ILLUMINATE_SIMD(RES, B, P, AXIS, ANGLE, PS, L)  \
	DlIlluminate((RES), (P), (AXIS), (ANGLE), \
		(PS), (L), (B), numMp, i_context )

#define ILLUMINATE_END( illum_flag, Ps, L, Cl )  \
	DlIlluminateEnd( (illum_flag), (Ps), (L), (Cl), 0, 1, i_context ); 

#define ILLUMINATE_END_SIMD( illum_flag, Ps, L, Cl, B )  \
	DlIlluminateEnd( (illum_flag), (Ps), (L), (Cl), (B), numMp, i_context ); 

#define SOLAR(IS_ILLU, PS, AXIS, ANGLE, LIGHT)  \
	DlSolar((IS_ILLU), (PS), (AXIS), (ANGLE), (LIGHT), 0, 1, i_context )

#define SOLAR_SIMD(IS_ILLU, B, PS, AXIS, ANGLE, LIGHT)  \
	DlSolar((IS_ILLU), (PS), (AXIS), (ANGLE), (LIGHT), (B), numMp, i_context )

#define SOLAR_END( illum_flag, Ps, L, Cl )  \
	DlSolarEnd( (illum_flag), (Ps), (L), (Cl), 0x0, 1, i_context ); 

#define SOLAR_END_SIMD( illum_flag, Ps, L, Cl, B )  \
	DlSolarEnd( (illum_flag), (Ps), (L), (Cl), (B), numMp, i_context ); 

#define ILLUMINANCE(B0, P1, P2, F3, S4) \
	DlIlluminance( \
		(B0), (P1), (P2), (F3), (S4), \
		0, 1, i_lights, lightCnt, polyCnt)

#define ILLUMINANCE_SIMD(B0, P1, P2, F3, S4, B5)  \
	DlIlluminance( \
		(B0), (P1), (P2), (F3), (S4), \
		(B5),  numMp, i_lights, lightCnt, 0)

#define ILLUMINANCE_BEGIN(B, L, Cl, Ol, evalPoints, category, \
		nbInArgs, inArgs, nbOutArgs, outArgs, typeArgs ) \
	DlIlluminanceCounter(lightCnt, (B), \
		(DlVector3 **) (L), (DlRGBColor **) (Cl), (DlRGBColor **)(Ol), \
		nbInArgs, inArgs, nbOutArgs, outArgs, typeArgs, \
		0, evalPoints, category, i_numLights, i_lights, numMp, i_context )

#define ILLUMINANCE_INCR(B, L, Cl, Ol, evalPoints, category, \
		nbInArgs, inArgs, nbOutArgs, outArgs, typeArgs ) \
	DlIlluminanceCounter(lightCnt, (B), \
		(DlVector3 **) (L), (DlRGBColor **) (Cl), (DlRGBColor **)(Ol), \
		nbInArgs, inArgs, nbOutArgs, outArgs, typeArgs, \
		1, evalPoints, category, i_numLights, i_lights, numMp, i_context )

#define GATHER_BEGIN(Category, P, Pdu, Pdv, Dir, Angle, \
	Samples, Handle, nbIn, In, NbOut, Names, Out, Types, B) \
	DlGatherBegin(Category, P, Pdu, Pdv, Dir, Angle, &(Samples), \
		&(Handle), nbIn, In, \
		NbOut, Names, Out, Types, \
		(B), numMp, i_context )

#define GATHER_CHECK(Handle, Count, B0, Out, B) \
	DlGatherCheck(Handle, Count, B0, Out, (B), \
		numMp, i_context)

#define GATHER_END(H) \
	DlGatherEnd(H)

////////////////////////////////////////////////////////////////////////////
// CALL_TEXTURE_1D
//
// R                               : return result
// TX                              : name of the texture
// CH                              : channel
// S, T                            : texture points
// DSDU, DSDV, DTDU, DTDV          : texture derivates
// DUMMY1, DUMMY2                  : paceholders
// V                               : variance
// SPLES                           : samples
// SB, TB                          : blur on 's' and 't' directions
// SW, TW                          : width on 's' and 't' directions
// FLTR                            : filter
// FIL                             : filler
// LERP                            : lerp (1=yes)
////////////////////////////////////////////////////////////////////////////

#define CALL_TEXTURE_1D( R, TX, CH, S, T, DSDU, DTDU, DSDV, DTDV, DUMMY1, DUMMY2, ARGS, NB ) \
	DlTexture( \
		(R), 1, (TX), unsigned(CH), (S), (T), (DSDU), (DTDU), (DSDV), (DTDV), \
		ARGS, NB, 1, 0, i_context)

#define CALL_TEXTURE_1D_SIMD( \
	R, B, TX, CH, S, T, DSDU, DTDU, DSDV, DTDV, DUMMY1, DUMMY2, ARGS, NB ) \
	DlTexture( \
		(R), 1, (TX), unsigned(CH), (S), (T), (DSDU), (DTDU), (DSDV), (DTDV), \
		ARGS, NB, numMp, (B), i_context )

////////////////////////////////////////////////////////////////////////////
// CALL_TEXTURE_3D
//
// C1                              : return result
// TX                              : name of the texture
// CH                              : channel
// S, T                            : texture points
// DSDU, DSDV, DTDU, DTDV          : texture derivates
// DUMMY1, DUMMY2                  : paceholders
// V                               : variance
// SPLES                           : samples
// SB, TB                          : blur on 's' and 't' directions
// SW, TW                          : width on 's' and 't' directions
// FLTR                            : filter
// FIL                             : filler
////////////////////////////////////////////////////////////////////////////

#define CALL_TEXTURE_3D( R, TX, CH, S, T, DSDU, DTDU, DSDV, DTDV, DUMMY1, DUMMY2, ARGS, NB ) \
	DlTexture( \
		(float*)((R)), 3, (TX), unsigned(CH), (S), (T), (DSDU), (DTDU), (DSDV), (DTDV), \
		ARGS, NB, 1, 0, i_context)

#define CALL_TEXTURE_3D_SIMD( \
	R, B, TX, CH, S, T, DSDU, DTDU, DSDV, DTDV, DUMMY1, DUMMY2, ARGS, NB ) \
	DlTexture( \
		(float*)(R), 3, (TX), unsigned(CH), (S), (T), (DSDU), (DTDU), (DSDV), (DTDV), \
		ARGS, NB, numMp, (B), i_context)

////////////////////////////////////////////////////////////////////////////
// CALL_ENVIRONEMENT_1D
//
// R              : return result (float)
// TX             : name of the texture
// CH             : channel
// P              : points of the texture
// DPDU, DPDV     : derivates of P
// DUMMY          : paceholder
// V              : variance
// SPLES          : samples
// SB, TB         : blur on 's' and 't' directions
// SW, TW         : width on 's' and 't' directions
// FLTR           : filter
// FIL            : filler
////////////////////////////////////////////////////////////////////////////

#define CALL_ENVIRONMENT_1D( R, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
	DlEnvironment((R), 1, (TX), unsigned(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, 1, polyCnt, 0x0, i_context )

#define CALL_ENVIRONMENT_1D_SIMD(R, B, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB) \
	DlEnvironment((R), 1, (TX), unsigned(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, numMp, 0, (B), i_context)

////////////////////////////////////////////////////////////////////////////
// CALL_ENVIRONEMENT_3D
//
// R              : return result (color)
// TX             : name of the texture
// CH             : channel
// P              : points of the texture
// DPDU, DPDV     : derivates of P
// DUMMY          : paceholder
////////////////////////////////////////////////////////////////////////////

#define CALL_ENVIRONMENT_3D( R, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
	DlEnvironment((float*)((R)), 3, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, 1, polyCnt, 0x0, i_context )

#define CALL_ENVIRONMENT_3D_SIMD( \
	R, B, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
	DlEnvironment((float*)(R), 3, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, numMp, 0, (B), i_context )

////////////////////////////////////////////////////////////////////////////
// CALL_SHADOW_1D
//
// R              : return result (opacity)
// TX             : name of the texture
// CH             : channel
// P              : points of the texture
// DPDU, DPDV     : derivates of P
// DUMMY          : paceholder
////////////////////////////////////////////////////////////////////////////

#define CALL_SHADOW_1D(R, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
{ \
	DlShadow( \
		(R), 1, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, \
		1, polyCnt, 0x0, i_context ); \
}

#define CALL_SHADOW_1D_SIMD( R, B, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
{ \
	DlShadow( \
		(R), 1, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB,\
		numMp, 0, (B), i_context ); \
}

////////////////////////////////////////////////////////////////////////////
// CALL_SHADOW_3D
//
// R              : return result (opacity)
// TX             : name of the texture
// CH             : channel
// P              : points of the texture
// DPDU, DPDV     : derivates of P
// DUMMY          : paceholder
////////////////////////////////////////////////////////////////////////////

#define CALL_SHADOW_3D(R, TX, CH, P,  DPDU, DPDV, DUMMY, ARGS, NB ) \
{ \
	DlShadow( \
		(float*)((R)), 3, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, \
		1, polyCnt, 0x0, i_context ); \
}

#define CALL_SHADOW_3D_SIMD(R, B, TX, CH, P, DPDU, DPDV, DUMMY, ARGS, NB ) \
{ \
	DlShadow( \
		(float*)(R), 3, (TX), int(CH), (P), (DPDU), (DPDV), \
		ARGS, NB, \
		numMp, 0, (B), i_context ); \
}

/*
	CALL_PRIM_BAKE

	filename      : filename to bake to. uniform.
	s, t          : coordinates to bake to.
	value         : values to bake
	channelname   : an optional parameter specifying a certain channel name
*/
#define CALL_PRIM_BAKE_SIMD( B, filename, s, t, value, channelname ) \
	DlBake( B, filename, 1, s, t, value, channelname, numMp, 0, i_context )

#define CALL_PRIM_BAKE( filename, s, t, value, channelname ) \
	DlBake( 0x0, filename, 1, s, t, value, channelname, 1, polyCnt, i_context )

#define CALL_PRIM_BAKE_POINT_SIMD( B, filename, s, t, value, channelname ) \
       DlBake( B, filename, 3, s, t, (float *)(value), \
               channelname, numMp, 0, i_context )

#define CALL_PRIM_BAKE_POINT( filename, s, t, value, channelname ) \
       DlBake( 0x0, filename, 3, s, t, (float *)(value), \
               channelname, 1, polyCnt, i_context )

/*
	CALL_PRIM_BAKE3D
	CALL_PRIM_TEXTURE3D
*/
#define CALL_PRIM_BAKE3D( res, B, filename, displaychannels, P, N, ARGS, NB ) \
		DlBake3d( res, filename, displaychannels, P, N, \
		ARGS, NB, numMp, B, i_context );

#define CALL_PRIM_TEXTURE3D( res, B, filename, P, N, ARGS, NB ) \
		DlTexture3d( res, filename, P, N, ARGS, NB, numMp, B, i_context );


////////////////////////////////////////////////////////////////////////////
// MACROS FOR MATRICES FUNCTIONS
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_INVERSE(M1, M2)       DlInverse(&(M1), &(M2))

#define CALL_PRIM_DETERMINANT(F, M)     DlDeterminant(&(F), &(M))

#define CALL_PRIM_TRANSLATE(M1, M2, P)  DlTranslate(&(M1), &(M2), &(P))

#define CALL_PRIM_ROTATE(M1, M2, F, V)  DlRotate(&(M1), &(M2), F, &(V))

#define CALL_PRIM_SCALE(M1, M2, P)      DlScale(&(M1), &(M2), &(P))

////////////////////////////////////////////////////////////////////////////
// Functions for access functions
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_SHADERNAME(RES) DlShaderName(&(RES), i_context)

#define CALL_PRIM_TEXTUREINFO(RES, F, PTR, NAME, IS_VAR, TYPE, NB_ELEMS)  \
{ \
	DlTextureInfo(&(RES), F, (void *)(PTR), NAME, TYPE, NB_ELEMS, i_context); \
}

#define CALL_PRIM_ATTRIBUTE(RES, PTR, NAME, IS_VAR, TYPE, NB_ELEMS)  \
{ \
	DlAttribute(&(RES), (void *)(PTR), NAME, TYPE, NB_ELEMS, &i_context, 0x0); \
}

#define CALL_PRIM_OPTION(RES, PTR, NAME, IS_VAR, TYPE, NB_ELEMS)  \
{ \
	DlOption(&(RES), (void *)(PTR), NAME, TYPE, NB_ELEMS, &i_context, 0x0); \
}

#define CALL_PRIM_RENDERERINFO(RES, PTR, NAME, IS_VAR, TYPE, NB_ELEMS)  \
{ \
	DlRendererInfo(&(RES), (void *)(PTR), NAME, TYPE, NB_ELEMS); \
}

#define CALL_PRIM_RAYINFO(RES, PTR, NAME, IS_VAR, TYPE, NB_ELEMS)  \
{ \
	DlRayInfo(&(RES), (void *)(PTR), NAME, TYPE, NB_ELEMS, i_context ); \
}

#define CALL_PRIM_SURFACE(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS)  \
	DlMessageParam((RES),  e_SurfaceParam, \
		(NAME), (VAR), IS_VAR, TYPE, NB_ELEMS, \
		(B),  numMp, i_context, 0)

#define CALL_PRIM_DISPLACEMENT(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS) \
    DlMessageParam((RES),  e_DisplacementParam, \
		(NAME), (VAR), IS_VAR, TYPE, NB_ELEMS, \
		(B), numMp, i_context, 0)

#define CALL_PRIM_LIGHTSOURCE(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS)  \
    DlMessageParam((RES), e_LightsourceParam, \
		(NAME), (VAR), IS_VAR, TYPE, NB_ELEMS, \
		(B), numMp, i_context, lightCnt)

#define CALL_PRIM_ATMOSPHERE(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS)  \
    DlMessageParam((RES),  e_AtmosphereParam, \
		(NAME), (VAR), IS_VAR, \
		TYPE, NB_ELEMS, \
		(B), numMp, i_context, 0)

#define CALL_PRIM_INCIDENT(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS)  \
    DlMessageParam((RES),  e_IncidentParam, \
		(NAME), (VAR), IS_VAR, TYPE, NB_ELEMS, \
		(B), numMp, i_context, 0)

#define CALL_PRIM_OPPOSITE(RES, B, NAME, VAR, IS_VAR, TYPE, NB_ELEMS)  \
    DlMessageParam((RES),  e_OppositeParam, \
		(NAME), (VAR), IS_VAR, TYPE, NB_ELEMS, \
		(B), numMp, i_context, 0)

////////////////////////////////////////////////////////////////////////////
// BOOLEAN FLAGS MACROS
////////////////////////////////////////////////////////////////////////////

#define SET_UNIFORM_BOOL_FLAG(BF, VF)  \
{  \
   (BF) = DL_SB_FALSE;  \
   for (polyCnt = 0; polyCnt < numMp; ++polyCnt)  \
   {  \
      if ((VF)[polyCnt]) { (BF) = DL_SB_TRUE; break; } \
   }  \
}

#define SET_VARYING_BOOL_FLAG(VF, BF)  \
{  \
   if (BF) \
	   for (polyCnt = 0; polyCnt < numMp; ++polyCnt)  \
	   {  \
    	  (VF)[polyCnt] = DL_SB_TRUE; \
	   }  \
}

#define INVERT_BOOL_FLAG(RF, BF)  \
{  \
   for (polyCnt = 0; polyCnt < numMp; ++polyCnt)  \
   {  \
      (RF)[polyCnt] = !(BF)[polyCnt]; \
   }  \
}

#define INVERT_INNER_BOOL_FLAG(RF, BF, OF)  \
{  \
   for (polyCnt = 0; polyCnt < numMp; ++polyCnt)  \
   {  \
      (RF)[polyCnt] = ((OF)[polyCnt] && !((BF)[polyCnt])); \
   }  \
}

#define OR_BOOL_FLAG(BF, LF, RF)  \
{  \
   for (polyCnt = 0; polyCnt < numMp; ++polyCnt)  \
   {  \
      (BF)[polyCnt] = ((LF)[polyCnt] || (RF)[polyCnt]); \
   } \
}

#define CALL_PRIM_TRANSMISSION(C, P1, P2, args, nbArgs, DP1U, DP1V, DP2U, DP2V)\
	DlTransmission((C), (P1), (DP1U), (DP1V), (P2), \
		(DP2U), (DP2V), true, args, nbArgs, \
		0x0, \
		1, polyCnt, \
		i_context )

#define CALL_PRIM_TRANSMISSION_SIMD(\
	C, B, P1, P2, args, nbArgs, DP1U, DP1V, DP2U, DP2V) \
	DlTransmission( (C), (P1), (DP1U), (DP1V), (P2), \
		(DP2U), (DP2V), true, args, nbArgs, \
		(B), \
		numMp, 0, \
		i_context )

#define CALL_PRIM_ISSHADOWRAY(F) \
	DlIsShadowRay(&(F), i_context )

#define CALL_PRIM_ISINDIRECTRAY( F ) \
	DlIsIndirectRay(&(F), i_context )

#define CALL_PRIM_RAYLEVEL(F) \
	DlRayLevel(&(F), i_context )

/* float trace(point, vector) shadeop */
#define CALL_PRIM_RAYHITTEST(RES, FROM, DIR) \
	DlRayHitTest((RES), (FROM), (DIR), 0x0, 1, i_context)

#define CALL_PRIM_RAYHITTEST_SIMD(RES, B, FROM, DIR) \
	DlRayHitTest((RES), (FROM), (DIR), (B), numMp, i_context)

////////////////////////////////////////////////////////////////////////////
// FIXME: 'pack' and 'unpack' functions
// Looks like they are DEPRECATED.
////////////////////////////////////////////////////////////////////////////

#define CALL_PRIM_PACK(F1, F2, F3, F4, F5) \
     { \
          float temp1 = (F2); \
          float temp2 = (F3); \
          float temp3 = (F4); \
          float temp4 = (F5); \
          DlPack(&(F1), &(temp1), &(temp2), &(temp3), &(temp4), 0, 1); \
     }

#define CALL_PRIM_PACK_SIMD(F1, B6, F2, F3, F4, F5) \
	DlPack((F1), (F2), (F3), (F4), (F5), (B6), numMp)

#define CALL_PRIM_UNPACK(F1, F2, F3, F4, F5) \
{ \
     float temp1 = (F1); \
     DlUnpack(&temp, &(F2), &(F3), &(F4), &(F5), 0, 1); \
}

#define CALL_PRIM_UNPACK_SIMD(F1, B6, F2, F3, F4, F5) \
     DlUnpack((F1), (F2), (F3), (F4), (F5), (B6),  numMp)

#endif
