/*
	LIBRARY
	  3delight
	FILENAME
	  slo.h
	VERSION
	  $Revision$
	DATE RELEASED
	  $Date: 2005-12-29 20:41:34 -0500 (Thu, 29 Dec 2005) $
	RCSID
	  $Id: slo.h 8433 2005-12-30 01:41:34Z aghiles $
*/

#ifndef __slo_h
#define __slo_h

#undef DL_INTERFACE

#if defined(_WIN32) && !defined(STATIC_3DELIGHT)
#	ifdef DL_COMPILING_3DELIGHT
#		define DL_INTERFACE __declspec(dllexport)
#	else
#		define DL_INTERFACE __declspec(dllimport)
#	endif
#else
#   define DL_INTERFACE
#endif


#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

typedef enum
{
	SLO_TYPE_UNKNOWN,
	SLO_TYPE_POINT,
	SLO_TYPE_COLOR,
	SLO_TYPE_SCALAR,
	SLO_TYPE_STRING,
	SLO_TYPE_SURFACE,
	SLO_TYPE_LIGHT,
	SLO_TYPE_DISPLACEMENT,
	SLO_TYPE_VOLUME,
	SLO_TYPE_TRANSFORMATION,
	SLO_TYPE_IMAGER,
	/* New types (RiSpec 3.1) */
	SLO_TYPE_VECTOR,
	SLO_TYPE_NORMAL,
	SLO_TYPE_MATRIX

} SLO_TYPE;

/* Storage class of the symbol */
typedef enum
{
	SLO_STOR_UNKNOWN,
	SLO_STOR_CONSTANT,
	SLO_STOR_VARIABLE,
	SLO_STOR_TEMPORARY,
	SLO_STOR_PARAMETER,
	SLO_STOR_OUTPUTPARAMETER,
	SLO_STOR_GSTATE
} SLO_STORAGE;

/* Variance of the symbol. Of course, in a shader, only varying & consant
   are useful */
typedef enum
{
	SLO_DETAIL_UNKNOWN,
	SLO_DETAIL_VARYING,
	SLO_DETAIL_UNIFORM
} SLO_DETAIL;

typedef struct {
	float xval;
	float yval;
	float zval;
} SLO_POINT;

typedef float SLO_SCALAR;

typedef float* SLO_MATRIX;

/* Visible fields of a shader symbol definition */
typedef struct slovissymdef
{
	/* name of symbol */
	const char *svd_name;

	SLO_TYPE svd_type;
	SLO_STORAGE svd_storage;
	SLO_DETAIL svd_detail;

	/* name of space in which to interpret symbol's value */
	const char *svd_spacename;

	union
	{
		/* pointers to default value */
		const SLO_POINT *pointval;
		const SLO_SCALAR *scalarval;
		const char *stringval;
		const float *matrixval;
	} svd_default;

	/* set to 0 if the value is not valid */
    unsigned svd_valisvalid:1;

	/* array length if the variable is an array */
	int svd_arraylen;

} SLO_VISSYMDEF;

DL_INTERFACE int Slo_SetPath(const char *i_path);
DL_INTERFACE int Slo_SetShader(const char *i_name);
DL_INTERFACE const char* Slo_GetName();
DL_INTERFACE SLO_TYPE Slo_GetType();
DL_INTERFACE int Slo_GetNArgs();

DL_INTERFACE SLO_VISSYMDEF *Slo_GetArgById(int i_id);
DL_INTERFACE SLO_VISSYMDEF *Slo_GetArgByName(const char *i_name);
DL_INTERFACE SLO_VISSYMDEF *Slo_GetArrayArgElement(
	SLO_VISSYMDEF *i_array, int i_index );

DL_INTERFACE int Slo_GetNAnnotations();
DL_INTERFACE const char *Slo_GetAnnotationKeyById(int i_id);
DL_INTERFACE const char *Slo_GetAnnotationByKey(const char *i_key);

DL_INTERFACE void Slo_EndShader();
DL_INTERFACE const char *Slo_TypetoStr(SLO_TYPE i_type);
DL_INTERFACE const char *Slo_StortoStr(SLO_STORAGE i_storage);
DL_INTERFACE const char *Slo_DetailtoStr(SLO_DETAIL i_detail);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
