
#include "stdafx.h"
#include "AVIFile.h"

// CAVIFile
CAVIFile::CAVIFile()
{
    // must be called before writing an AVI
    AVIFileInit();

    // init private members
    m_hAVIDC = NULL;
    m_frameIndex = NULL;
    m_pAVIFile = NULL;
    m_pAVIStream = NULL;
    m_pAVICompressedStream = NULL;

    m_fileName = "";
    m_errorMsg = "";
}

CAVIFile::~CAVIFile()
{
    ReleaseMemory();

    // must be called when finishing an AVI write
    AVIFileExit();
}

// Displays codec dialog, returns S_OK if a codec has been chosen
HRESULT CAVIFile::ChooseCodec(HWND hwnd)
{
    COMPVARS comp;
    ZeroMemory(&comp, sizeof(COMPVARS));
    comp.cbSize = sizeof(comp);
    if (ICCompressorChoose(hwnd, ICMF_CHOOSE_ALLCOMPRESSORS | ICMF_CHOOSE_DATARATE | ICMF_CHOOSE_KEYFRAME, NULL, NULL, &comp, NULL))
    {
        m_fccHandler = comp.fccHandler;
        return S_OK;
    }
    else
        return E_FAIL;
}

void CAVIFile::ReleaseMemory()
{
    if (m_hAVIDC)
    {
        DeleteDC(m_hAVIDC);
        m_hAVIDC=NULL;
    }
    if (m_pAVICompressedStream)
    {
        AVIStreamRelease(m_pAVICompressedStream);
        m_pAVICompressedStream=NULL;
    }
    if (m_pAVIStream)
    {
        AVIStreamRelease(m_pAVIStream);
        m_pAVIStream=NULL;
    }
    if (m_pAVIFile)
    {
        AVIFileRelease(m_pAVIFile);
        m_pAVIFile=NULL;
    }
}

// this now does little more than set some private members
HRESULT CAVIFile::CreateFile(CStringA filename, int framerate)
{
    m_fileName = filename;
    m_frameRate = framerate;

    return S_OK;
}

// this is what now actually creates the movie
HRESULT CAVIFile::InitMovie(int width, int height, int bpp)
{
    DWORD bufsize = width * height * 4;

    m_hAVIDC = CreateCompatibleDC(NULL);
    if(m_hAVIDC == NULL)	
    {
	    SetErrorMessage("Unable to create DC");
	    return E_FAIL;
    }
    
    if(FAILED(AVIFileOpen(&m_pAVIFile, (LPCSTR)m_fileName, OF_CREATE|OF_WRITE, NULL)))
    {
	    SetErrorMessage("Unable to create file");
	    return E_FAIL;
    }

    ZeroMemory(&m_AVIStreamInfo,sizeof(AVISTREAMINFO));
    m_AVIStreamInfo.fccType		= streamtypeVIDEO;
    m_AVIStreamInfo.fccHandler	= m_fccHandler;
    m_AVIStreamInfo.dwScale		= 1;
    m_AVIStreamInfo.dwRate		= m_frameRate;
    m_AVIStreamInfo.dwQuality	= -1; // default quality
    m_AVIStreamInfo.dwSuggestedBufferSize = bufsize;
    SetRect(&m_AVIStreamInfo.rcFrame, 0, 0, width, height);
    _tcscpy(m_AVIStreamInfo.szName, _T("Animation rendered with Caligari trueSpace via dribble"));

    if(FAILED(AVIFileCreateStream(m_pAVIFile, &m_pAVIStream, &m_AVIStreamInfo)))
    {
	    SetErrorMessage("Unable to create videostream");
	    return E_FAIL;
    }

    ZeroMemory(&m_AVICompressOptions,sizeof(AVICOMPRESSOPTIONS));
    m_AVICompressOptions.fccType = streamtypeVIDEO;
    m_AVICompressOptions.fccHandler = m_AVIStreamInfo.fccHandler;
    m_AVICompressOptions.dwFlags = AVICOMPRESSF_KEYFRAMES|AVICOMPRESSF_VALID;
    m_AVICompressOptions.dwKeyFrameEvery = 1;
    

    if(FAILED(AVIMakeCompressedStream(&m_pAVICompressedStream,m_pAVIStream,&m_AVICompressOptions,NULL)))
    {
	    SetErrorMessage("Unable to create compressed stream - bad codec?");
	    return E_FAIL;
    }

    BITMAPINFO bmpInfo;
    ZeroMemory(&bmpInfo,sizeof(BITMAPINFO));
    bmpInfo.bmiHeader.biPlanes		= 1;
    bmpInfo.bmiHeader.biWidth		= width;
    bmpInfo.bmiHeader.biHeight		= height;
    bmpInfo.bmiHeader.biCompression	= BI_RGB;
    bmpInfo.bmiHeader.biBitCount	= bpp;
    bmpInfo.bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
    bmpInfo.bmiHeader.biSizeImage	= bmpInfo.bmiHeader.biWidth * bmpInfo.bmiHeader.biHeight * bmpInfo.bmiHeader.biBitCount / 8;

    if (FAILED(AVIStreamSetFormat(m_pAVICompressedStream, 0, (LPVOID)&bmpInfo, bmpInfo.bmiHeader.biSize)))
    {
        SetErrorMessage("Stream format cannot be set - codec error?");
        return E_FAIL;
    }

    return S_OK;
}

HRESULT CAVIFile::AddFrame(int width, int height, LPVOID pBits, int bpp)
{
    // create the movie if not already
    if (!m_hAVIDC)
        InitMovie(width, height, bpp);

    DWORD bufsize = width * height * (bpp / 8);

    if (FAILED(AVIStreamWrite(m_pAVICompressedStream, m_frameIndex++, 1, pBits, bufsize, 0, NULL, NULL)))
	{
        SetErrorMessage(_T("Unable to add frame to AVI"));

		ReleaseMemory();

		return E_FAIL;
	}

    return S_OK;
}
