/////////////////////////////////////////////////////////////////////////////
//
// NexgenIPL - Next Generation Image Processing Library
// Copyright (c) 1999-2001 Binary Technologies
// All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in this file is used in any commercial application 
// then a simple email would be nice.
//
// THIS SOFTWARE IS PROVIDED BY BINARY TECHNOLOGIES ``AS IS'' AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BINARY TECHNOLOGIES OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// Binary Technologies
// http://www.binary-technologies.com
// info@binary-technologies.com
//
/////////////////////////////////////////////////////////////////////////////

#ifndef BTDEFINES_INCLUDED
#define BTDEFINES_INCLUDED

#pragma once

#if defined(UNICODE) || defined(_UNICODE)
	typedef wchar_t          bt_char;
	typedef wchar_t          bt_uchar;

	#define _BTTEXT(x)       L ## x
#else
	typedef char             bt_char;
	typedef unsigned char    bt_uchar;

	#define _BTTEXT(x)       x
#endif

typedef unsigned long        BTCOLORREF;

#define BTRGB(r,g,b)         ((unsigned long)(((unsigned char)(r)|((unsigned short)((unsigned char)(g))<<8))|(((unsigned long)(unsigned char)(b))<<16)))
#define BTRGBA(r,g,b,a)      ((unsigned long)(((unsigned char)(r)|((unsigned short)((unsigned char)(g))<<8))|(((unsigned long)(unsigned char)(b))<<16)|(((unsigned long)(a))<<24)))

#define BTGetRValue(rgba)    ((unsigned char)(rgba))
#define BTGetGValue(rgba)    ((unsigned char)(((unsigned short)(rgba)) >> 8))
#define BTGetBValue(rgba)    ((unsigned char)((rgba)>>16))
#define BTGetAValue(rgba)    ((unsigned char)((rgba)>>24))

#define BTMAKEINTRESOURCE(i) (bt_char *)((unsigned long)((unsigned short)(i)))

typedef struct tagBTRGBQUAD
{
	unsigned char rgbBlue;
	unsigned char rgbGreen;
	unsigned char rgbRed;
	unsigned char rgbReserved;
} BTRGBQUAD;

enum BTFileType
{
	BTFILETYPE_UNKNOWN = -1,
	BTFILETYPE_BMP = 0,
	BTFILETYPE_PCX,
	BTFILETYPE_PNG,
	BTFILETYPE_PPM,
	BTFILETYPE_PGM,
	BTFILETYPE_PBM,
	BTFILETYPE_JPEG,
	BTFILETYPE_TGA,
	BTFILETYPE_TIFF,
	BTFILETYPE_GIF,
	BTFILETYPE_ILBM,
	BTFILETYPE_RAS,
	BTFILETYPE_EPS,
	BTFILETYPE_MNG,
	BTFILETYPE_WMF,
	BTFILETYPE_EMF,
	BTFILETYPE_APM,
	BTFILETYPE_PSP,
	BTFILETYPE_JP2,
	BTFILETYPE_CUT,
	BTFILETYPE_EXTERN
};

enum BTCodecType
{
	BTCODECTYPE_DECODER = 0,
	BTCODECTYPE_ENCODER
};

enum BTEncodingType
{
	BTENCODINGTYPE_ASCII = 0,
	BTENCODINGTYPE_BINARY
};

enum BTResolutionMetric
{
	BTMETRIC_NONE = -1,      // Metric unknown
	BTMETRIC_CENTIMETER = 0, // Resolution is in centimeters (pixels/centimeter)
	BTMETRIC_METER,          // Resolution is in meters (pixels/meter)
	BTMETRIC_INCH            // Resolution is in inches (pixels/inch)
};

typedef struct tagBTResolution
{
	int    nResolutionMetric;
	double HResolution;
	double VResolution;

	tagBTResolution::tagBTResolution() : nResolutionMetric(BTMETRIC_NONE),
                                         HResolution(0.0),
										 VResolution(0.0) {};
} BTResolution;

typedef struct tagBTParamBMP
{
	bool bEnableRLE; // [in]

	tagBTParamBMP::tagBTParamBMP() : bEnableRLE(true) {};
} BTParamBMP;

typedef struct tagBTParamTGA
{
	bool bEnableRLE; // [in]

	tagBTParamTGA::tagBTParamTGA() : bEnableRLE(true) {};
} BTParamTGA;

typedef struct tagBTParamMNG
{
	int nCompressionLevel; // [in] 1..9 = Speed..Best, 0 = no compression, -1 = default.

	tagBTParamMNG::tagBTParamMNG() : nCompressionLevel(1) {};
} BTParamMNG;

typedef struct tagBTParamJPEG
{
	enum BTDCTMethod { DCT_ISlow,   // Slow but accurate integer algorithm.
                       DCT_IFast,   // Faster, less accurate integer method.
                       DCT_Float }; // Floating-point: accurate, fast on fast HW.

	int  nQuality;         // [in] Quality of compression on a scale from 0..100 [low..best].
	int  nSmoothingFactor; // [in] 1..100, or 0 for no input smoothing.
	bool bOptimizeCoding;  // [in] true = optimize entropy encoding parms.
	int  nDCTMethod;       // [in|out] DCT/IDCT algorithm option.

	tagBTParamJPEG::tagBTParamJPEG()	:	nQuality(0),
											nSmoothingFactor(0),
											bOptimizeCoding(false),
											nDCTMethod(DCT_Float) {};
} BTParamJPEG;

typedef struct tagBTParamPGM
{
	int   nEncodingType;    // [in|out] The encoding type of the image (Ascii or Binary/Raw).
	char* pszDescription;	// [in] Text that will be saved in the file. Each line
							// must start with a '#':
							//
							// BTParamPGM Param;
							// Param.pszDescription = "# 1. Line\n# 2. Line\n# 3. Line";

	tagBTParamPGM::tagBTParamPGM()	:	nEncodingType( BTENCODINGTYPE_BINARY),
										pszDescription( NULL) {};
} BTParamPGM;

typedef struct tagBTParamPPM
{
	int   nEncodingType;    // [in|out] The encoding type of the image (Ascii or Binary/Raw).
	char* pszDescription;	// [in] Text that will be saved in the file. Each line
							// must start with a '#':
							//
							// BTParamPPM Param;
							// Param.pszDescription = "# 1. Line\n# 2. Line\n# 3. Line";

	tagBTParamPPM::tagBTParamPPM()	:	nEncodingType( BTENCODINGTYPE_BINARY),
										pszDescription( NULL) {};
} BTParamPPM;

typedef struct tagBTParamPBM
{
	int nEncodingType; // [in|out] The encoding type of the image (Ascii or Binary/Raw).

	tagBTParamPBM::tagBTParamPBM() : nEncodingType( BTENCODINGTYPE_BINARY) {};
} BTParamPBM;

typedef struct tagBTParamTIFF
{
	enum BTTIFFCodec	{	None          = 1,
							CCITTRLE      = 2,
							CCITTRLEW     = 32771,
							CCITTFAX3     = 3,
							CCITT_T4      = 3,
							CCITTFAX4     = 4,
							CCITT_T6      = 4,
							JPEG          = 7,
							PACKBITS      = 32773,
							DEFLATE       = 32946,
							ADOBE_DEFLATE = 8 };

	char* pszTitle;       // [in]
	char* pszAuthor;      // [in]
	char* pszDescription; // [in]
	char* pszSoftware;    // [in]
	char* pszCopyright;   // [in]
	long  lCodec;         // [in]

	tagBTParamTIFF::tagBTParamTIFF() :	pszTitle(NULL),
										pszAuthor(NULL),
										pszDescription(NULL),
										pszSoftware(NULL),
										pszCopyright(NULL),
										lCodec(None) {};
} BTParamTIFF;

typedef struct tagBTParamPNG
{
	char* pszTitle;       // [in]
	char* pszAuthor;      // [in]
	char* pszDescription; // [in]
	char* pszCopyright;   // [in]
	char* pszSoftware;    // [in]
	char* pszDisclaimer;  // [in]
	char* pszComment;     // [in]
} BTParamPNG;

typedef struct tagBTParamGIF
{
	int           nTransparentColor; // [in|out] Palette index of transparent color.
	int           nBackgroundColor;  // [out]
	int           nDelay;            // [out]
	int           nImageIndex;       // [in]     Requested image.
	int           nImageCount;       // [out]    Number of images.
	unsigned char szComment[256];    // [in|out]

	tagBTParamGIF::tagBTParamGIF()	:	nTransparentColor(-1),
										nBackgroundColor(-1),
										nDelay(0),
										nImageIndex(1),
										nImageCount(0) {};
} BTParamGIF;

typedef struct tagBTParamJP2
{
	enum BTMode	{	MODE_Int,
					MODE_Real };

	int    nMode;
	double dCompressionRate; // [in] e.g. 100:1 = 0.01, 200:1 = 0.005.

	tagBTParamJP2::tagBTParamJP2()	:	nMode(MODE_Int),
										dCompressionRate(1.0) {};
} BTParamJP2;

#endif // BTDEFINES_INCLUDED
