#include "stdafx.h"
//#include "dribble.h"
#include "RibExport.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




// GoLights
// Recursively looks through scene, looking for lights to write out
// Inputs: pGNode - pointer to GNode to search from
// Outputs:
// Returns: 
void CRibExport::GoLights(tsxGNODE *pGNode)
{
	while (tsxIsGNode((tsxSOBJ*)pGNode) == e_tsxTRUE)
	{
		// if there's a child, recurse through this function again
		tsxGNODE* pChild = tsxGNodeGetFirstChild(pGNode);
		if (tsxIsGNode((tsxSOBJ*)pChild) == e_tsxTRUE)
			GoLights(pChild);

		tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);
		switch (ObjType)
		{
			case e_tsxLIGHT:
			{
				if ( IsVisible( pGNode ) )
				{
					if (JustShadowmaps)
					{
						if (tsxLightGetShadType((tsxLIGHT*)pGNode) == e_tsxLS_SHADOWMAP)
							MakeShadowMap((tsxLIGHT*)pGNode);
					}
					else
						WriteLight((tsxLIGHT*) pGNode);
				}
			}
			break;

			// look for emissive materials too
			case e_tsxPOLYHEDRON:
			{
				if ( IsVisible( pGNode ) )
				{
					CheckForAreaLight( pGNode );
				}

			}
			break;
		}
		
		pGNode = tsxGNodeGetNext(pGNode);
	}
}

tsxLIGHT* CRibExport::GetEmissiveLight( tsxPOLYHEDRON *pPolyh )
{
#ifndef TSX66
	return NULL;
#else

	if ( tsxPolyhGetNbrMaterials( pPolyh ) < 1 )
		return NULL;

	tsxMATERIAL *pMaterial = tsxPolyhGetIndexedMaterial( pPolyh, tsxPolyhGetFirstMaterialIdx( pPolyh ) );
	if ( !pMaterial )
		return NULL;
	
	tsxLIGHT *pLight = tsxMaterialGetEmission( pMaterial );
	if ( !pLight )
		return NULL;

	CtsxColor lightcol;
	tsxMaterialFixColor( pMaterial );
	tsxMaterialGetColor( pMaterial, &lightcol );
	
	tsxLightSetColor( pLight, &lightcol );

	return pLight;
#endif
}

void CRibExport::CheckForAreaLight( tsxGNODE *pGNode )
{
	tsxPOLYHEDRON *pPolyh = (tsxPOLYHEDRON*)pGNode;

	tsxLIGHT *pLight = GetEmissiveLight( pPolyh );
	if ( !pLight )
		return;
	
	WriteLight( pLight, pPolyh );
}


// WriteLight
//	writes out a light
//	uses the relatively new "uberlight" as it is now standard
//	in BMRT and has more options
// Inputs: pLight - pointer to light to use
// Outputs:
// Returns: 
void CRibExport::WriteLight(tsxLIGHT *pLight, tsxPOLYHEDRON *pPolyh)
{	
	CString ObjName = GetName((tsxGNODE*) pLight);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));
	RtString shadmapname = (RtString)tsxMalloc(_MAX_FNAME * sizeof(char));
	sprintf(shadmapname, "%s%s%d.dsm", ShadmapDirectory, name, LightNum);

	if ( !Binary )
		RiArchiveRecord(RI_COMMENT, "Current light source name = %s", name);
	tsxFree(name);

	//RiTransformBegin();
		CtsxTxmx3f pMatrix;
		tsxGNodeGetTransform((tsxGNODE*) pLight, &pMatrix);
		RtMatrix rtxmx;
		tSmxToRtmx(rtxmx, pMatrix);

		tsxLIGHT_TYPE ltype = tsxLightGetType(pLight);
		//if ( ( ltype != e_tsxLT_INFINITE) && ( ltype != e_tsxLT_SKY ) )
		//	RiTransform(rtxmx);
	

		CtsxColor pColor;
		tsxLightGetColor(pLight, &pColor);

		float FogLight = 0;
		if (tsxLightIsVolumetric(pLight) == e_tsxTRUE)
			FogLight = 1;
		
		float doshadow = 0.0f;
		tsxLIGHT_SHADOWING shadtype = tsxLightGetShadType(pLight);
		RtString shad = "off";
		RtFloat shadblur = 0.1;		
		RtFloat shadnsamps = 1;
		if ( shadtype != e_tsxLS_NOSHADOWS )
		{
			if ( (shadtype == e_tsxLS_SHADOWMAP) && (m_alwaysRayShadows == false) )
			{
				//float sharp = tsxLightGetShmapSharpnessCustom(pLight);
				//shadblur = ((sharp / 100.0f));
				tsxLIGHT_SMSHARPNESS sharp = tsxLightGetShmapSharpness( pLight );
				switch ( sharp )
				{
				case e_tsxSMS_LOW:
					shadblur = 0.05f;
					break;

				case e_tsxSMS_MEDIUM:
					shadblur = 0.01f;
					break;

				case e_tsxSMS_HIGH:
					shadblur = 0.0f;
					break;

				default:
					shadblur = 0.01f;
					break;
				}
				float qual = tsxLightGetShmapQualityCustom(pLight);
				//shadnsamps = (qual / 9.0) * 1024;
				//shadnsamps = qual * 16.0f;
				//shadnsamps = qual;
				shadnsamps = 1;
				doshadow = 0.0f;
			}
			else
			// raytraced, whether specified or converted using alwaysRayShadows
			{
				shad = "on";	
				doshadow = 1.0f;
				strcpy( shadmapname, "raytrace" );
				shadblur = 0.0f;
			}
		}
		else
			strcpy( shadmapname, "" );

		//RiAttribute("light", "shadows", &shad, RI_NULL);

        if ( m_bUseIrradiancePhotons || m_bUseCaustics )
        {
       		RtString on = "on";
    		RiAttribute( "light", "string emitphotons", &on, RI_NULL );
        }

		
		if (tsxGNodeGetUnits((tsxGNODE*)pLight) != e_tsxUNITS_METERS)
			tsxGNodeSetUnits((tsxGNODE*)pLight, e_tsxUNITS_METERS);

		CtsxVector3f coords;
		tsxGNodeGetLocation((tsxGNODE*)pLight, &coords);

		RtColor lightcol;
		tSColorToRtColor(lightcol, pColor);
		RtFloat intensity = tsxLightGetIntensity(pLight) * LightMultiplier;

		// falloff
		tsxLIGHT_FALLOFF tsfalloff = tsxLightGetFalloff(pLight);
		RtFloat FalloffCoeff = tsxLightGetFalloffCoeff( pLight );
		RtFloat Falloff;
		switch (tsfalloff)
		{
			case e_tsxLF_ZERO : 
				Falloff = 0;
			break;

			case e_tsxLF_LINEAR : 
				Falloff = 1;
			break;
			
			case e_tsxLF_SQUARED: 
				Falloff = 2;
			break;
		}

		switch (ltype)
		{
			case e_tsxLT_GONIOMETRIC:
				// fallthrough
			case e_tsxLT_LOCAL:
			{
				RtPoint from, to;
				from[0] = (RtFloat)coords.x;
				from[1] = (RtFloat)coords.z;
				from[2] = (RtFloat)coords.y;
				RiLightSource( "tSpointlight", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "float decay", &Falloff, "float falloffcoeff", &FalloffCoeff, "string shadowmap", &shadmapname, "float blur", &shadblur, "float doshadow", &doshadow, "float __foglight", &FogLight, RI_NULL );
			}
			break;
			case e_tsxLT_SPOT:
			{
				RtFloat coneangle = tsxLightGetSpotAngle( pLight );
				RtFloat ratio = tsxLightGetSpotRatio( pLight );
				RtFloat hotspotangle = coneangle * ratio;
				RtFloat beamdist = 8.0f;
				
				CtsxTxmx3f matrix;
				CtsxVector3f lightto, v1;
				tsxGNodeGetPureTransform((tsxGNODE*)pLight,&matrix);
				v1.x = v1.y = 0.f;
				v1.z = 1.f;
				tsxTransformVecs3f(1,&lightto,&v1,matrix);
				RtPoint from, to;
				from[0] = coords.x;
				from[1] = coords.z;
				from[2] = coords.y;
				
				to[0] = lightto.x;
				to[1] = lightto.z;
				to[2] = lightto.y;
				RiLightSource( "tSspotlight", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "point to", &to, "float decay", &Falloff, "float falloffcoeff", &FalloffCoeff, "float conedeltaangle", &hotspotangle, "float coneangle", &coneangle, "float beamdistribution", &beamdist, "string shadowmap", &shadmapname, "float blur", &shadblur, "float doshadow", &doshadow, "float __foglight", &FogLight, RI_NULL );
			}
			break;
			case e_tsxLT_PROJECTOR:
			{
				//RiTransformBegin();
				//RiTransform( rtxmx );

				RtFloat coneangle = tsxLightGetProjectorAngle( pLight );
				RtFloat ratio = 1.0f;
				RtFloat hotspotangle = coneangle * ratio;
				
				CtsxTxmx3f matrix;
				CtsxVector3f lightto, v1;
				tsxGNodeGetPureTransform((tsxGNODE*)pLight,&matrix);
				v1.x = v1.y = 0.f;
				v1.z = 1.f;
				tsxTransformVecs3f(1,&lightto,&v1,matrix);
				RtPoint from, to;
				from[0] = coords.x;
				from[1] = coords.z;
				from[2] = coords.y;
				
				to[0] = lightto.x;
				to[1] = lightto.z;
				to[2] = lightto.y;

				RtString texmapname = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
				tsxLightGetProjectorFilename( pLight, texmapname, _MAX_PATH );
				CString newtexname = MakeTexture( texmapname );
				strcpy( texmapname, newtexname );
				RiLightSource( "tSprojectorlight", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "point to", &to, "float decay", &Falloff, "float falloffcoeff", &FalloffCoeff, "float coneangle", &coneangle, "string texturemap", &texmapname, "string shadowmap", &shadmapname, "float blur", &shadblur, "float doshadow", &doshadow, "float __foglight", &FogLight, RI_NULL );
				tsxFree( texmapname );

				//RiTransformEnd();
			}
			break;
			case e_tsxLT_INFINITE:
			{
				CtsxTxmx3f matrix;
				CtsxVector3f lightto, v1;
				tsxGNodeGetPureTransform((tsxGNODE*)pLight,&matrix);
				v1.x = v1.y = 0.f;
				v1.z = 1.f;
				tsxTransformVecs3f(1,&lightto,&v1,matrix);
				RtPoint from, to;
				from[0] = coords.x;
				from[1] = coords.z;
				from[2] = coords.y;
				
				to[0] = lightto.x;
				to[1] = lightto.z;
				to[2] = lightto.y;

				/*if ( shadtype != e_tsxLS_NOSHADOWS )
				{
					RtString shad = "raytrace";
					RtFloat shadnsampsf = 1.0f;
					RiLightSource( "shadowdistant", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "point to", &to, "string shadowname", &shad, "float samples", &shadnsampsf, RI_NULL );
				}
				else
					RiLightSource( "distantlight", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "point to", &to, RI_NULL );*/
				RiLightSource( "tSinfinitelight", "float intensity", &intensity, "color lightcolor", &lightcol, "point from", &from, "point to", &to, "string shadowname", &shadmapname, "float samples", &shadnsamps, "float blur", &shadblur, RI_NULL );
			}
			break;
			case e_tsxLT_AREA:
			{
				RiTransformBegin();
					RiConcatTransform( rtxmx );
					RiAttributeBegin();
						RiSurface("constant", RI_NULL);
						RiAttribute( "light", "integer nsamples", &m_skySamples, RI_NULL );
						RtInt on = 1;
#ifdef TSX66
						if ( tsxLightAreaGetShowGeometry( pLight ) == e_tsxFALSE )
							on = 0;
#endif
						RiAttribute( "visibility", "integer camera", &on, RI_NULL );
						RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
						RiAttribute( "visibility", "integer specular", &on, RI_NULL );
						RtLightHandle light = RiAreaLightSource("arealight", "lightcolor", &lightcol, "intensity", &intensity, RI_NULL);
						
						RtPoint points[4];
						points[0][0] = -0.5;
						points[0][1] = 0.5;
						points[0][2] = 0.0;
						
						points[3][0] = 0.5;
						points[3][1] = 0.5;
						points[3][2] = 0.0;
						
						points[2][0] = 0.5;
						points[2][1] = -0.5;
						points[2][2] = 0.0;
						
						points[1][0] = -0.5;
						points[1][1] = -0.5;
						points[1][2] = 0.0;
						RiPolygon(4, RI_P, (RtPointer)points, RI_NULL);

					RiAttributeEnd();
				RiTransformEnd();
				RiIlluminate( light, 1 );
			}
			break;
#ifdef TSX66
			case e_tsxLT_OBJECT:
			{
				if ( pPolyh )
				{
					RiTransformBegin();
					tsxGNodeGetTransform((tsxGNODE*) pPolyh, &pMatrix);
					RtMatrix rtxmx;
					tSmxToRtmx(rtxmx, pMatrix);

						RiConcatTransform( rtxmx );
						RiAttributeBegin();
							RiSurface("constant", RI_NULL);
							RiAttribute( "light", "integer nsamples", &m_skySamples, RI_NULL );
							RtLightHandle light = RiAreaLightSource("arealight", "lightcolor", &lightcol, "intensity", &intensity, RI_NULL);
							
							//WritePolyhedronAsFaces( pPolyh, false );
							WritePolyhedronAsFaces( pPolyh );

						RiAttributeEnd();
					RiTransformEnd();
					RiIlluminate( light, 1 );
				}
			}
			break;
#endif
			// renderers that support a hemispherical light can be supported via a sky light
			case e_tsxLT_SKY:
			{				
				RtFloat occsamples = (RtFloat)m_skySamples;
				CtsxTxmx3f matrix;
				CtsxVector3f lightto, v1, direction;
				tsxGNodeGetPureTransform((tsxGNODE*)pLight,&matrix);
				v1.x = v1.y = 0.f;
				v1.z = 1.f;
				tsxTransformVecs3f(1,&lightto,&v1,matrix);
				
				direction.x = coords.x - lightto.x;
				direction.y = coords.z - lightto.z;
				direction.z = coords.y - lightto.y;
				tsxNormalize3f( direction );
				RtPoint dir;
				dir[0] = direction.x;
				dir[1] = direction.y;
				dir[2] = direction.z;

				RtFloat maxdist = 10.0f;
				RtFloat bias = 0.005f;
				RiLightSource( "tSskylight", "float intensity", &intensity, "color lightcolor", &lightcol, "vector direction", &dir, "float samples", &occsamples, "float maxdist", &maxdist, "float bias", &bias, RI_NULL );
	
			}
			break;
			default:
			{
				if ( !Binary )
					RiArchiveRecord(RI_COMMENT, "UNSUPPORTED LIGHT TYPE");
			}


		}
	//RiTransformEnd();

	tsxFree(shadmapname);

	LightNum++;
}


