#include "stdafx.h"
//#include "dribble.h"
#include "RibExport.h"
//#include "BTCImageObject.h"
#include "wfmmshaders.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




// GoSceneTextures
// Recursively looks through scene, looking for textures to write out
// Inputs: pGNode - pointer to GNode to search from
// Outputs:
// Returns: 
void CRibExport::GoSceneTextures(tsxGNODE *pGNode)
{
	while (tsxIsGNode((tsxSOBJ*)pGNode) == e_tsxTRUE)
	{
		// if there's a child, recurse through this function again
		tsxGNODE* pChild = tsxGNodeGetFirstChild(pGNode);
		if (tsxIsGNode((tsxSOBJ*)pChild) == e_tsxTRUE)
			GoSceneTextures(pChild);

		// if the object is invisble, we just skip it
		if (tsxMNodeIsVisible((tsxMNODE*) pGNode) == e_tsxTRUE)
		{
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pGNode, ExtDataID);

			tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);
			switch (ObjType)
			{
				case e_tsxMBALLOBJ:
				case e_tsxMETANURBS:
				{
					// If it's a metaobject, we need to get the rendering polyh
					tsxPOLYHEDRON* pMetanode;
					pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
					if (pMetanode)
					{
						WritePolyhedronTextures(pMetanode, (tsxGNODE*)pMetanode);
					}
				}
				break;
				case e_tsxPOLYHEDRON:
				{
					WritePolyhedronTextures((tsxPOLYHEDRON*) pGNode, pGNode);
				}
				break;
				case e_tsxNPATCH_POLYH:
				{
					WritePatchTextures(tsxNPatchPolyhGetNPatch((tsxPOLYHEDRON*)pGNode), (tsxPOLYHEDRON*)pGNode);
				}
				break;
				case e_tsxSUBDIV:
				{
					tsxPOLYHEDRON *pCage = tsxSubDivGetCagePtr((tsxMNODE*)pGNode);
					WritePolyhedronTextures(pCage, pGNode);
				}
				break;


			}
		} // visible
		
		pGNode = tsxGNodeGetNext(pGNode);
	}
}


///////////////////////////////////////////////////////////////////////////////





// WritePolyhedronTextures
// Inputs: pInPolyh - pointer to polyhedron to use
// Outputs:
// Returns: 
void CRibExport::WritePolyhedronTextures(tsxPOLYHEDRON *pPolyh, tsxGNODE* pTopNode)
{
	// get the object name
	CString ObjName = GetName(pTopNode);
	MakeNameFilename(&ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// get some basic polyh info
	int NumPolys = tsxPolyhGetNbrFaces(pPolyh);
	CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pPolyh);
	CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pPolyh);
	CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pPolyh);


		for (int poly = 0; poly < NumPolys; poly++)
		{
			// let's have some dynamic notification - less frequently for big objects
			/*int modulus = 10;
			if (poly > 10000)
				modulus = 2500;
			else
				if (poly > 1000)
					modulus = 250;
				else
					if (poly > 100)
						modulus = 50;

			/*if ((poly % modulus) == 0)
			{
				char msg[255];
				sprintf(msg, "processing %s, face %d of %d..", name, poly, NumPolys);
				tsxSetHelpMessage(msg);
			}*/

		
			// now look at materials
			// only do so if the object has no custom data
			// uses a material caching system so we don't have to write
			// out the same material multiple times
			tsxMATERIAL* pMaterial;
			tsxMaterialCreate(&pMaterial);
			tsxPolyhGetMaterial(pPolyh, pFaceArray[poly]->materialIdx, pMaterial);
			int Faceting = tsxMaterialGetFacetingType(pMaterial);
	
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
			if (!pData)
			{
				if ( (poly == 0) || (pFaceArray[poly]->materialIdx != OldMaterialIdx) )
				{
					RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
					sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), pFaceArray[0]->materialIdx);

					
					bool Painted = IsMaterialPainted(pMaterial, TxrName);

					// for prman we have to make tex files
					//if (Renderer == Renderer_Prman)
					if (Painted)
					{
						MakeTexture( TxrName );
					}
					if (TxrName)
						tsxFree(TxrName);

					// now look for the displacement
					RtString DisName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
					sprintf(DisName, "%s%d", ObjName.GetBuffer(_MAX_PATH), pFaceArray[0]->materialIdx);
					
					float amp;
					bool Displaced = IsMaterialDisplaced(pMaterial, DisName, amp);

					// for prman we have to make tex files
					//if (Renderer == Renderer_Prman)
					if (Displaced)
					{
						MakeTexture( DisName );
					}

					if (DisName)
						tsxFree(DisName);
	

				}
				OldMaterialIdx = pFaceArray[poly]->materialIdx;
			}


			// destroy working material
			tsxMaterialDestroy(pMaterial);
		}


	tsxFree(name);
}




// WriteMaterial
//	writes out a tS material as Renderman calls
// Inputs: pMaterial - pointer to material to use
// Outputs:
// Returns: 
void CRibExport::WriteMaterial( tsxGNODE *pGNode, tsxMATERIAL *pMaterial, CString ObjName, int MatNbr)
{
	bool TransShadows = false;

    RtString shadingmodel = new char[255];
    strcpy( shadingmodel, "matte" );

    CtsxColor Color;
	RtColor BaseColor;
	tsxMaterialFixColor( pMaterial );
	tsxMaterialFixTransparency( pMaterial );
	tsxMaterialGetColor( pMaterial, &Color );
	tSColorToRtColor( BaseColor, Color);

	float Ka = 0.0f;
	float Kd = 1.0f;
	float Ks = 0.0f;
	float roughness = 0.0f;
	float Kr = 0.0f;	
	float Kt = 0.0f;
	float ior = 1.0f;
	tssColor specularcolor = {1.0f, 1.0f, 1.0f};

	if (!JustShadowmaps)
	{
		if (DeclareParams)
		{
			tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_TRANSPARENCY);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_REFLECTANCE);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);
			WriteShinfo(pShinfo);
		}

        Kd = 0.0f;
		
		// first we're going to work with the reflectance shader
		tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_REFLECTANCE);
		char ShaderName[255];
		if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) != e_tsxFAILURE)
		{
			RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), MatNbr);
			
			bool Painted = IsMaterialPainted(pMaterial, TxrName);

			// for prman we have to make tex files
			if (Painted)
			{
				CString newtexname = MakeTexture( TxrName );
				strcpy( TxrName, newtexname );
			}



			if (lstrcmpi(ShaderName, "caligari metal") == 0)
			{
				
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);

                strcpy( shadingmodel, "chrome" );
				/*// for caustics
				if (m_bUseCaustics)
				{
					tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/

                if ( Kr > 0.0f )
                {
                    if ( Painted )
                    {
                        CtsxVector2f uvreps, uvoffset;
		                uvreps.x = uvreps.y = 1.0f;
		                uvoffset.x = uvoffset.y = 0.0f;
		                GetUVInfo( pMaterial, uvreps, uvoffset );            		
					    RiSurface("tSpaintedmetalreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "float Kr", &Kr, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                    }
				    else
					    RiSurface("tSmetalreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "float Kr", &Kr, RI_NULL);
                }
                else
                {
                    if ( Painted )
                    {
                        CtsxVector2f uvreps, uvoffset;
		                uvreps.x = uvreps.y = 1.0f;
		                uvoffset.x = uvoffset.y = 0.0f;
		                GetUVInfo( pMaterial, uvreps, uvoffset );            		
					    RiSurface("tSpaintedmetal", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                    }
				    else
					    RiSurface("tSmetal", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, RI_NULL);
                }

				
			}
			else
			if (lstrcmpi(ShaderName, "glass") == 0)
			{
				Ka = 0.0f;
				Kd = 0.0f;
				
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
				

				// for caustics
				/*if (m_bUseCaustics)
				{
					tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/
                strcpy( shadingmodel, "glass" );

				switch (Renderer)
				{
					case Renderer_AIR:
						{
							tssColor transmitcolor = { 1.0f, 1.0f, 1.0f };
							RiDeclare("transmitcolor", "color");
							RiDeclare("ior", "float");
							RiDeclare("Kt", "float");
							RiSurface("SIglass", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "Kr", &Kr, "Kt", &Kt, "roughness", &roughness, "transmitcolor", &transmitcolor, "ior", &ior, RI_NULL);
						}
						break;
					default:
						RiSurface("glass", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "Kr", &Kr, "roughness", &roughness, RI_NULL);
				}
			}
			else
			if (lstrcmpi(ShaderName, "mirror") == 0)
			{
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                
				//char name[255];
				//tsxShaderInfoGetParameterName( pShinfo, 4, &name[0] );

				
				roughness = 1.f/(100.f*roughness) - 0.1f;
				roughness = 1.0f - roughness;
				
				


				// for caustics
				/*if (m_bUseCaustics)
				{
					tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/
                /*strcpy( shadingmodel, "chrome" );

				if (Painted)
					RiSurface("paintedplastic", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "texturename", &TxrName, RI_NULL);
				else
					switch (Renderer)
					{
						case Renderer_AIR:
							RiSurface("SIshinymetal", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "Kr", &Kr, "roughness", &roughness, RI_NULL);
							break;
						default:
							RiSurface("shiny", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "Kr", &Kr, "roughness", &roughness, RI_NULL);
					}*/
				/*roughness = 169.0f - 140.0f * (3.0f - roughness);
				roughness = (sqrt(roughness) - 13.0f) / 70.0f;
                roughness = 1.0f - roughness;*/
				
				

				// for caustics
				/*if (m_bUseCaustics)
				{
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/

				if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSpaintedstandardreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "Kr", &Kr, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					RiSurface("tSstandardreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "Kr", &Kr, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "metal") == 0)
			{
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
                
				// for caustics
				/*if (m_bUseCaustics)
				{
					tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/
                strcpy( shadingmodel, "chrome" );

                // metal shaders have the same base color for specular and no diffuse
                Kd = 0.0f;
                
                if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSpaintedmetal", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "float roughness", &roughness, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					RiSurface("tSmetal", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "float roughness", &roughness, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "caligari phong") == 0)
			{
				
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
				//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
                roughness = 169.0f - 140.0f * (3.0f - roughness);
				roughness = (sqrt(roughness) - 13.0f) / 70.0f;
                roughness = 1.0f - roughness;
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
				

				// for caustics
				/*if (m_bUseCaustics)
				{
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/

				if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );         
					if ( Kr > 0.0f )
						RiSurface("tSpaintedstandardreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "float Kr", &Kr, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
					else
						RiSurface("tSpaintedstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					if ( Kr > 0.0f )
						RiSurface("tSstandardreflect", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "float Kr", &Kr, RI_NULL);
					else
						RiSurface("tSstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "phong") == 0)
			{
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
				roughness = 169.0f - 140.0f * (3.0f - roughness);
				roughness = (sqrt(roughness) - 13.0f) / 70.0f;
                roughness = 1.0f - roughness;
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

				// for caustics
				/*if (m_bUseCaustics)
				{
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/

				if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSpaintedstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					RiSurface("tSstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "plastic") == 0)
			{
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

				// for caustics
				/*if (m_bUseCaustics)
				{
					RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
				}*/

				if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSpaintedstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					RiSurface("tSstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "constant") == 0)
			{
				RiSurface("constant", RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "matte") == 0)
			{
				Ks = 0.0f;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);

				
				if (Painted)
                {
                    CtsxVector2f uvreps, uvoffset;
		            uvreps.x = uvreps.y = 1.0f;
		            uvoffset.x = uvoffset.y = 0.0f;
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSpaintedstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
                }
				else
					RiSurface("tSstandard", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, RI_NULL);
			}
			else
			if (lstrcmpi(ShaderName, "translucent plastic") == 0)
			{

				// for SSS
				float Ksub;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
                roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ksub);
				
                strcpy( shadingmodel, "transparent" );

                CtsxVector2f uvreps, uvoffset;
	            uvreps.x = uvreps.y = 1.0f;
	            uvoffset.x = uvoffset.y = 0.0f;

				if (Painted)
				{
		            GetUVInfo( pMaterial, uvreps, uvoffset );            		
					RiSurface("tSsss", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "texturename", &TxrName, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL);
				}
				else
					RiSurface("tSsss", "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, RI_NULL);

				// TODO: this should be the object name, or preferably the parent group name
				tsxGNODE* pParent = tsxGNodeGetRoot( pGNode );
				CString groupname = GetName( pParent );
				RtString sssname = (RtString)tsxMalloc(255 * sizeof(char));
				strcpy(sssname, groupname.GetBuffer(255));

				RiAttribute( "visibility", "string subsurface", &sssname, RI_NULL );
				tsxFree( sssname );

				RtColor scattering = { 2.19f, 2.62f, 3.00f };
				RtColor absorption = { 0.0021f, 0.0041f, 0.0071f } ;
				RtFloat ior = 1.33f;
				RtFloat scale = 1.0f;
				RtFloat shadingrate = 50.0f;

				RiAttribute( "subsurface", "color scattering", &scattering, "color absorption", &absorption, "float refractionindex", &ior, "float scale", &scale, "float shadingrate", &shadingrate, RI_NULL );

			}
			else
			// no recognizable shader, so we use a default
			{
				Kd = 1.0f;
				RiSurface("matte", "Ka", &Ka, "Kd", &Kd, RI_NULL);
			}

			if (TxrName)
				tsxFree(TxrName);

		}
	}

	// now look at transparency
	tssColor BaseOpacity = { 1, 1, 1 };
	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_TRANSPARENCY);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) != e_tsxFAILURE)
	{
		if (lstrcmpi(ShaderName, "c_alpha") == 0)
		{
			float alpha;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&alpha);
			BaseOpacity[0] = BaseOpacity[1] = BaseOpacity[2] = alpha;
		}

		if (lstrcmpi(ShaderName, "plain") == 0)
		{
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&BaseOpacity);
		}
	}

    if ( strcmp( shadingmodel, "matte" ) == 0 )
    {
        if ( BaseOpacity[0] < 1.0f )
            strcpy( shadingmodel, "transparent" );
    }


			
	// now we will look at displacement
	RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
	sprintf(TxrName, "%sbump%d", ObjName.GetBuffer(_MAX_PATH), MatNbr);
		
	RtFloat amplitude = 0.0f;
	bool Displaced = IsMaterialDisplaced(pMaterial, TxrName, amplitude);
	
	if (Displaced)
	{	
		CString newtexname = MakeTexture( TxrName );
		strcpy( TxrName, newtexname );

		//RiDeclare("Km", "float");
		//RiDeclare("texturename", "string");
		//RiDisplacement("bumpy", "Km", &amplitude, "texturename", &TxrName, RI_NULL);
		float truedisp = 1.0f;
		CtsxVector2f uvreps, uvoffset;
		uvreps.x = uvreps.y = 1.0f;
		uvoffset.x = uvoffset.y = 0.0f;
		GetUVInfo( pMaterial, uvreps, uvoffset );
		RiDisplacement( "tSbumpmap", "float Km", &amplitude, "string texturename", &TxrName, "float truedisplace", &truedisp, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL );
		
		/*switch (Renderer)
		{
			case Renderer_Air:
			case Renderer_BMRT:
				RiDisplacement("bumpy", "Km", &amplitude, "texturename", &FileName, RI_NULL);
				break;
			case Renderer_Prman:
				// for prman we have to make tex files
				RtString TexName = (RtString)tsxMalloc(255 & sizeof(char));
				strcpy(TexName, FileName);
				strcat(TexName, ".tex");
				RiDisplacement("bumpy", "amplitude", &amplitude, "texturename", &TexName, RI_NULL);
				tsxFree(TexName);
				break;
		}*/	

		//if (FileName)
		//	tsxFree(FileName);

		RtFloat radius = amplitude * 0.5f;
		RtString coordsys = "shader";
		RiAttribute( "displacementbound", "float sphere", &radius, "string coordinatesystem", &coordsys, RI_NULL );
	}

	
	if (TxrName)
		tsxFree(TxrName);



	if (!JustShadowmaps)
	{
		// now handle the basic color and opacity		
		RiColor( BaseColor );
		RiOpacity( BaseOpacity );
	}


	// now the visibility options
	RtInt on;
	/*if ( ( m_doRaytrace ) && ( Kd > 0.0f ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) && ( Ks > 0.0f ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer specular", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) && ( tsxMNodeCastShadows( (tsxMNODE*)pGNode ) == e_tsxTRUE ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer transmission", &on, RI_NULL );

		RtString hitmode = "primitive";
		if ( !TransShadows )
			RiAttribute( "visibility", "string transmissionhitmode", &hitmode, RI_NULL );
	}
	if ( ( (BaseOpacity[0] + BaseOpacity[1] + BaseOpacity[2]) == 3.0f ) && ( Kt == 0.0f ) )
	{
		RtString hitmode = "primitive";
		RiAttribute( "visibility", "string camerahitmode", &hitmode, RI_NULL );
	}*/
	if ( ( m_doRaytrace ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer specular", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer transmission", &on, RI_NULL );

		RtString hitmode = "primitive";
		if ( !TransShadows )
			RiAttribute( "visibility", "string transmissionhitmode", &hitmode, RI_NULL );
	}
	if ( ( (BaseOpacity[0] + BaseOpacity[1] + BaseOpacity[2]) == 3.0f ) && ( Kt == 0.0f ) )
	{
		RtString hitmode = "primitive";
		RiAttribute( "visibility", "string camerahitmode", &hitmode, RI_NULL );
	}


	

	/*// finally, shadow-casting options

	RtString shadtype = new char[255];
	RtInt shad;
	if ( tsxMNodeCastShadows( (tsxMNODE*)pGNode ) == e_tsxTRUE )
	{
		if ( TransShadows )
			strcpy( shadtype, "shader" );
		else
			strcpy( shadtype, "Os" );

	}
	else
	{
		strcpy( shadtype, "transparent" );
	}
	RiAttribute( "visibility", "string transmission", &shadtype, RI_NULL );
    
	delete []shadtype;*/

    // shading model for photons
    if ( m_bUseIrradiancePhotons || m_bUseCaustics )
    {
        switch ( Renderer )
        {
        case Renderer_3Delight:
            RiAttribute( "photon", "string shadingmodel", &shadingmodel, RI_NULL );    
            break;
        }
    }

    delete []shadingmodel;


}

// WriteShinfo
//	writes out details of a shaderinfo structure
//	called recursively to handle layers
//	For now, this just dumps out all shader info as comments
// Inputs: pShinfo - pointer to shaderinfo to use
// Outputs:
// Returns: 
void CRibExport::WriteShinfo(tsxSHINFO *pShinfo)
{
	tsxSHCLASS shclass = tsxShaderInfoGetClass(pShinfo);

	char CurrentName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &CurrentName[0]) == e_tsxFAILURE)
		return;
	
	if (lstrcmpi(CurrentName, "none") == 0)
		return;

	switch (shclass)
	{
		case tsxSHCLASS_COLOR:
			RiArchiveRecord(RI_COMMENT, "Color Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_TRANSPARENCY:
			RiArchiveRecord(RI_COMMENT, "Transparency Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_REFLECTANCE:
			RiArchiveRecord(RI_COMMENT, "Reflectance Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_DISPLACEMENT:
			RiArchiveRecord(RI_COMMENT, "Displacement Shader: %s", CurrentName);
		break;
	}

	// first declare the variables
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, n);
		tsxSHPARAM_TYPE ParamType = tsxParameterValueGetType(pVal);
		switch (ParamType)
		{
			case tsxSHPARAM_BOOL:
			{
				RiDeclare(ParamName, "integer");
			}
			break;

			case tsxSHPARAM_FLOAT:
			case tsxSHPARAM_DOUBLE:
			{
				RiDeclare(ParamName, "float");
			}
			break;
			
			case tsxSHPARAM_BYTE:
			case tsxSHPARAM_SHORT:
			case tsxSHPARAM_LONG:
			case tsxSHPARAM_WORD:
			case tsxSHPARAM_DWORD:
			{
				RiDeclare(ParamName, "integer");
			}
			break;

			case tsxSHPARAM_VECTOR:
			{
				RiDeclare(ParamName, "vector");
			}
			break;
			case tsxSHPARAM_POINT:
			{
				RiDeclare(ParamName, "point");
			}
			break;
			case tsxSHPARAM_COLOR:
			case tsxSHPARAM_COLOR4:			
			{
				RiDeclare(ParamName, "color");
			}
			break;

			case tsxSHPARAM_STRING:
			{
				RiDeclare(ParamName, "string");
			}
			break;			

			default:
				RiArchiveRecord(RI_COMMENT, "\tUNKNOWN PARAMETER TYPE");

		}
	}

	// now write em
	char strParams[1048];
	int chars = 0;
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, n);
		tsxSHPARAM_TYPE ParamType = tsxParameterValueGetType(pVal);
		chars += sprintf(strParams + chars, " ""%s""", ParamName);
		switch (ParamType)
		{
			case tsxSHPARAM_BOOL:
			{
				bool b;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&b);
				if (b)
					chars += sprintf(strParams + chars, " 1");
				else
					chars += sprintf(strParams + chars, " 0");
			}
			break;

			case tsxSHPARAM_FLOAT:
			case tsxSHPARAM_DOUBLE:
			{
				float f;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&f);
				chars += sprintf(strParams + chars, " %g", f);
			}
			break;
			
			case tsxSHPARAM_BYTE:
			case tsxSHPARAM_SHORT:
			case tsxSHPARAM_LONG:
			case tsxSHPARAM_WORD:
			case tsxSHPARAM_DWORD:
			{
				int i;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&i);
				chars += sprintf(strParams + chars, " %d", i);
			}
			break;

			case tsxSHPARAM_VECTOR:
			case tsxSHPARAM_POINT:
			case tsxSHPARAM_COLOR:
			{
				tssColor c;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&c);
				chars += sprintf(strParams + chars, " [ %g, %g, %g ]", c[0], c[1], c[2]);
			}
			break;

			case tsxSHPARAM_COLOR4:
			{
				tssColor4 c;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&c);
				chars += sprintf(strParams + chars, " [ %g, %g, %g %g ]", c[0], c[1], c[2], c[3]);
			}
			break;

			case tsxSHPARAM_STRING:
			{
				char* pC;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&pC);
				chars += sprintf(strParams + chars, " ""%s""", pC);
			}
			break;

		}
	}
	sprintf(strParams + chars, "\n");
	RiArchiveRecord(RI_COMMENT, strParams);
}


///////////////////////////////////////////////////////////////////////////////


void CRibExport::WritePatchTextures(tsxNPATCH *pPatch, tsxPOLYHEDRON *pPolyh)
{
	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	MakeNameFilename(&ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));


		// now look at materials
		int NbrMaterials = tsxPolyhGetNbrMaterials(pPolyh);

		tsxMATERIAL* pMaterial;
		tsxMaterialCreate(&pMaterial);
		tsxPolyhGetMaterial(pPolyh, 0, pMaterial);
		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
		if (!pData)
		{
			RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), 0);

			
			bool Painted = IsMaterialPainted(pMaterial, TxrName);

			// for prman we have to make tex files
			//if (Renderer == Renderer_Prman)
			if (Painted)
			{
				MakeTexture( TxrName );

				/*if (FileName)
					tsxFree(FileName);
				if (TexName)
					tsxFree(TexName);*/
			}
			if (TxrName)
				tsxFree(TxrName);

			// now look for the displacement
			RtString DisName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(DisName, "%s%d", ObjName.GetBuffer(_MAX_PATH), 0);
			
			float amp;
			bool Displaced = IsMaterialDisplaced(pMaterial, DisName, amp);

			// for prman we have to make tex files
			//if (Renderer == Renderer_Prman)
			if (Displaced)
			{
				MakeTexture( DisName );
			}

			if (DisName)
				tsxFree(DisName);

		}
		
		// destroy working material
		tsxMaterialDestroy(pMaterial);

	tsxFree(name);

}

void CRibExport::WriteTempMaterial(tsxSOBJ *pObj)
{
	if (!pObj)
		return;

	CString ObjName = GetName((tsxGNODE*) pObj);
	MakeNameFilename(&ObjName);


	char tempdir[255];
	GetTempPath(255, tempdir);
	char filename[255];
	sprintf(filename, "%s\\material.rib", tempdir);

	RiBegin(filename);

	RiWorldBegin();

		tsxMATERIAL* pMaterial = tsxPolyhGetIndexedMaterial((tsxPOLYHEDRON*)pObj, tsxPolyhGetFirstMaterialIdx((tsxPOLYHEDRON*)pObj));
		WriteMaterial( (tsxGNODE*)pObj, pMaterial, ObjName, tsxPolyhGetFirstMaterialIdx((tsxPOLYHEDRON*)pObj));
	
		RiWorldEnd();

	RiEnd();

}

bool CRibExport::IsMaterialPainted(tsxMATERIAL *pMaterial, char *pTxrName )
{
	if (!pMaterial)
		return FALSE;

	if (!pTxrName)
		return FALSE;

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return FALSE;
	
	if (lstrcmpi(ShaderName, "plain") == 0)
		return FALSE;


	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari texture") == 0)
	{
		if (ConvertTextureShaders)
		{
			ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
		}
		else
		{
			char* pC;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
			strcpy(pTxrName, pC);	
		}
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		if (ConvertTextureShaders)
		{
			ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
		}
		else
		{
			char* pC;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
			strcpy(pTxrName, pC);	
		}
	}
	else
	if (ConvertProceduralShaders)
	{
		ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
	}

	

	return TRUE;
	
}

void CRibExport::GetUVInfo(tsxMATERIAL *pMaterial, CtsxVector2f& uvreps, CtsxVector2f& uvoffset)
{
	if (!pMaterial)
		return;
	
	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return;
	
	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari texture") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 15);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
	}

	pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return;
	
	if (lstrcmpi(ShaderName, "none") == 0)
		return;


	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari bump") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 15);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
	}
	
}


bool CRibExport::IsMaterialDisplaced(tsxMATERIAL *pMaterial, char *pTxrName, float& amplitude)
{
	if (!pMaterial)
		return FALSE;

	if (!pTxrName)
		return FALSE;

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return FALSE;
	
	if (lstrcmpi(ShaderName, "none") == 0)
		return FALSE;


	// now write em
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		TRACE("%d - %s\n", n, ParamName);
	}

	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari bump") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);	
		float f;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&f);
		amplitude = f;
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);			
		float f;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 11);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&f);
		amplitude = f;
	}
	

	return TRUE;
	
}

bool CRibExport::ConvertProcedural(tsxMATERIAL *pMaterial, int width, int height, char *pFilename)
{
	tsxERR err;

	tsxImage TextureImg;
	tsxImageCreateData(&TextureImg, width, height, e_tsxIMAGE_RGBA);

	// generate texture
	err = tsxMaterialGetTextureFromProceduralColor(pMaterial, &TextureImg);

	CtsxColor *texture = (CtsxColor*)TextureImg.data;
	
	if (!texture)
		return FALSE;

	/*BTCImageObject	ImageObject;
	ImageObject.GetObjectDataPtr()->Create(width,height,24);
	BTCImageData *pImageData = ImageObject.GetObjectDataPtr();
	unsigned char *pImageBits = pImageData->GetBits();
	

	// copy internal data to ImageObject
	for( register int y = 0; y < height; y++)
	{
		for( register int x = 0; x < width; x++)
		{
			pImageData->SetColorForPixel(x, y, RGB(texture[y * width + x].red, texture[y * width + x].green, texture[y * width + x].blue));			
		}

	}

	char FullName[_MAX_PATH];
	sprintf(FullName, "%s\\%s.tif", TextureDirectory, pFilename);
	strcpy(pFilename, FullName);

	ImageObject.Save(pFilename, BTFILETYPE_TIFF);


	if (texture)
		tsxImageDestroyData(&TextureImg);*/

	return TRUE;
}


///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////

CString CRibExport::MakeTexture( RtString filename )
{
	CString tiffile, outfile;
	
	char drive[_MAX_PATH];
	char dir[_MAX_PATH];
	char fname[_MAX_PATH];
	char ext[_MAX_PATH];

	_splitpath( filename, drive, dir, fname, ext );

	// first check to see if it's in a format that 3Delight understands,
	// and if not, make it so
	bool canread = false;
	if ( ( strcmpi( ext, ".jpg" ) == 0 ) ||
		( strcmpi( ext, ".jpeg" ) == 0 ) ||
		( strcmpi( ext, ".tif" ) == 0 ) ||
		( strcmpi( ext, ".tiff" ) == 0 ) ||
		( strcmpi( ext, ".iff" ) == 0 ) ||
		( strcmpi( ext, ".sgi" ) == 0 ) ||
		( strcmpi( ext, ".exr" ) == 0 ) ||
		( strcmpi( ext, ".gif" ) == 0 ) ||
		( strcmpi( ext, ".hdr" ) == 0 ) ||
		( strcmpi( ext, ".rad" ) == 0 ) ||
		( strcmpi( ext, ".psd" ) == 0 ) )
	{
		canread = true;
		tiffile = filename;
	}
	
	if ( !canread )
	{
		char tempdir[_MAX_PATH];
		GetTempPath( _MAX_PATH, tempdir );

		tiffile.Format("%s%s.tif", tempdir, fname );


		/*BTCImageObject ImageObject;
		if ( ImageObject.Load( filename ) )
		{
			ImageObject.Save( tiffile, BTFILETYPE_TIFF );		
		}
		else
			return _T("BADIMAGE");*/
		ILuint devilimg;
		InitDevIL( devilimg );
		
		if ( ilLoadImage( filename ) )
		{
			ILint width = ilGetInteger( IL_IMAGE_WIDTH );
			ILint height = ilGetInteger( IL_IMAGE_HEIGHT );
			ilConvertImage( IL_RGBA, IL_UNSIGNED_BYTE );
			ilEnable( IL_FILE_OVERWRITE );
			ilSave( IL_TIF, (char*)((LPCSTR)tiffile) );
			ShutdownDevIL( devilimg );
		}
		else
		{
			ShutdownDevIL( devilimg );
			return _T("BADIMAGE");
		}
	}

	// new method is to save the Renderman texture to the same dir as the texture
	// but skip if it already exists
	outfile.Format("%s%s%s.tdl", drive, dir, fname );
	if ( FileExists( outfile.GetBuffer() ) )
	{
		MakeUnixPath( &outfile );
		return outfile;
	}

	
	// calling this pops up a window, and we don't want that!
	//RiMakeTexture( (char*)(LPCSTR)tiffile, (char*)(LPCSTR)outfile, RI_PERIODIC, RI_PERIODIC, RiBoxFilter, 1.0f, 1.0f, RI_NULL );

	// instead we'll call tdlmake.exe manually
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;

	char cmd[255];
	sprintf( cmd, "tdlmake -mode periodic -quality high \"%s\" \"%s\"", tiffile, outfile );

    // Start the child process. 
    CreateProcess( NULL,   // No module name (use command line). 
        (LPTSTR)cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi );             // Pointer to PROCESS_INFORMATION structure.
    
    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );


	return outfile;

}

///////////////////////////////////////////////////////////////////////////////
