#include "stdafx.h"
#include "dribble.h"
#include "RibExport.h"
#include "BTCImageObject.h"
#include "wfmmshaders.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static bool LastDisplaced = false;



// GoSceneTextures
// Recursively looks through scene, looking for textures to write out
// Inputs: pGNode - pointer to GNode to search from
// Outputs:
// Returns: 
void CRibExport::GoSceneTextures(tsxGNODE *pGNode)
{
	while (tsxIsGNode((tsxSOBJ*)pGNode) == e_tsxTRUE)
	{
		// if there's a child, recurse through this function again
		tsxGNODE* pChild = tsxGNodeGetFirstChild(pGNode);
		if (tsxIsGNode((tsxSOBJ*)pChild) == e_tsxTRUE)
			GoSceneTextures(pChild);

		// if the object is invisble, we just skip it
		if ( IsVisible( pGNode ) )
		{
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pGNode, ExtDataID);

			tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);
			switch (ObjType)
			{
				case e_tsxMBALLOBJ:
				case e_tsxMETANURBS:
				{
					// If it's a metaobject, we need to get the rendering polyh
					tsxPOLYHEDRON* pMetanode;
					pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
					if (pMetanode)
					{
						WritePolyhedronTextures(pMetanode, (tsxGNODE*)pMetanode);
					}
				}
				break;
				case e_tsxPOLYHEDRON:
				{
					WritePolyhedronTextures((tsxPOLYHEDRON*) pGNode, pGNode);
				}
				break;
				case e_tsxNPATCH_POLYH:
				{
					WritePatchTextures(tsxNPatchPolyhGetNPatch((tsxPOLYHEDRON*)pGNode), (tsxPOLYHEDRON*)pGNode);
				}
				break;
				case e_tsxSUBDIV:
				{
					tsxPOLYHEDRON *pCage = tsxSubDivGetCagePtr((tsxMNODE*)pGNode);
					WritePolyhedronTextures(pCage, pGNode);
				}
				break;


			}
		} // visible
		
		pGNode = tsxGNodeGetNext(pGNode);
	}
}


///////////////////////////////////////////////////////////////////////////////





// WritePolyhedronTextures
// Inputs: pInPolyh - pointer to polyhedron to use
// Outputs:
// Returns: 
void CRibExport::WritePolyhedronTextures(tsxPOLYHEDRON *pPolyh, tsxGNODE* pTopNode)
{
	// get the object name
	CString ObjName = GetName(pTopNode);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// get some basic polyh info
	int NumPolys = tsxPolyhGetNbrFaces(pPolyh);
	CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pPolyh);
	CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pPolyh);
	CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pPolyh);


		for (int poly = 0; poly < NumPolys; poly++)
		{
			// now look at materials
			// only do so if the object has no custom data
			// uses a material caching system so we don't have to write
			// out the same material multiple times
			tsxMATERIAL* pMaterial;
			tsxMaterialCreate(&pMaterial);
			tsxPolyhGetMaterial(pPolyh, pFaceArray[poly]->materialIdx, pMaterial);
			int Faceting = tsxMaterialGetFacetingType(pMaterial);
	
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
			if (!pData)
			{
				if ( (poly == 0) || (pFaceArray[poly]->materialIdx != OldMaterialIdx) )
				{
					RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
					sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), pFaceArray[0]->materialIdx);

					
					bool Painted = IsMaterialPainted(pMaterial, TxrName);

					// for prman we have to make tex files
					//if (Renderer == Renderer_Prman)
					if (Painted)
					{
						MakeTexture( TxrName );
					}
					if (TxrName)
						tsxFree(TxrName);

					// now look for the displacement
					RtString DisName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
					sprintf(DisName, "%s%d", ObjName.GetBuffer(_MAX_PATH), pFaceArray[0]->materialIdx);
					
					float amp;
					bool truedisp;
					bool normalmap;
					bool Displaced = IsMaterialDisplaced(pMaterial, DisName, amp, truedisp, normalmap);

					// for prman we have to make tex files
					//if (Renderer == Renderer_Prman)
					if (Displaced)
					{
						MakeTexture( DisName );
					}

					if (DisName)
						tsxFree(DisName);
	

				}
				OldMaterialIdx = pFaceArray[poly]->materialIdx;
			}


			// destroy working material
			tsxMaterialDestroy(pMaterial);
		}


	tsxFree(name);
}




// WriteMaterial
//	writes out a tS material as Renderman calls
// Inputs: pMaterial - pointer to material to use
// Outputs:
// Returns: 
void CRibExport::WriteMaterial( tsxGNODE *pGNode, tsxMATERIAL *pInMaterial, CString ObjName, int MatNbr)
{
	if ( !pInMaterial )
		return;

	tsxMATERIAL *pMaterial = NULL;
	tsxMaterialCreateCopy( &pMaterial, pInMaterial );
	if ( IsMaterialLayered( pMaterial ) )
	{
		//WriteMaterial( pGNode, pFirstLayer, ObjName, MatNbr );
	}

	bool TransShadows = false;

    RtString shadingmodel = new char[255];
    strcpy( shadingmodel, "matte" );

    CtsxColor Color;
	RtColor BaseColor;
	tsxMaterialFixColor( pMaterial );
	tsxMaterialFixTransparency( pMaterial );
	tsxMaterialGetColor( pMaterial, &Color );
	tSColorToRtColor( BaseColor, Color);

	float Ka = 0.0f;
	float Kd = 1.0f;
	float Ks = 0.0f;
	float roughness = 0.0f;
	float Kr = 0.0f;	
	float Kt = 0.0f;
	float ior = 1.0f;
	tssColor specularcolor = {1.0f, 1.0f, 1.0f};
	const tssColor BLACK = { 0.0f, 0.0f, 0.0f };

	tssColor BaseOpacity = { 1, 1, 1 };
			
	RtFloat metal = 0.0f;
	RtFloat raytracing = (RtFloat)m_doRaytrace;
	RtFloat usephong = 1.0f;

	bool Masked = false;
	RtFloat maskfilter = 0.0f;

	// look at transparency first because we need to see if we'll have a mask
	CtsxVector2f MaskUVReps, MaskUVOffset;
	MaskUVReps.x = MaskUVReps.y = 1.0f;
	MaskUVOffset.x = MaskUVOffset.y = 0.0f;
				
	RtString MaskTxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
	//sprintf(MaskTxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), MatNbr);
	strcpy( MaskTxrName, "" );

	RtString EnvTxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
	strcpy( EnvTxrName, "" );	

	RtString KaMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( KaMapTxrName, "" );

	RtString KdMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( KdMapTxrName, "" );
	
	RtString KsMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( KsMapTxrName, "" );
	
	RtString KrMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( KrMapTxrName, "" );

	RtString KtMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( KtMapTxrName, "" );

	RtString roughnessMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( roughnessMapTxrName, "" );

	RtString iorMapTxrName = (RtString)tsxMalloc( _MAX_PATH * sizeof(char) );
	strcpy( iorMapTxrName, "" );
	
	
	
	
	
	if ( m_environmentShowReflections )
	{
		CString strTextureName;	
		strTextureName = MakeEnviroTexture( (RtString)((LPCSTR)m_environmentName), true );
		strcpy( EnvTxrName, (LPCSTR)strTextureName );
	}
				       

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_TRANSPARENCY);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) != e_tsxFAILURE)
	{
		if (lstrcmpi(ShaderName, "c_alpha") == 0)
		{
			float alpha;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&alpha);
			BaseOpacity[0] = BaseOpacity[1] = BaseOpacity[2] = alpha;
		}

		if (lstrcmpi(ShaderName, "plain") == 0)
		{
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&BaseOpacity);
		}

		if (lstrcmpi(ShaderName, "c_wrapped_mask") == 0)
		{
			Masked = IsMaterialMasked( pMaterial, MaskTxrName );
			if ( Masked )
			{
				GetUVInfo( pShinfo, MaskUVReps, MaskUVOffset );   
				CString newtexname = MakeTexture( MaskTxrName );
				strcpy( MaskTxrName, newtexname );
			}
		}

		if (lstrcmpi(ShaderName, "wrapped image") == 0)
		{
			maskfilter = 1.0f;
			Masked = IsMaterialMasked( pMaterial, MaskTxrName );
			if ( Masked )
			{
				GetUVInfo( pShinfo, MaskUVReps, MaskUVOffset );   
				CString newtexname = MakeTexture( MaskTxrName );
				strcpy( MaskTxrName, newtexname );
			}
		}
	}

	// handle transparency for shadow maps
	if ( JustShadowmaps )
	{
		if ( Masked )
		{
			raytracing = 0;
			Ka = 1;
			Kd = 0;
			Ks = 0;
			Kr = 0;
			Kt = 0;
			RiSurface("tSstandard", 
				"Ka", &Ka, 
				"Kd", &Kd, 
				"Ks", &Ks, 
				"roughness", &roughness, 
				"specularcolor", &specularcolor, 
				"Kr", &Kr,
				"Kt", &Kt,
				"ior", &ior,
				"float metal", &metal,
				"float raytracing", &raytracing,
				"string envmap", &EnvTxrName,
				"float envexposure", &m_environmentExposure,
				"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
				RI_NULL);
		}
	}
	else
	{
		/*if (DeclareParams)
		{
			tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_TRANSPARENCY);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_REFLECTANCE);
			WriteShinfo(pShinfo);
			pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);
			WriteShinfo(pShinfo);
		}*/

        Kd = 0.0f;
		
		

		// first we're going to work with the reflectance shader
		pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_REFLECTANCE);
		if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) != e_tsxFAILURE)
		{
			

			RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), MatNbr);
			
			bool Painted = false;

			CtsxVector2f TxrUVReps, TxrUVOffset;
			TxrUVReps.x = TxrUVReps.y = 1.0f;
			TxrUVOffset.x = TxrUVOffset.y = 0.0f;

			bool isSimbiont = false;

			if (lstrcmpi(ShaderName, "SimbiontTS") == 0)
			{
				char *pC;
				tsxSHPARAMVAL *pVal = tsxShaderInfoGetParameterValue( pShinfo, 1 );
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
				strcpy(TxrName, pC);
				RiSurface( "DTE_SimpleSurf", "string DT_Surface", &TxrName, RI_NULL );
				isSimbiont = true;
			}
			else
				Painted = IsMaterialPainted(pMaterial, TxrName);

			if ( Painted )
			{
				tsxSHINFO *pColShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
				GetUVInfo( pColShinfo, TxrUVReps, TxrUVOffset );   
				CString newtexname = MakeTexture( TxrName );
				strcpy( TxrName, newtexname );
			}
			else
				strcpy( TxrName, "" );

			

			if ( isSimbiont == false )
			{
				if ( (lstrcmpi(ShaderName, "caligari metal") == 0) || (lstrcmpi(ShaderName, "c metal enhanced") == 0) )
				{
					metal = 1.0f;
					usephong = 0.0f;

					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);

					strcpy( shadingmodel, "chrome" );
					


					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);

					
				}
				else
				if (lstrcmpi(ShaderName, "glass") == 0)
				{
					Ka = 0.0f;
					Kd = 0.0f;
					usephong = 0.0f;
					
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
					

					// for caustics
					/*if (m_bUseCaustics)
					{
						tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/
					strcpy( shadingmodel, "glass" );

					RtFloat sharpness = 0.0f;
					RiSurface("tSglass", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"float sharpness", &sharpness,
					"Kr", &Kr,
					"float Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "dielectric") == 0)
				{
					metal = 1.0f;
					usephong = 0.0f;
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);				
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
					

					// for caustics
					/*if (m_bUseCaustics)
					{
						tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/
					strcpy( shadingmodel, "glass" );

					RtFloat sharpness = 0.8f;
					RiSurface("tSglass", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"float sharpness", &sharpness,
					"Kr", &Kr,
					"float Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "mirror") == 0)
				{
					//metal = 1.0f;
					usephong = 0.0f;

					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
	                
					//char name[255];
					//tsxShaderInfoGetParameterName( pShinfo, 4, &name[0] );

					
					//roughness = 1.f/(100.f*roughness) - 0.1f;
					//roughness = 1.0f - roughness;
					
					


					// for caustics
					/*if (m_bUseCaustics)
					{
						tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/
					strcpy( shadingmodel, "chrome" );

					
					/*roughness = 169.0f - 140.0f * (3.0f - roughness);
					roughness = (sqrt(roughness) - 13.0f) / 70.0f;
					roughness = 1.0f - roughness;*/
					
					

					// for caustics
					/*if (m_bUseCaustics)
					{
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);

				}
				else
				if (lstrcmpi(ShaderName, "metal") == 0)
				{
					metal = 1.0f;
					usephong = 0.0f;

					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
	                
					// for caustics
					/*if (m_bUseCaustics)
					{
						tssColor specularcolor = { 1.0f, 1.0f, 1.0f };
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/
					strcpy( shadingmodel, "chrome" );

					// metal shaders have the same base color for specular and no diffuse
					Kd = 0.0f;
	                
					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &BLACK,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "caligari phong") == 0)
				{
					
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					//roughness = 169.0f - 140.0f * (3.0f - roughness);
					//roughness = (sqrt(roughness) - 13.0f) / 70.0f;
					//roughness = 1.0f - roughness;
					
					//roughness = 1.f/(100.f*roughness) - 0.1f;
					//roughness = 35.f*roughness*roughness+13.f*roughness+3.f;
					//roughness = 1.0f / roughness;

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
									
					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"ior", &ior,
					"float metal", &metal,
					"float raytracing", &raytracing,
					"color bg_color", &BLACK,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "c phong enhanced") == 0)
				{
					
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
									
					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"ior", &ior,
					"float metal", &metal,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "phong") == 0)
				{
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 169.0f - 140.0f * (3.0f - roughness);
					//roughness = (sqrt(roughness) - 13.0f) / 70.0f;
					//roughness = 1.0f - roughness;
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

					// for caustics
					/*if (m_bUseCaustics)
					{
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "plastic") == 0)
				{
					usephong = 0.0f;
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

					// for caustics
					/*if (m_bUseCaustics)
					{
						RiAttribute("caustic", "specularcolor", &specularcolor, RI_NULL);
					}*/

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"float usephong", &usephong,
					"specularcolor", &specularcolor, 
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "constant") == 0)
				{
					RiSurface("constant", RI_NULL);
				}
				else
				if (lstrcmpi(ShaderName, "matte") == 0)
				{
					Ks = 0.0f;
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);

					
					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);

				}
				else
				if (lstrcmpi(ShaderName, "translucent plastic") == 0)
				{

					// for SSS
					float Ksub;
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					roughness = 1.0f / (1.0f - roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ksub);
					
					strcpy( shadingmodel, "transparent" );


					if (Painted)
					{
						RiSurface("tSsss", "translucency", &Ksub, "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "texturename", &TxrName, "float ureps", &TxrUVReps.x, "float vreps", &TxrUVReps.y, "float uoffset", &TxrUVOffset.x, "float voffset", &TxrUVOffset.x, RI_NULL);
					}
					else
						RiSurface("tSsss", "translucency", &Ksub, "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, RI_NULL);

					// TODO: this should be the object name, or preferably the parent group name
					tsxGNODE* pParent = tsxGNodeGetRoot( pGNode );
					CString groupname = GetName( pParent );
					RtString sssname = (RtString)tsxMalloc(255 * sizeof(char));
					strcpy(sssname, groupname.GetBuffer(255));

					RiAttribute( "visibility", "string subsurface", &sssname, RI_NULL );
					tsxFree( sssname );

					// marble
					RtColor scattering = { 2.19f, 2.62f, 3.00f };
					RtColor absorption = { 0.0021f, 0.0041f, 0.0071f } ;
					// potato
					RtColor meanfreepath = { 18.5f, 10.4f, 3.5f };
					RtColor diffusereflectance;
					diffusereflectance[0] = BaseColor[0];
					diffusereflectance[1] = BaseColor[1];
					diffusereflectance[2] = BaseColor[2];
					RtFloat ior = 1.3f;
					RtFloat scale = 0.01f;
					RtFloat shadingrate = 1.0f;

					//RiAttribute( "subsurface", "color meanfreepath", &meanfreepath, "color reflectance", &diffusereflectance, "float refractionindex", &ior, "float scale", &scale, "float shadingrate", &shadingrate, RI_NULL );
					RiAttribute( "subsurface", "color scattering", &scattering, "color absorption", &absorption, "float refractionindex", &ior, "float scale", &scale, "float shadingrate", &shadingrate, RI_NULL );

				}
				else
				if (lstrcmpi(ShaderName, "drbSimpleSSS") == 0)
				{

					// for SSS
					float Ksub = 1.0f;
					// marble
					RtColor scattering = { 2.19f, 2.62f, 3.00f };
					RtColor absorption = { 0.0021f, 0.0041f, 0.0071f } ;
					// potato
					RtColor meanfreepath = { 18.5f, 10.4f, 3.5f };
					RtColor diffusereflectance;
					diffusereflectance[0] = BaseColor[0];
					diffusereflectance[1] = BaseColor[1];
					diffusereflectance[2] = BaseColor[2];
					RtFloat ior = 1.3f;
					RtFloat scale = 0.01f;
					RtFloat shadingrate = 1.0f;

					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					roughness = 1.0f / (1.0f - roughness);
					//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&scattering);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&absorption);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 9);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&shadingrate);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&scale);
					

						


					strcpy( shadingmodel, "transparent" );


					if (Painted)
					{
						RiSurface("tSsss", "translucency", &Ksub, "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "float Kr", &Kr, "texturename", &TxrName, "float ureps", &TxrUVReps.x, "float vreps", &TxrUVReps.y, "float uoffset", &TxrUVOffset.x, "float voffset", &TxrUVOffset.x, RI_NULL);
					}
					else
						RiSurface("tSsss", "translucency", &Ksub, "Ka", &Ka, "Kd", &Kd, "Ks", &Ks, "roughness", &roughness, "specularcolor", &specularcolor, "float Kr", &Kr, RI_NULL);

					// TODO: this should be the object name, or preferably the parent group name
					tsxGNODE* pParent = tsxGNodeGetRoot( pGNode );
					CString groupname = GetName( pParent );
					RtString sssname = (RtString)tsxMalloc(255 * sizeof(char));
					strcpy(sssname, groupname.GetBuffer(255));

					RiAttribute( "visibility", "string subsurface", &sssname, RI_NULL );
					tsxFree( sssname );

					

					//RiAttribute( "subsurface", "color meanfreepath", &meanfreepath, "color reflectance", &diffusereflectance, "float refractionindex", &ior, "float scale", &scale, "float shadingrate", &shadingrate, RI_NULL );
					RiAttribute( "subsurface", "color scattering", &scattering, "color absorption", &absorption, "float refractionindex", &ior, "float scale", &scale, "float shadingrate", &shadingrate, RI_NULL );

				}
				else
				if ( lstrcmpi(ShaderName, "wrapped mirror map") == 0 )
				{
					//metal = 1.0f;
					usephong = 0.0f;
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					char* pC = NULL;
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 5 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						// lightworks quirk
						if ( strcmp( pC, "unavailable" ) != 0 )
						{
							strcpy( KrMapTxrName, pC);	
							CString newtexname = MakeTexture( KrMapTxrName );
							strcpy( KrMapTxrName, newtexname );
						}
					}
					

					
					//roughness = 1.f/(100.f*roughness) - 0.1f;
					//roughness = 1.0f - roughness;
					

					strcpy( shadingmodel, "chrome" );

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string Kr_map", &KrMapTxrName,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if ( lstrcmpi(ShaderName, "c mapped phong") == 0 )
				{
					//metal = 1.0f;
					
					char* pC;
					
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 1 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KaMapTxrName, pC);	
						CString newtexname = MakeTexture( KaMapTxrName );
						strcpy( KaMapTxrName, newtexname );
					}
					
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 3 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KdMapTxrName, pC);	
						CString newtexname = MakeTexture( KdMapTxrName );
						strcpy( KdMapTxrName, newtexname );
					}

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 5 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KsMapTxrName, pC);	
						CString newtexname = MakeTexture( KsMapTxrName );
						strcpy( KsMapTxrName, newtexname );
					}


					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 7 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( roughnessMapTxrName, pC);	
						CString newtexname = MakeTexture( roughnessMapTxrName );
						strcpy( roughnessMapTxrName, newtexname );
					}


					pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 9 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						//strcpy( speccolMapTxrName, pC);	
						//CString newtexname = MakeTexture( speccolMapTxrName );
						//strcpy( speccolMapTxrName, newtexname );
					}


					pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 11 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KtMapTxrName, pC);	
						CString newtexname = MakeTexture( KtMapTxrName );
						strcpy( KtMapTxrName, newtexname );
					}

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 12);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 13 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KrMapTxrName, pC);	
						CString newtexname = MakeTexture( KrMapTxrName );
						strcpy( KrMapTxrName, newtexname );
					}

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 15 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( iorMapTxrName, pC);	
						CString newtexname = MakeTexture( iorMapTxrName );
						strcpy( iorMapTxrName, newtexname );
					}

					
					/*roughness = 1.f/(100.f*roughness) - 0.1f;
					roughness = 1.0f - roughness;*/
					//roughness = 169.0f - 140.0f * (3.0f - roughness);
					//roughness = (sqrt(roughness) - 13.0f) / 70.0f;
					//roughness = 1.0f - roughness;
					

					strcpy( shadingmodel, "chrome" );

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string Ka_map", &KaMapTxrName,
					"string Kd_map", &KdMapTxrName,
					"string Ks_map", &KsMapTxrName,
					"string Kr_map", &KrMapTxrName,
					"string Kt_map", &KtMapTxrName,
					"string roughness_map", &roughnessMapTxrName,
					"string ior_map", &iorMapTxrName,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				if ( lstrcmpi(ShaderName, "c mapped metal") == 0 )
				{
					metal = 1.0f;
					usephong = 0.0f;
					
					char* pC;

					Kd = 0.0f;
					
					tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 1 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KaMapTxrName, pC);	
						CString newtexname = MakeTexture( KaMapTxrName );
						strcpy( KaMapTxrName, newtexname );
					}
									
					pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 3 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KsMapTxrName, pC);	
						CString newtexname = MakeTexture( KsMapTxrName );
						strcpy( KsMapTxrName, newtexname );
					}


					pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 5 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( roughnessMapTxrName, pC);	
						CString newtexname = MakeTexture( roughnessMapTxrName );
						strcpy( roughnessMapTxrName, newtexname );
					}



					pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 7 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KtMapTxrName, pC);	
						CString newtexname = MakeTexture( KtMapTxrName );
						strcpy( KtMapTxrName, newtexname );
					}

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 9 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( KrMapTxrName, pC);	
						CString newtexname = MakeTexture( KrMapTxrName );
						strcpy( KrMapTxrName, newtexname );
					}

					pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
					pVal = tsxShaderInfoGetParameterValue( pShinfo, 11 );
					tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
					if ( pC )
					{
						strcpy( iorMapTxrName, pC);	
						CString newtexname = MakeTexture( iorMapTxrName );
						strcpy( iorMapTxrName, newtexname );
					}


					//roughness = 1.f/(100.f*roughness) - 0.1f;
					//roughness = 1.0f - roughness;
					

					strcpy( shadingmodel, "chrome" );

					RiSurface("tSstandard", 
					"Ka", &Ka, 
					"Kd", &Kd, 
					"Ks", &Ks, 
					"roughness", &roughness, 
					"specularcolor", &specularcolor, 
					"Kr", &Kr,
					"Kt", &Kt,
					"float ior", &ior,
					"float metal", &metal,
					"float usephong", &usephong,
					"float raytracing", &raytracing,
					"color bg_color", &m_bgColor,
					"string envmap", &EnvTxrName,
					"float envexposure", &m_environmentExposure,
					"string Ka_map", &KaMapTxrName,
					"string Kd_map", &KdMapTxrName,
					"string Ks_map", &KsMapTxrName,
					"string Kr_map", &KrMapTxrName,
					"string Kt_map", &KtMapTxrName,
					"string roughness_map", &roughnessMapTxrName,
					"string ior_map", &iorMapTxrName,
					"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
					"string maskmap", &MaskTxrName, "float maskureps", &MaskUVReps.x, "float maskvreps", &MaskUVReps.y, "float maskuoffset", &MaskUVOffset.x, "float maskvoffset", &MaskUVOffset.y, "float maskfilter", &maskfilter,
					RI_NULL);
				}
				else
				// no recognizable shader, so we use a default
				{
					Kd = 1.0f;
					RiSurface("matte", "Ka", &Ka, "Kd", &Kd, RI_NULL);
				}
			} // simbiont

			if ( TxrName )
				tsxFree( TxrName );

			if ( KaMapTxrName )
				tsxFree( KaMapTxrName );

			if ( KdMapTxrName )
				tsxFree( KdMapTxrName );

			if ( KsMapTxrName )
				tsxFree( KsMapTxrName );

			if ( KrMapTxrName )
				tsxFree( KrMapTxrName );

			if ( KtMapTxrName )
				tsxFree( KtMapTxrName );

			if ( roughnessMapTxrName )
				tsxFree( roughnessMapTxrName );

			if ( iorMapTxrName )
				tsxFree( iorMapTxrName );

		}
	}

	

    if ( strcmp( shadingmodel, "matte" ) == 0 )
    {
        if ( BaseOpacity[0] < 1.0f )
            strcpy( shadingmodel, "transparent" );
    }


			
	// now we will look at displacement
	pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);

	RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
	sprintf(TxrName, "%sbump%d", ObjName.GetBuffer(_MAX_PATH), MatNbr);
		
	RtFloat amplitude = 0.0f;
	bool truedisp;
	bool normalmap;
	bool Displaced = IsMaterialDisplaced(pMaterial, TxrName, amplitude, truedisp, normalmap );
	
	if (Displaced)
	{	
		LastDisplaced = true;
		
		CtsxVector2f uvreps, uvoffset;
		uvreps.x = uvreps.y = 1.0f;
		uvoffset.x = uvoffset.y = 0.0f;
		RtFloat disp = 0.0f;
		if ( truedisp )
			disp = 1.0f;

		tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]);
		if (lstrcmpi(ShaderName, "SimbiontTS") == 0)
		{
			char *pC;
			tsxSHPARAMVAL *pVal = tsxShaderInfoGetParameterValue( pShinfo, 1 );
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
			strcpy(TxrName, pC);
			RiDisplacement( "DTE_Displace", "string DT_Surface", &TxrName, RI_NULL );
			truedisp = true;
			disp = 1.0f;
			amplitude = 1.0f;
		}
		else
		{

			GetUVInfo( pShinfo, uvreps, uvoffset );
			
			CString newtexname = MakeTexture( TxrName );
			strcpy( TxrName, newtexname );

			

			if ( !normalmap )
			{
				amplitude *= 10.0f;
				RiDisplacement( "tSbumpmap", "float Km", &amplitude, "string texturename", &TxrName, "float truedisplace", &disp, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL );
			}
			else
			{
				RiDisplacement( "tSnormalmap", "float Km", &amplitude, "string texturename", &TxrName, "float truedisplace", &disp, "float ureps", &uvreps.x, "float vreps", &uvreps.y, "float uoffset", &uvoffset.x, "float voffset", &uvoffset.x, RI_NULL );
			}
		}

		
		if ( truedisp )
		{
			RtFloat radius = fabs(amplitude);
			RtString coordsys = "world";
			RiAttribute( "displacementbound", "float sphere", &radius, "string coordinatesystem", &coordsys, RI_NULL );
			RtInt on = 1;
			if ( !m_doRaytraceDisplacements )
				on = 0;
			RiAttribute( "trace", "integer displacements", &on, RI_NULL );
		}
	}
	else
	{
		// clear any displacement if it was previously there
		if ( LastDisplaced )
		{
			RiDisplacement( "", RI_NULL );
			LastDisplaced = false;
		}
	}

	
	if (TxrName)
		tsxFree(TxrName);

	if ( MaskTxrName )
		tsxFree( MaskTxrName );

	if ( EnvTxrName )
		tsxFree( EnvTxrName );


	if (!JustShadowmaps)
	{
		// now handle the basic color and opacity		
		RiColor( BaseColor );
		RiOpacity( BaseOpacity );
	}


	// now the visibility options
	RtInt on;
	/*if ( ( m_doRaytrace ) && ( Kd > 0.0f ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) && ( Ks > 0.0f ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer specular", &on, RI_NULL );
	}
	if ( ( m_doRaytrace ) && ( tsxMNodeCastShadows( (tsxMNODE*)pGNode ) == e_tsxTRUE ) )
	{
		on = 1;
		RiAttribute( "visibility", "integer transmission", &on, RI_NULL );

		RtString hitmode = "primitive";
		if ( !TransShadows )
			RiAttribute( "visibility", "string transmissionhitmode", &hitmode, RI_NULL );
	}
	if ( ( (BaseOpacity[0] + BaseOpacity[1] + BaseOpacity[2]) == 3.0f ) && ( Kt == 0.0f ) )
	{
		RtString hitmode = "primitive";
		RiAttribute( "visibility", "string camerahitmode", &hitmode, RI_NULL );
	}*/

	// need to use visibility options even without raytracing
	//if ( m_doRaytrace )
	{
		on = 1;
		RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
		RiAttribute( "visibility", "integer specular", &on, RI_NULL );

		RtInt transmit = 0;
		if ( tsxMNodeCastShadows( (tsxMNODE*)pGNode ) == e_tsxTRUE )
		{
			transmit = 1;
			
			RtString hitmode = new char[255];
			strcpy( hitmode, "primitive" );
			if ( ( Masked ) || ( Kt > 0.0f ) )
				strcpy( hitmode, "shader" );
			
			RiAttribute( "shade", "string transmissionhitmode", &hitmode, RI_NULL );
			RiAttribute( "shade", "string camerahitmode", &hitmode, RI_NULL );

			// now for GI
			strcpy( hitmode, "primitive" );
			if ( m_bHighQualityGI )
				strcpy( hitmode, "shader" );
			RiAttribute( "shade", "string diffusehitmode", &hitmode, RI_NULL );

			delete []hitmode;
		}
		RiAttribute( "visibility", "integer transmission", &transmit, RI_NULL );
	}

	

	/*// finally, shadow-casting options

	RtString shadtype = new char[255];
	RtInt shad;
	if ( tsxMNodeCastShadows( (tsxMNODE*)pGNode ) == e_tsxTRUE )
	{
		if ( TransShadows )
			strcpy( shadtype, "shader" );
		else
			strcpy( shadtype, "Os" );

	}
	else
	{
		strcpy( shadtype, "transparent" );
	}
	RiAttribute( "visibility", "string transmission", &shadtype, RI_NULL );
    
	delete []shadtype;*/

    // shading model for photons
    if ( m_bUseIrradiancePhotons || m_bUseCaustics )
    {
        switch ( Renderer )
        {
        case Renderer_3Delight:
            RiAttribute( "photon", "string shadingmodel", &shadingmodel, RI_NULL );    
            break;
        }
    }

    delete []shadingmodel;

	tsxMaterialDestroy( pMaterial );

}

// WriteShinfo
//	writes out details of a shaderinfo structure
//	called recursively to handle layers
//	For now, this just dumps out all shader info as comments
// Inputs: pShinfo - pointer to shaderinfo to use
// Outputs:
// Returns: 
void CRibExport::WriteShinfo(tsxSHINFO *pShinfo)
{
	tsxSHCLASS shclass = tsxShaderInfoGetClass(pShinfo);

	char CurrentName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &CurrentName[0]) == e_tsxFAILURE)
		return;
	
	if (lstrcmpi(CurrentName, "none") == 0)
		return;

	switch (shclass)
	{
		case tsxSHCLASS_COLOR:
			RiArchiveRecord(RI_COMMENT, "Color Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_TRANSPARENCY:
			RiArchiveRecord(RI_COMMENT, "Transparency Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_REFLECTANCE:
			RiArchiveRecord(RI_COMMENT, "Reflectance Shader: %s", CurrentName);
		break;
		case tsxSHCLASS_DISPLACEMENT:
			RiArchiveRecord(RI_COMMENT, "Displacement Shader: %s", CurrentName);
		break;
	}

	// first declare the variables
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, n);
		tsxSHPARAM_TYPE ParamType = tsxParameterValueGetType(pVal);
		switch (ParamType)
		{
			case tsxSHPARAM_BOOL:
			{
				RiDeclare(ParamName, "integer");
			}
			break;

			case tsxSHPARAM_FLOAT:
			case tsxSHPARAM_DOUBLE:
			{
				RiDeclare(ParamName, "float");
			}
			break;
			
			case tsxSHPARAM_BYTE:
			case tsxSHPARAM_SHORT:
			case tsxSHPARAM_LONG:
			case tsxSHPARAM_WORD:
			case tsxSHPARAM_DWORD:
			{
				RiDeclare(ParamName, "integer");
			}
			break;

			case tsxSHPARAM_VECTOR:
			{
				RiDeclare(ParamName, "vector");
			}
			break;
			case tsxSHPARAM_POINT:
			{
				RiDeclare(ParamName, "point");
			}
			break;
			case tsxSHPARAM_COLOR:
			case tsxSHPARAM_COLOR4:			
			{
				RiDeclare(ParamName, "color");
			}
			break;

			case tsxSHPARAM_STRING:
			{
				RiDeclare(ParamName, "string");
			}
			break;			

			default:
				RiArchiveRecord(RI_COMMENT, "\tUNKNOWN PARAMETER TYPE");

		}
	}

	// now write em
	char strParams[1048];
	int chars = 0;
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, n);
		tsxSHPARAM_TYPE ParamType = tsxParameterValueGetType(pVal);
		chars += sprintf(strParams + chars, " ""%s""", ParamName);
		switch (ParamType)
		{
			case tsxSHPARAM_BOOL:
			{
				bool b;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&b);
				if (b)
					chars += sprintf(strParams + chars, " 1");
				else
					chars += sprintf(strParams + chars, " 0");
			}
			break;

			case tsxSHPARAM_FLOAT:
			case tsxSHPARAM_DOUBLE:
			{
				float f;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&f);
				chars += sprintf(strParams + chars, " %g", f);
			}
			break;
			
			case tsxSHPARAM_BYTE:
			case tsxSHPARAM_SHORT:
			case tsxSHPARAM_LONG:
			case tsxSHPARAM_WORD:
			case tsxSHPARAM_DWORD:
			{
				int i;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&i);
				chars += sprintf(strParams + chars, " %d", i);
			}
			break;

			case tsxSHPARAM_VECTOR:
			case tsxSHPARAM_POINT:
			case tsxSHPARAM_COLOR:
			{
				tssColor c;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&c);
				chars += sprintf(strParams + chars, " [ %g, %g, %g ]", c[0], c[1], c[2]);
			}
			break;

			case tsxSHPARAM_COLOR4:
			{
				tssColor4 c;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&c);
				chars += sprintf(strParams + chars, " [ %g, %g, %g %g ]", c[0], c[1], c[2], c[3]);
			}
			break;

			case tsxSHPARAM_STRING:
			{
				char* pC;
				tsxParameterValueGetTypedValue(pVal, ParamType, (void*)&pC);
				chars += sprintf(strParams + chars, " ""%s""", pC);
			}
			break;

		}
	}
	sprintf(strParams + chars, "\n");
	RiArchiveRecord(RI_COMMENT, strParams);
}


///////////////////////////////////////////////////////////////////////////////


void CRibExport::WritePatchTextures(tsxNPATCH *pPatch, tsxPOLYHEDRON *pPolyh)
{
	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));


		// now look at materials
		int NbrMaterials = tsxPolyhGetNbrMaterials(pPolyh);

		tsxMATERIAL* pMaterial;
		tsxMaterialCreate(&pMaterial);
		tsxPolyhGetMaterial(pPolyh, 0, pMaterial);
		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
		if (!pData)
		{
			RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), 0);

			
			bool Painted = IsMaterialPainted(pMaterial, TxrName);

			// for prman we have to make tex files
			//if (Renderer == Renderer_Prman)
			if (Painted)
			{
				MakeTexture( TxrName );

				/*if (FileName)
					tsxFree(FileName);
				if (TexName)
					tsxFree(TexName);*/
			}
			if (TxrName)
				tsxFree(TxrName);

			// now look for the displacement
			RtString DisName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
			sprintf(DisName, "%s%d", ObjName.GetBuffer(_MAX_PATH), 0);
			
			float amp;
			bool truedisp;
			bool normalmap;
			bool Displaced = IsMaterialDisplaced(pMaterial, DisName, amp, truedisp, normalmap );

			// for prman we have to make tex files
			//if (Renderer == Renderer_Prman)
			if (Displaced)
			{
				MakeTexture( DisName );
			}

			if (DisName)
				tsxFree(DisName);

		}
		
		// destroy working material
		tsxMaterialDestroy(pMaterial);

	tsxFree(name);

}

void CRibExport::WriteTempMaterial(tsxSOBJ *pObj)
{
	if (!pObj)
		return;

	CString ObjName = GetName((tsxGNODE*) pObj);
	ObjName = MakeNameFilename(ObjName);


	char tempdir[255];
	GetTempPath(255, tempdir);
	char filename[255];
	sprintf(filename, "%s\\material.rib", tempdir);

	RiBegin(filename);

	RiWorldBegin();

		tsxMATERIAL* pMaterial = tsxPolyhGetIndexedMaterial((tsxPOLYHEDRON*)pObj, tsxPolyhGetFirstMaterialIdx((tsxPOLYHEDRON*)pObj));
		WriteMaterial( (tsxGNODE*)pObj, pMaterial, ObjName, tsxPolyhGetFirstMaterialIdx((tsxPOLYHEDRON*)pObj));
	
		RiWorldEnd();

	RiEnd();

}

bool CRibExport::IsMaterialPainted(tsxMATERIAL *pMaterial, char *pTxrName )
{
	if (!pMaterial)
		return FALSE;

	if (!pTxrName)
		return FALSE;

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_COLOR);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return FALSE;
	
	if (lstrcmpi(ShaderName, "plain") == 0)
		return FALSE;


	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari texture") == 0)
	{
		if (ConvertTextureShaders)
		{
			ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
		}
		else
		{
			char* pC;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
			strcpy(pTxrName, pC);	
		}
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		if (ConvertTextureShaders)
		{
			ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
		}
		else
		{
			char* pC;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
			strcpy(pTxrName, pC);	
		}
	}
	else
	if (ConvertProceduralShaders)
	{
		ConvertProcedural(pMaterial, TexWidth, TexHeight, pTxrName);
	}

	

	return TRUE;
	
}

bool CRibExport::IsMaterialMasked(tsxMATERIAL *pMaterial, char *pTxrName )
{
	if (!pMaterial)
		return FALSE;

	if (!pTxrName)
		return FALSE;

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_TRANSPARENCY);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return false;
	
	if (lstrcmpi(ShaderName, "plain") == 0)
		return false;


	// standard tS mask
	if (lstrcmpi(ShaderName, "c_wrapped_mask") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);	

		return true;
	}

	if (lstrcmpi(ShaderName, "wrapped image") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);	

		return true;
	}
	

	return false;
	
}



void CRibExport::GetUVInfo(tsxSHINFO *pShinfo, CtsxVector2f& uvreps, CtsxVector2f& uvoffset)
{
	if (!pShinfo)
		return;
	
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return;
	
	tsxSHCLASS shclass = tsxShaderInfoGetClass( pShinfo );
	switch ( shclass )
	{
	case tsxSHCLASS_COLOR:
		{
			// standard tS texture map
			if (lstrcmpi(ShaderName, "caligari texture") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
				
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
			else
			// wfmm texadjust
			if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 15);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
		}
		break;

	case tsxSHCLASS_TRANSPARENCY:
		{
			// standard tS mask
			if (lstrcmpi(ShaderName, "c_wrapped_mask") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
				
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
			
		}
		break;

	case tsxSHCLASS_DISPLACEMENT:
		{			
			// standard tS texture map
			if (lstrcmpi(ShaderName, "caligari bump") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
			else
			// wfmm texadjust
			if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 15);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
			else if (lstrcmpi(ShaderName, "caligari normal map") == 0)
			{
				char* pC;
				tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);

				pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvreps.y);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.x);
				pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
				tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&uvoffset.y);
			}
		}
		break;

	}
	
}

bool CRibExport::IsMaterialDisplaced(tsxMATERIAL *pMaterial, char *pTxrName, float& amplitude, bool& truedisp, bool& normalmap )
{
	truedisp = false;
	normalmap = false;

	if (!pMaterial)
		return FALSE;

	if (!pTxrName)
		return FALSE;

	tsxSHINFO* pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_DISPLACEMENT);
	char ShaderName[255];
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) == e_tsxFAILURE)
		return FALSE;
	
	if (lstrcmpi(ShaderName, "none") == 0)
		return FALSE;


	// now write em
	for (int n = 0; n < tsxShaderInfoGetParametersNumber(pShinfo); n++)
	{
		char ParamName[255];
		tsxShaderInfoGetParameterName(pShinfo, n, ParamName);
		TRACE("%d - %s\n", n, ParamName);
	}

	// standard tS texture map
	if (lstrcmpi(ShaderName, "caligari bump") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);	
		float f;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&f);
		amplitude = f;
		int b;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_BOOL, (void*)&b);
		truedisp = (bool)b;
	}
	else
	// wfmm texadjust
	if (lstrcmpi(ShaderName, "wfmmtexadjust") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);			
		float f;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 11);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&f);
		amplitude = f;
	}
	else if (lstrcmpi(ShaderName, "caligari normal map") == 0)
	{
		char* pC;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_STRING, (void*)&pC);
		strcpy(pTxrName, pC);	
		float f;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&f);
		amplitude = 1.0f;
		int b;
		pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_BOOL, (void*)&b);
		truedisp = (bool)b;
		normalmap = true;
	}


	return TRUE;
	
}

bool CRibExport::ConvertProcedural(tsxMATERIAL *pMaterial, int width, int height, char *pFilename)
{
	tsxERR err;

	tsxImage TextureImg;
	tsxImageCreateData(&TextureImg, width, height, e_tsxIMAGE_RGBA);

	// generate texture
	err = tsxMaterialGetTextureFromProceduralColor(pMaterial, &TextureImg);

	CtsxColor *texture = (CtsxColor*)TextureImg.data;
	
	if (!texture)
		return FALSE;

	
	/*ILuint devilimg;
	theApp.InitDevIL( devilimg );
	
	// create the image
	ilTexImage( width, height, 1, 3, IL_RGB, IL_UNSIGNED_BYTE, TextureImg.data );

	char FullName[_MAX_PATH];
	sprintf(FullName, "%s\\%s.tif", TextureDirectory, pFilename);
	strcpy(pFilename, FullName);


	ilEnable( IL_FILE_OVERWRITE );
	ilSave( IL_TIF, pFilename );
	theApp.ShutdownDevIL( devilimg );*/
	BTCImageObject	ImageObject;
	ImageObject.GetObjectDataPtr()->Create(width,height, 24);
	BTCImageData *pImageData = ImageObject.GetObjectDataPtr();
	unsigned char *pImageBits = pImageData->GetBits();
	
	// copy internal data to ImageObject
	for( register int y = 0; y < height; y++)
	{
		for( register int x = 0; x < width; x++)
		{
			pImageData->SetColorForPixel(x, y, RGB(texture[y * width + x].red, texture[y * width + x].green, texture[y * width + x].blue));			
		}

	}

	
	char FullName[_MAX_PATH];
	sprintf(FullName, "%s\\%s.tif", TextureDirectory, pFilename);
	strcpy(pFilename, FullName);

	ImageObject.Save(pFilename, BTFILETYPE_TIFF);


	if (texture)
		tsxImageDestroyData(&TextureImg);


	return TRUE;
}


///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////

CString CRibExport::MakeTexture( RtString filename )
{
	CString tiffile, outfile;
	
	char drive[_MAX_PATH];
	char dir[_MAX_PATH];
	char fname[_MAX_PATH];
	char ext[_MAX_PATH];

	_splitpath( filename, drive, dir, fname, ext );

	// first check to see if it's in a format that 3Delight understands,
	// and if not, make it so
	bool canread = false;
	if ( ( strcmpi( ext, ".jpg" ) == 0 ) ||
		( strcmpi( ext, ".jpeg" ) == 0 ) ||
		( strcmpi( ext, ".tif" ) == 0 ) ||
		( strcmpi( ext, ".tiff" ) == 0 ) ||
		( strcmpi( ext, ".iff" ) == 0 ) ||
		( strcmpi( ext, ".sgi" ) == 0 ) ||
		( strcmpi( ext, ".exr" ) == 0 ) ||
		( strcmpi( ext, ".gif" ) == 0 ) ||
		( strcmpi( ext, ".hdr" ) == 0 ) ||
		( strcmpi( ext, ".rad" ) == 0 ) ||
		( strcmpi( ext, ".psd" ) == 0 ) )
	{
		canread = true;
		tiffile = filename;
	}
	
	if ( !canread )
	{
		char tempdir[_MAX_PATH];
		GetTempPath( _MAX_PATH, tempdir );

		tiffile.Format("%s%s.tif", tempdir, fname );
		
		/*ILuint devilimg;
		theApp.InitDevIL( devilimg );
		
		if ( ilLoadImage( filename ) )
		{
			ILint width = ilGetInteger( IL_IMAGE_WIDTH );
			ILint height = ilGetInteger( IL_IMAGE_HEIGHT );
			ILint bpp = ilGetInteger( IL_IMAGE_BITS_PER_PIXEL );
			if ( bpp == 32 )
				ilConvertImage( IL_RGBA, IL_UNSIGNED_BYTE );
			else
				ilConvertImage( IL_RGB, IL_UNSIGNED_BYTE );
			ilEnable( IL_FILE_OVERWRITE );
			ilSave( IL_TIF, (char*)((LPCSTR)tiffile) );
			theApp.ShutdownDevIL( devilimg );
		}
		else
		{
			theApp.ShutdownDevIL( devilimg );
			return _T("");
		}*/
		BTCImageObject ImageObject;
		if ( ImageObject.Load( filename ) )
		{
			int channels = ImageObject.GetObjectData().GetPlanes();
			if ( channels == 4 )
				ImageObject.GetObjectData().ConvertTo32BPP();
			else
				ImageObject.GetObjectData().ConvertTo24BPP();

			ImageObject.Save( tiffile, BTFILETYPE_TIFF );		
		}
		else
			return _T("");
	}

	// new method is to save the Renderman texture to the same dir as the texture
	// but skip if it already exists
	outfile.Format("%s%s%s.tdl", drive, dir, fname );

	if ( !m_cacheOverride )
	{
		if ( FileExists( outfile.GetBuffer() ) )
		{
			MakeUnixPath( &outfile );
			return outfile;
		}
	}

	
	// calling this pops up a window, and we don't want that!
	//RiMakeTexture( (char*)(LPCSTR)tiffile, (char*)(LPCSTR)outfile, RI_PERIODIC, RI_PERIODIC, RiBoxFilter, 1.0f, 1.0f, RI_NULL );

	// instead we'll call tdlmake.exe manually
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;

	char cmd[255];
	sprintf( cmd, "tdlmake -mode periodic \"%s\" \"%s\"", tiffile, outfile );

    // Start the child process. 
    CreateProcess( NULL,   // No module name (use command line). 
        (LPTSTR)cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi );             // Pointer to PROCESS_INFORMATION structure.
    
    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );


	return outfile;

}

///////////////////////////////////////////////////////////////////////////////

CString CRibExport::MakeEnviroTexture( RtString filename, bool probe )
{
	CString tiffile, outfile;
	
	char drive[_MAX_PATH];
	char dir[_MAX_PATH];
	char fname[_MAX_PATH];
	char ext[_MAX_PATH];

	_splitpath( filename, drive, dir, fname, ext );

	// first check to see if it's in a format that AIR understands,
	// and if so, just return the same filename
	bool canread = false;
	if ( ( strcmpi( ext, ".jpg" ) == 0 ) ||
		( strcmpi( ext, ".jpeg" ) == 0 ) ||
		( strcmpi( ext, ".tif" ) == 0 ) ||
		( strcmpi( ext, ".tiff" ) == 0 ) ||
		( strcmpi( ext, ".png" ) == 0 ) ||
		( strcmpi( ext, ".sgi" ) == 0 ) ||
		( strcmpi( ext, ".rgb" ) == 0 ) ||
		( strcmpi( ext, ".hdr" ) == 0 ) ||
		( strcmpi( ext, ".rad" ) == 0 ) ||
		( strcmpi( ext, ".psd" ) == 0 ) )
	{
		canread = true;
		tiffile = filename;
	}

	// TODO: actually try and convert it
	if ( !canread )
	{
		return _T("");
	}

	// new method is to save the Renderman texture to the same dir as the texture
	// but skip if it already exists
	outfile.Format("%s%s%s.tdl", drive, dir, fname );
	if ( !m_cacheOverride )
	{
		if ( FileExists( (char*)((LPCSTR)outfile) ) )
		{
			MakeUnixPath( &outfile );
			return outfile;
		}
	}

	
	// calling this pops up a window, and we don't want that!
	//RiMakeTexture( (char*)(LPCSTR)tiffile, (char*)(LPCSTR)outfile, RI_PERIODIC, RI_PERIODIC, RiBoxFilter, 1.0f, 1.0f, RI_NULL );

	// instead we'll call tdlmake.exe manually
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;

	char cmd[255];
	if ( probe )
		sprintf( cmd, "tdlmake -lightprobe \"%s\" \"%s\"", tiffile, outfile );
	else
		sprintf( cmd, "tdlmake -mode periodic \"%s\" \"%s\"", tiffile, outfile );

    // Start the child process. 
    CreateProcess( NULL,   // No module name (use command line). 
        (LPTSTR)cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi );             // Pointer to PROCESS_INFORMATION structure.
    
    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );


	return outfile;

}

///////////////////////////////////////////////////////////////////////////////

// look for the layered material and set to the first layer if found
bool CRibExport::IsMaterialLayered(tsxMATERIAL *pMaterial )
{
	if ( !pMaterial )
		return false;

	tsxSHINFO* pColorShinfo = tsxMaterialGetShaderInfo( pMaterial, tsxSHCLASS_COLOR );
	tsxSHINFO* pReflShinfo = tsxMaterialGetShaderInfo( pMaterial, tsxSHCLASS_REFLECTANCE );
	tsxSHINFO* pTransShinfo = tsxMaterialGetShaderInfo( pMaterial, tsxSHCLASS_TRANSPARENCY );
	tsxSHINFO* pDispShinfo = tsxMaterialGetShaderInfo( pMaterial, tsxSHCLASS_DISPLACEMENT );
	char ShaderName[255];
	if ( tsxShaderInfoGetShaderName(pColorShinfo, &ShaderName[0]) == e_tsxFAILURE )
		return false;
	if ( tsxShaderInfoGetShaderName(pTransShinfo, &ShaderName[0]) == e_tsxFAILURE )
		return false;
	if ( tsxShaderInfoGetShaderName(pDispShinfo, &ShaderName[0]) == e_tsxFAILURE )
		return false;
	if ( tsxShaderInfoGetShaderName(pReflShinfo, &ShaderName[0]) == e_tsxFAILURE )
		return false;
	
	
	if ( lstrcmpi(ShaderName, "r_layers") == 0 )
	{
			
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue( pColorShinfo, 0 );
		tsxSHINFO* pBaseColorShinfo = tsxShaderInfoCreate();
		tsxShaderInfoSetShader( pBaseColorShinfo, tsxShaderInfoGetShader( tsxParameterValueGetShaderInfo( pVal ) ) );
		tsxShaderInfoCopy( pBaseColorShinfo, tsxParameterValueGetShaderInfo( pVal ) );

		pVal = tsxShaderInfoGetParameterValue( pReflShinfo, 0 );
		tsxSHINFO* pBaseReflShinfo = tsxShaderInfoCreate();
		tsxShaderInfoSetShader( pBaseReflShinfo, tsxShaderInfoGetShader( tsxParameterValueGetShaderInfo( pVal ) ) );
		tsxShaderInfoCopy( pBaseReflShinfo, tsxParameterValueGetShaderInfo( pVal ) );

		pVal = tsxShaderInfoGetParameterValue( pDispShinfo, 0 );
		tsxSHINFO* pBaseDispShinfo = tsxShaderInfoCreate();
		tsxShaderInfoSetShader( pBaseDispShinfo, tsxShaderInfoGetShader( tsxParameterValueGetShaderInfo( pVal ) ) );
		tsxShaderInfoCopy( pBaseDispShinfo, tsxParameterValueGetShaderInfo( pVal ) );
		
		
		
	
		
		tsxMaterialSetShaderInfo( pMaterial, pBaseColorShinfo );
		tsxMaterialSetShaderInfo( pMaterial, pBaseReflShinfo );
		tsxMaterialSetShaderInfo( pMaterial, pBaseDispShinfo );

		tsxShaderInfoDestroy( pBaseColorShinfo );
		tsxShaderInfoDestroy( pBaseReflShinfo );
		tsxShaderInfoDestroy( pBaseDispShinfo );

		return true;
	}
	

	return false;
}

///////////////////////////////////////////////////////////////////////////////

// look for the fur shader
bool CRibExport::IsFurMaterial(tsxMATERIAL *pMaterial )
{
	if ( !pMaterial )
		return false;

	tsxSHINFO* pDisplShinfo = tsxMaterialGetShaderInfo( pMaterial, tsxSHCLASS_DISPLACEMENT );
	char ShaderName[255];
	if ( tsxShaderInfoGetShaderName(pDisplShinfo, &ShaderName[0]) == e_tsxFAILURE )
		return false;
		
	if ( lstrcmpi(ShaderName, "drbSimpleFur") == 0 )
		return true;	

	return false;
}

///////////////////////////////////////////////////////////////////////////////

void CRibExport::GetStdSurfaceParameters( tsxMATERIAL *pMaterial, float& Ka, float& Kd, float& Ks, float& roughness, tssColor& specularcolor, float& Kr, float& Kt, float& ior, float& metal, float& usephong  )
{
	Ka = 0.0f;
	Kd = 1.0f;
	Ks = 0.0f;
	roughness = 0.0f;
	Kr = 0.0f;	
	Kt = 0.0f;
	ior = 1.0f;
	specularcolor[0] = specularcolor[1] = specularcolor[2] = 1.0f;
	const tssColor BLACK = { 0.0f, 0.0f, 0.0f };
	metal = 0.0f;
	usephong = 1.0f;

	// first we're going to work with the reflectance shader
	char ShaderName[255];
	tsxSHINFO *pShinfo = tsxMaterialGetShaderInfo(pMaterial, tsxSHCLASS_REFLECTANCE);
	if (tsxShaderInfoGetShaderName(pShinfo, &ShaderName[0]) != e_tsxFAILURE)
	{
		if ( (lstrcmpi(ShaderName, "caligari metal") == 0) || (lstrcmpi(ShaderName, "c metal enhanced") == 0) )
		{
			metal = 1.0f;
			usephong = 0.0f;

			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
		}
		else
		if (lstrcmpi(ShaderName, "glass") == 0)
		{
			Ka = 0.0f;
			Kd = 0.0f;
			usephong = 0.0f;
			
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		if (lstrcmpi(ShaderName, "dielectric") == 0)
		{
			metal = 1.0f;
			usephong = 0.0f;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);				
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		if (lstrcmpi(ShaderName, "mirror") == 0)
		{
			//metal = 1.0f;
			usephong = 0.0f;

			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
		}
		else
		if (lstrcmpi(ShaderName, "metal") == 0)
		{
			metal = 1.0f;
			usephong = 0.0f;

			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			// metal shaders have the same base color for specular and no diffuse
			Kd = 0.0f;
		}
		else
		if (lstrcmpi(ShaderName, "caligari phong") == 0)
		{
			
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);

			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
							
		}
		else
		if (lstrcmpi(ShaderName, "c phong enhanced") == 0)
		{
			
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			

			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		if (lstrcmpi(ShaderName, "phong") == 0)
		{
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

		}
		else
		if (lstrcmpi(ShaderName, "plastic") == 0)
		{
			usephong = 0.0f;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);

		}
		else
		if (lstrcmpi(ShaderName, "constant") == 0)
		{
			Ka = 1.0f;
			Kd = 1.0f;
			Ks = 0.0f;
		}
		else
		if (lstrcmpi(ShaderName, "matte") == 0)
		{
			Ks = 0.0f;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
		}
		else
		if (lstrcmpi(ShaderName, "translucent plastic") == 0)
		{

			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			roughness = 1.0f / (1.0f - roughness);
			//roughness = 1.0f - (( 1.0f / roughness - 10.0f ) / 100.0f);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
		}
		else
		if (lstrcmpi(ShaderName, "drbSimpleSSS") == 0)
		{
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			roughness = 1.0f / (1.0f - roughness);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		if ( lstrcmpi(ShaderName, "wrapped mirror map") == 0 )
		{
			//metal = 1.0f;
			usephong = 0.0f;
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 1);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 3);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
		}
		else
		if ( lstrcmpi(ShaderName, "c mapped phong") == 0 )
		{
			//metal = 1.0f;
			
			char* pC;
			
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kd);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&specularcolor);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 12);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 14);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		if ( lstrcmpi(ShaderName, "c mapped metal") == 0 )
		{
			metal = 1.0f;
			usephong = 0.0f;

			Kd = 0.0f;
			
			tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 0);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ka);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 2);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Ks);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 4);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&roughness);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 6);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kt);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kr);
			pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
			tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&ior);
		}
		else
		// no recognizable shader, so we use a default
		{
			Kd = 1.0f;
		}
	}
}