#include "stdafx.h"
//#include "dribble.h"
#include "RibExport.h"
#include "wfmmshaders.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// ParseObjects
// Recursively looks through scene, looking for polyhedra to write out
// Inputs: pGNode - pointer to GNode to search from
// Outputs:
// Returns: 
void CRibExport::ParseObjects(tsxGNODE *pGNode, bool FirstPass )
{
	while (tsxIsGNode((tsxSOBJ*)pGNode) == e_tsxTRUE)
	{
		// if there's a child, recurse through this function again
		tsxGNODE* pChild = tsxGNodeGetFirstChild(pGNode);
		if (tsxIsGNode((tsxSOBJ*)pChild) == e_tsxTRUE)
			ParseObjects(pChild, FirstPass);

		// if the object is invisble, we just skip it
		if ( IsVisible( pGNode ) )
		{
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pGNode, ExtDataID);


			// only write if it's geometry
			tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);
			switch (ObjType)
			{
				case e_tsxMBALLOBJ:
				case e_tsxMETANURBS:
				case e_tsxPOLYHEDRON:
				case e_tsxNPATCH_POLYH:
				case e_tsxSUBDIV:
				//case e_tsxNCURVE:
					WriteObject( pGNode, FirstPass );
			}
		} // visible
		
		pGNode = tsxGNodeGetNext(pGNode);
	}
}


///////////////////////////////////////////////////////////////////////////////


// wrapper function, outputs a polyhedron using one of two methods
void CRibExport::OutputPolyhedron(tsxPOLYHEDRON *pPolyh)
{	
	if (IndividualFaces)
		WritePolyhedronAsFaces(pPolyh);
	else
		WritePolyhedronAsMesh(pPolyh);
}


///////////////////////////////////////////////////////////////////////////////



// WritePolyhedronAsFaces
//	writes each face of the polyhedron separately.. takes a lot more space but
//	allows greater control
// Inputs: pInPolyh - pointer to polyhedron to use
// Outputs:
// Returns: 
void CRibExport::WritePolyhedronAsFaces(tsxPOLYHEDRON *pInPolyh, bool IncludeMaterial)
{
	// make a copy of the polyhedron and (optionally) triangulate it
	tsxPOLYHEDRON* pPolyh;
	tsxGNodeCopy((tsxGNODE*) pInPolyh, (tsxGNODE**) &pPolyh);
	//if ( Renderer == Renderer_3Delight )
	//	tsxPolyhInvertFaceNormals( pPolyh );

	// TODO: properly handle holes (there's something wrong) so we can remove this
	// auto-triangulation
	if ( ( m_doTriangulate ) || ( tsxPolyhContainHole( pPolyh ) == e_tsxTRUE ) )
		tsxPolyhTessellate(pPolyh);



	// get some basic polyh info
	int NumPolys = tsxPolyhGetNbrFaces(pPolyh);
	CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pPolyh);
	CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pPolyh);
	CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pPolyh);

	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// write object header
	/*RiAttributeBegin();
		
		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);

		if ( tsxMNodeIsDoubleSidedGeometry( (tsxMNODE*)pPolyh ) == e_tsxTRUE )
			RiSides( 2 );

		WriteTransform((tsxGNODE*) pInPolyh);*/
		
		for (int poly = 0; poly < NumPolys; poly++)
		{
			
			// get the face normal for all "Np" params
			CtsxVector3f PlaneNormal;
			tsxPolyhGetFaceNormal(pPolyh, pFaceArray[poly], &PlaneNormal);
			PlaneNormal.x = -PlaneNormal.x;
			PlaneNormal.y = -PlaneNormal.y;
			PlaneNormal.z = -PlaneNormal.z;



			int NumVxs = pFaceArray[poly]->nbrVxs;
			int NbrUV = tsxPolyhGetNbrUV(pPolyh);
			CtsxUV *pUV = tsxPolyhGetUVArray(pPolyh);
		

			// now look at materials
			// only do so if the object has no custom data
			// uses a material caching system so we don't have to write
			// out the same material multiple times
			tsxMATERIAL* pMaterial;
			tsxMaterialCreate(&pMaterial);
			tsxPolyhGetMaterial(pPolyh, pFaceArray[poly]->materialIdx, pMaterial);
			int Faceting = tsxMaterialGetFacetingType(pMaterial);
	
			CtsxColor BaseColor;
			tsxMaterialGetColor( pMaterial, &BaseColor );
			
			char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pInPolyh, ExtDataID);
			if (pData)
			{
#ifndef PRMANLIB
				RiArchiveRecord(RI_VERBATIM, pData);
#endif
			}
			else
			{
				if ( (poly == 0) || (pFaceArray[poly]->materialIdx != OldMaterialIdx) )
				{
					if ( IncludeMaterial )
						WriteMaterial( (tsxGNODE*)pInPolyh, pMaterial, ObjName, pFaceArray[poly]->materialIdx);
				}
				OldMaterialIdx = pFaceArray[poly]->materialIdx;
			}
		
			// allocate array memory
			int NumHoles = tsxFaceGetNbrHoles(pFaceArray[poly]);
			int* pHoles = (int*)tsxMalloc((NumHoles+1) * sizeof(int));

			// in order to do so, we need to first scan through the face and see what the holes
			// add to the amount
			int AllocSize = NumVxs + 1;
			
			if (NumHoles > 0)
			{
				CtsxHole** pHoleArray = tsxFaceGetHoleptrArray(pFaceArray[poly]);
				for (int hole = 0; hole < NumHoles; hole++)
				{		
					AllocSize += pHoleArray[hole]->nbrVxs;
				}
			}

			CtsxVector3f* pVxs = (CtsxVector3f*)tsxMalloc(AllocSize * sizeof(CtsxVector3f));
			CtsxVector3f* pNormals = (CtsxVector3f*)tsxMalloc(AllocSize * sizeof(CtsxVector3f));
			RtFloat* pU = (RtFloat*)tsxMalloc(NumVxs * sizeof(RtFloat));
			RtFloat* pV = (RtFloat*)tsxMalloc(NumVxs * sizeof(RtFloat));
			RtColor *pCs = (RtColor*)tsxMalloc(NumVxs * sizeof(RtColor));
			

			int vx;
			bool HasVertexColors = false;
			for (vx = 0; vx < NumVxs; vx++)
			{
				pVxs[vx].x = pVertArray[pFaceArray[poly]->pFVxs[vx].vix].x;
				pVxs[vx].y = pVertArray[pFaceArray[poly]->pFVxs[vx].vix].y;
				pVxs[vx].z = pVertArray[pFaceArray[poly]->pFVxs[vx].vix].z;
				CtsxVector3f normal;
				float facetcos = cos((float)tsxMaterialGetAutofacetAngle(pMaterial));
				tsxPolyhGetFaceVertexAutoFacetNormal(pPolyh, pFaceArray[poly], vx, facetcos, &normal);
				pNormals[vx].x = -normal.x;
				pNormals[vx].y = -normal.y;
				pNormals[vx].z = -normal.z;
				pU[vx] = pUV[pFaceArray[poly]->pFVxs[vx].uvix].u;
				pV[vx] = 1.0 - pUV[pFaceArray[poly]->pFVxs[vx].uvix].v;
				
				// vertex colors
				if ( pFaceArray[poly]->pFVxs[vx].color.alpha != 0 )
				{
					pCs[vx][0] = pFaceArray[poly]->pFVxs[vx].color.red / 255.0f;
					pCs[vx][1] = pFaceArray[poly]->pFVxs[vx].color.green / 255.0f;
					pCs[vx][2] = pFaceArray[poly]->pFVxs[vx].color.blue / 255.0f;
				}
				else
				{
					HasVertexColors = true;
					pCs[vx][0] = BaseColor.red / 255.0f;
					pCs[vx][1] = BaseColor.green / 255.0f;
					pCs[vx][2] = BaseColor.blue / 255.0f;
				}
			}
			
			
			if (NumHoles == 0)
			{
				if (Faceting == tsxFACET_FACETED)
				{
					if ( HasVertexColors )
						RiPolygon(NumVxs, "P", (RtPointer)pVxs, "Np", &PlaneNormal, "Cs", (RtPointer)pCs, RI_NULL);
					else
						RiPolygon(NumVxs, "P", (RtPointer)pVxs, "Np", &PlaneNormal, RI_NULL);
				}
				else
				{
					if ( HasVertexColors )
						RiPolygon(NumVxs, "P", (RtPointer)pVxs, "N", (RtPointer)pNormals, "Np", &PlaneNormal, "s", (RtPointer)pU, "t", (RtPointer)pV, "Cs", (RtPointer)pCs, RI_NULL);
					else
						RiPolygon(NumVxs, "P", (RtPointer)pVxs, "N", (RtPointer)pNormals, "Np", &PlaneNormal, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL);
					
				}
			}
			else
			{
				pHoles[0] = NumVxs;
				CtsxHole** pHoleArray = tsxFaceGetHoleptrArray(pFaceArray[poly]);
				for (int hole = 0; hole < NumHoles; hole++)
				{
					
					pHoles[hole+1] = pHoleArray[hole]->nbrVxs;
					for (int hvx = 0; hvx < pHoleArray[hole]->nbrVxs; hvx++)
					{
						TRACE("hole: %d  vx: %d   hole vx index: %d\n", hole, hvx, pHoleArray[hole]->pFVxs[hvx].vix);
						pVxs[vx+hvx].x = pVertArray[pHoleArray[hole]->pFVxs[hvx].vix].x;
						pVxs[vx+hvx].y = pVertArray[pHoleArray[hole]->pFVxs[hvx].vix].y;
						pVxs[vx+hvx].z = pVertArray[pHoleArray[hole]->pFVxs[hvx].vix].z;
						CtsxVector3f normal;
						float facetcos = cos((float)tsxMaterialGetAutofacetAngle(pMaterial));
						tsxPolyhGetHoleVertexAutoFacetNormal(pPolyh, pFaceArray[poly], pHoleArray[hole], hvx, facetcos, &normal);
						pNormals[vx+hvx].x = -normal.x;
						pNormals[vx+hvx].y = -normal.y;
						pNormals[vx+hvx].z = -normal.z;
					}
					vx += pHoleArray[hole]->nbrVxs;
				}

				if (Faceting == tsxFACET_FACETED)
					RiGeneralPolygon(NumHoles + 1, pHoles, "P", (RtPointer)pVxs, "Np", &PlaneNormal, RI_NULL);
				else
					RiGeneralPolygon(NumHoles + 1, pHoles, "P", (RtPointer)pVxs, "N", (RtPointer)pNormals, "Np", &PlaneNormal, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL);
				
			}
			
			tsxFree(pVxs);
			tsxFree(pNormals);
			tsxFree(pHoles);
			tsxFree(pU);
			tsxFree(pV);
			tsxFree(pCs);

			// destroy working material
			tsxMaterialDestroy(pMaterial);
		}


	/*// end of object info
	RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if (Verbose)
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	tsxFree(name);

	// delete copy
	tsxSobjDelete((tsxSOBJ*)pPolyh);

}

// Generates a random point in a triangle with corresponding normal
void RandPoint(CtsxVector3f* Outvec, CtsxVector3f* Outnorm, CtsxUV *OutUV, CtsxVector3f inA, CtsxVector3f inB, CtsxVector3f inC, CtsxVector3f nA, CtsxVector3f nB, CtsxVector3f nC, CtsxUV uvA, CtsxUV uvB, CtsxUV uvC)
{
   float a, b, c;

     a = tssRandom();
     b = tssRandom();

     if ((a+b) > 1)
     {
          a = 1-a;
          b = 1-b;
     }

     c = 1-a-b;

	 
     Outvec->x = (a * inA.x) + (b * inB.x) + (c * inC.x);
     Outvec->y = (a * inA.y) + (b * inB.y) + (c * inC.y);
     Outvec->z = (a * inA.z) + (b * inB.z) + (c * inC.z);

	 Outnorm->x = (a * nA.x) + (b * nB.x) + (c * nC.x);
     Outnorm->y = (a * nA.y) + (b * nB.y) + (c * nC.y);
     Outnorm->z = (a * nA.z) + (b * nB.z) + (c * nC.z);

	 OutUV->u = (a * uvA.u) + (b * uvB.u) + (c * uvC.u);
     OutUV->v = (a * uvA.v) + (b * uvB.v) + (c * uvC.v);
     

	 
}

// WritePolyhedronAsFur
//	grows fur for each face of the polyhedron separately
// Inputs: pInPolyh - pointer to polyhedron to use
// Outputs:
// Returns: 
void CRibExport::WritePolyhedronAsFur(tsxPOLYHEDRON *pInPolyh)
{
	// make a copy of the polyhedron and (optionally) triangulate it
	tsxPOLYHEDRON* pPolyh;
	tsxGNodeCopy((tsxGNODE*) pInPolyh, (tsxGNODE**) &pPolyh);
	//if ( Renderer == Renderer_3Delight )
	//	tsxPolyhInvertFaceNormals( pPolyh );

	tsxPolyhTessellate(pPolyh);



	// get some basic polyh info
	int NumPolys = tsxPolyhGetNbrFaces(pPolyh);
	CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pPolyh);
	CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pPolyh);
	CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pPolyh);

	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// write object header
	/*RiAttributeBegin();
		
		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);

		if ( tsxMNodeIsDoubleSidedGeometry( (tsxMNODE*)pPolyh ) == e_tsxTRUE )
			RiSides( 2 );

		WriteTransform((tsxGNODE*) pInPolyh);*/

		CtsxTxmx3f ObjTxmx, InvObjTxmx;
		tsxGNodeGetPureTransform( (tsxGNODE*)pPolyh, &ObjTxmx);
		tsxInvertMx3f( InvObjTxmx, ObjTxmx );

		RtFloat width[2];
		int NumStrands = 10;
		float BaseLength = 0.1f;
		float LengthVar = 0.0f;
		int PointsPerStrand = 4;
		float Stiffness = 0.0f;
		float Kink = 0.5f;
		tssColor RootColor, TipColor;
		float TipFade = 0.001f;
		
		int matidx = tsxPolyhGetFirstMaterialIdx( pPolyh );
		tsxMATERIAL *pFurMaterial = tsxPolyhGetIndexedMaterial( pPolyh, matidx );

		CtsxColor Color;
		RtColor BaseColor;
		tsxMaterialFixColor( pFurMaterial );
		tsxMaterialFixTransparency( pFurMaterial );
		tsxMaterialGetColor( pFurMaterial, &Color );
		tSColorToRtColor( BaseColor, Color);
		RiColor( BaseColor );

		tsxSHINFO *pShinfo = tsxMaterialGetShaderInfo( pFurMaterial, tsxSHCLASS_DISPLACEMENT );

		float Density;
		tsxSHPARAMVAL* pVal = tsxShaderInfoGetParameterValue(pShinfo, 5);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Density);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 7);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&BaseLength);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 8);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&LengthVar);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 15);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Stiffness);
		Stiffness = 1.0f - (Stiffness / 100.0f);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 19);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&Kink);
		Kink = (Kink / 100.0f);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 9);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&width[0]);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 10);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&width[1]);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 12);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&RootColor);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 13);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_COLOR, (void*)&TipColor);

		pVal = tsxShaderInfoGetParameterValue(pShinfo, 22);
		tsxParameterValueGetTypedValue(pVal, tsxSHPARAM_FLOAT, (void*)&TipFade);

		NumStrands = (int)Density;

		CtsxVector2f TxrUVReps, TxrUVOffset;
		TxrUVReps.x = TxrUVReps.y = 1.0f;
		TxrUVOffset.x = TxrUVOffset.y = 0.0f;

		RtString TxrName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
		RtString lengthMapName = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
		sprintf(TxrName, "%s%d", ObjName.GetBuffer(_MAX_PATH), matidx);
		bool painted = IsMaterialPainted(pFurMaterial, TxrName);
		if ( painted )
		{
			strcpy( lengthMapName, TxrName );
			tsxSHINFO *pColShinfo = tsxMaterialGetShaderInfo(pFurMaterial, tsxSHCLASS_COLOR);
			GetUVInfo( pColShinfo, TxrUVReps, TxrUVOffset );   
			CString newtexname = MakeTexture( TxrName );
			strcpy( TxrName, newtexname );
		}
		else
		{
			strcpy( TxrName, "" );
			strcpy( lengthMapName, "" );
		}

		float Ka = 0.0f;
		float Kd = 1.0f;
		float Ks = 0.0f;
		float roughness = 0.0f;
		float Kr = 0.0f;	
		float Kt = 0.0f;
		float ior = 1.0f;
		tssColor specularcolor = {1.0f, 1.0f, 1.0f};
		RtFloat metal = 0.0f;
		RtFloat usephong = 1.0f;
		GetStdSurfaceParameters( pFurMaterial, Ka, Kd, Ks, roughness, specularcolor, Kr, Kt, ior, metal, usephong  );
		/*RiSurface( "tShair_solid", 
			"float Ka", &Ka, "float Kd", &Kd, "float Ks", &Ks, "float roughness", &roughness, "color specularcolor", &specularcolor, "float usephong", &usephong, "float metal", &metal,
			"color rootcolor", &RootColor, "color tipcolor", &TipColor, "float tipfade", &TipFade, 
			"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
			RI_NULL );*/
		RiSurface( "tSfur", 
			"float Ka", &Ka, "float Kd", &Kd, "float Ks", &Ks, "float roughness", &roughness, "color specularcolor", &specularcolor,
			"color rootcolor", &RootColor, "color tipcolor", &TipColor, "float tipfade", &TipFade, 
			"string texmap", &TxrName, "float texureps", &TxrUVReps.x, "float texvreps", &TxrUVReps.y, "float texuoffset", &TxrUVOffset.x, "float texvoffset", &TxrUVOffset.y, 
			RI_NULL );
		tsxFree( TxrName );

		tsxImage lengthMap;
		lengthMap.data = NULL;
		if ( painted )
			tsxImageLoadData( &lengthMap, lengthMapName );

		tsxFree( lengthMapName );

		int NumCurves = NumPolys * NumStrands;
		int NumPoints = NumCurves * PointsPerStrand;
		CtsxVector3f* pCntlPoints = (CtsxVector3f*)tsxMalloc(NumPoints * sizeof(CtsxVector3f));

		RtFloat *pU = (RtFloat*)tsxMalloc( NumCurves * sizeof(RtFloat) );
		RtFloat *pV = (RtFloat*)tsxMalloc( NumCurves * sizeof(RtFloat) );
		CtsxVector3f* pSurfNormals = (CtsxVector3f*)tsxMalloc(NumCurves * sizeof(CtsxVector3f));
		RtFloat *pLengths = (RtFloat*)tsxMalloc( NumCurves * sizeof(RtFloat) );
		RtInt *pVertsPerCurve = (RtInt*)tsxMalloc(NumCurves * sizeof(RtInt));
		RtFloat *pWidths = (RtFloat*)tsxMalloc(NumCurves * 2 * sizeof(RtFloat));
		int curvepos = 0;
		for ( int i = 0; i < NumCurves; i++ )
		{
			pWidths[curvepos] = width[0];
			pWidths[curvepos+1] = width[1];
			curvepos += 2;
		}

		int CurveCount = 0;
		int PointCount = 0;
		int UVCount = 0;
		
		CtsxVector3f Gravity;
		Gravity.x = 0.0f;
		Gravity.y = 0.0f;
		Gravity.z = -1.0f;

		tsxRotateVecs3f(1, &Gravity, &Gravity, InvObjTxmx);
		tsxNormalize3f( Gravity );

		CtsxVector3f GravVec;

		for (int poly = 0; poly < NumPolys; poly++)
		{
			// get the face normal for all "Np" params
			CtsxVector3f PlaneNormal;
			tsxPolyhGetFaceNormal(pPolyh, pFaceArray[poly], &PlaneNormal);
			PlaneNormal.x = -PlaneNormal.x;
			PlaneNormal.y = -PlaneNormal.y;
			PlaneNormal.z = -PlaneNormal.z;



			int NumVxs = pFaceArray[poly]->nbrVxs;
			int NbrUV = tsxPolyhGetNbrUV(pPolyh);
			CtsxUV *pUV = tsxPolyhGetUVArray(pPolyh);
		

	
			

			for ( int j = 0; j < NumStrands; j++ )
			{
				

				CtsxVector3f coords1 = pVertArray[pFaceArray[poly]->pFVxs[0].vix];
				CtsxVector3f coords2 = pVertArray[pFaceArray[poly]->pFVxs[1].vix];
				CtsxVector3f coords3 = pVertArray[pFaceArray[poly]->pFVxs[2].vix];
				

				CtsxVector3f* pNormArray = tsxPolyhGetVxNormalsArray( pPolyh );
				CtsxVector3f norm1 = pNormArray[pFaceArray[poly]->pFVxs[0].vix];
				CtsxVector3f norm2 = pNormArray[pFaceArray[poly]->pFVxs[1].vix];
				CtsxVector3f norm3 = pNormArray[pFaceArray[poly]->pFVxs[2].vix];
				
				if ( tsxMaterialGetFacetingType(pFurMaterial) == tsxFACET_FACETED ) 
				{
					norm1 = PlaneNormal;
					norm2 = PlaneNormal;
					norm3 = PlaneNormal;
				}

				if ( tsxMaterialGetFacetingType(pFurMaterial) == tsxFACET_AUTO ) 
				{
					float facetcos = cos( (float)tsxMaterialGetAutofacetAngle(pFurMaterial) );
					tsxPolyhGetFaceVertexAutoFacetNormal( pPolyh, pFaceArray[poly], 0, facetcos, &norm1 );
					tsxPolyhGetFaceVertexAutoFacetNormal( pPolyh, pFaceArray[poly], 1, facetcos, &norm2 );
					tsxPolyhGetFaceVertexAutoFacetNormal( pPolyh, pFaceArray[poly], 2, facetcos, &norm3 );
				}
				

				CtsxUV uv1 = pUV[pFaceArray[poly]->pFVxs[0].uvix];
				CtsxUV uv2 = pUV[pFaceArray[poly]->pFVxs[1].uvix];
				CtsxUV uv3 = pUV[pFaceArray[poly]->pFVxs[2].uvix];
				

				CtsxVector3f RealPos, Normal;
				CtsxUV UV;
				RandPoint(&RealPos, &Normal, &UV, coords1,coords2, coords3, norm1, norm2, norm3, uv1, uv2, uv3);
				//tsxTransformVecs3f(1, &RealPos, &RealPos, ObjTxmx);
				//tsxRotateVecs3f(1, &Normal, &Normal, ObjTxmx);
				
				//float tmp;
				//tmp = RealPos.z;
				//RealPos.z = RealPos.y;
				//RealPos.y = tmp;
				
				// invert interpolated normal, tS has it backwards
				Normal.x = -Normal.x;
				Normal.y = -Normal.y;
				Normal.z = -Normal.z;
				
				UV.v = 1.0f - UV.v;

				float Length = BaseLength;
				if ( LengthVar > 0.0f )
					Length -= tssRandom() * (LengthVar * 0.01f);
				//if ( (numpts % 2) != 0 )
				//	numpts--;

				if ( lengthMap.data != NULL )
				{
					int x;
					int y;

					x = (int)(UV.u * (float)lengthMap.width);
					y = (int)(UV.v * (float)lengthMap.height);

					CtsxColor pixel = GetUnfilteredPixel( x, y, lengthMap );

					Length *= ((float)pixel.alpha / 255.0f);
				}

				
				pCntlPoints[PointCount] = RealPos;
				pU[UVCount] = UV.u;
				pV[UVCount] = UV.v;
				
				PointCount++;
				UVCount++;

				MixNormal( GravVec, Normal, Gravity, 0.25f * Stiffness );
				if ( Kink > 0)
				{
					GravVec.x += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.y += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.z += (tssRandom() - 0.5f) * 2.0f * Kink;
				}
				tsxNormalize3f( GravVec );
				pCntlPoints[PointCount].x = RealPos.x + (GravVec.x * Length);
				pCntlPoints[PointCount].y = RealPos.y + (GravVec.y * Length);
				pCntlPoints[PointCount].z = RealPos.z + (GravVec.z * Length);
				//pU[PointCount] = UV.u;
				//pV[PointCount] = UV.v;
				PointCount++;

				MixNormal( GravVec, Normal, Gravity, 0.50f * Stiffness );
				if ( Kink > 0)
				{
					GravVec.x += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.y += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.z += (tssRandom() - 0.5f) * 2.0f * Kink;
				}
				tsxNormalize3f( GravVec );
				pCntlPoints[PointCount].x = pCntlPoints[PointCount-1].x + (GravVec.x * Length);
				pCntlPoints[PointCount].y = pCntlPoints[PointCount-1].y + (GravVec.y * Length);
				pCntlPoints[PointCount].z = pCntlPoints[PointCount-1].z + (GravVec.z * Length);
				//pU[PointCount] = UV.u;
				//pV[PointCount] = UV.v;
				PointCount++;

				MixNormal( GravVec, Normal, Gravity, 0.75f * Stiffness );				
				if ( Kink > 0)
				{
					GravVec.x += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.y += (tssRandom() - 0.5f) * 2.0f * Kink;
					GravVec.z += (tssRandom() - 0.5f) * 2.0f * Kink;
				}
				tsxNormalize3f( GravVec );
				pCntlPoints[PointCount].x = pCntlPoints[PointCount-1].x + (GravVec.x * Length);
				pCntlPoints[PointCount].y = pCntlPoints[PointCount-1].y + (GravVec.y * Length);
				pCntlPoints[PointCount].z = pCntlPoints[PointCount-1].z + (GravVec.z * Length);
				//pU[UVCount] = UV.u;
				//pV[UVCount] = UV.v;
				//UVCount++;
				PointCount++;
				
				pSurfNormals[CurveCount] = Normal;
				pLengths[CurveCount] = Length;
				pVertsPerCurve[CurveCount] = PointsPerStrand;
				
				



				
				
				CurveCount++;
			}

		}


		// render the curves

		RiCurves(RI_CUBIC, NumCurves, pVertsPerCurve, RI_NONPERIODIC, "width", pWidths, "P", pCntlPoints, "uniform float s", pU, "uniform float t", pV, "uniform normal surface_normal", pSurfNormals, "uniform float hair_length", pLengths, RI_NULL);

		tsxFree(pCntlPoints);
		tsxFree(pVertsPerCurve);
		tsxFree( pWidths );
		tsxFree( pU );
		tsxFree( pV );
		tsxFree( pSurfNormals );
		tsxFree( pLengths );


	// end of object info
	/*RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if (Verbose)
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	tsxFree(name);

	// delete copy
	tsxSobjDelete((tsxSOBJ*)pPolyh);

	tsxImageDestroyData( &lengthMap );
}


// WritePolyhedronAsMesh
//	writes polyhedron as PontsPolygon
// Inputs: pInPolyh - pointer to polyhedron to use
// Outputs:
// Returns: 
// TODO: Break into separate materials
void CRibExport::WritePolyhedronAsMesh(tsxPOLYHEDRON *pInPolyh)
{
	// make a copy of the polyhedron and (optionally) triangulate it
	tsxPOLYHEDRON* pPolyh;
	tsxGNodeCopy((tsxGNODE*) pInPolyh, (tsxGNODE**) &pPolyh);
	//if ( Renderer == Renderer_3Delight )
	//	tsxPolyhInvertFaceNormals( pPolyh );

	if ( m_doTriangulate )
		tsxPolyhTessellate(pPolyh);



	// get some basic polyh info
	int NumPolys = tsxPolyhGetNbrFaces(pPolyh);
	CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pPolyh);
	CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pPolyh);
	CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pPolyh);
	CtsxUV* pUV = tsxPolyhGetUVArray(pPolyh);

	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// write object header
	/*RiAttributeBegin();

		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);

		if ( tsxMNodeIsDoubleSidedGeometry( (tsxMNODE*)pPolyh ) == e_tsxTRUE )
			RiSides( 2 );

		
		WriteTransform((tsxGNODE*) pInPolyh);*/
		
		// now look at materials
		// only do so if the object has no custom data
		// uses a material caching system so we don't have to write
		// out the same material multiple times
		tsxMATERIAL* pMaterial;
		tsxMaterialCreate(&pMaterial);
		int MaterialIdx = tsxPolyhGetFirstMaterialIdx(pPolyh);
		tsxPolyhGetMaterial(pPolyh, MaterialIdx, pMaterial);
		int Faceting = tsxMaterialGetFacetingType(pMaterial);

		
		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pInPolyh, ExtDataID);
		if (pData)
		{
#ifndef PRMANLIB
			RiArchiveRecord(RI_VERBATIM, pData);
#endif
		}
		else
		{
			WriteMaterial( (tsxGNODE*)pInPolyh, pMaterial, ObjName, MaterialIdx);
		}


		RtInt *pVertsPerFace = (RtInt*)tsxMalloc(NumPolys * sizeof(RtInt));
		int vxcount = 0;
		for (int poly = 0; poly < NumPolys; poly++)
		{
			pVertsPerFace[poly] = pFaceArray[poly]->nbrVxs;
			vxcount += pVertsPerFace[poly];
		}

		RtInt *pVerts = (RtInt*)tsxMalloc(vxcount * sizeof(RtInt));
		RtFloat* pU = (RtFloat*)tsxMalloc(vxcount * sizeof(RtFloat));
		RtFloat* pV = (RtFloat*)tsxMalloc(vxcount * sizeof(RtFloat));
		CtsxVector3f* pNormals = (CtsxVector3f*)tsxMalloc(vxcount * sizeof(CtsxVector3f));
		CtsxVector3f* pOutPoints = (CtsxVector3f*)tsxMalloc(vxcount * sizeof(CtsxVector3f));
		
		CMapStringToString hashtable;

		// fill in the points and normals
		vxcount = 0;			
		for (int poly = 0; poly < NumPolys; poly++)
		{
			/*if ((poly % 250) == 0)
			{
				char msg[255];
				sprintf(msg, "processing %s, face %d of %d..", name, poly, NumPolys);
				tsxSetHelpMessage(msg);
			}*/

			for (int vx = 0; vx < pVertsPerFace[poly]; vx++)
			{
				CtsxVector3f normal;
				float facetcos = cos((float)tsxMaterialGetAutofacetAngle(pMaterial));
				tsxPolyhGetFaceVertexAutoFacetNormal(pPolyh, pFaceArray[poly], vx, facetcos, &normal);
				
				int tsindex = pFaceArray[poly]->pFVxs[vx].vix;
				
				pOutPoints[vxcount].x = pVertArray[tsindex].x;
				pOutPoints[vxcount].y = pVertArray[tsindex].y;
				pOutPoints[vxcount].z = pVertArray[tsindex].z;
				pNormals[vxcount].x = -normal.x;
				pNormals[vxcount].y = -normal.y;
				pNormals[vxcount].z = -normal.z;
				pU[vxcount] = pUV[pFaceArray[poly]->pFVxs[vx].uvix].u;
				pV[vxcount] = 1.0 - pUV[pFaceArray[poly]->pFVxs[vx].uvix].v;

				CString strkey;
				strkey.Format("[%f, %f, %f]  [%f, %f, %f]", pVertArray[tsindex].x, pVertArray[tsindex].y, pVertArray[tsindex].z, normal.x, normal.y, normal.z);
				CString strvalue;
				strvalue.Format("%d", vxcount);
				hashtable[strkey] = strvalue;
				vxcount++;
			}
			
		}

		int oldvxcount = vxcount;

		// now go back and fill in the face-vx indices
		vxcount = 0;			
		for (int poly = 0; poly < NumPolys; poly++)
		{
			/*if ((poly % 250) == 0)
			{
				char msg[255];
				sprintf(msg, "calculating normals for %s, face %d of %d..", name, poly, NumPolys);
				tsxSetHelpMessage(msg);
			}*/

			for (int vx = 0; vx < pVertsPerFace[poly]; vx++)
			{
				CtsxVector3f normal;
				float facetcos = cos((float)tsxMaterialGetAutofacetAngle(pMaterial));
				tsxPolyhGetFaceVertexAutoFacetNormal(pPolyh, pFaceArray[poly], vx, facetcos, &normal);
				
				int tsindex = pFaceArray[poly]->pFVxs[vx].vix;
				/*CtsxVector3f point;
				point.x = pVertArray[tsindex].x;
				point.y = pVertArray[tsindex].y;
				point.z = pVertArray[tsindex].z;
				int riindex = -1;
				for (int n=0; n < oldvxcount; n++)
				{
					if ( (pNormals[n].x == normal.x) && (pNormals[n].y == normal.y)  && (pNormals[n].z == normal.z) && (pOutPoints[n].x == point.x) && (pOutPoints[n].y == point.y)  && (pOutPoints[n].z == point.z))
					{
						riindex = n;
						n = oldvxcount + 1;
					}
				}*/
				CString strkey;
				strkey.Format("[%f, %f, %f]  [%f, %f, %f]", pVertArray[tsindex].x, pVertArray[tsindex].y, pVertArray[tsindex].z, normal.x, normal.y, normal.z);
				CString strvalue;
				hashtable.Lookup(strkey, strvalue);
				int riindex = atoi(strvalue);

				
				pVerts[vxcount] = riindex;

				vxcount++;
			}
			
		}

	
		RiPointsPolygons(NumPolys, pVertsPerFace, pVerts, "P", (RtPointer)pOutPoints, "N", (RtPointer)pNormals, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL); 
		tsxFree(pOutPoints);
		tsxFree(pNormals);
		tsxFree(pVerts);
		tsxFree(pVertsPerFace);
		tsxFree(pU);
		tsxFree(pV);

		// destroy working material
		tsxMaterialDestroy(pMaterial);


	/*// end of object info
	RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if (Verbose)
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	tsxFree(name);

	// delete copy
	tsxSobjDelete((tsxSOBJ*)pPolyh);
}


///////////////////////////////////////////////////////////////////////////////


void CRibExport::WritePatch(tsxNPATCH *pPatch, tsxPOLYHEDRON *pPolyh)
{
	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	/*// write object header
	RiAttributeBegin();

		RiOrientation( "lh" );

		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);

		if ( tsxMNodeIsDoubleSidedGeometry( (tsxMNODE*)pPolyh ) == e_tsxTRUE )
			RiSides( 2 );

		
		WriteTransform((tsxGNODE*)pPolyh);*/
			
		// now look at materials
		int NbrMaterials = tsxPolyhGetNbrMaterials(pPolyh);
		if ( !Binary )
			RiArchiveRecord(RI_COMMENT, "Materials: %d", NbrMaterials);

		tsxMATERIAL* pMaterial;
		tsxMaterialCreate(&pMaterial);
		tsxPolyhGetMaterial(pPolyh, 0, pMaterial);
		int Faceting = tsxMaterialGetFacetingType(pMaterial);
		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
		if (pData)
		{
#ifndef PRMANLIB
			RiArchiveRecord(RI_VERBATIM, pData);
#endif
		}
		else
			WriteMaterial( (tsxGNODE*)pPolyh, pMaterial, ObjName, 0);
		
		// destroy working material
		tsxMaterialDestroy(pMaterial);
	
		RtInt nu = tsxNPatchGetNumUCntlPts(pPatch);
		RtInt uorder = tsxNPatchGetUOrder(pPatch);
		int numuknots = tsxNPatchGetNumUKnots(pPatch);
		//RtFloat pUKnots = (RtFloat*)tsxMalloc((numuknots) * sizeof(RtFloat));
		RtFloat* pUKnots = tsxNPatchGetUKnots(pPatch);
		RtFloat umin = 0.0f;
		RtFloat umax = 1.0f;
		
		RtInt nv = tsxNPatchGetNumVCntlPts(pPatch);
		RtInt vorder = tsxNPatchGetVOrder(pPatch);
		int numvknots = tsxNPatchGetNumVKnots(pPatch);
		//RtFloat pVKnots = (RtFloat*)tsxMalloc((numvknots) * sizeof(RtFloat));
		RtFloat* pVKnots = tsxNPatchGetVKnots(pPatch);
		RtFloat vmin = 0.0f;
		RtFloat vmax = 1.0f;


		CtsxVector3f* pVxs = tsxNPatchGetCntlPts(pPatch);
		

		//RiNuPatch(nu, uorder, pUKnots, umin, umax, nv, vorder, pVKnots, vmin, vmax, "P", (RtPointer)pVxs, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL);		
		RiNuPatch(nu, uorder, pUKnots, umin, umax, nv, vorder, pVKnots, vmin, vmax, "P", (RtPointer)pVxs, RI_NULL);		

	// end of object info
	/*RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if ( Verbose )
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	if (name)
		tsxFree(name);
}


///////////////////////////////////////////////////////////////////////////////


void CRibExport::WriteSDS(tsxMNODE *pSDS, tsxPOLYHEDRON *pPolyh)
{
	// air does not support subdivision surfaces,
	// so we write them out as polyhedral
	/*if (Renderer == Renderer_AIR)
	{
		tsxPOLYHEDRON* pPolyh;
		tsxGNodeCopy((tsxGNODE*) pSDS, (tsxGNODE**) &pPolyh);
		tsxSubDivExtractFinalMesh((tsxMNODE*)pPolyh, e_tsxFALSE, e_tsxFALSE, e_tsxFALSE);
		OutputPolyhedron(pPolyh);
		tsxSobjDelete((tsxSOBJ*)pPolyh);
		return;
	}*/
	

	// get the object name
	CString ObjName = GetName((tsxGNODE*) pPolyh);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// write object header
	/*RiAttributeBegin();

		RiOrientation( "rh" );

		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);
		
		WriteTransform((tsxGNODE*)pSDS);*/

	
		// now look at materials
		int NbrMaterials = tsxPolyhGetNbrMaterials(pPolyh);
		if ( !Binary )
			RiArchiveRecord(RI_COMMENT, "Materials: %d", NbrMaterials);

		tsxMATERIAL* pMaterial;
		tsxMaterialCreate(&pMaterial);
		tsxPolyhGetMaterial(pPolyh, 0, pMaterial);
		int Faceting = tsxMaterialGetFacetingType(pMaterial);

		CtsxColor BaseColor;
		tsxMaterialGetColor( pMaterial, &BaseColor );

		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pPolyh, ExtDataID);
		if (pData)
		{
#ifndef PRMANLIB
			RiArchiveRecord(RI_VERBATIM, pData);
#endif
		}
		else
			WriteMaterial( (tsxGNODE*)pPolyh, pMaterial, ObjName, 0);
		
		// destroy working material
		tsxMaterialDestroy(pMaterial);
	
		tsxPOLYHEDRON *pCage = tsxSubDivGetCagePtr(pSDS);

		// get some basic polyh info
		int NumPolys = tsxPolyhGetNbrFaces(pCage);
		CtsxFace** pFaceArray = tsxPolyhGetFaceptrArray(pCage);
		CtsxVector3f* pVertArray = tsxPolyhGetVxArray(pCage);
		CtsxVector3f* pNormalArray = tsxPolyhGetVxNormalsArray(pCage);
		CtsxUV* pUV = tsxPolyhGetUVArray(pCage);
		RtInt NumVxs = tsxPolyhGetNbrVxs(pCage);
		

		RtInt *pVertsPerFace = (RtInt*)tsxMalloc(NumPolys * sizeof(RtInt));
		int vxcount = 0;
		for (int poly = 0; poly < NumPolys; poly++)
		{
			pVertsPerFace[poly] = pFaceArray[poly]->nbrVxs;
			vxcount += pVertsPerFace[poly];
		}

		RtInt *pVerts = (RtInt*)tsxMalloc(vxcount * sizeof(RtInt));
		RtFloat* pU = (RtFloat*)tsxMalloc(vxcount * sizeof(RtFloat));
		RtFloat* pV = (RtFloat*)tsxMalloc(vxcount * sizeof(RtFloat));
		RtColor *pCs = (RtColor*)tsxMalloc(vxcount * sizeof(RtColor));
			
			
				

		vxcount = 0;
		bool HasVertexColors = false;
		for (int poly = 0; poly < NumPolys; poly++)
		{
			for (int vx = 0; vx < pVertsPerFace[poly]; vx++)
			{
				pVerts[vxcount] = pFaceArray[poly]->pFVxs[vx].vix;			
				pU[vxcount] = pUV[pFaceArray[poly]->pFVxs[vx].uvix].u;
				pV[vxcount] = 1.0 - pUV[pFaceArray[poly]->pFVxs[vx].uvix].v;

				// vertex colors
				if ( pFaceArray[poly]->pFVxs[vx].color.alpha != 0 )
				{
					pCs[vxcount][0] = pFaceArray[poly]->pFVxs[vx].color.red / 255.0f;
					pCs[vxcount][1] = pFaceArray[poly]->pFVxs[vx].color.green / 255.0f;
					pCs[vxcount][2] = pFaceArray[poly]->pFVxs[vx].color.blue / 255.0f;
				}
				else
				{
					HasVertexColors = true;
					pCs[vxcount][0] = BaseColor.red / 255.0f;
					pCs[vxcount][1] = BaseColor.green / 255.0f;
					pCs[vxcount][2] = BaseColor.blue / 255.0f;
				}
				vxcount++;
			}
			
		}

		int NumEdges = tsxSubDivGetEdges(pSDS, NULL);

		
		RtInt NumTags = NumEdges;


		RtToken *pTags = NULL;
		RtInt *pNargs = NULL;
		RtInt *pIntargs = NULL;
		RtFloat *pFloatargs = NULL;

		if ( NumTags > 0 )
		{
			pTags = (RtToken*)tsxMalloc(NumTags * sizeof(RtToken));
			pNargs = (RtInt*)tsxMalloc(NumTags * 2 * sizeof(RtInt));
			pIntargs = (RtInt*)tsxMalloc(NumTags * 2 * sizeof(RtInt));
			pFloatargs = (RtFloat*)tsxMalloc(NumTags * sizeof(RtFloat));
		}


		CtsxSubDivEdge *pEdges = (CtsxSubDivEdge*)tsxMalloc(NumEdges * sizeof(CtsxSubDivEdge));
		tsxSubDivGetEdges(pSDS, pEdges);
		int nargcount=0;
		int floatargcount=0;
		int intargcount=0;
		for (int edge=0; edge < NumEdges; edge++)
		{
			pTags[edge] = "crease";
			pNargs[nargcount] = 2;
			pNargs[nargcount+1] = 1;
			pIntargs[intargcount] = pEdges[edge].iStartVertex;
			pIntargs[intargcount+1] = pEdges[edge].iEndVertex;
			
			if ( (pEdges[edge].Infinity == e_tsxTRUE) || (pEdges[edge].Sharpness >= 10) )
				pFloatargs[floatargcount] = RI_INFINITY;
			else
				pFloatargs[floatargcount] = pEdges[edge].Sharpness;
			nargcount += 2;
			intargcount += 2;
			floatargcount++;
		}

		/*
		// this is the original for linking with BMRT's libribout
		if (NumEdges == 0)
			RiSubdivisionMesh("catmull-clark", NumPolys, pVertsPerFace, pVerts, 0, RI_NULL, RI_NULL, RI_NULL, RI_NULL, "P", (RtPointer)pVertArray, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL);
		else
		{
			RiSubdivisionMesh("catmull-clark", NumPolys, pVertsPerFace, pVerts, NumTags, pTags, pNargs, pIntargs, pFloatargs, "P", (RtPointer)pVertArray, "s", (RtPointer)pU, "t", (RtPointer)pV, RI_NULL);
		}
		*/
		if ( HasVertexColors )
			RiSubdivisionMesh("catmull-clark", NumPolys, pVertsPerFace, pVerts, NumTags, pTags, pNargs, pIntargs, pFloatargs, "P", (RtPointer)pVertArray, "facevertex float s", (RtPointer)pU, "facevertex float t", (RtPointer)pV, "facevertex color Cs", (RtPointer)pCs, RI_NULL);
		else
			RiSubdivisionMesh("catmull-clark", NumPolys, pVertsPerFace, pVerts, NumTags, pTags, pNargs, pIntargs, pFloatargs, "P", (RtPointer)pVertArray, "facevertex float s", (RtPointer)pU, "facevertex float t", (RtPointer)pV, RI_NULL);

		tsxFree(pVertsPerFace);
		tsxFree(pVerts);
		tsxFree(pU);
		tsxFree(pV);
		tsxFree(pEdges);
		tsxFree(pCs);

		if ( NumTags > 0 )
		{
			tsxFree(pTags);
			tsxFree(pNargs);
			tsxFree(pIntargs);
			tsxFree(pFloatargs);
		}
	// end of object info
	/*RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if (Verbose)
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	tsxFree(name);
}


// writes out a tsxNCURVE as a RiCurves
// if the input curve has an odd number of points, the last point will be discarded
// to make a cubic spline
void CRibExport::WriteNURBSCurve(tsxNCURVE *pCurve)
{
	// only prman and air support RiCurves
	if (Renderer == Renderer_BMRT)
		return;


	// get the object name
	CString ObjName = GetName((tsxGNODE*) pCurve);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));

	// write object header
	/*RiAttributeBegin();

		if ( !Binary )
			RiAttribute("identifier", "name", &name, RI_NULL);
		
		WriteTransform((tsxGNODE*)pCurve);*/
		
		// now look at materials
		//int NbrMaterials = tsxPolyhGetNbrMaterials((tsxPOLYHEDRON*)pCurve);
		//RiArchiveRecord(RI_COMMENT, "Materials: %d", NbrMaterials);

		//tsxMATERIAL* pMaterial;
		//tsxMaterialCreate(&pMaterial);
		//tsxPolyhGetMaterial((tsxPOLYHEDRON*)pCurve, 0, pMaterial);
		//int Faceting = tsxMaterialGetFacetingType(pMaterial);
		char *pData = (char*)tsxSobjGetData((tsxSOBJ*)pCurve, ExtDataID);
		if (pData)
		{
#ifndef PRMANLIB
			RiArchiveRecord(RI_VERBATIM, pData);
#endif
		}
		else
			//WriteMaterial(pMaterial, ObjName, 0);
			RiSurface("constant", RI_NULL);
		
		// destroy working material
		//tsxMaterialDestroy(pMaterial);
	

		RtInt numcurves = 1;
		RtInt numpts = tsxNCurveGetNumCntlPts(pCurve);
		CtsxVector3f* pVxs = tsxNCurveGetCntlPts(pCurve);
		

		RtInt *pVertsPerCurve = (RtInt*)tsxMalloc(numcurves * sizeof(RtInt));
		
		if ( (numpts % 2) != 0 )
			numpts--;

		CtsxVector3f* pCntlPoints = (CtsxVector3f*)tsxMalloc(numpts * sizeof(CtsxVector3f));
		for (int pt=0; pt < numpts; pt++)
			pCntlPoints[pt] = pVxs[pt];

		pVertsPerCurve[0] = numpts;
		
		RtFloat width = 0.01f;

		// check whether curve is open or closed, and create the right one
		if (tsxNCurveGetClosed(pCurve) == e_tsxTRUE)
			RiCurves(RI_CUBIC, numcurves, pVertsPerCurve, RI_PERIODIC, RI_CONSTANTWIDTH, &width, "P", pCntlPoints, RI_NULL);
		else
			RiCurves(RI_CUBIC, numcurves, pVertsPerCurve, RI_NONPERIODIC, RI_CONSTANTWIDTH, &width, "P", pCntlPoints, RI_NULL);


		tsxFree(pVertsPerCurve);
		tsxFree(pCntlPoints);

	// end of object info
	/*RiAttributeEnd();
	
	// add a couple of comment lines to break up the RIB
	if (Verbose)
	{
		RiArchiveRecord(RI_COMMENT, "-");
		RiArchiveRecord(RI_COMMENT, "------------ end of %s ------------", name);
		RiArchiveRecord(RI_COMMENT, "-");
	}*/

	tsxFree(name);
}


///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////

// writes the geometry of an object only
void CRibExport::WriteGeometry( tsxGNODE *pGNode, bool IncludeMaterial )
{
	tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);
	switch (ObjType)
	{
		case e_tsxMBALLOBJ:
		case e_tsxMETANURBS:
		{
			// If it's a metaobject, we need to get the rendering polyh
			tsxPOLYHEDRON* pMetanode;
			pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
			if (pMetanode)
			{
				if ( IsFurObject( pMetanode ) )
					WritePolyhedronAsFur( pMetanode );
				else
					OutputPolyhedron( pMetanode );
			}
		}
		break;
		case e_tsxPOLYHEDRON:
		{
			if ( IsFurObject( (tsxPOLYHEDRON*) pGNode ) )
				WritePolyhedronAsFur( (tsxPOLYHEDRON*)pGNode );
			else
				OutputPolyhedron((tsxPOLYHEDRON*) pGNode);
		}
		break;
		case e_tsxNPATCH_POLYH:
		{
			//WritePatch(tsxNPatchPolyhGetNPatch((tsxPOLYHEDRON*)pGNode), (tsxPOLYHEDRON*)pGNode);
			// If it's a metaobject, we need to get the rendering polyh
			tsxPOLYHEDRON* pMetanode;
			pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
			if (pMetanode)
			{
				if ( IsFurObject( pMetanode ) )
					WritePolyhedronAsFur( pMetanode );
				else
					WritePatch(tsxNPatchPolyhGetNPatch((tsxPOLYHEDRON*)pGNode), (tsxPOLYHEDRON*)pGNode);
			}
		}
		break;
		case e_tsxSUBDIV:
		{
			//WriteSDS((tsxMNODE*)pGNode, (tsxPOLYHEDRON*)pGNode);
			tsxPOLYHEDRON* pMetanode;
			pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
			if (pMetanode)
			{
				if ( IsFurObject( pMetanode ) )
					WritePolyhedronAsFur( pMetanode );
				else
					WriteSDS((tsxMNODE*)pGNode, (tsxPOLYHEDRON*)pGNode);
			}

		}
		break;
		case e_tsxNCURVE:
		{
			WriteNURBSCurve((tsxNCURVE*)pGNode);
		}
		break;


	}
}

///////////////////////////////////////////////////////////////////////////////

void CRibExport::WriteObject( tsxGNODE *pGNode, bool FirstPass )
{
	tsxSOBJTYPE ObjType = tsxSobjGetType((tsxSOBJ*) pGNode);

	// bail if it's a metaball subobject
	if ( ObjType == e_tsxPOLYHEDRON )
	{
		tsxGNODE* pParent = tsxGNodeGetRoot( pGNode );
		if ( tsxSobjGetType( (tsxSOBJ*)pParent ) == e_tsxMBALLOBJ )
			return;
	}

	// bail if it's an emissive light
	tsxLIGHT *pLight = GetEmissiveLight( (tsxPOLYHEDRON*)pGNode );
	if ( pLight )
		return;

	CString ObjName = GetName( pGNode );
	int namelength = ObjName.GetLength();
	if ( namelength == 0 )
	{
		MessageBox( NULL, "Object with blank name?", "Output error", MB_OK );
	}
	else
	{
		//ObjName = MakeNameFilename( GetName( pGNode ) );	
		//ObjName.Format( "%s_%d", ObjName, objID );
		ObjName.Format( "%s_%d", MakeNameFilename( GetName( pGNode ) ), objID );
	}
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));


	if ( FirstPass )
	{
		RiArchiveBegin( name, RI_NULL );

			WriteGeometry( pGNode );

		RiArchiveEnd();
	}
	else
	{
		
		switch (ObjType)
		{
			case e_tsxMBALLOBJ:
			case e_tsxMETANURBS:
			{
				// If it's a metaobject, we need to get the rendering polyh
				tsxPOLYHEDRON* pMetanode;
				pMetanode = tsxMNodeGetPolyhedron((tsxMNODE*)pGNode); 
				if (pMetanode)
				{
					RiOrientation( "rh" );
				}
			}
			break;
			case e_tsxPOLYHEDRON:
			{
				RiOrientation( "rh" );
			}
			break;
			case e_tsxNPATCH_POLYH:
			{
				RiOrientation( "lh" );
			}
			break;
			case e_tsxSUBDIV:
			{
				RiOrientation( "rh" );
			}
			break;
			case e_tsxNCURVE:
			{
				RiOrientation( "lh" );
			}
			break;


		}
			

		RiAttributeBegin();
		
			if ( !Binary )
				RiAttribute("identifier", "name", &name, RI_NULL);

			if ( tsxMNodeIsDoubleSidedGeometry( (tsxMNODE*)pGNode ) == e_tsxTRUE )
				RiSides( 2 );

			//if ( !IsFurObject( (tsxPOLYHEDRON*)pGNode ) )
				WriteTransform( pGNode );

			RiReadArchive( name, NULL, RI_NULL );

		RiAttributeEnd();
	}

	objID++;

	tsxFree( name );
}

///////////////////////////////////////////////////////////////////////////////

bool CRibExport::IsFurObject(tsxPOLYHEDRON *pPolyh)
{
	if ( tsxPolyhGetNbrMaterials( pPolyh ) < 1 )
		return false;

	tsxMATERIAL *pMaterial = tsxPolyhGetIndexedMaterial( pPolyh, tsxPolyhGetFirstMaterialIdx( pPolyh ) );
	if ( !pMaterial )
		return false;

	return IsFurMaterial( pMaterial );
}