// RibExport.cpp: implementation of the CRibExport class.
// class for exporting tS scenes to RIB format
// (c) 2003 Simon Windmill, windmill fraser multimedia, inc.

//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "dribble.h"
#include "RibExport.h"
#include "wfmmshaders.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CFile Batchfile;
bool WriteBatch = FALSE;

//#define PRMANLIB 1

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRibExport::CRibExport()
{
	// init some member variables
	Filename = "output.rib";
	ImageName = "output.tga";

	RIBDirectory = "d:\\tmp\\dribble";

	TextureDirectory = "d:\\tmp\\dribble";

	ShadmapDirectory = "d:\\tmp\\dribble";

	tsxAnimGetPlayRanges(&FirstFrame, &LastFrame);
	
	Sides = 1;

	ShadingRate = 1.0f;
	PixelSamplesX = 2;
	PixelSamplesY = 2;
	PixelFilterWidthX = 1;
	PixelFilterWidthY = 1;


	m_skySamples = 32;

	RadiositySteps = 50;
	RadiosityMinPatchSamples = 2;

	OldMaterialIdx = -1;

	Verbose = TRUE;
	m_doTriangulate = true;

	m_bUseIrradiancePhotons = true;
	m_bUseGI = false;
	m_bHighQualityGI = false;
	m_bUseGIOcclusion = false;
    m_fGISamples = 64;
    m_fGIMaxError = 1.0f;
    m_fGIShadingRate = 4.0f;
    m_fGIIntensity = 1.0f;
    m_iPhotons = 100000;

	m_bUseCaustics = FALSE;
	SaveSeedFile = FALSE;
	UseSeedFile = FALSE;
	SeedFile = "";
	SeedBlur = 0.005;

	Busy = FALSE;
	DeclareParams = TRUE;

	LightNum = 0;
	
	ConvertTextureShaders = FALSE;
	ConvertProceduralShaders = TRUE;
	TexWidth = 256;
	TexHeight = 256;

	Renderer = Renderer_Prman;

	PatchPoly = FALSE;
	IndividualFaces = FALSE;

	AIR_RaytraceShadows = TRUE;
	AIR_RaytraceReflections = TRUE;

	JustShadowmaps = FALSE;

	LightMultiplier = 1.0f;
	ShadowBias = 0.1f;

	PixFilter = PixFilter_Box;

	FormatWidth = 640;
	FormatHeight = 480;
	FormatAspect = 1.0;

	UseDOF = true;
	DOF_FocalLength = 2.0;
	DOF_FStop = 4.0;

	m_alwaysRayShadows = false;

	m_useRaytraceHider = false;

	m_useMotionBlur = false;

	m_blurMultiplier = 4;
}

CRibExport::~CRibExport()
{
}


///////////////////////////////////////////////////////////////////////////////
// Utility functions

// tSColorToRtColor
// Converts tS-style color to Renderman-style color
// Inputs: CtsxColor incolor - tS-style color
// Outputs: RtColor result - Renderman-style color
// Returns: 
void CRibExport::tSColorToRtColor(RtColor result, CtsxColor incolor)
{
	result[0] = incolor.red / 255.0f;
	result[1] = incolor.green / 255.0f;
	result[2] = incolor.blue / 255.0f;
}



// tSOpacityToRtColor
// Converts tS-style opacity to Renderman-style opacity
// Inputs: CtsxColor incolor - tS-style color
// Outputs: RtColor result - Renderman-style color
// Returns: 
void CRibExport::tSOpacityToRtColor(RtColor result, CtsxColor incolor)
{
	result[0] = incolor.alpha / 255.0f;
	result[1] = incolor.alpha / 255.0f;
	result[2] = incolor.alpha / 255.0f;
}



// tSmxToRtmx
// converts tS-style matrices to Renderman-style matrices
// Inputs: CtsxTxmx3f inmx - tS-style matrix
// Outputs: RtMatrix result - Renderman-style matrix
// Returns: 
void CRibExport::tSmxToRtmx(RtMatrix result, CtsxTxmx3f inmx)
{
	result[0][0] = inmx.matrix[0][0];
    result[0][1] = inmx.matrix[2][0];
    result[0][2] = inmx.matrix[1][0];
    result[0][3] = 0;
    result[1][0] = inmx.matrix[0][1];
    result[1][1] = inmx.matrix[2][1];
    result[1][2] = inmx.matrix[1][1];
    result[1][3] = 0;
    result[2][0] = inmx.matrix[0][2];
    result[2][1] = inmx.matrix[2][2];
    result[2][2] = inmx.matrix[1][2];
    result[2][3] = 0;
    result[3][0] = inmx.matrix[0][3];
    result[3][1] = inmx.matrix[2][3];
    result[3][2] = inmx.matrix[1][3];
    result[3][3] = 1;
}



// AimZ
// rotate the world so the directionvector points in
// positive z by rotating about the y axis, then x. The cosine
// of each rotation is given by components of the normalized
// direction vector. Before the y rotation the direction vector
// might be in negative z, but not afterward.
// from Renderman Companion
// Inputs: CtsxVector direction - direction to look at
// Outputs:
// Returns:
void CRibExport::AimZ(CtsxVector3f direction)
{
	double xzlen, yzlen, xrot, yrot;

	if ( (direction.x == 0) && (direction.y == 0) && (direction.z == 0) )
		return;

	xzlen = sqrt(direction.x * direction.x + direction.y * direction.y);
	if (xzlen == 0)
		if (direction.z < 0)
			yrot = 180;
		else
			//yrot = 0;
			yrot = (direction.z < 0) ? 180 : 0;
	else
		yrot = 180*acos(direction.y / xzlen) / M_PI;

	yzlen = sqrt(direction.z * direction.z + xzlen * xzlen);
	xrot = 180 * acos(xzlen/yzlen) / M_PI;

	if (direction.z > 0)
		RiRotate(xrot, 1, 0, 0);
	else
		RiRotate(-xrot, 1, 0, 0);

	if (direction.x > 0)
		RiRotate(-yrot, 0, 1, 0);
	else
		RiRotate(yrot, 0, 1, 0);
}

// PlaceCamera
// establish a viewpoint, viewing direction and orientation
// for a scene. This routine must be called before RiWorldBegin().
// position: a point giving the camera position
// direction: a point giving the camera direction relative to position
// roll: an optional rotation of the camera about its direction axis
// from the Renderman Companion
// Inputs: CtsxVector3f position - location of camera
//			CtsxVector3f direction - direction of camera
//			float roll - camera banking
// Outputs:
// Returns:
void CRibExport::PlaceCamera(CtsxVector3f position, CtsxVector3f direction, float roll, bool perspective)
{
	// first do the projection
	float fov;

	if (!JustShadowmaps)
	{
		float zoom = tsxAViewGetZoom();
		float factor = 1.0f;
		float ratio = 1.0f;
		RiArchiveRecord(RI_COMMENT, "Camera info: %d, %d, %f, %f", FormatWidth, FormatHeight, zoom, (float)max(FormatWidth,FormatHeight)/(float)min(FormatWidth,FormatHeight) );
		
		if ( FormatHeight > FormatWidth )
		{
			ratio = (float)FormatWidth / (float)FormatHeight;
		}
		if ( perspective )
		{
			factor = 2.5f;
			factor *= zoom;
		}
		else
		{
			factor = 0.5f;
			
			factor *= (1.0f / zoom);
		}

		
			
		if ( perspective )
		{
			factor *= (float)max(FormatWidth,FormatHeight)/(float)min(FormatWidth,FormatHeight);
			fov = tsxRad2Deg( (float)atan( 1.0f / factor ) ) * 2.0f;
		
			RiProjection(RI_PERSPECTIVE, "fov", &fov, RI_NULL);	
		}
		else
		{
			factor *= 0.01f;
			
			RiProjection( RI_ORTHOGRAPHIC, RI_NULL );
			float normwidth = ((float)FormatWidth) * factor;
			float normheight = ((float)FormatHeight) * factor;
			
			normwidth *= 0.5f;
			normheight *= 0.5f;
			RiScreenWindow(-normwidth, normwidth, -normheight, normheight);
		}
	}

	RiFrameAspectRatio(  (float)FormatWidth / (float)FormatHeight );
	
	
	// now position
	RiIdentity();
	RiRotate(-roll, 0, 0, 1);
	AimZ(direction);
	RiTranslate(-position.x, -position.z, -position.y);
	// TESTING
	RiRotate( 90, 0, 1, 0 );

	// for DOF
	if (!JustShadowmaps)
	{
		if (UseDOF)
		{
			CtsxVector3f camto;
			RtFloat FocusDistance;
			tsxGNODE *pCam = (tsxGNODE*)tsxAViewGetCameraObj();

			FocusDistance = DOF_FocusDistance;

			if (tsxIsGNode((tsxSOBJ*)pCam))
			{
				tsxGNODE *pTarget;
				if (tsxGNodeGetLookAt(pCam, &pTarget) == e_tsxTRUE)
				{
					tsxGNodeGetLocation(pTarget, &camto);		
					FocusDistance = tsxDistanceToPoint3f( camto, position );		
				}
				else
				{
					CtsxTxmx3f matrix;
					CtsxVector3f v1;
					tsxGNodeGetPureTransform( pCam, &matrix );
					v1.x = v1.y = 0.f;
					v1.z = 1.f;
					tsxTransformVecs3f( 1, &camto, &v1, matrix );
				}
			}
			else
			{
				camto.x = 0.0f;
				camto.y = 0.0f;
				camto.z = 0.0f;
			}

			
			RiDepthOfField( DOF_FStop, DOF_FocalLength, FocusDistance );
		}
	}
}


// MakeNameFilename
// make an object name a "good" filename by replacing objectionable
// chars with an underscore
// Inputs: CString *name - pointer to object name
// Outputs: CString *name - pointer to name
// Returns: 
CString CRibExport::MakeNameFilename(const CString name)
{
	CString str( name );
	str.MakeLower();
	str.Replace(" ", "_");
	str.Replace(",", "_");
	return str;
}


// MakeUnixPath
// converts an NT path to a UNIX-style path
// path - pointer to path
void CRibExport::MakeUnixPath(CString *path)
{
	path->Replace("\\", "/");
}



// GetName
//	gets the name of a GNode
// Inputs: tsxGNODE *pGNode - pointer to gnode
// Outputs:
// Returns: name of gnode, as CString
CString CRibExport::GetName(tsxGNODE *pGNode)
{
	CString str;
	char* pBuffer;
	pBuffer = (char*)tsxCalloc(255, sizeof(char));
	if (tsxGNodeGetName(pGNode, pBuffer, 255) != 0)
	{
		str = pBuffer;
	}
	tsxFree(pBuffer);
	return str;
}

	///////////////////////////////////////////////////////////////////////////////


char * strnset (char * string, int val, size_t count)
{
	char *start = string;

    while (count-- && *string)
    *string++ = (char)val;

    return(start);
}

// ExportScene
// The main public function, this exports each frame to RIB
// Inputs: 
// Outputs: 
// Returns: 
void CRibExport::ExportScene( bool tofile )
{
	if (Busy)
		return;

	Busy = TRUE;

	WriteRIB = tofile;
	

	// let's just do some cleanup work here - automatically clearing any settings
	// that are incompatible with a particular renderer, and set some settings that are
	// dependent on others
	switch (Renderer)
	{
		case Renderer_AIR:
			m_bUseCaustics = FALSE;
			if ((m_bUseGI) && (!UseSeedFile))
			{
				AIR_RaytraceShadows = TRUE;
				AIR_RaytraceReflections = TRUE;
			}
			break;
		case Renderer_Prman:
			m_bUseGI = false;
			m_bUseCaustics = false;
			break;
	}
	if ( m_bUseIrradiancePhotons )
	{
		if ( !m_bUseGI )
			m_bUseIrradiancePhotons = false;
	}

	if ( m_bUseGI )
	{
		if ( !m_doRaytrace )
		{
			m_doRaytrace = true;
			m_rayDepth = 2;
		}
	}

	DeclareParams = FALSE;

	if ( Binary )
		Verbose = FALSE;

	// chop off '.rib' from Filename
	BaseRIBFilename = Filename.Left(Filename.GetLength() - 4);
	CString fname;
	
	if (FirstFrame == LastFrame)
	{
		fname = RIBDirectory + "\\" + BaseRIBFilename + ".rib";
		//ExportFrame(fname, FirstFrame);
	}

	{

		WriteBatch = false;

		CString batchname = RIBDirectory + "\\" + BaseRIBFilename + ".bat";
				
		CFileException e;
		bool canwrite = true;
		if ( WriteBatch )
			canwrite = ( Batchfile.Open( batchname, CFile::modeCreate | CFile::modeWrite, &e ) );

		if ( canwrite )
		{

			// loop for each frame - we're dumping out separate RIBs instead of one big one
			for (float frame = FirstFrame; frame <= LastFrame; frame++)
			{
				if ( FirstFrame == LastFrame )
					fname.Format("%s\\%s.rib", RIBDirectory, BaseRIBFilename );
				else
					fname.Format("%s\\%s%04d.rib", RIBDirectory, BaseRIBFilename, (int)frame);
				
				ExportFrame(fname, (int)frame);

				if ( WriteBatch )
				{
					// write out line to our batchfile
					CString batchline;

					switch (Renderer)
					{
						case Renderer_3Delight:
							batchline.Format( "renderdl %s\n", fname );
							break;

						case Renderer_AIR:
							batchline.Format( "air %s\n", fname );
							break;

						case Renderer_Aqsis:
							batchline.Format( "aqsis %s\n", fname );
							break;
						
						case Renderer_BMRT:
							batchline.Format( "rendrib %s\n", fname );
							break;

						case Renderer_Pixie:
							batchline.Format( "rndr %s\n", fname );
							break;

						case Renderer_Prman:
							batchline.Format("prman %s\n", fname );
							break;
					}
					
					
					Batchfile.Write(batchline, batchline.GetLength());

				}
				
			} // frame loop

			// close batch file			
			if ( WriteBatch )
				Batchfile.Close();

		} // batch file
	}

	Busy = FALSE;
}





// callback to update status bar with 3Delight render progress
void RenderProgress( float i_progress )
{
	char msg[255];
	sprintf( msg, "Rendering with dribble (%d%%)", (RtInt)i_progress );
	tsxSetHelpMessage( msg );
}

// ExportFrame
//	writes out an entire frame to RIB
// Inputs: char *fname - filename, full thing expected
//			int frame - frame number
// Outputs:
// Returns: 
void CRibExport::ExportFrame(CString fname, int frame)
{
	// set seed
	srand( 6395 );

	tsxSetHelpMessage( "Preparing to render scene with dribble.." );
	
	tsxAnimSetActiveTime(frame);
	
	LightNum = 0;

	// first we look for any shadow maps we may need to generate
	if ( !m_alwaysRayShadows )
	{
		JustShadowmaps = true;
		GoLights(tsxSceneGetFirstNode());
	}
	

	JustShadowmaps = false;
	LightNum = 0;

	char *ri2rib_ascii = "Ascii";
	char *ri2rib_binary = "Binary";
	RiOption ( "RI2RIB_Output", (RtToken) "Type", Binary ? (RtPointer) &ri2rib_binary : (RtPointer) &ri2rib_ascii, RI_NULL );
	
	RtString ribname = (RtString)tsxMalloc(fname.GetLength() * sizeof(RtString));
	strcpy(ribname, fname);

	if ( WriteRIB )
		RiBegin(ribname);
	else
		RiBegin( RI_NULL );

	if ( WriteRIB )
		if ( !Binary )
		{
			RiArchiveRecord(RI_COMMENT, "Scene %s", fname);
			RiArchiveRecord(RI_COMMENT, "Creator dribble: trueSpace->RIB plugin");
			
			//char Date[255];
			//_strdate(Date);
			//RiArchiveRecord(RI_COMMENT, "CreationDate %s", Date);
			
			char User[255];
			DWORD size = 255;
			GetUserName(User, &size);
			RiArchiveRecord(RI_COMMENT, "For %s", User);
			
			RiArchiveRecord(RI_COMMENT, "Frames %d", LastFrame-FirstFrame);
		}


	// dump out the geometry archives
	objID = 0;
	ParseObjects( tsxSceneGetFirstNode(), true );

		// Write frame header
		//RiFrameBegin((RtInt)(frame+1));

			// write the options
			RtInt bucketsize[] = {32, 32}; 
			RtInt gridsize = (RtInt)( ((float)bucketsize[0] * (float)bucketsize[1]) / ShadingRate );
			RiOption( "limits", "integer bucketsize[2]", &bucketsize, RI_NULL );
			RiOption( "limits", "integer gridsize", &gridsize, RI_NULL );

			RtString bucketorder = "circle";
			RiOption( "render", "string bucketorder", &bucketorder, RI_NULL );

			RiOption( "trace", "integer maxdepth", &m_rayDepth, RI_NULL );
			/*if ( m_doRaytraceDisplacements )
			{
				RtInt on = 1;
				RiAttribute( "trace", "integer displacements", &on, RI_NULL );
			}*/


			RtPointer progresscb = &RenderProgress;
			RiOption( "statistics", "progresscallback", &progresscb, RI_NULL );

			char display[255];			
			/*if (FirstFrame == LastFrame)
				sprintf(display, "%s.tif", ImageName);
			else
				sprintf(display, "%s%04d.tif", ImageName, (int)frame);*/
			sprintf( display, "%s%s.tif", RIBDirectory, ImageName );
			


			// TESTING toon
			RtString edgevars = "z";
			RtFloat edgethreshold = 0.1;
			//RiDeclare( "outline", "color" );
			//RiQuantize( "outline", 255, 0, 255, 0.5 );

			// layered output
			RiDeclare( "__ambient", "color" );
			RiDeclare( "__diffuse", "color" );
			RiDeclare( "__specular", "color" );
			RiDeclare( "__reflect", "color" );
			RiDeclare( "__refract", "color" );

			RiQuantize( "__ambient", 255, 0, 255, 0.5 );
			RiQuantize( "__diffuse", 255, 0, 255, 0.5 );
			RiQuantize( "__specular", 255, 0, 255, 0.5 );
			RiQuantize( "__reflect", 255, 0, 255, 0.5 );
			RiQuantize( "__refract", 255, 0, 255, 0.5 );
				
			if ( WriteRIB )
				RiDisplay(display, RI_FILE, RI_RGB, RI_NULL);
				//RiDisplay( display, RI_FILE, "outline", "string edgevariables", &edgevars, "float edgethreshold", &edgethreshold, RI_NULL );
			else
			{
				HWND hwnd = tsxAViewGetHwnd();
				CRect rectWin;
				::GetClientRect( hwnd, &rectWin );
				//FormatWidth = rectWin.Width();
				//FormatHeight = rectWin.Height();
				long DPWidth, DPHeight;
				tsxAViewGetDrawportSize( &DPWidth, &DPHeight );
				
				HWND rndwnd = tsxAViewGetHwnd();
				RtString mainclassname = new char[255];
				RtString renclassname = new char[255];
				GetClassName( hwnd, &mainclassname[0], 255 );
				GetClassName( rndwnd, &renclassname[0], 255 );
				int toscreen;
				/*if ( ( pSize ) && ( !pSubRect ) )
					toscreen = 0;
				else*/
					toscreen = 1;
				
				RtInt cropped = 0;
				RtInt cropx1 = 0;
				RtInt cropy1 = 0;

				if ( !m_SubRect.IsRectEmpty() )
				{
					// calculate cropping as percentage, and limit it
					// (else Renderman ignores the cropping)
					RtFloat x1 = (RtFloat)m_SubRect.left / (RtFloat)FormatWidth;
					RtFloat y1 = (RtFloat)m_SubRect.top / (RtFloat)FormatHeight;
					RtFloat x2 = (RtFloat)m_SubRect.right / (RtFloat)FormatWidth;
					RtFloat y2 = (RtFloat)m_SubRect.bottom / (RtFloat)FormatHeight;

					if ( x1 < 0 )
						x1 = 0;
					if ( y1 < 0 )
						y1 = 0;
					if ( x2 > 1 )
						x2 = 1;
					if ( y2 > 1 )
						y2 = 1;
					RiCropWindow( x1, x2, y1, y2 );

					cropped = 1;
					cropx1 = m_SubRect.left;
					cropy1 = m_SubRect.top;
				}


				
				RiDisplay( display, "tsdd", "rgba", "string mainclassname", &mainclassname, "string renclassname", &renclassname, "integer toscreen", &toscreen, "integer cropped", &cropped, "integer cropx1", &cropx1, "integer cropy1", &cropy1, "integer renhwnd", &rndwnd, "integer mainhwnd", &hwnd, "integer windowwidth", &DPWidth, "integer windowheight", &DPHeight, RI_NULL );				
				
				// TESTING toon								
				//RiDisplay( display, "tsdd", "outline", "string edgevariables", &edgevars, "float edgethreshold", &edgethreshold, "string mainclassname", &mainclassname, "string renclassname", &renclassname, "integer toscreen", &toscreen, "integer cropped", &cropped, "integer cropx1", &cropx1, "integer cropy1", &cropy1, "integer renhwnd", &rndwnd, "integer mainhwnd", &hwnd, "integer windowwidth", &DPWidth, "integer windowheight", &DPHeight, RI_NULL );

				// TESTING PSD
				
				if ( PSDOutput )
				{
					char psdfile[255];
					//sprintf( psdfile, "+%s%s.psd", OutputDirectory, ImageName );
					sprintf( psdfile, "+%s", RawImageName );
					RiDisplay( psdfile, "psd", "__ambient,__diffuse,__specular,__reflect,__refract", RI_NULL );				
				}


				
				

				delete []mainclassname;
				delete []renclassname;
			}

			if (SaveSeedFile)
			{
				RtString pSeed = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
				strcpy(pSeed, SeedFile);
				RiDisplay(pSeed, RI_FILE, "+indirect", RI_NULL);
				tsxFree(pSeed);
			}
			
			
			RiFormat(FormatWidth, FormatHeight, FormatAspect);

			
			
			RiShadingRate(ShadingRate);
			RtInt samplesx = PixelSamplesX;
			RtInt samplesy = PixelSamplesY;
			if ( UseDOF || m_useMotionBlur )
			{
				samplesx *= m_blurMultiplier;
				samplesy *= m_blurMultiplier;;
			}
			RiPixelSamples(samplesx, samplesy);

			switch (PixFilter)
			{
				case PixFilter_Box:
					RiPixelFilter(RiBoxFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
				case PixFilter_Triangle:
					RiPixelFilter(RiTriangleFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
				case PixFilter_CatmullRom:
					RiPixelFilter(RiCatmullRomFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
				case PixFilter_Gaussian:
					RiPixelFilter(RiGaussianFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
				case PixFilter_Sinc:
					RiPixelFilter(RiSincFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
#ifndef _3DELIGHT
				case PixFilter_Disk:
					RiPixelFilter(RiDiskFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
#endif
				case PixFilter_Bessel:
					RiPixelFilter(RiBesselFilter, PixelFilterWidthX, PixelFilterWidthY);
					break;
				default:
					RiPixelFilter(RiBoxFilter, PixelFilterWidthX, PixelFilterWidthY);
			}

			// Write camera position
			CtsxVector3f posn, dirn;
			float roll;
			tsxAViewGetPosition(&posn);
			tsxAViewGetDirection(&dirn);
			roll = tsxRad2Deg(tsxAViewGetBankAngle());
			tsxVIEWMODE mode = tsxAViewGetViewMode();
			bool perspective = ( ( mode == e_tsxVM_PERSPECTIVE ) || ( mode == e_tsxVM_CAMERA ) );
			PlaceCamera(posn, dirn, roll, perspective);
			
			

			// for motion blur
			RiShutter(0.0f, 1.0f);
			RtInt flag = 0;

			if ( m_useRaytraceHider )
				RiHider( "raytrace", "jitter", (RtPointer)&flag, RI_NULL );
			else
				RiHider( "hidden", "jitter", (RtPointer)&flag, RI_NULL );

			if ( m_bUseGI )
			{
                switch ( Renderer )
                {
                case Renderer_3Delight:
                    break;
                }
				
                /*if (m_bUseCaustics)
				{
					int nphotons = 50000;
					RiDeclare("nphotons", "integer");
					RiAttribute("light", "nphotons", &nphotons, RI_NULL);
				}*/
			}

			/*int truedisp = 1;
			RiAttribute("render", "integer truedisplacement", &truedisp, RI_NULL);

			// using this globally in 3DL causes significant slowdown
			RtString coordsys = (RtString)tsxMalloc(255 * sizeof(char));
			strcpy(coordsys, "world");
			RiAttribute("displacementbound", "sphere", &DisplacementBound, "coordinatesystem", &coordsys, RI_NULL);
			tsxFree(coordsys);*/
						
			
			/*if ( m_bRaytraceMotionBlur )
			{
				RtInt on = 1;
				RiAttribute( "trace", "integer samplemotion", &on, RI_NULL );
			}*/

			
			// create imager shaders
			WriteBackground();

			// handle foreground shaders, including grabbing enviro map
			WriteForeground();



			// and now rip through the scene looking for textures
			//GoSceneTextures(tsxSceneGetFirstNode());


            if ( m_bUseGI || m_bUseCaustics )
			{
                switch ( Renderer )
                {
                case Renderer_3Delight:
                    {
						if ( m_bUseIrradiancePhotons || m_bUseCaustics )
						{

							RtInt photons = m_iPhotons * 1000;
							RiOption( "photon", "emit", &photons, RI_NULL );
							RtString globmap = "global.ph";
							RiAttribute( "photon", "globalmap", &globmap, RI_NULL );
				
							RtInt vis = 1;
							RiAttribute( "visibility", "integer photon", &vis, RI_NULL );
						}

						if ( m_bUseGI )
						{
							RtInt nsamples = (RtInt)m_fGISamples;
							RiAttribute( "irradiance", "integer nsamples", &nsamples, RI_NULL );
							RiAttribute( "irradiance", "maxerror", &m_fGIMaxError, RI_NULL );
							RiAttribute( "irradiance", "shadingrate", &m_fGIShadingRate, RI_NULL );

							RtFloat finalgather = 1.0f;
							RtFloat useocclusion = 0.0f;
							if ( m_bUseGIOcclusion )
								useocclusion = 1.0f;
							RiLightSource( "tSindirectlight", "float samples", &m_fGISamples, "float intensity", &m_fGIIntensity, "float finalgather", &finalgather, "float useocclusion", &useocclusion, RI_NULL );
						}

                        if ( m_bUseCaustics )
                        {
       			            RtString causticmap = "caustic.ph";
    				        RiAttribute( "photon", "causticmap", &causticmap, RI_NULL );

                            RiLightSource( "tScausticlight", "float samples", &m_fGISamples, "float intensity", &m_fGIIntensity, RI_NULL );
                        }
                    }
                    break;
                }
				/*if (UseSeedFile)
				{
					RiDeclare("mapname", "string");
					RiDeclare("blur", "float");
					RtString pSeed = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
					strcpy(pSeed, SeedFile);
					RiLightSource("indirect", "mapname", &pSeed, "blur", &SeedBlur, RI_NULL);						
					tsxFree(pSeed);						
				}
				else
					RiLightSource("indirect", RI_NULL);

				if (m_bUseCaustics)
					RiLightSource("caustic", RI_NULL);*/
			}

			// Write world header
			RiWorldBegin();

				RiOrientation("lh");

				RiSides( Sides );
				
				// atmosphere shaders
				//WriteVolumetrics();

				// now we do the lights
				if ( Verbose )
				{
					RiArchiveRecord(RI_COMMENT, "+");
					RiArchiveRecord(RI_COMMENT, "+++++++++++++++ lights +++++++++++++++");
					RiArchiveRecord(RI_COMMENT, "+");
				}
				
				// always include an ambient light
				RiLightSource( "tSambientlight", RI_NULL );
				
				// do the environment
				/*// TODO: testing
				m_doEnvironment = true;
				m_environmentName = "d:\\lightprobes\\hdr\\kitchen_probe.hdr";
				m_doEnvironmentLight = true;
				m_environmentLightIntensity = 0.1f;
				m_environmentLightSamples = 64;
				m_environmentLightShadows = true;
				m_environmentLightExposure = 0.0f;
				m_environmentSize = 100.0f;
				m_environmentBlur = 0.0f;
				m_environmentExposure = 0.0f;*/
				m_doEnvironmentLight = (m_environmentLightIntensity > 0.0f );

				
				if ( ( m_doEnvironmentLight ) && ( m_environmentName != "" ) )
				{
					RtString TextureName = new char[_MAX_PATH];
					strcpy( TextureName, "" );
						
					CString str( m_environmentName );
					str.Replace( _T("_cross"), _T("_probe") );
					CString src( str );
					
					CString strTextureName;
					
					strTextureName = MakeEnviroTexture( (RtString)((LPCSTR)src), true );
					strcpy( TextureName, (LPCSTR)strTextureName );


					RtFloat intensity = m_environmentLightIntensity;
					RtFloat exposure = m_environmentLightExposure;
					RtFloat samples = m_environmentLightSamples;

					RtFloat Kocc = m_environmentLightShadows ? 1.0f : 0.0f;

					RtString envspace = "world";
					RiLightSource( "tSenvlight", "float intensity", &intensity, "float exposure_compensation_stops", &exposure, "float Kocc", &Kocc, "string envmap", &TextureName, "float samples", &samples, "string envspace", &envspace, RI_NULL );
					
					delete []TextureName;
				
				}
				// TESTING
				RiRotate( -90, 0, 1, 0 );
					
				// and now rip through the scene writing out lights
				GoLights(tsxSceneGetFirstNode());			

				if ( ( m_environmentName != "" ) && ( ( m_environmentShowBG ) || ( m_environmentShowReflections ) ) )
				{
					//WriteEnviroCube();
					WriteEnviroSphere();
				}

				if (Verbose)
				{
					RiArchiveRecord(RI_COMMENT, "+");
					RiArchiveRecord(RI_COMMENT, "+++++++++++ end of lights ++++++++++++");
					RiArchiveRecord(RI_COMMENT, "+");
				}
				


				// now do the geometry
				objID = 0;
				ParseObjects(tsxSceneGetFirstNode(), false );

			RiWorldEnd();

		//RiFrameEnd();
	

	// end rib
	RiEnd();

	if (ribname)
		tsxFree(ribname);

	// make sure status bar is clear
	tsxSetHelpMessage(NULL);
	
}


// WriteVolumetrics
//	checks to see if the tS Volumetrics shader is active,
//	if so, converts values to Renderman "noisysmoke" atmospheric
//	shader and writes it out
// Inputs: 
// Outputs:
// Returns: 
void CRibExport::WriteVolumetrics()
{
	// look to see if the tS shader is active
	char GlobalShaderName[255];
	tsxSceneGetShaderName(tsxSHCLASS_FOREGROUND, GlobalShaderName);
	if (lstrcmpi(GlobalShaderName, "light scattering") == 0)
	{
		float Density;
		float StepSize;
		float Octaves = 3.0f;
		float Freq;
		float UseNoise;

		// rip through the shader params, looking for the useful ones
		for (int n = 0; n < tsxSceneGetShaderParametersNumber(tsxSHCLASS_FOREGROUND); n++)
		{
			char name[255];
			tsxSceneGetShaderParameterName(tsxSHCLASS_FOREGROUND, n, name);

			if (lstrcmpi(name, "fog density") == 0)
			{
				tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_FOREGROUND, n, tsxSHPARAM_FLOAT, &Density);
			}
			else
			if (lstrcmpi(name, "samples") == 0)
			{
				long Samples;
				tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_FOREGROUND, n, tsxSHPARAM_LONG, &Samples);
				StepSize = (((1.0f - ((float)Samples * 0.01f)) * 5.0f)) + 0.01f;
			}
			else
			if (lstrcmpi(name, "noise amplitude") == 0)
			{
				tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_FOREGROUND, n, tsxSHPARAM_FLOAT, &UseNoise);
			}
			else
			if (lstrcmpi(name, "noise scale") == 0)
			{
				tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_FOREGROUND, n, tsxSHPARAM_FLOAT, &Freq);
			}
		}

		RiDeclare("density", "float");
		RiDeclare("stepsize", "float");
		RiDeclare("octaves", "float");
		RiDeclare("freq", "float");
		RiDeclare("use_noise", "float");

		RiAtmosphere("noisysmoke", "density", &Density, "stepsize", &StepSize, "octaves", &Octaves, "freq", &Freq, "use_noise", &UseNoise, RI_NULL);
		
	}

}









void CRibExport::MakeShadowMap(tsxLIGHT *pLight)
{
	CString ObjName = GetName((tsxGNODE*) pLight);
	ObjName = MakeNameFilename(ObjName);
	RtString name = (RtString)tsxMalloc(255 * sizeof(char));
	strcpy(name, ObjName.GetBuffer(255));
	RtString shadmapname = (RtString)tsxMalloc(255 * sizeof(char));
	sprintf(shadmapname, "%s%d.dsm", name, LightNum);

	tsxFree(name);

	int frame = (int)tsxAnimGetActiveTime();

	RtString shadmapfilename = (RtString)tsxMalloc(_MAX_PATH * sizeof(char));
	
	if ( FirstFrame == LastFrame )
		sprintf(shadmapfilename, "%s\\%s", ShadmapDirectory, shadmapname );
	else
		sprintf(shadmapfilename, "%s\\%s", ShadmapDirectory, shadmapname);
		
	// DEBUG
	char shadmapribfilename[255];
	strcpy( shadmapribfilename, "d:\\tmp\\dribble\\shadmap.rib" );
	if ( ( FileExists( shadmapfilename ) == false ) || ( m_refreshShadmaps ) )
	{
		// DEBUG
		//RiBegin(shadmapribfilename);
		RiBegin( RI_NULL );

			RiDisplay(shadmapfilename, "dsm", RI_RGBAZ, RI_NULL);
			RiDeclare("Np", "float");
			
			float mapsize = tsxLightGetShadowMapResolution(pLight);
			RiFormat(mapsize, mapsize, 1);
			RiShadingRate(1);
			
			int samples = 1;
			tsxLIGHT_SMQUALITY qual = tsxLightGetShmapQuality( pLight );
			switch ( qual )
			{
			case e_tsxSMQ_LOW:
				samples = 1;
				break;

			case e_tsxSMQ_MEDIUM:
				samples = 4;
				break;

			case e_tsxSMQ_HIGH:
				samples = 8;
				break;

			case e_tsxSMQ_CUSTOM:
				short cust = tsxLightGetShmapQualityCustom( pLight );
				samples = cust;
				break;				
			}

			RiPixelSamples( samples, samples );

			RiPixelFilter(RiBoxFilter, 1, 1);
			RtInt flag = 0;
			RiHider("hidden", "jitter", (RtPointer)&flag, RI_NULL);
			
			tsxLIGHT_TYPE ltype = tsxLightGetType(pLight);
			switch (ltype)
			{
				case e_tsxLT_SPOT:
				{
					RtFloat coneangle = tsxRad2Deg(tsxLightGetSpotAngle(pLight));
					RiProjection(RI_PERSPECTIVE, "fov", &coneangle, RI_NULL);

					CtsxVector3f pos;
					CtsxAxes3f orient;
					tsxGNodeGetAxesPosition((tsxGNODE*)pLight, &pos);
					tsxGNodeGetAxesOrientation((tsxGNODE*)pLight, &orient);

					RiIdentity();
					AimZ( orient.zaxis );
					RiTranslate(-pos.x, -pos.z, -pos.y);
				}
				break;

				case e_tsxLT_PROJECTOR:
				{
					RtFloat coneangle = tsxRad2Deg(tsxLightGetProjectorAngle(pLight));
					RiProjection(RI_PERSPECTIVE, "fov", &coneangle, RI_NULL);

					CtsxVector3f pos;
					CtsxAxes3f orient;
					tsxGNodeGetAxesPosition((tsxGNODE*)pLight, &pos);
					tsxGNodeGetAxesOrientation((tsxGNODE*)pLight, &orient);

					RiIdentity();
					AimZ( orient.zaxis );
					RiTranslate(-pos.x, -pos.z, -pos.y);
				}
				break;

				case e_tsxLT_INFINITE:
				{
					RiProjection( RI_ORTHOGRAPHIC, RI_NULL );
					RiScreenWindow( -45.0f, 45.0f, -45.0f, 45.0f );

					RiIdentity();
					RiTranslate( 0.0f, 0.0f, 100.0f );
					CtsxAxes3f orient;
					tsxGNodeGetAxesOrientation((tsxGNODE*)pLight, &orient);
					AimZ( orient.zaxis );
				}
				break;

				case e_tsxLT_GONIOMETRIC:
					// fallthrough
				case e_tsxLT_LOCAL:
				{
					RtFloat coneangle = 45.0f;
					RiProjection(RI_PERSPECTIVE, "fov", &coneangle, RI_NULL);

					CtsxVector3f pos;
					CtsxAxes3f orient;
					tsxGNodeGetAxesPosition((tsxGNODE*)pLight, &pos);
					tsxGNodeGetAxesOrientation((tsxGNODE*)pLight, &orient);

					RiIdentity();
					AimZ( orient.zaxis );
					RiTranslate(-pos.x, -pos.z, -pos.y);
				}
				break;
			}
				
			RiRotate( 90, 0, 1, 0 );

			objID = 0;
			ParseObjects(tsxSceneGetFirstNode(), true);
			
			RiWorldBegin();
						
				RiRotate( -90, 0, 1, 0 );

				objID = 0;
				ParseObjects(tsxSceneGetFirstNode(), false);

			RiWorldEnd();

			/*//if (Renderer == Renderer_Prman)
			{
				RtString actualshadmapname = (RtString)tsxMalloc(255 * sizeof(char));
				sprintf(actualshadmapname, "%s.shd", shadmapfilename);
				RiMakeShadow(shadmapfilename, actualshadmapname, RI_NULL);
				tsxFree(actualshadmapname);
			}*/

		RiEnd();
	}

	tsxFree(shadmapname);

	// write out line to our batchfile
	/*if (WriteBatch)
	{
		CString batchline;

		switch (Renderer)
		{
			case Renderer_3Delight:
				batchline.Format( "renderdl %s\n", shadmapribfilename );
				break;

			case Renderer_AIR:
				batchline.Format( "air %s\n", shadmapribfilename );
				break;

			case Renderer_Aqsis:
				batchline.Format( "aqsis %s\n", shadmapribfilename );
				break;
			
			case Renderer_BMRT:
				batchline.Format( "rendrib %s\n", shadmapribfilename );
				break;

			case Renderer_Pixie:
				batchline.Format( "rndr %s\n", shadmapribfilename );
				break;

			case Renderer_Prman:
				batchline.Format("prman %s\n", shadmapribfilename );
				break;
		}
		
		
		Batchfile.Write(batchline, batchline.GetLength());
	}*/

	if (shadmapfilename)
		tsxFree(shadmapfilename);

	LightNum++;

}


void CRibExport::SplitPath(char *path, char *drive, char *dir, char *fname, char *ext)
{
 register char *p;
        char *last_slash = NULL, *dot = NULL;
        unsigned len;

        /* we assume that the path argument has the following form, where any
         * or all of the components may be missing.
         *
         *  <drive><dir><fname><ext>
         *
         * and each of the components has the following expected form(s)
         *
         *  drive:
         *  0 to _MAX_DRIVE-1 characters, the last of which, if any, is a
         *  ':'
         *  dir:
         *  0 to _MAX_DIR-1 characters in the form of an absolute path
         *  (leading '/' or '\') or relative path, the last of which, if
         *  any, must be a '/' or '\'.  E.g -
         *  absolute path:
         *      \top\next\last\     ; or
         *      /top/next/last/
         *  relative path:
         *      top\next\last\  ; or
         *      top/next/last/
         *  Mixed use of '/' and '\' within a path is also tolerated
         *  fname:
         *  0 to _MAX_FNAME-1 characters not including the '.' character
         *  ext:
         *  0 to _MAX_EXT-1 characters where, if any, the first must be a
         *  '.'
         *
         */

        /* extract drive letter and :, if any */

        if ((_tcslen(path) >= (_MAX_DRIVE - 2)) && (*(path + _MAX_DRIVE - 2) == _T(':'))) {
            if (drive) {
                strncpy(drive, path, _MAX_DRIVE - 1);
                *(drive + _MAX_DRIVE-1) = _T('\0');
            }
            path += _MAX_DRIVE - 1;
        }
        else if (drive) {
            *drive = _T('\0');
        }

        /* extract path string, if any.  Path now points to the first character
         * of the path, if any, or the filename or extension, if no path was
         * specified.  Scan ahead for the last occurence, if any, of a '/' or
         * '\' path separator character.  If none is found, there is no path.
         * We will also note the last '.' character found, if any, to aid in
         * handling the extension.
         */

        for (last_slash = NULL, p = (char *)path; *p; p++) {
            if (*p == _T('/') || *p == _T('\\'))
                /* point to one beyond for later copy */
                last_slash = p + 1;
            else if (*p == _T('.'))
                dot = p;
        }

        if (last_slash) {

            /* found a path - copy up through last_slash or max. characters
             * allowed, whichever is smaller
             */

            if (dir) {
                len = __min(((char *)last_slash - (char *)path) / sizeof(char),
                    (_MAX_DIR - 1));
                strncpy(dir, path, len);
                *(dir + len) = _T('\0');
            }
            path = last_slash;
        }
        else if (dir) {

            /* no path found */

            *dir = _T('\0');
        }

        /* extract file name and extension, if any.  Path now points to the
         * first character of the file name, if any, or the extension if no
         * file name was given.  Dot points to the '.' beginning the extension,
         * if any.
         */

        if (dot && (dot >= path)) {
            /* found the marker for an extension - copy the file name up to
             * the '.'.
             */
            if (fname) {
                len = __min(((char *)dot - (char *)path) / sizeof(char),
                    (_MAX_FNAME - 1));
                strncpy(fname, path, len);
                *(fname + len) = _T('\0');
            }
            /* now we can get the extension - remember that p still points
             * to the terminating nul character of path.
             */
            if (ext) {
                len = __min(((char *)p - (char *)dot) / sizeof(char),
                    (_MAX_EXT - 1));
                strncpy(ext, dot, len);
                *(ext + len) = _T('\0');
            }
        }
        else {
            /* found no extension, give empty extension and copy rest of
             * string into fname.
             */
            if (fname) {
                len = __min(((char *)p - (char *)path) / sizeof(char),
                    (_MAX_FNAME - 1));
                strncpy(fname, path, len);
                *(fname + len) = _T('\0');
            }
            if (ext) {
                *ext = _T('\0');
            }
        }
}




// writes out the object's name
void CRibExport::WriteName(tsxGNODE *pNode)
{

}


// writes out the object's transformation matrix, including motion block if needed
bool CRibExport::WriteTransform(tsxGNODE *pNode)
{
	bool showblur = false;

	if ( m_useMotionBlur )
	{
		// look for rigid transformations
		float firstframe = tsxSobjGetFirstTime((tsxSOBJ*)pNode, e_tsxKFT_MOVE);
		float lastframe = tsxSobjGetLastTime((tsxSOBJ*)pNode, e_tsxKFT_MOVE);
		bool moved = ( firstframe != lastframe );
		firstframe = tsxSobjGetFirstTime( (tsxSOBJ*)pNode, e_tsxKFT_ROTATE );
		lastframe = tsxSobjGetLastTime( (tsxSOBJ*)pNode, e_tsxKFT_ROTATE );
		bool rotated = ( firstframe != lastframe );
		firstframe = tsxSobjGetFirstTime( (tsxSOBJ*)pNode, e_tsxKFT_SCALE );
		lastframe = tsxSobjGetLastTime( (tsxSOBJ*)pNode, e_tsxKFT_SCALE );
		bool scaled = ( firstframe != lastframe );
		
		
		// check for parent - maybe this should be recursive
		tsxGNODE *pParent = tsxGNodeGetParent( pNode );
		if (!( moved || rotated || scaled ))
		{
			
			if ( pParent )
			{
				firstframe = tsxSobjGetFirstTime((tsxSOBJ*)pParent, e_tsxKFT_MOVE);
				float lastframe = tsxSobjGetLastTime((tsxSOBJ*)pParent, e_tsxKFT_MOVE);
				moved = ( firstframe != lastframe );
				firstframe = tsxSobjGetFirstTime( (tsxSOBJ*)pParent, e_tsxKFT_ROTATE );
				lastframe = tsxSobjGetLastTime( (tsxSOBJ*)pParent, e_tsxKFT_ROTATE );
				rotated = ( firstframe != lastframe );
				firstframe = tsxSobjGetFirstTime( (tsxSOBJ*)pParent, e_tsxKFT_SCALE );
				lastframe = tsxSobjGetLastTime( (tsxSOBJ*)pParent, e_tsxKFT_SCALE );
				scaled = ( firstframe != lastframe );			
			}
		}


		if ( moved || rotated || scaled )
		{
			showblur = true;
			float OldTime = tsxAnimGetActiveTime();

			RtFloat Times[2];
			Times[0] = 0.0f;
			Times[1] = 1.0f;

			RiMotionBegin(2, 0.0f, 1.0f);
				
				tsxAnimSetActiveTimeObject( pParent ? (tsxSOBJ*)pParent : (tsxSOBJ*)pNode, OldTime);
				// write out the transform
				CtsxTxmx3f pMatrix;
				tsxGNodeGetTransform(pNode, &pMatrix);
				RtMatrix rtmx;
				tSmxToRtmx(rtmx, pMatrix);
				RiConcatTransform(rtmx);

				tsxAnimSetActiveTimeObject( pParent ? (tsxSOBJ*)pParent : (tsxSOBJ*)pNode, OldTime + 1 );
				// write out the transform
				tsxGNodeGetTransform(pNode, &pMatrix);
				tSmxToRtmx(rtmx, pMatrix);
				RiConcatTransform(rtmx);
					

			RiMotionEnd();

		}
		else
			showblur = false;
	}
	
	// just the regular transform, please
	if ( showblur == false )
	{
		// write out the transform
		CtsxTxmx3f pMatrix;
		tsxGNodeGetTransform(pNode, &pMatrix);
		RtMatrix rtmx;
		tSmxToRtmx(rtmx, pMatrix);
		RiConcatTransform(rtmx);
	}

	RtInt on;
	if ( showblur )
		on = 0;
	else
		on = 1;
	RiAttribute( "trace", "int samplemotion", &on, RI_NULL );

	return showblur;
}




void CRibExport::WriteBackground()
{
	char GlobalShaderName[255];
	tsxSceneGetShaderName(tsxSHCLASS_BACKGROUND, GlobalShaderName);
	
	m_bgColor[0] = m_bgColor[1] = m_bgColor[2] = 0.0f;

	// DEBUG
	/*for (int n = 0; n < tsxSceneGetShaderParametersNumber(tsxSHCLASS_BACKGROUND); n++)
	{
		char name[255];
		tsxSceneGetShaderParameterName(tsxSHCLASS_BACKGROUND, n, name);

		tsxSHPARAM_TYPE thetype = tsxSceneGetShaderParameterType(tsxSHCLASS_BACKGROUND, n );

		n = n;
	}*/

	if ( tsxSceneIsEnvironmentMapped() == e_tsxTRUE )
	{
		RtColor color = { 0.0f, 0.0f, 0.0f };
		
		RiImager( "tSBGcolor", "color bgcolor", (RtColor*)&color, RI_NULL );

		m_bgColor[0] = color[0];
		m_bgColor[1] = color[1];
		m_bgColor[2] = color[2];
	}
	else if (lstrcmpi(GlobalShaderName, "plain") == 0)
	{
		tssColor color;
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 0, tsxSHPARAM_COLOR, &color);

		RiImager( "tSBGcolor", "color bgcolor", (RtColor*)&color, RI_NULL );

		m_bgColor[0] = color[0];
		m_bgColor[1] = color[1];
		m_bgColor[2] = color[2];
	}		
	else if (lstrcmpi(GlobalShaderName, "c scaled image") == 0)
	{
		char filename[_MAX_PATH];
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 0, tsxSHPARAM_STRING, (void*)&filename);
		
		// this causes a corrupted stack for some reason?
		//bool filter;
		//tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 1, tsxSHPARAM_BOOL, (void*)&filter);

		CString newtexname = MakeTexture( filename );
		RtString newname = new char[255];
		strcpy( newname, newtexname );
		RiImager( "tSBGimage", "string texturename", &newname, RI_NULL );
		delete []newname;
	}
	else if (lstrcmpi(GlobalShaderName, "graduated") == 0)
	{
		tssColor color, topcolor, bottomcolor;
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 0, tsxSHPARAM_COLOR, &topcolor);
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 1, tsxSHPARAM_COLOR, &bottomcolor);

		RiImager( "tSBGgradient", "color topcolor", (RtColor*)&topcolor, "color bottomcolor", (RtColor*)&bottomcolor, RI_NULL );

		tssColorMix( color, topcolor, bottomcolor, 0.5f );
		m_bgColor[0] = color[0];
		m_bgColor[1] = color[1];
		m_bgColor[2] = color[2];
		
	}
	else if ( lstrcmpi(GlobalShaderName, "clouds") == 0 )
	{
		tssColor color, backcolor, cloudcolor;
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 0, tsxSHPARAM_COLOR, &backcolor);
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 1, tsxSHPARAM_COLOR, &cloudcolor);

		float scale;
		tsxSceneGetShaderParameterTypedValue( tsxSHCLASS_BACKGROUND, 2, tsxSHPARAM_FLOAT, &scale );

		long detail;
		tsxSceneGetShaderParameterTypedValue( tsxSHCLASS_BACKGROUND, 3, tsxSHPARAM_LONG, &detail );

		// TODO: write a bg clouds shader :)
		//RiImager( "tSBGclouds", "color backcolor", (RtColor*)&backcolor, "color cloudcolor", (RtColor*)&cloudcolor, "float scale", (RtFloat*)&scale, "integer detail", (RtLong*)&detail, RI_NULL );
		// write plain background temporarily
		RiImager( "tSBGcolor", "color bgcolor", (RtColor*)&backcolor, RI_NULL );

		tssColorMix( color, backcolor, cloudcolor, 0.5f );
		m_bgColor[0] = color[0];
		m_bgColor[1] = color[1];
		m_bgColor[2] = color[2];
	}
}

///////////////////////////////////////////////////////////////////////////////

void CRibExport::WriteForeground()
{
	// DEBUG
	/*for (int n = 0; n < tsxSceneGetShaderParametersNumber(tsxSHCLASS_FOREGROUND); n++)
	{
		char name[255];
		tsxSceneGetShaderParameterName(tsxSHCLASS_FOREGROUND, n, name);

		tsxSHPARAM_TYPE thetype = tsxSceneGetShaderParameterType(tsxSHCLASS_FOREGROUND, n );

		n = n;
	}*/

	char envname[255];
	tsxSceneGetEnvironmentFilename( envname, 255 );
	if ( tsxSceneIsEnvironmentMapped() == e_tsxTRUE )
		m_environmentName.Format( "%s", envname );
	else
		m_environmentName = "";

	char GlobalShaderName[255];
	tsxSceneGetShaderName( tsxSHCLASS_FOREGROUND, GlobalShaderName );
	
	if ( lstrcmpi(GlobalShaderName, "depth cue") == 0 )
	{
		//tssColor color, backcolor, cloudcolor;
		//tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 0, tsxSHPARAM_COLOR, &backcolor);
		//tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_BACKGROUND, 1, tsxSHPARAM_COLOR, &cloudcolor);

		float mindist, maxdist;
		tsxSceneGetShaderParameterTypedValue( tsxSHCLASS_FOREGROUND, 0, tsxSHPARAM_FLOAT, &mindist );
		tsxSceneGetShaderParameterTypedValue( tsxSHCLASS_FOREGROUND, 1, tsxSHPARAM_FLOAT, &maxdist );

		RiAtmosphere( "tSdepthcue", "color background", &m_bgColor, "float mindistance", &mindist, "float maxdistance", &maxdist, RI_NULL );
	}
	else
	if ( lstrcmpi(GlobalShaderName, "fog") == 0 )
	{
		tssColor fogcolor;
		tsxSceneGetShaderParameterTypedValue(tsxSHCLASS_FOREGROUND, 1, tsxSHPARAM_COLOR, &fogcolor);
		
		float distance;
		tsxSceneGetShaderParameterTypedValue( tsxSHCLASS_FOREGROUND, 0, tsxSHPARAM_FLOAT, &distance );
		
		float ground = 0.0f;
		RiAtmosphere( "tSfog", "float ground", &ground, "color fogcolor", &fogcolor, "float distance", &distance, RI_NULL );
	}
	else
	if ( lstrcmpi(GlobalShaderName, "light scattering") == 0 )
	{
	}

	

	
}

///////////////////////////////////////////////////////////////////////////////


// creates a large cube with inverted normals and UV coords to map
// an HDR environment image in vertical-cross format
void CRibExport::WriteEnviroCube()
{
	RtPoint Vxs[4];	
	RtFloat U[4];
	RtFloat V[4];
	
	RtFloat size = m_environmentSize;
	Vxs[0][0] = size;   Vxs[0][1] = size;   Vxs[0][2] = size;
	Vxs[1][0] = -size;  Vxs[1][1] = size;   Vxs[1][2] = size;
	Vxs[2][0] = -size;  Vxs[2][1] = -size;  Vxs[2][2] = size;
	Vxs[3][0] = size;   Vxs[3][1] = -size;  Vxs[3][2] = size;
	
	
	RtString TextureName = new char[_MAX_PATH];
	strcpy( TextureName, "" );
		
	CString src( m_environmentName );
	
	CString strTextureName;
	
	strTextureName = MakeEnviroTexture( (RtString)((LPCSTR)src), true );
	strcpy( TextureName, (LPCSTR)strTextureName );

	
						
	

	// now do the visible cube for the background
	//if ( m_doEnvironment )
	{
		RiAttributeBegin();

		RiOrientation( RI_RH );

			RtInt on;
			if ( ( m_doRaytrace ) )
			{
				on = 1;
				RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
			}
			if ( ( m_doRaytrace ) )
			{
				on = 1;
				RiAttribute( "visibility", "integer specular", &on, RI_NULL );
			}
			if ( ( m_doRaytrace ) )
			{
				on = 0;
				RiAttribute( "visibility", "integer transmission", &on, RI_NULL );

				RtString hitmode = "primitive";
				RiAttribute( "shade", "string transmissionhitmode", &hitmode, RI_NULL );
			}
			RtString hitmode = "primitive";
			RiAttribute( "shade", "string camerahitmode", &hitmode, RI_NULL );
			
			RtString maptype = "planar";
			RtFloat intensity = 1.0;
			//RtFloat blur = 0.0;
			RtFloat blur = m_environmentBlur;
			RtFloat exposure = m_environmentExposure;
			RiSurface( "tShdri", "string envname", &TextureName, "string mappingtype", &maptype, "float intensity", &intensity, "float blur", &blur, "float exposure_compensation_stops", &exposure, RI_NULL );
		
			const RtFloat onethird = 0.339f;
			const RtFloat twothird = 0.659f;
			const RtFloat half = 0.499f;
			const RtFloat quarter = 0.251f;
			const RtFloat one = 1.0f;
			const RtFloat zero = 0.0f;
			RiTransformBegin();

				// far
				U[0] = onethird;  U[1] = twothird;  U[2] = twothird;  U[3] = onethird;
				V[0] = quarter;  V[1] = quarter;  V[2] = half;  V[3] = half;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );

				// right
				RiRotate( 90.0f, 0.0f, 1.0f, 0.0f );
				U[0] = zero;  U[1] = onethird;  U[2] = onethird;  U[3] = zero;
				V[0] = quarter;  V[1] = quarter;  V[2] = half;  V[3] = half;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );

				// near
				RiRotate( 90.0f, 0.0f, 1.0f, 0.0f );
				U[0] = twothird;  U[1] = onethird;  U[2] = onethird;  U[3] = twothird;
				V[0] = one;  V[1] = one;  V[2] = 0.75f;  V[3] = 0.75f;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );

				// left
				RiRotate( 90.0f, 0.0f, 1.0f, 0.0f );
				U[0] = twothird;  U[1] = one;  U[2] = one;  U[3] = twothird;
				V[0] = quarter;  V[1] = quarter;  V[2] = half;  V[3] = half;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );
			
			RiTransformEnd();

			RiTransformBegin();
				// bottom
				RiRotate( 90.0f, 1.0f, 0.0f, 0.0f );
				U[0] = onethird;  U[1] = twothird;  U[2] = twothird;  U[3] = onethird;
				V[0] = half;  V[1] = half;  V[2] = 0.75f;  V[3] = 0.75f;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );
			RiTransformEnd();

			RiTransformBegin();
				// top
				RiRotate( -90.0f, 1.0f, 0.0f, 0.0f );
				U[0] = onethird;  U[1] = twothird;  U[2] = twothird;  U[3] = onethird;
				V[0] = zero;  V[1] = zero;  V[2] = quarter;  V[3] = quarter;	 
				RiPolygon( 4, "P", (RtPointer)&Vxs, "s", (RtPointer)&U, "t", (RtPointer)&V, RI_NULL );
			RiTransformEnd();

		RiAttributeEnd();
	}

	delete []TextureName;

}


// creates a large sphere with inverted normals and UV coords to map
// an HDR environment image in spherical probe format
void CRibExport::WriteEnviroSphere()
{
	
	RtString TextureName = new char[_MAX_PATH];
	strcpy( TextureName, "" );
		
	CString src( m_environmentName );
	
	CString strTextureName;
	
	strTextureName = MakeEnviroTexture( (RtString)((LPCSTR)src), true );
	strcpy( TextureName, (LPCSTR)strTextureName );

	
						
	

	// now do the visible sphere for the background
	if ( ( m_environmentShowBG ) || ( m_environmentShowReflections ) )
	{
		RiAttributeBegin();

		RiOrientation( RI_RH );

			RtInt on;
			on = 0;
			RiAttribute( "visibility", "integer diffuse", &on, RI_NULL );
			on = 0;
			RiAttribute( "visibility", "integer transmission", &on, RI_NULL );			
			on = m_environmentShowBG;
			RiAttribute( "visibility", "integer camera", &on, RI_NULL );
			on = m_environmentShowReflections;
			RiAttribute( "visibility", "integer specular", &on, RI_NULL );
			
			RtString hitmode = "primitive";
			RiAttribute( "shade", "string transmissionhitmode", &hitmode, RI_NULL );
			RiAttribute( "shade", "string camerahitmode", &hitmode, RI_NULL );
			
			RtString maptype = "environment";
			RtString envspace = "object";
			RtFloat intensity = 1.0;
			//RtFloat blur = 0.0;
			RtFloat blur = m_environmentBlur;
			RtFloat exposure = m_environmentExposure;
			RiSurface( "tShdri", "string envname", &TextureName, "string envspace", &envspace, "string mappingtype", &maptype, "float intensity", &intensity, "float blur", &blur, "float exposure_compensation_stops", &exposure, RI_NULL );
			RiTransformBegin();
				
				RiRotate( -90.0f, 0.0f, 1.0f, 0.0f );
				RiSphere( m_environmentSize, -m_environmentSize, m_environmentSize, 360, RI_NULL );
							
			RiTransformEnd();
		

		RiAttributeEnd();
	}

	delete []TextureName;

}


///////////////////////////////////////////////////////////////////////////////

bool CRibExport::IsVisible( tsxGNODE *pGNode )
{
	bool visible = true;

	if ( tsxSobjGetType((tsxSOBJ*) pGNode) != e_tsxLIGHT )
	{
		if ( tsxMNodeIsVisible((tsxMNODE*) pGNode) == e_tsxFALSE )
			visible = false;
	}

#ifdef TSX66
	// get layer index
	int idx = tsxGetObjectLayerIndex( (tsxSOBJ*)pGNode );

	if ( tsxIsLayerVisible( idx ) == e_tsxFALSE )
		visible = false;
#endif

	return visible;
}