// RibExport.h: interface for the CRibExport class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RIBEXPORT_H__0CEFF8C6_DAC2_45FF_AF3E_B35D87ECE2FE__INCLUDED_)
#define AFX_RIBEXPORT_H__0CEFF8C6_DAC2_45FF_AF3E_B35D87ECE2FE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//#define LIBAQSIS

#ifdef LIBAQSIS
#include ".\libri2rib\ri.h"
#ifdef _DEBUG
#pragma comment ( lib, "libri2rib/libri2rib_debug.lib" )
#pragma comment ( lib, "libri2rib/libz.lib" )
#else
#pragma comment ( lib, "libri2rib/libri2rib.lib" )
#pragma comment ( lib, "libri2rib/libz.lib" )
#endif
#endif

#ifdef _3DELIGHT
#include "3delight/ri.h"
#pragma comment ( lib, "3delight/3delight.lib" )
#endif


#include "tsx.h"
//#include "IL/il.h"
//#include "IL/ilu.h"


const int Renderer_3Delight = 0;
const int Renderer_AIR = 1;
const int Renderer_Aqsis = 2;
const int Renderer_BMRT = 3;
const int Renderer_Pixie = 4;
const int Renderer_Prman = 5;


const int PixFilter_Box = 0;
const int PixFilter_Triangle = 1;
const int PixFilter_CatmullRom = 2;
const int PixFilter_Gaussian = 3;
const int PixFilter_Sinc = 4;
const int PixFilter_Disk = 5;
const int PixFilter_Bessel = 6;

class CRMObject
{
	tsxGNODE	*pGNode;
	RtInt		ObjectHandle;
};

class CRibExport  
{
public:
	// raytracing
	bool	m_doRaytrace;
	RtInt	m_rayDepth;
	bool	m_doRaytraceDisplacements;
	
	// other render options
	bool	m_doTriangulate;
	RtInt	m_blurMultiplier;
	
	// lighting
	bool	m_alwaysRayShadows;
	int		m_skySamples;

	// environment
	int		m_environmentSize;
	CString	m_environmentName;
	bool	m_environmentShowBG;
	bool	m_environmentShowReflections;
	bool	m_doEnvironmentLight;
	float	m_environmentBlur;
	float	m_environmentExposure;
	float	m_environmentLightIntensity;
	float	m_environmentLightExposure;
	float	m_environmentLightSamples;
	bool	m_environmentLightShadows;

	RtColor	m_bgColor;

	bool	m_cacheOverride;			// when true, textures/shaders/etc. will be created even if they already exist

	bool	m_useRaytraceHider;

	bool	m_useMotionBlur;
	bool	m_doRaytraceMotionBlur;

	bool Binary;
	RtFloat DOF_FocalLength;
	RtFloat DOF_FStop;
	bool UseDOF;
	RtFloat	DOF_FocusDistance;

	int FormatWidth;
	int FormatHeight;
	float FormatAspect;
	int PixFilter;
	float LightMultiplier;
	bool IndividualFaces;
	int Renderer;
	bool PatchPoly;
	bool ConvertProceduralShaders;
	bool ConvertTextureShaders;
	int TexHeight;
	int TexWidth;
	void WriteTempMaterial(tsxSOBJ *pObj);
	bool DeclareParams;
	int ExtDataID;
	void ExportFlow();
	float PixelSamplesX;
	float PixelSamplesY;
	float PixelFilterWidthX;
	float PixelFilterWidthY;
	int RadiosityMinPatchSamples;
	int RadiositySteps;
	
    bool m_bUseGI;
    float m_fGISamples;
    float m_fGIMaxError;
    float m_fGIShadingRate;
    float m_fGIIntensity;
    int m_iPhotons;
    bool m_bUseCaustics;
	bool m_bUseIrradiancePhotons;
	bool m_bUseGIOcclusion;
	bool m_bHighQualityGI;
	

	bool UseSeedFile;
	bool SaveSeedFile;
	CString SeedFile;
	bool AIR_RaytraceShadows;
	bool AIR_RaytraceReflections;
	float SeedBlur;
	bool Verbose;
	int Sides;
	float ShadingRate;
	float LastFrame;
	void ExportScene( bool tofile );
	void ExportFrame(CString fname, int frame);
	float FirstFrame;
		
	bool	m_refreshShadmaps;

	float ShadowBias;
	CString ImageName;
	CString Filename;
	
	CString RIBDirectory;
	CString TextureDirectory;
	CString ShadmapDirectory;
	CString BaseRIBFilename;
	CString OutputDirectory; // only for PSD right now
	CString RawImageName;
	CRibExport();
	virtual ~CRibExport();

	void ReadtSRenderOptions();

	bool	PSDOutput;

	CRect	m_SubRect;

	int		objID;

private:
	
	void WriteForeground();
	void WriteBackground();
	void WriteEnviroCube();
	void WriteEnviroSphere();

	void OutputPolyhedron(tsxPOLYHEDRON* pPolyh);
	bool WriteTransform(tsxGNODE *pNode);
	void WriteName(tsxGNODE *pNode);
	void WriteNURBSCurve(tsxNCURVE *pCurve);
	void SplitPath(char *path, char *drive, char *dir, char *fname, char *ext);
	bool JustShadowmaps;
	
	CString MakeTexture( RtString filename );
	CString MakeEnviroTexture( RtString filename, bool probe );
	void MakeUnixPath(CString *path);
	void MakeShadowMap(tsxLIGHT *pLight);
	bool ConvertProcedural(tsxMATERIAL* pMaterial, int width, int height, char *pFilename);
	//void GetUVInfo(tsxMATERIAL *pMaterial, CtsxVector2f& uvreps, CtsxVector2f& uvoffset);
	void GetUVInfo(tsxSHINFO *pShinfo, CtsxVector2f& uvreps, CtsxVector2f& uvoffset);
	bool IsMaterialPainted(tsxMATERIAL *pMaterial, char *pTxrName);
	bool IsMaterialDisplaced(tsxMATERIAL *pMaterial, char *pTxrName, float& amplitude, bool& truedisp, bool& normalmap);
	bool IsMaterialMasked(tsxMATERIAL *pMaterial, char *pTxrName);
	bool IsMaterialLayered(tsxMATERIAL *pMaterial );
	bool IsFurMaterial(tsxMATERIAL *pMaterial);
	bool IsFurObject(tsxPOLYHEDRON *pPolyh);
	void WritePatch(tsxNPATCH *pPatch, tsxPOLYHEDRON *pPolyh);
	void WriteSDS(tsxMNODE *pSDS, tsxPOLYHEDRON *pPolyh);
	void WritePatchTextures(tsxNPATCH *pPatch, tsxPOLYHEDRON *pPolyh);
	bool Busy;
	CString GetName(tsxGNODE *pGNode);
	void WriteVolumetrics();
	
	void WriteShinfo(tsxSHINFO* pShinfo);
	void WriteMaterial( tsxGNODE *pGNode, tsxMATERIAL* pMaterial, CString ObjName, int MatNbr);
	void GetStdSurfaceParameters( tsxMATERIAL* pMaterial, float& Ka, float& Kd, float& Ks, float& roughness, tssColor& speccol, float& Kr, float& Kt, float& ior, float& metal, float& usephong  );
	unsigned short OldMaterialIdx;
	void WritePolyhedronAsFaces(tsxPOLYHEDRON* pPolyh, bool IncludeMaterial = true);
	void WritePolyhedronTextures(tsxPOLYHEDRON* pPolyh, tsxGNODE* pTopNode);
	void WriteGeometry( tsxGNODE *pGNode, bool IncludeMaterial = true );
	void WriteObject( tsxGNODE *pGNode, bool FirstPass );
	void WriteLight(tsxLIGHT* pLight, tsxPOLYHEDRON *pPolyh = NULL);
	void GoLights(tsxGNODE* pGNode);
	void CheckForAreaLight(tsxGNODE *pGNode);
	tsxLIGHT* GetEmissiveLight(tsxPOLYHEDRON *pPolyh);
	void WritePolyhedronAsMesh(tsxPOLYHEDRON* pInPolyh);
	CString MakeNameFilename(const CString name);
	void PlaceCamera(CtsxVector3f position, CtsxVector3f direction, float roll, bool perspective);
	void AimZ(CtsxVector3f direction);
	void tSmxToRtmx(RtMatrix result, CtsxTxmx3f inmx);
	void tSOpacityToRtColor(RtColor result, CtsxColor incolor);
	void tSColorToRtColor(RtColor result, CtsxColor incolor);
	
	void ParseObjects( tsxGNODE* pGNode, bool FirstPass );
	void GoSceneTextures(tsxGNODE* pGNode);

	bool IsVisible( tsxGNODE* pGNode );
	
	void WritePolyhedronAsFur(tsxPOLYHEDRON* pPolyh);

protected:
	int LightNum;
	bool WriteRIB;
};

#endif // !defined(AFX_RIBEXPORT_H__0CEFF8C6_DAC2_45FF_AF3E_B35D87ECE2FE__INCLUDED_)
