// functions for converting tS render options to Renderman
// equivalents

#include "stdafx.h"
#include "RibExport.h"


void CRibExport::ReadtSRenderOptions()
{
	// get tS render options from UI
	tsxRenderData rData;
	tsxInitRenderData( &rData );

	// anti-aliasing
	switch ( rData.antialias )
	{
	case e_tsxRENDER_ANTIALIAS_ADAPTIVE:
		// fallthrough
	case e_tsxRENDER_ANTIALIAS_2X:
		PixelSamplesX = 2;
		PixelSamplesY = 2;
		break;

	case e_tsxRENDER_ANTIALIAS_3X:
		PixelSamplesX = 3;
		PixelSamplesY = 3;
		break;

	case e_tsxRENDER_ANTIALIAS_4X:
		PixelSamplesX = 4;
		PixelSamplesY = 4;
		break;

	case e_tsxRENDER_ANTIALIAS_NONE:
		// fallthrough
	default:
		PixelSamplesX = 1;
		PixelSamplesY = 1;
		break;
	}

	// raytracing
	m_doRaytrace = (rData.rayTrace == e_tsxTRUE);
	if ( !m_doRaytrace )
		m_rayDepth = 0;
	else
		m_rayDepth = rData.rayTraceMaxDepth;

	// misc options
	m_doTriangulate = (rData.triangulate == e_tsxTRUE);
	if ( rData.singleSided == e_tsxTRUE )
		Sides = 1;
	else
		Sides = 2;

	// depth of field
	if ( tsxSceneGetShaderIndexParametersNumber( tsxSHCLASS_POSTPROCESS, e_tsxDEPTH_OF_FIELD ) )
	{
		UseDOF = true;
		tsxSceneGetShaderIndexParameterTypedValue( tsxSHCLASS_POSTPROCESS, e_tsxDEPTH_OF_FIELD, 0, tsxSHPARAM_FLOAT, &DOF_FocalLength );
		tsxSceneGetShaderIndexParameterTypedValue( tsxSHCLASS_POSTPROCESS, e_tsxDEPTH_OF_FIELD, 2, tsxSHPARAM_FLOAT, &DOF_FStop );
		tsxSceneGetShaderIndexParameterTypedValue( tsxSHCLASS_POSTPROCESS, e_tsxDEPTH_OF_FIELD, 3, tsxSHPARAM_FLOAT, &DOF_FocusDistance );

		// tweak these values
		DOF_FocalLength *= 0.1f;
		DOF_FStop *= 2.0f;

	}
	else
		UseDOF = false;

	
}
