// XTabCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "XTabCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CXTabCtrl

CXTabCtrl::CXTabCtrl()
{
	m_iSelectedTab = 0;
	m_ptTabs.x = 4;
	m_ptTabs.y = 28;

	m_crSelected = 0;
	m_crDisabled = 0;
	m_crNormal = 0;
	m_crMouseOver = 0;

	m_bColorSelected  = false;
	m_bColorDisabled  = false;
	m_bColorNormal    = false;
	m_bColorMouseOver = false;


	m_iIndexMouseOver = -1;

	m_bMouseOver = false;

	m_hPenLight = ::CreatePen(PS_SOLID,1, ::GetSysColor(COLOR_3DHILIGHT));
	m_hPenShadow = ::CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DDKSHADOW));
	m_hPenItemShadow = ::CreatePen(PS_SOLID,1,::GetSysColor(COLOR_3DSHADOW));

}

CXTabCtrl::~CXTabCtrl()
{
	m_arrayStatusTab.RemoveAll();
}


BEGIN_MESSAGE_MAP(CXTabCtrl, CTabCtrl)
	//{{AFX_MSG_MAP(CXTabCtrl)
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnSelchange)
	ON_NOTIFY_REFLECT(TCN_SELCHANGING, OnSelchanging)
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()

	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXTabCtrl message handlers

void CXTabCtrl::SetTopLeftCorner(CPoint pt)
{
	m_ptTabs.x = pt.x;
	m_ptTabs.y = pt.y;
}

void CXTabCtrl::SetMouseOverColor(COLORREF cr) 
{
	m_bColorMouseOver = true;
	m_crMouseOver = cr;
}


void CXTabCtrl::SetDisabledColor(COLORREF cr) 
{
	m_bColorDisabled = true;
	m_crDisabled = cr;
}

void CXTabCtrl::SetSelectedColor(COLORREF cr)
{
	m_bColorSelected = true;
	m_crSelected = cr;
}


void CXTabCtrl::SetNormalColor(COLORREF cr)
{
	m_bColorNormal = true;
	m_crNormal = cr;
}



void CXTabCtrl::AddTab(CWnd* pWnd, LPTSTR lpszCaption, int iImage)
{
	ASSERT_VALID(pWnd);

	TCITEM item;
	item.mask = TCIF_TEXT|TCIF_PARAM|TCIF_IMAGE;
	item.lParam = (LPARAM) pWnd;
	item.pszText = lpszCaption;
	item.iImage = iImage;

	int iIndex = m_arrayStatusTab.GetSize();
	InsertItem(iIndex, &item);

	pWnd->SetWindowPos(NULL, m_ptTabs.x, m_ptTabs.y , 0, 0,
						SWP_FRAMECHANGED | SWP_NOSIZE | SWP_NOZORDER);
	
	pWnd->ShowWindow(iIndex ? SW_HIDE : SW_SHOW);

	//** the initial status is enabled
	m_arrayStatusTab.Add(TRUE); 

}

void CXTabCtrl::EnableTab(int iIndex, BOOL bEnable)
{
	ASSERT(iIndex < m_arrayStatusTab.GetSize());

	//** if it should change the status ----
	if (m_arrayStatusTab[iIndex] != bEnable)
	{
		m_arrayStatusTab[iIndex] = bEnable;

		//** redraw the item -------
		CRect rect;

		GetItemRect(iIndex, &rect);
		InvalidateRect(rect);
	}
}

void CXTabCtrl::DeleteAllTabs()
{
	m_arrayStatusTab.RemoveAll();
	
	DeleteAllItems();

}

void CXTabCtrl::DeleteTab(int iIndex)
{
	ASSERT(iIndex < m_arrayStatusTab.GetSize());

	m_arrayStatusTab.RemoveAt(iIndex);

	DeleteItem(iIndex);

}


BOOL CXTabCtrl::IsTabEnabled(int iIndex)
{
	ASSERT(iIndex < m_arrayStatusTab.GetSize());
	
	return m_arrayStatusTab[iIndex];
}


BOOL CXTabCtrl::SelectTab(int iIndex)
{
	ASSERT(iIndex < m_arrayStatusTab.GetSize());

	if (GetCurSel() == iIndex)
		return TRUE;

	//** check if the tab is enabled --------
	if (m_arrayStatusTab[iIndex])
	{
		TCITEM item;
		CWnd* pWnd;
		item.mask = TCIF_PARAM;

		GetItem(GetCurSel(), &item);
		pWnd = reinterpret_cast<CWnd*> (item.lParam);
		ASSERT_VALID(pWnd);
		pWnd->ShowWindow(SW_HIDE);

		SetCurSel(iIndex);
		GetItem(iIndex, &item);
		pWnd = reinterpret_cast<CWnd*> (item.lParam);
		ASSERT_VALID(pWnd);
		pWnd->ShowWindow(SW_SHOW);

		return TRUE;
	}

	return FALSE;
}

void CXTabCtrl::OnSelchange(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	int iNewTab = GetCurSel();

	if (!IsTabEnabled(iNewTab))
	{
		SetCurSel(m_iSelectedTab);
	}
	else
	{
		TCITEM item;
		CWnd* pWnd;

		item.mask = TCIF_PARAM;
		
		//** hide the current tab ---------
		GetItem(m_iSelectedTab, &item);
		pWnd = reinterpret_cast<CWnd*> (item.lParam);
		ASSERT_VALID(pWnd);
		pWnd->ShowWindow(SW_HIDE);

		//** show the selected tab --------
		GetItem(iNewTab, &item);
		pWnd = reinterpret_cast<CWnd*> (item.lParam);
		ASSERT_VALID(pWnd);
		pWnd->ShowWindow(SW_SHOW);

	}

	*pResult = 0;
}

void CXTabCtrl::OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	m_iSelectedTab = GetCurSel();

	*pResult = 0;
}



void CXTabCtrl::PreSubclassWindow() 
{
	// TODO: Add your specialized code here and/or call the base class
	CTabCtrl::PreSubclassWindow();
	ModifyStyle(0, TCS_OWNERDRAWFIXED);
}


void CXTabCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{	
	CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);
	if (!pDC)
		return;

	int nSavedDC = pDC->SaveDC();

	CRect rect = lpDrawItemStruct->rcItem;

	pDC->SetBkMode(TRANSPARENT);
	pDC->FillSolidRect(rect, RGB(192,192,192));


	rect.top += ::GetSystemMetrics(SM_CYEDGE);

	int nTabIndex = lpDrawItemStruct->itemID;
	
	if (nTabIndex < 0) return;

	BOOL bSelected = (nTabIndex == GetCurSel());

	COLORREF crSelected = m_bColorSelected ? m_crSelected : GetSysColor(COLOR_BTNTEXT);
	COLORREF crNormal  = m_bColorNormal   ? m_crNormal   : GetSysColor(COLOR_BTNTEXT);
	COLORREF crDisabled = m_bColorDisabled ? m_crDisabled : GetSysColor(COLOR_GRAYTEXT);


	char label[64];
	TC_ITEM item;
	item.mask = TCIF_TEXT|TCIF_IMAGE;
	item.pszText = label;     
	item.cchTextMax = 63;    	
	if (!GetItem(nTabIndex, &item))
		return;

	

	CRect rectItem;
	POINT pt;

	GetItemRect(nTabIndex, &rectItem);
	GetCursorPos(&pt);
	ScreenToClient(&pt);

	if (rectItem.PtInRect(pt))
		m_iIndexMouseOver = nTabIndex;



	
	//**  Draw the image
	CImageList* pImageList = GetImageList();
	if (pImageList && item.iImage >= 0) 
	{

		rect.left += pDC->GetTextExtent(_T(" ")).cx;

		IMAGEINFO info;
		pImageList->GetImageInfo(item.iImage, &info);
		CRect ImageRect(info.rcImage);
		int nYpos = rect.top;

		pImageList->Draw(pDC, item.iImage, CPoint(rect.left, nYpos), ILD_TRANSPARENT);
		rect.left += ImageRect.Width();
	}

	CFont CaptionFont;	
	CFont* oldfont;
	
	if (!IsTabEnabled(nTabIndex))
	{
		CaptionFont.CreateFont(-11,0,0,0,FW_NORMAL,FALSE,FALSE,
		0,ANSI_CHARSET,OUT_TT_PRECIS,
		CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,
		FF_SWISS,"Arial");
		oldfont = pDC->SelectObject(&CaptionFont);
		pDC->SetTextColor(crDisabled);
		rect.top -= ::GetSystemMetrics(SM_CYEDGE);
		pDC->DrawText(label, rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER);
	}
	else
	{
		//** selected item -----
		if (bSelected)
		{
			CaptionFont.CreateFont(-11,0,0,0,FW_BOLD,FALSE,FALSE,
			0,ANSI_CHARSET,OUT_TT_PRECIS,
			CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,
			FF_SWISS,"Arial");
			oldfont = pDC->SelectObject(&CaptionFont);
			pDC->SetTextColor(crSelected);
		}
		else //** other item ---
		{
			CaptionFont.CreateFont(-11,0,0,0,FW_NORMAL,FALSE,FALSE,
			0,ANSI_CHARSET,OUT_TT_PRECIS,
			CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,
			FF_SWISS,"Arial");
			oldfont = pDC->SelectObject(&CaptionFont);
			if (m_bColorMouseOver && nTabIndex == m_iIndexMouseOver) 
			{
				pDC->SetTextColor(m_crMouseOver);	
			}
			else
			{
				pDC->SetTextColor(crNormal);
			}
		}

		rect.top -= ::GetSystemMetrics(SM_CYEDGE);
		pDC->DrawText(label, rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER);

	}


	pDC->SelectObject(oldfont);

	pDC->RestoreDC(nSavedDC);
}

void CXTabCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	//** if we should change the color of the tab ctrl ---
	if (m_bColorMouseOver)
	{
		SetTimer(1,10,NULL);

		if (m_iIndexMouseOver != -1)
		{
			CRect rectItem;
			GetItemRect(m_iIndexMouseOver, rectItem);
			if (!rectItem.PtInRect(point))
			{
				CRect rectOldItem;
				GetItemRect(m_iIndexMouseOver, rectOldItem);
				m_iIndexMouseOver = -1;
				InvalidateRect(rectOldItem);
				return;
			}
		}
	

		if (!m_bMouseOver)
		{
			TCHITTESTINFO hitTest;

			m_bMouseOver = true;
			hitTest.pt = point;

			int iItem = HitTest(&hitTest);
			if (iItem != -1 && m_arrayStatusTab[iItem])
			{
				RECT rectItem;
				GetItemRect(iItem, &rectItem);

				InvalidateRect(&rectItem);
			}
		}
	}
	
	CTabCtrl::OnMouseMove(nFlags, point);
}

void CXTabCtrl::OnTimer(UINT_PTR nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	POINT pt;
	GetCursorPos(&pt);
	CRect rectItem, rectScreen;

	GetItemRect(m_iIndexMouseOver, rectItem);
	rectScreen = rectItem;
	ClientToScreen(rectScreen);
	
	// If mouse leaves, show normal
	if (!rectScreen.PtInRect(pt)) 
	{
		KillTimer (1);
		m_bMouseOver = false;
		m_iIndexMouseOver = -1;
		InvalidateRect(rectItem);
	}


	CTabCtrl::OnTimer(nIDEvent);
}

void CXTabCtrl::ChangeTab(int iIndex, CWnd *pNewTab, LPTSTR lpszCaption, int iImage)
{
	ASSERT_VALID(pNewTab);

	TCITEM item;
	item.mask = TCIF_TEXT|TCIF_PARAM|TCIF_IMAGE;
	item.lParam = (LPARAM) pNewTab;
	item.pszText = lpszCaption;
	item.iImage = iImage;

	//** update item 
	VERIFY(SetItem(iIndex, &item));

	pNewTab->SetWindowPos(NULL, m_ptTabs.x, m_ptTabs.y , 0, 0,
						SWP_FRAMECHANGED | SWP_NOSIZE | SWP_NOZORDER);
	
	pNewTab->ShowWindow(SW_HIDE);

	//** the initial status is enabled
	m_arrayStatusTab[iIndex] = TRUE; 

	//** updates the screen
	GetItem(GetCurSel(), &item);
	((CWnd*)item.lParam)->ShowWindow(SW_SHOW);
}

BOOL CXTabCtrl::SelectTabByName(LPCTSTR lpszTabCaption)
{
	TCHAR szName[64];
	TCITEM item;
	
	memset(&item, 0, sizeof(item));

	item.mask       = TCIF_TEXT;
	item.cchTextMax = sizeof(szName);
	item.pszText    = szName;

	for (int i=0; i < GetItemCount(); i++)
	{
		GetItem(i, &item);

		if (strcmp(lpszTabCaption, item.pszText) == 0)
			return SelectTab(i);
	}

	return FALSE;
}

BOOL CXTabCtrl::SelectNextTab(BOOL bForward)
{
	int iSum = bForward ? 1 : -1;

	int iCurSel = GetCurSel();
	int iCurSelSave(iCurSel);

	do
	{
		iCurSel += iSum;

		if (iCurSel < 0 || iCurSel == GetItemCount())
			return FALSE;

		if (IsTabEnabled(iCurSel))
		{
			if (iCurSel < GetItemCount())
			{
				SelectTab(iCurSel);

				return TRUE;
			}

			return FALSE;
		}

	}
	while (iCurSel != iCurSelSave);
	
	return FALSE;
}


void CXTabCtrl::DrawItemRect(CDC &dc, CRect &rcItem,int nHint)
{
	//Let's draw items rect.
	rcItem.bottom-=1;
	if(rcItem.left>0)
	{
		bool bDoNotCut=true;

		::SelectObject(dc.m_hDC,m_hPenLight);
		
		if(nHint & HINT_ACTIVE_ITEM)
			dc.MoveTo(rcItem.left,rcItem.bottom+2);
		else
			dc.MoveTo(rcItem.left,rcItem.bottom);
		
		dc.LineTo(rcItem.left,rcItem.top+2);
		dc.LineTo(rcItem.left+2,rcItem.top);
		/*if(!m_rectUpDn.IsRectEmpty() && rcItem.right >= m_rectUpDn.left)
		{
			//	CLBTabCtrl is not in stacked mode and current item overlaps
			//	msctls_updown32 control. So let's decrease current item's  
			//	width.
			rcItem.right=m_rectUpDn.left-SHIFT_FROM_CUT_TO_SPIN;
			dc.LineTo(rcItem.right,rcItem.top);
			bDoNotCut=false;
		}
		else*/
			dc.LineTo(rcItem.right-2,rcItem.top);
		
		if(!nHint)
			::SelectObject(dc.m_hDC,m_hPenItemShadow);
		else
		{
			if(nHint & HINT_ACTIVATE_RIGHT_SIDE)
			{
				::SelectObject(dc.m_hDC,m_hPenShadow);
			}
		}
		if(bDoNotCut)
		{
			if(nHint & HINT_ACTIVE_ITEM)
			{
				dc.LineTo(rcItem.right,rcItem.top+2);
				dc.LineTo(rcItem.right,rcItem.bottom+2);
			}
			else
			{
				dc.LineTo(rcItem.right,rcItem.top+2);
				dc.LineTo(rcItem.right,rcItem.bottom+1);
			}
		}
		else
		{
			//	CLBTabCtrl is not in stacked mode and current item overlaps
			//	msctls_updown32 control. So let's draw current item's rect 
			//	as cut.
			int nYPos=rcItem.top+2;

			nYPos+=rcItem.Height()/5;
			rcItem.right+=1;
			if(nYPos < rcItem.bottom)
				dc.LineTo(rcItem.right,nYPos);

			nYPos+=rcItem.Height()/3;
			rcItem.right-=2;
			if(nYPos < rcItem.bottom)
				dc.LineTo(rcItem.right,nYPos);

			nYPos+=rcItem.Height()/5;
			rcItem.right+=1;
			if(nYPos < rcItem.bottom)
				dc.LineTo(rcItem.right,nYPos);

			nYPos+=rcItem.Height()/5;
			rcItem.right+=1;
			if(nYPos < rcItem.bottom)
				dc.LineTo(rcItem.right,nYPos);
			if(nHint & HINT_ACTIVE_ITEM)
			{
				dc.LineTo(rcItem.right,rcItem.bottom+2);
			}
			else
				dc.LineTo(rcItem.right,rcItem.bottom+1);

		}
	}
}

UINT CXTabCtrl::FindLeftDownItem() const
{
	//Helper function
	//Finds leftmost down item.

	CRect rctLeft(100000,0,0,0),rctCur;
	int nCount = GetItemCount()-1;
	UINT nItem=0;
	for(int i=0;i<nCount;i++)
	{
		GetItemRect(i,&rctCur);
		if(rctCur.left > 0 && rctCur.left <= rctLeft.left)
		{
			if(rctCur.bottom > rctLeft.bottom)
			{
				rctLeft=rctCur;
				nItem=i;	
			}
		}
	}
	return nItem;
}

void CXTabCtrl::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	DRAWITEMSTRUCT drawItemStruct;
	CRect rcItem, rcPnt, rcActive;
	rcPnt = dc.m_ps.rcPaint;

	CRect rctItem;
	GetItemRect(0,&rctItem);
	GetClientRect(&m_rctPage);
	// fill the background
	dc.FillSolidRect(m_rctPage, RGB(192,192,192));
	m_rctPage.top=rctItem.bottom;

	HPEN hOldPen;
	hOldPen = (HPEN)::GetCurrentObject(dc.m_hDC,OBJ_PEN);

	

	int m_nActiveItem = GetCurSel();
	GetItemRect(m_nActiveItem,&rcActive);

	CRect rectTemp;
	//if(rectTemp.IntersectRect(m_rctPage,rcPnt) && rcActive.bottom == m_rctPage.top)
	{
		//Let's draw Property Page bounding rectangle as 3D.

		rectTemp=m_rctPage;
		::SelectObject(dc.m_hDC,m_hPenLight);
		dc.MoveTo(rectTemp.left,rectTemp.bottom+1);
		dc.LineTo(rectTemp.left,rectTemp.top+1);

		if(rcActive.left>=0)
		{
			dc.LineTo(rcActive.left,rcActive.bottom+1);
			//if(m_rectUpDn.IsRectEmpty())
			{
				dc.MoveTo(rcActive.right,rcActive.bottom+1);
			}
			/*else
			{
				if(rcActive.right >= m_rectUpDn.left)
				{
					dc.MoveTo(m_rectUpDn.left-SHIFT_FROM_CUT_TO_SPIN+2,
						rcActive.bottom+1);
				}
				else
				{
					dc.MoveTo(rcActive.right,rcActive.bottom+1);

				}
			}*/
			dc.LineTo(rectTemp.right-2,rcActive.bottom+1);
		}
		else
		{
			CRect rectItemLeftmost;
			UINT nItemLeftmost=FindLeftDownItem();
			GetItemRect(nItemLeftmost,&rectItemLeftmost);
			dc.LineTo(rectTemp.right-2,rcActive.bottom+1);
		}
		::SelectObject(dc.m_hDC,m_hPenItemShadow);
		dc.LineTo(rectTemp.right-2,rectTemp.bottom-2);
		dc.LineTo(rectTemp.left,rectTemp.bottom-2);

		::SelectObject(dc.m_hDC,m_hPenShadow);
		dc.MoveTo(rectTemp.right-1,rcActive.bottom+1);
		dc.LineTo(rectTemp.right-1,rectTemp.bottom-1);
		dc.LineTo(rectTemp.left-2,rectTemp.bottom-1);

	}

	int nCount = GetItemCount();

	for(int i=0;i< nCount;i++)
	{
		//if(i!=m_nActiveItem)
		{
			//Let's draw all inactive  items which intersects invalidated
			//rect (rcPaint).
			GetItemRect(i,&rcItem);
			//if (rectTemp.IntersectRect(&rcItem,&rcPnt))
			{
				//rcItem.DeflateRect(ITEM_DEFLATE,0);

				//Draw item's text,bitmap,...
				drawItemStruct.rcItem=rcItem;
				drawItemStruct.hDC=dc.GetSafeHdc();
				drawItemStruct.itemID=i;
				DrawItem(&drawItemStruct);
				DrawItemRect(dc,rcItem);
			}
		}
	}

	//Restore device context defaults.
	::SelectObject(dc.m_hDC,hOldPen);
}


BOOL CXTabCtrl::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	return FALSE;
}