; dribble installer
; new version

;--------------------------------
;Include Modern UI

  !include "MUI2.nsh"

;--------------------------------
;General

XPStyle on

  ;Name and file
  Name "dribble 1.04wip080808"
  OutFile "dribble1.04wip080808_setup.exe"
  !define MUI_ICON ".\installer.ico"
  !define MUI_UNICON ".\installer.ico"

  ;Default installation folder
  ;TODO find tS dir
  InstallDir "$PROGRAMFILES\trueSpace76\ts"
  
  ;Get installation folder from registry if available
  InstallDirRegKey HKCU "Software\cool powers\dribble" "Install_Dir"

;--------------------------------
;Interface Settings

  !define MUI_ABORTWARNING


Var 3DERoot
Var tSRoot
Var UsingTS7

; determines whether 3Delight is installed and aborts if so
Function CheckFor3DE
	ReadEnvStr "$3DERoot" "DELIGHT"
	IfFileExists "$3DERoot\bin\renderdl.exe" Found3DE NotFound3DE
NotFound3DE:
  MessageBox MB_OK "3Delight was not found.  Please install 3Delight, then reinstall dribble."
  Quit
Found3DE:  
FunctionEnd

Function .onInit
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace76" "AppDir"
  IfFileExists $tSRoot\*.* FoundTS7Dir
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace75" "AppDir"
  IfFileExists $tSRoot\*.* FoundTS7Dir
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace7" "AppDir"
  IfFileExists $tSRoot\*.* FoundTS7Dir
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace6.5" "AppDir"
  IfFileExists $tSRoot\*.* FoundTSDir
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace6" "AppDir"
  IfFileExists $tSRoot\*.* FoundTSDir
  ReadRegStr "$tSRoot" HKCU "Software\Caligari trueSpace5" "AppDir"
  IfFileExists $tSRoot\*.* FoundTSDir
  strcpy $INSTDIR "$PROGRAMFILES\trueSpace76\ts"
  goto Leave
FoundTS7Dir:
  strcpy $INSTDIR $tSRoot\ts
  goto Leave
FoundTSDir:
  strcpy $INSTDIR $tSRoot
Leave:
FunctionEnd

; searches for a variety of tS executables
Function CheckForTS
  IfFileExists "$INSTDIR\tS7.exe" FoundTS7 TryTS6
TryTS6:
  IfFileExists "$INSTDIR\tS6.exe" FoundTS TryTS5
TryTS5:
  IfFileExists "$INSTDIR\tS5.exe" FoundTS NoTS
NoTS:
  MessageBox MB_OK "No trueSpace executable was found in this directory.  Please make sure you are installing to your trueSpace directory - for trueSpace7 users, this means e.g. c:\trueSpace75\ts\"
  Abort "No trueSpace executable found"
FoundTS7:
  strcpy $UsingTS7 "yes"
FoundTS:
FunctionEnd

;--------------------------------
;Pages

  !define MUI_PAGE_CUSTOMFUNCTION_PRE CheckFor3DE
  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE ".\License.txt"
  !define MUI_DIRECTORYPAGE_TEXT_TOP "dribble must be installed within your trueSpace directory to ensure all the components are installed correctly.  Here are some examples:$\n$\nc:\trueSpace6\$\nc:\trueSpace75\ts\$\n$\nNote the trailing \ts\ directory for trueSpace7 users."
  !define MUI_DIRECTORYPAGE_TEXT_DESTINATION "Please locate your trueSpace folder:"
  !define MUI_PAGE_CUSTOMFUNCTION_LEAVE CheckForTS
  ;!define MUI_PAGE_CUSTOMFUNCTION_SHOW FindTSDir
  !insertmacro MUI_PAGE_DIRECTORY
  ;!insertmacro MUI_PAGE_COMPONENTS  
  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !define MUI_UNPAGE_CUSTOMFUNCTION_PRE CheckFor3DE
  !insertmacro MUI_UNPAGE_WELCOME
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH

;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections



Section "Plugin" SecPlugin
  
  ;strcmp $UsingTS7 "yes" 0 NoTS7
	;SetOutPath "$INSTDIR\..\"
	;goto BTIPL
NoTS7:
	; tS root (or should be)
  SetOutPath "$INSTDIR"

BTIPL:
  ; install BTNexGen DLLs
  File ".\data\BTNexgenIPL32.dll"
  
  ; make tsx directory
  CreateDirectory "$INSTDIR\tSX\dribble"
  CreateDirectory "$INSTDIR\tSX\dribble\docs"
  CreateDirectory "$INSTDIR\tSX\dribble\workingdata"
  SetOutPath "$INSTDIR\tSX\dribble"
  File ".\data\dribble1.04wip080808_ts51.tsx"
  File ".\data\dribble1.04wip080808_ts6or7.tsx"
  File ".\data\default.drb"
  File ".\dribble.txt"
  
  SetOutPath "$INSTDIR\Shaders\Material"
  File ".\data\tsshaders\*.tss"
  
  
  SetOutPath "$INSTDIR\tSX\dribble\workingdata"
  File ".\data\workingdata\readme.txt"
  
  SetOutPath "$INSTDIR\tSX\dribble\docs"
  File /r ".\data\docs\*.*"
  
  ; 3Delight shader path
  SetOutPath "$3DERoot\shaders"
  File ".\data\3deshaders\*.*"
  
  ; 3Delight displays path
  SetOutPath "$3DERoot\displays"
  File ".\data\3dedisplays\*.*"

  ;Store installation folder
  WriteRegStr HKCU "Software\cool powers\dribble" "Install_Dir" $INSTDIR

  ;Create uninstaller
  WriteUninstaller "$INSTDIR\tSX\dribble\dribble1.04wip080808_uninstall.exe"
  
  ; show the quickstart
  ExecShell "open" "$INSTDIR\tSX\dribble\docs\quickstart.html"

SectionEnd

;--------------------------------


;Descriptions

  ;Language strings
  LangString DESC_SecPlugin ${LANG_ENGLISH} "The plugin files"

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !insertmacro MUI_DESCRIPTION_TEXT ${SecPlugin} $(DESC_SecPlugin)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  Delete "$INSTDIR\tSX\dribble\docs\*.*"
  Delete "$INSTDIR\tSX\dribble\docs\images\*.*"
  RMDir "$INSTDIR\tSX\dribble\docs\images"
  RMDir "$INSTDIR\tSX\dribble\docs"
  
	Delete "$INSTDIR\tSX\dribble\workingdata\*.*"
  RMDir "$INSTDIR\tSX\dribble\workingdata"
  
  Delete "$INSTDIR\tSX\dribble\dribble1.04wip080808_ts51.tsx"
  Delete "$INSTDIR\tSX\dribble\dribble1.04wip080808_ts6or7.tsx"
  Delete "$INSTDIR\tSX\dribble\dribble.txt"
  Delete "$INSTDIR\tSX\dribble\default.drb"
  
  Delete "$INSTDIR\Shaders\Material\drbSimpleSSS.tss"
  Delete "$INSTDIR\Shaders\Material\drbSimpleFur.tss"
  
  
  ; delete 3DE stuff
  Delete "$3DERoot\shaders\tSambientlight.sdl"
  Delete "$3DERoot\shaders\tSBGcolor.sdl"
  Delete "$3DERoot\shaders\tSBGgradient.sdl"
  Delete "$3DERoot\shaders\tSBGimage.sdl"
  Delete "$3DERoot\shaders\tSbumpmap.sdl"
  Delete "$3DERoot\shaders\tScausticlight.sdl"
  Delete "$3DERoot\shaders\tSdepthcue.sdl"
  Delete "$3DERoot\shaders\tSenvlight.sdl"
  Delete "$3DERoot\shaders\tSfog.sdl"
  Delete "$3DERoot\shaders\tSfur.sdl"
  Delete "$3DERoot\shaders\tSglass.sdl"
  Delete "$3DERoot\shaders\tShair.sdl"
  Delete "$3DERoot\shaders\tShair_solid.sdl"
  Delete "$3DERoot\shaders\tShdri.sdl"
  Delete "$3DERoot\shaders\tSindirectlight.sdl"
  Delete "$3DERoot\shaders\tSinfinitelight.sdl"
  Delete "$3DERoot\shaders\tSnormalmap.sdl"
	Delete "$3DERoot\shaders\tSpointlight.sdl"
	Delete "$3DERoot\shaders\tSprojectorlight.sdl"
	Delete "$3DERoot\shaders\tSskylight.sdl"
	Delete "$3DERoot\shaders\tSspotlight.sdl"
	Delete "$3DERoot\shaders\tSsss.sdl"
	Delete "$3DERoot\shaders\tSstandard.sdl"
	
	Delete "$3DERoot\displays\tsdd.dpy"

  Delete "$INSTDIR\tSX\dribble\dribble1.04wip080808_uninstall.exe"

  RMDir "$INSTDIR\tSX\dribble"

  DeleteRegKey /ifempty HKCU "Software\cool powers\dribble"

SectionEnd






