/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Team.                                         */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/

/*
	A light source for image based lighting. 
	
	PARAMETERS
	Kocc      : if >0, use occlusion() to compute the bent normal and to
	            to scale the result (occluded regions will appear darker)
	samples   : number of samples to use when calling occlusion().

	---
	Aghiles Kheffache.
	Mon Jan  5 01:24:42 EST 2004
*/
light DIenvlight(
  float intensity = 1.0; 
  float exposure_compensation_stops = 0.0;
  float Kocc = 0.0;
  string envmap = "", envspace = "world";
  float samples = 64; float __nonspecular = 1; )
{
	normal shading_normal = faceforward( Ns, I );

	illuminate( Ps + shading_normal ) /* shade all surface points */
	{
		if( intensity > 0.0 )
		{
			vector envdir = vector shading_normal;
			float occ = 0;

			if( Kocc != 0.0 )
			{
				occ = Kocc * occlusion( Ps, normalize(shading_normal), samples,
					"environmentdir", envdir, "distribution", envmap,
					"environmentspace", envspace );
			}

			vector world_envdir = normalize( vtransform("world", envdir) );
			float exposure = pow(2,exposure_compensation_stops);
			Cl =  indirectdiffuse( envmap, world_envdir ) * (intensity * exposure) * (1-occ);
		}
		else
			Cl = 0;
	}
}
