/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Team.                                         */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/

/*
	A light source for image based lighting. 
	
	PARAMETERS
	Kocc      : if >0, use occlusion() to compute the bent normal and to
	            to scale the result (occluded regions will appear darker)
	samples   : number of samples to use when calling occlusion().

	---
	Aghiles Kheffache.
	Mon Jan  5 01:24:42 EST 2004
*/
light tSenvlight(
  float intensity = 1.0; 
  float exposure_compensation_stops = 0.0;
  float Kocc = 0.0;
  string envmap = "", envspace = "world";
  float maxdist = 1e10;
  float bias = -1;
  float samples = 64; float __nonspecular = 0; )
{
	normal shading_normal = faceforward( Ns, I );
	uniform float t1 = 0.025 * sqrt(samples);
	uniform float t2 = 0.05 * sqrt(samples);
	uniform float exposure = pow(2,exposure_compensation_stops);
	
	illuminate( Ps + shading_normal ) /* shade all surface points */
	{
		Cl = 0;
		if( intensity > 0.0 )
		{
			/*vector envdir = vector shading_normal;
			vector world_envdir = normalize( vtransform("world", envdir) );
			float occ = 0;

			if( Kocc != 0.0 )
			{
			  occ = Kocc * occlusion( Ps, normalize(shading_normal), samples,
					"environmentdir", envdir, "distribution", envmap,
					"environmentspace", envspace, "maxdist", maxdist, "bias", bias );
			}

			
			
			
			
			float exposure = pow(2,exposure_compensation_stops);
			Cl =  indirectdiffuse( envmap, world_envdir ) * (intensity * exposure) * (1-occ);*/
			
			string gather_cat = concat("environment:", envmap);
			color envcolor = 0;
			vector envdir = 0, raydir = 0;
			float solidangle = 0;
			
			gather(
				gather_cat, 0, 0, 0, samples,
				"environment:color", envcolor,
				"ray:direction", raydir,
				"environment:direction", envdir,
				"environment:solidangle", solidangle )
			{
				
				raydir = vtransform( envspace, "current", raydir );
				envdir = vtransform( envspace, "current", envdir );
				
				vector world_envdir = normalize( vtransform("current", shading_normal) );
			

				float atten = max( world_envdir . normalize(envdir), 0 );
				
				float kocc = 1 - smoothstep( t1, t2, solidangle );
				kocc *= Kocc;

				color trs = 1;
				if( kocc > 0 && atten > 0 )
				{
					trs = transmission(
						Ps, Ps + raydir * maxdist, "bias", bias );
					trs = 1 - kocc * (1 - trs);
				}

				Cl += (envcolor * (intensity * exposure)) * trs * atten;
			}
		}
	}
}
