color sharpspecular(normal N; vector V; float roughness, sharpness;)
{
    color C = 0;
    extern point P;

    if (sharpness==0)
      C=specularstd(N,V,roughness); /* standard specular */
    else {
      illuminance (P, N, PI/2) {
        extern vector L;  extern color Cl;
        float nonspec = 0;
        lightsource ("__nonspecular", nonspec);
        if (nonspec < 1) {
           float f=comp(specularbrdf(L,N,V,roughness),0);
           f=mix(f,smoothstep(sharpness*.495,1-sharpness*.495,f),sharpness);
           C += Cl * ((1-nonspec) * f);
        }
      }
    }
    return C;
}

surface tSglass(

float Ka = .1;
string Ka_map = "";

float Kd = .6;
string Kd_map = "";

float Ks = .7; 
string Ks_map = "";

float roughness = .04;
string roughness_map = "";
float sharpness = 0.77;

color specularcolor = 1;

float Kr = 0; 
string Kr_map = "";

float Kt = 0; 
string Kt_map = "";

float ior = 1.3;
string ior_map = "";

float metal = 0;

float maskureps = 1;
float maskvreps = 1;
float maskuoffset = 0;
float maskvoffset = 0;
string maskmap = ""; 
float maskfilter = 0;

float texureps = 1;
float texvreps = 1;
float texuoffset = 0;
float texvoffset = 0;
string texmap = ""; 

float raytracing = 0;
string envmap = "";
float envexposure = 0.0;
color bg_color = 0;

output varying color __ambient = color(0);
output varying color __diffuse = color(0);
output varying color __specular = color(0);
output varying color __reflect = color(0);
output varying color __refract = color(0);

 )
{

	float calcKa = Ka;
	float calcKd = Kd;
	float calcKs = Ks;
	float calcroughness = roughness;
	float calcKr = Kr;
	float calcKt = Kt;
	float calcior = ior;
	
	normal Nn = normalize(N);
    normal Nf = faceforward(Nn, I);
    vector V = normalize(-I);
    vector In = normalize( I );
        
	
	Oi = Os;
	
	if ( maskmap != "" )
	{
		float ss = (s + maskuoffset) * maskureps;
		float tt = (t + maskvoffset) * maskvreps;
		
		if ( maskfilter > 0 )
		{
			Oi = 1 - texture( maskmap, ss, tt, "filter", "gaussian" );
		}
		else
		{
			float bpp;
			textureinfo( maskmap, "channels", bpp );
			if ( bpp == 4 )
			{
				float alpha = texture( maskmap[3], ss, tt, "filter", "gaussian", "fill", 1.0 );
				Oi = color(alpha, alpha, alpha);
			}
			else
			{
				color pixel = texture( maskmap, ss, tt, "filter", "gaussian" );
				//float val = (0.3*comp(pixel,0) + 0.59*comp(pixel,1) + 0.11*comp(pixel,2));
				float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
				Oi = color( val, val, val );
			}
		}
		
	}
	
	color Ct = Cs;
	if ( texmap != "" )
	{
		float ss = (s + texuoffset) * texureps;
		float tt = (t + texvoffset) * texvreps;
		
		Ct = texture( texmap, ss, tt, "filter", "gaussian" );
	}
		
	color speccol = specularcolor;
    if ( metal > 0.0 )
		speccol = Ct;
		
	// now filter the channels by the maps if needed
	if ( Ka_map != "" )
	{
		color pixel = texture( Ka_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcKa *= val;
	}
	if ( Kd_map != "" )
	{
		color pixel = texture( Kd_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcKd *= val;
	}
	if ( Ks_map != "" )
	{
		color pixel = texture( Ks_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcKs *= val;
	}
	if ( roughness_map != "" )
	{
		color pixel = texture( roughness_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcroughness *= val;
	}
	if ( Kr_map != "" )
	{
		color pixel = texture( Kr_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcKr *= val;
	}
	if ( Kt_map != "" )
	{
		color pixel = texture( Kt_map, "filter", "gaussian" );
		float val = (comp(pixel,0) + comp(pixel,1) + comp(pixel,2)) / 3;
		calcKt *= val;
	}
		
	float kr, kt;
	vector Rdir, Tdir;
	fresnel ( In, Nn, (I.N < 0) ? 1.0/calcior : calcior, kr, kt, Rdir, Tdir);
	kt = 1 - kr;
	kr *= calcKr;
	kt *= calcKt;
	
	uniform float ray_depth;
	uniform string ray_type;
	rayinfo( "depth", ray_depth );
	rayinfo( "type", ray_type );
			
	if ( ray_type == "transmission" )
	{

      if ( N.I > 0)
        Oi = 0;
      else
        //Oi = (1+Ct*kt*Nf.In); /* not correct, but looks ok */ /*(1-kt);*/
        //Oi = ( (1 - kt) * (1 - Ct) );
        //Oi = (Ct - kt);
        Oi *= 1 - (kt * Ct);
      Ci = Ct;
    
    } 
    else 
    {
		if ( ( kr > 0 ) || ( kt > 0 ) )	
		{
			
			
			
			if ( ray_depth > 1 )
				kr = 0;
			if ( ray_depth > 6 )
				kt = 0;
				
			//if ( raylevel() > 0 )
			//	kr = 0;
			
			color envcolor = 0;
			if ( envmap != "" )
			{
				envcolor = environment( envmap, vtransform( "world", Rdir ) );
				envcolor *= pow( 2, envexposure );
			}
			else
				envcolor = bg_color;
				
			if ( kr > 0 )
			{
				color hitcolor = 0;			
				gather("illuminance", P, Rdir, 0, 1, "surface:Ci", hitcolor)
				{
					   __reflect = hitcolor;
				}
				else
				{
						__reflect = envcolor;
				}
					
				if ( metal > 0 )
					__reflect *= Ct;
						
				__reflect *= kr;
				
				__reflect *= Oi;
			}
			
			if ( kt > 0 )
			{
				
				if ( raytracing > 0 )
				{
					__refract = trace( P, Tdir );
					//__refract = environment( "raytrace", Tdir );
					//Oi = 1;
				}
				else
					__refract = 0;
					
				__refract *= Ct;					
				__refract *= kt;
				__refract *= Oi;
			}		
		}
		
		__ambient = Oi * Ct * calcKa * ambient();
		__diffuse = Oi * Ct * calcKd * diffuse(Nn);
		__specular = Oi * speccol * calcKs * specularstd( Nf, -In, calcroughness );
		
		Ci = __ambient + __diffuse + __specular + __reflect + __refract;
	}
	
}
