#include "stdafx.h"
#include "drbHair.h"
#include "MaterialSample.h"

#include "../WriteMaterial.h"

void RenderMaterialSample()
{
	char tempdir[_MAX_PATH];
	GetTempPath(_MAX_PATH, tempdir);
	CString temppath(tempdir);
		
	CStdioFile vib;
	CStdioFile vs;

	vib.Open(temppath + "vomatsample.vib", CFile::modeCreate | CFile::modeWrite | CFile::typeText);
	vs.Open(temppath + "vomatsample.vs", CFile::modeCreate | CFile::modeWrite | CFile::typeText);

	// shader file basics
	vs.WriteString("ReadArchive \"statics.vib\"\n");
	vs.WriteString("Declare stripe = SpectrumOfColors(\n");
	vs.WriteString("[0.000, '0.3, 0.3, 0.3', 0.04999, '0.3, 0.3, 0.3']\n");
	vs.WriteString("[0.050, '1, 1, 1', 0.0999, '1, 1, 1']\n");
	vs.WriteString("[0.100, '0.3, 0.3, 0.3', 0.14999, '0.3, 0.3, 0.3']\n");
	vs.WriteString("[0.150, '1, 1, 1', 0.2, '1, 1, 1'])\n");
	vs.WriteString("Declare BackgroundShader = Shader [ FunctionalSurface [\n");
	vs.WriteString("	Color stripe[fmod(u+v, 0.2)]\n");
	vs.WriteString("	Ka 1.000000\n");
	vs.WriteString("]\n");
	vs.WriteString("]\n");

	tsxMATERIAL *pMaterial;
	tsxMaterialCreate(&pMaterial);
	//tsxRenderMaterialSampleGetMaterial(&pMaterial);
	tsxMaterialCopyActive(pMaterial);
	MaterialOptions opt;
	opt.f = &vs;
	opt.UseCaustics = true;
	opt.ConvertProceduralShaders = true;
	opt.TexWidth = 128;
	opt.TexHeight = 128;
	opt.OneTexFile = true;
	opt.Filename = temppath + "vomatsample.vib";
	WriteMaterial(opt, pMaterial, "Ball", 0);
	tsxMaterialDestroy(pMaterial);

	vib.WriteString("Camera [\n");
	vib.WriteString("	(-0.086987,-0.014500,5.925819), (-0.086987,0.002952,4.925971), (0.000000,0.000000,1.000000)\n");
	vib.WriteString("	FieldOfView 28.134003\n");
	vib.WriteString("	Format (128, 128)\n");
	vib.WriteString("	FrameAspectRatio 1.0000\n");
	vib.WriteString("	LensModel PERSPECTIVE\n");
	vib.WriteString("]\n");
	vib.WriteString("Options [\n");
	vib.WriteString("	AdaptiveDepth 1\n");
	vib.WriteString("	ShadowBias 0.001000\n");
	vib.WriteString("	MaxRayLevel 6\n");
	vib.WriteString("	MaxBlurLevel 3\n");
	vib.WriteString("	MaxBlurSamples 4\n");
	vib.WriteString("]\n");
	vib.WriteString("DirectionalLight [\n");
	vib.WriteString("	(0.450813,0.388035,0.803863)\n");
	vib.WriteString("	Intensity '1.000000,1.000000,1.000000'*0.500000\n");
	vib.WriteString("	LightingAttributes SPECULAR\n");
	vib.WriteString("]\n");
	vib.WriteString("\n");
	vib.WriteString("\n");
	vib.WriteString("\n");
	vib.WriteString("Shape [\n");
	vib.WriteString("Sphere('0,0,0', 1.25)\n");
	vib.WriteString("ShapeSubdivisions(16,16)\n");
	vib.WriteString("RenderBound (0,0,1.00075,1.00075)\n");
	vib.WriteString("BallShader\n");
	vib.WriteString("ShadingAttributes 191\n");
	vib.WriteString("]\n");
	vib.WriteString("\n");
	vib.WriteString("\n");
	vib.WriteString("\n");
	vib.WriteString("Shape [\n");
	vib.WriteString("	NURBS(\n");
	vib.WriteString("	4, 4, [ 0.000000, 0.000000, 0.000000, 0.000000, 1.000000, 1.000000, 1.000000, 1.000000 ], \n");
	vib.WriteString("	4, 4, [ 0.000000, 0.000000, 0.000000, 0.000000, 1.000000, 1.000000, 1.000000, 1.000000 ], \n");
	vib.WriteString("	[\n");
	vib.WriteString("	[ (-6.244532,6.244532,0.000000) , (-6.244532,2.081511,0.000000) , (-6.244532,-2.081511,0.000000) , (-6.244532,-6.244532,0.000000)  ],\n");
	vib.WriteString("	[ (-2.081511,6.244532,0.000000) , (-2.081511,2.081511,0.000000) , (-2.081511,-2.081511,0.000000) , (-2.081511,-6.244532,0.000000)  ],\n");
	vib.WriteString("	[ (2.081511,6.244532,0.000000) , (2.081511,2.081511,0.000000) , (2.081511,-2.081511,0.000000) , (2.081511,-6.244532,0.000000)  ],\n");
	vib.WriteString("	[ (6.244532,6.244532,0.000000) , (6.244532,2.081511,0.000000) , (6.244532,-2.081511,0.000000) , (6.244532,-6.244532,0.000000) ]\n");
	vib.WriteString("	] )\n");
	vib.WriteString("	BackgroundShader\n");
	vib.WriteString("	ShapeSubdivisions(16,16)\n");
	vib.WriteString("	RenderBound (0,0,1.00075,1.00075)\n");
	vib.WriteString("]\n");


	vs.Close();
	vib.Close();

}